#!/usr/bin/perl -w

###############################################################################
###############################################################################
##
## This file is part of CG-seq.
##
##  CG-seq is free software: you can redistribute it and/or modify
##  it under the terms of the GNU General Public License as published by
##  the Free Software Foundation, either version 3 of the License, or
##  (at your option) any later version.
##
##  CG-seq is distributed in the hope that it will be useful,
##  but WITHOUT ANY WARRANTY; without even the implied warranty of
##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##  GNU General Public License for more details.
##
##  You should have received a copy of the GNU General Public License
##  along with CG-seq.  If not, see <http://www.gnu.org/licenses/>.
##
###############################################################################
###############################################################################


###############################################################################
###############################################################################
##
## 	Script : CG-seq.pl
##	Author : Benjamin Grenier-Boley <benjamin.grenier-boley@inria.fr>
##
##	Date : July 2009
##
##	Goal : Execute the comparative genomics software pipeline
##             according to the configuration files
##
###############################################################################
###############################################################################


use strict;
use warnings;
use Getopt::Long qw (:config noauto_abbrev);
use Pod::Usage;
use File::Spec;
use File::Basename;
use FindBin;
use lib "$FindBin::Bin/perl_modules";

use configuration_handler;
use software_handler;
use general_functions;


###############################################################################
###############################################################################
##
##
##			     DEFAULT CONFIGURATION
##
##
###############################################################################
###############################################################################

#Default directory for configuration files
my $default_conf_dir = File::Spec->catdir(dirname($0),"cfg");

#All pipeline wanted value
my $all = 0;

#Masking coding regions values
my $mask_coding_wanted = 0;
my $mask_coding_conf = File::Spec->catfile($default_conf_dir,
					   "mask_regions.conf");
my $mask_coding_outdir_name = "mask_regions_genomes";
my $cmd_mask_coding = "";
my $mask_coding_available = 0;
my $mask_cds_wanted = 0;
my $mask_rna_wanted = 0;

#Masking redundant regions values
my $mask_redundancy_wanted = 0;
my $mask_redundancy_conf = File::Spec->catfile($default_conf_dir,
					      "mask_redundancy.conf");
my $mask_redundancy_outdir_name = "mask_redundancy_genomes";
my $cmd_mask_redundancy = "";
my $mask_redundancy_available = 0;


#Alignments values
my $alignments_outdir_name = "alignments";

#Yass values
my $yass_wanted = 0;
my $yass_conf = File::Spec->catfile($default_conf_dir,
				    "yass.conf");
my $yass_outdir_name = "yass";
my $cmd_yass = "";
my $yass_available = 0;


#Blast values
my $blast_wanted = 0;
my $blast_conf = File::Spec->catfile($default_conf_dir,
				     "blast.conf");
my $blast_outdir_name = "blast";
my $cmd_blast = "";
my $blast_available = 0;

#CGseqcore values
my $cg_seq_wanted = 0;
my $cg_seq_conf = File::Spec->catfile($default_conf_dir,
				      "cgseqcore.conf");
my $cg_seq_outdir_name = "conserved_regions";
my $cmd_cg_seq = "";
my $cg_seq_available = 0;

#Structure inference directory
my $structure_outdir_name = "structure_inference";

#Predicted RNA
my $rna_predicted_outdir_name = "RNA_predicted";

#Carnac values
my $carnac_wanted = 0;
my $carnac_conf = File::Spec->catfile($default_conf_dir,
				      "carnac.conf");
my $carnac_outdir_name = "carnac";
my $cmd_carnac = "";
my $carnac_available = 0;


#RNAz values
my $rnaz_wanted = 0;
my $rnaz_conf = File::Spec->catfile($default_conf_dir,
				    "rnaz.conf");
my $rnaz_outdir_name = "rnaz";
my $cmd_rnaz = "";
my $rnaz_available = 0;

#rnazWindow.pl configuration
my $rnazwindow_conf = File::Spec->catfile($default_conf_dir,
					  "rnazwindow.conf");
my $rnazwindow_available = 0;
my $cmd_rnazwindow = "";


#RNAplot configuration
my $rnaplot_conf = File::Spec->catfile($default_conf_dir,
				       "rnaplot.conf");
my $cmd_rnaplot = "";
my $rnaplot_available = 0;

#ClustalW configuration
my $clustalw_conf = File::Spec->catfile($default_conf_dir,
					"clustalw.conf");
my $cmd_clustalw = "";
my $clustalw_available = 0;

#Help wanted
my $help = 0;

#Quiet
my $quiet = 0;




###############################################################################
###############################################################################
##
##
##			     GLOBAL VARIABLES
##
##
###############################################################################
###############################################################################


#Reference file
my $reference_file = "";

#Location of reference file after masking coding regions
my $reference_file_mask = "";

#Array of fasta files or directories provided by user
my @other;

#Reference on the hash table of real directories or fasta files returned
#by the check_provided_file function
my $other_files;

#Orignial other files before masking coding regions
my $original_other_files;

#Location of the outdir
my $outdir = "";

#start time
my $start;

#end time
my $end;

#Name of the warning report file
my $warning_filename = "warnings.txt";

#Content of the warning file
my $content_warning = "";


###############################################################################
###############################################################################
##
##
##				     MAIN
##
##
###############################################################################
###############################################################################


#Parse options
GetOptions('mask-regions:s' => sub{$mask_coding_wanted = 1; 
				   $mask_coding_conf = $_[1] if ($_[1] ne "");},
	   'mask-redundancy:s' => sub{$mask_redundancy_wanted = 1;
				      $mask_redundancy_conf = $_[1] if ($_[1] ne "");},
	   'yass:s' => sub{$yass_wanted = 1;
			   $yass_conf = $_[1] if ($_[1] ne "");},
	   'blast:s' => sub{$blast_wanted=1;
			    $blast_conf = $_[1] if ($_[1] ne "");},
	   'conserved:s' => sub{$cg_seq_wanted = 1;
				$cg_seq_conf = $_[1] if ($_[1] ne "");},
	   'RNAz:s' => sub{$rnaz_wanted=1;
			   $rnaz_conf = $_[1] if ($_[1] ne "");},
	   'carnac:s' => sub{$carnac_wanted=1;
			     $carnac_conf = $_[1] if ($_[1] ne "");},
	   'o=s' => \$outdir,
	   'all' => \$all,
	   'help' => \$help,
	   'h' => \$help,
	   'quiet' => \$quiet) or pod2usage(2);


#Help
pod2usage(1) if $help;

#If no options : all pipeline by default
if($mask_coding_wanted == 0 && $mask_redundancy_wanted == 0 && $yass_wanted == 0
   && $blast_wanted == 0 && $cg_seq_wanted == 0 && $rnaz_wanted == 0
   && $carnac_wanted == 0){
  $all = 1;
}

#Get outdir reference file and other species files or directories
my $nb_args = @ARGV;

if($nb_args < 2){
  pod2usage(1);
}
$reference_file = $ARGV[0];
for(my $i = 1; $i < $nb_args; $i++){
  push(@other, $ARGV[$i]);
}


#Get starting execution time
if(!$quiet){
  $start = time();
}

#Check outdir
if($outdir eq ""){
  print "\nERROR: You must provide an output directory\n\n";
  pod2usage(1);
}

#Check every pipeline software availability according to the pipeline wanted
#and get command lines to execute the software. If a critical error occured :
#stop pipeline.

#Mask coding step
if($mask_coding_wanted || $all){
  ($mask_coding_available,$cmd_mask_coding) = 
    &get_software_availability($mask_coding_conf,
			       \$content_warning);

  if(!$mask_coding_available){
    if(!$all && !$mask_redundancy_wanted && !$yass_wanted && !$blast_wanted && 
       !$cg_seq_wanted && !$carnac_wanted && !$rnaz_wanted){
      my $error = "\nERROR : The masking script is not available\n\n";
      exit_script($error);

    }
    else{
      $content_warning .= 
	"\nWARNING : The masking of regions step will be skipped\n\n";
    }
  }
  else{

    #Masking only CDSs
    if($cmd_mask_coding =~ /--mode[ \t]+0/){
      $mask_cds_wanted = 1;
    }
    else{
      #Masking only RNAs
      if($cmd_mask_coding =~ /--mode[ \t]+1/){
	$mask_rna_wanted = 1;
      }

      else{
	if($cmd_mask_coding =~ /--mode[ \t]+2/){
	  $mask_cds_wanted = 1;
	  $mask_rna_wanted = 1;
	}
      }
    }


  }
}

if($mask_redundancy_wanted || $all){
  ($mask_redundancy_available,$cmd_mask_redundancy) = 
    &get_software_availability($mask_redundancy_conf,
			       \$content_warning);
  if(!$mask_redundancy_available){
    if(!$all && !$mask_coding_wanted && !$yass_wanted && !$blast_wanted && 
       !$cg_seq_wanted && !$carnac_wanted && !$rnaz_wanted){
      my $error = 
	"\nERROR : The redundancy masking script is not available\n\n";
      exit_script($error);
    }
    else{
      $content_warning .= 
	"\nWARNING : The masking of redundant regions step will be skipped\n\n";
    }
  }
}


#Alignment step
#Yass
if($yass_wanted || $all){
  ($yass_available,$cmd_yass) = &get_software_availability($yass_conf,
							  \$content_warning);
  if(!$yass_available){
    $content_warning .=
      "\nWARNING : The alignment step with Yass will be skipped\n\n";
  }
}
#Blast
if($blast_wanted || $all){
  ($blast_available,$cmd_blast) = &get_software_availability($blast_conf,
							  \$content_warning);
   if(!$blast_available){
    $content_warning .=
      "\nWARNING : The alignment step with Blast will be skipped\n\n";
  }
}
#Check alignemts software availability
if((($yass_wanted || $all) && !$yass_available) &&
   (($blast_wanted || $all) && !$blast_available)){
  my $message =  "\nERROR : Neither \"Yass\" software nor \"Blast\" are ";
  $message .= "available\n\tCheck your configuration files or install at ";
  $message .= "least one of these two tools\n";
  exit_script($message);
}
else{
  if(!$blast_wanted && !$all){
    if((($yass_wanted) && !$yass_available)){
      my $message =  "\nERROR : \"Yass\" software is not available";
      $message .= "\n\tCheck your configuration file or install it\n";
      exit_script($message);
    }
  }
  else{
    if(!$yass_wanted && !$all){
      if((($blast_wanted) && !$blast_available)){
	my $message =  "\nERROR : \"Blast\" software is not available";
	$message .= "\n\tCheck your configuration file or install it\n";
	exit_script($message);
      }
    }
  }
}

#Conserved region software
if($all || $cg_seq_wanted){
  ($cg_seq_available,$cmd_cg_seq) = 
    &get_software_availability($cg_seq_conf,
			       \$content_warning);
  if(!$cg_seq_available){
    my $message =  "\nERROR : \"CGseqcore\" software is not available";
    $message .= "\n\tCheck your configuration file or install it\n";
    exit_script($message);
  }
}

#Structure predictors
#Carnac
if($carnac_wanted || $all){
  ($carnac_available,$cmd_carnac) = &get_software_availability($carnac_conf,
							       \$content_warning);
  if(!$carnac_available){
    my $message = "The structure prediction step with \"carnac\"  will ";
    $message .= "be skipped\n\n";
    $content_warning .= "\nWARNING : $message";
  }
}
#RNAz
if($rnaz_wanted || $all){
  ($rnaz_available,$cmd_rnaz) = &get_software_availability($rnaz_conf,
							  \$content_warning);
  ($clustalw_available,$cmd_clustalw) = 
    &get_software_availability($clustalw_conf,
			       \$content_warning);
  ($rnazwindow_available,$cmd_rnazwindow) = 
    &get_software_availability($rnazwindow_conf,
			       \$content_warning);
  if($rnazwindow_available){
    my $dev_null = File::Spec->devnull();
    $cmd_rnazwindow =~ s/1>$dev_null 2>&1//g;
  }

  if(!$clustalw_available){
    my $message = "\"Clustalw\" is not available\n\tCheck you configuration";
    $message .= " file or install it to use \"RNAz\"";
    $content_warning .= "WARNING : $message\n\n";
    $rnaz_available = 0;
  }
  if(!$rnaz_available){
    my $message = "The structure prediction step with \"RNAz\" will ";
    $message .= "be skipped\n\n";
    $content_warning .= "\nWARNING : $message";
  }
  else{
    if(!$rnazwindow_available){
      my $message = "\"rnazWindow.pl script not found: too long sequences";
      $message .= " will not be taken into account\n\n";
      $content_warning .= "\nWARNING : $message";
    }
  }
}
#RNAplot
if($all || $rnaz_wanted || $carnac_wanted){
  ($rnaplot_available,$cmd_rnaplot) = &get_software_availability($rnaplot_conf);
  if(!$rnaplot_available){
    my $message = "\"RNAplot\" is not available\n\tCheck your configuration ";
    $message .= "file or install it if you want to produce 2D drawings of RNA structures \n";
    $content_warning .= "\nWARNING : $message\n";
  }
}
#Check structure prediction software availability
if((($carnac_wanted || $all) && !$carnac_available) &&
   (($rnaz_wanted || $all) && !$rnaz_available)){
  my $message =  "\nERROR : Neither \"carnac\" software nor \"RNAz\" are ";
  $message .= "available\n\tCheck the path in the configuration files or install at ";
  $message .= "least one of these two tools\n";
  exit_script($message);
}
else{
  if(!$rnaz_wanted && !$all){
    if((($carnac_wanted) && !$carnac_available)){
      my $message =  "\nERROR : \"carnac\" software is not available";
      $message .= "\n\tCheck the path in the configuration file or install it\n";
      exit_script($message);
    }
  }
  else{
    if(!$carnac_wanted && !$all){
      if($rnaz_wanted && !$clustalw_available){
	my $message = "\nERROR : \"ClustalW\" software is not available ";
	$message .= "and needed to use \"RNAz\". Check you configuration file";
	$message .= " or install it\n";
	exit_script($message);
      }
      else{
	if($rnaz_wanted && !$rnaz_available){
	  my $message =  "\nERROR : \"RNAz\" software is not available";
	  $message .= "\n\tCheck your configuration file or install it\n";
	  exit_script($message);
	}
      }
    }
  }
}

#Check if the pipeline is correct : if at least one alignment software is
#wanted and at least one prediction structure software : check if the user
#used the option conserved to run CGseqcore
if(!$all){
  if(($yass_available || $blast_available) &&
     ($carnac_available || $rnaz_available) &&
     !$cg_seq_available){
    my $message = "You must use the option \"--conserved\" to find conserved";
    $message .= " regions if you want to run an alignment software and a ";
    $message .= "structure prediction one";
    exit_script("\nERROR : $message\n\n");

  }

}

#Check reference file and other files or directories existence
$other_files = check_provided_files($reference_file,\@other,\$content_warning);
$original_other_files = $other_files;


#Print information on the pipeline that will be executed
if(!$quiet){

 print "\n____________________________________\n\n"; 
 print "GENERAL INFORMATION ON EXECUTION\n\n";
  
  if($mask_coding_available){
    if($mask_cds_wanted && $mask_rna_wanted){
      print "   - Masking CDSs and RNAs\n";
    }
    else{
      if($mask_cds_wanted){
	print "   - Masking CDSs\n";
      }
      else{
	if($mask_rna_wanted){
	  print "  - Masking RNAs\n";
	}
      }
    }
  }
  if($mask_redundancy_available){
    print "   - Masking redundant regions between strains of a given species\n";
  }
  if($yass_available && $blast_available){
    print "   - Alignment step with Yass and Blast\n";
  }
  else{
    if($yass_available){
      print "   - Alignment step with Yass\n";
    }
    else{
      if($blast_available){
	print "   - Alignment step with Blast\n";
      }
    }
  }
 if($cg_seq_available){
     print "   - Find conserved regions with CGseqcore\n";
 }
 if($rnaz_available && $carnac_available){
     print "   - Structure prediction with caRNAc and RNAz\n";
 } 
 else{
     if($carnac_available){
	 print "   - Structure prediction with caRNAc\n";
     }
     else{
	 if($rnaz_available){
	     print "   - Structure prediction with RNAz\n";
	 }
     }
 } 
}


##############################################################################
#
#                          PIPELINE BEGINNING
#
##############################################################################


#Deal with outdirs



create_directory($outdir);

my $mask_coding_outdir = File::Spec->catdir($outdir,
					    $mask_coding_outdir_name);

my $mask_redundancy_outdir = File::Spec->catdir($outdir,
						$mask_redundancy_outdir_name);

my $alignments_outdir = File::Spec->catdir($outdir,
					   $alignments_outdir_name);

my $yass_outdir = File::Spec->catdir($alignments_outdir,
				     $yass_outdir_name);

my $blast_outdir = File::Spec->catdir($alignments_outdir,
				      $blast_outdir_name);

my $cg_seq_outdir = File::Spec->catdir($outdir,
					   $cg_seq_outdir_name);

my $structure_outdir = File::Spec->catdir($outdir,
					  $structure_outdir_name);

my $rna_predicted_outdir = File::Spec->catdir($outdir,
					      $rna_predicted_outdir_name);

my $carnac_outdir = File::Spec->catdir($structure_outdir,
				       $carnac_outdir_name);

my $carnac_predicted_outdir = 
  File::Spec->catdir($rna_predicted_outdir,
		     $carnac_outdir_name);

my $rnaz_outdir = File::Spec->catdir($structure_outdir,
				     $rnaz_outdir_name);
my $rnaz_predicted_outdir = 
  File::Spec->catdir($rna_predicted_outdir,
		     $rnaz_outdir_name);


########################## MASKING CODING STEP ###############################
 
 if(!$quiet){
 print "\n____________________________________\n\n";
 print "STEP 1: PREPROCESSING\n\n";
}

if($mask_coding_available){

  my $start_mask;
  my $end_mask;

  if(!$quiet){
    $start_mask = time();
    print "Masking annotated features \n\n";
  }


  #Execute masking step
  ($reference_file_mask,
   $other_files) = &execute_masking_coding($reference_file,
					   $other_files,
					   $cmd_mask_coding,
					   $mask_coding_outdir,
					   \$content_warning,
					   $quiet);




  if(!$quiet){
    $end_mask = time();
    my $time_diff = $end_mask - $start_mask;
    print "\nTime elapsed: $time_diff seconds\n";
  }




}
else{
  $reference_file_mask = $reference_file;
}


########################## MASKING REDUNDANCY STEP ###########################

if($mask_redundancy_available){

  my $start_mask;
  my $end_mask;

  if(!$quiet){
    $start_mask = time();
    print "Masking redundant regions within a species\n\n";
    
  }


  #Execute masking step
  $other_files = &execute_masking_redundancy($other_files,
					     $cmd_mask_redundancy,
					     $mask_redundancy_outdir,
					     \$content_warning,
					     $quiet);


  if(!$quiet){
    $end_mask = time();
    my $time_diff = $end_mask - $start_mask;
    print "\nElapsed time: $time_diff seconds\n\n";
  }


}


 if(!$quiet){
     print "\n____________________________________\n\n";
     print "STEP 2: ALIGNMENT\n\n";
}

######################## ALIGNMENT STEP WITH YASS ############################



if($yass_available){

  my $start_yass;
  my $end_yass;

  if(!$quiet){
    $start_yass = time();
   
    print "Running YASS\n";
  }

  #Create general alignments outdir if not exists
  &create_directory($alignments_outdir);

  #Execute alignments with Yass
  &execute_yass($reference_file_mask,
 		$other_files,
 		$cmd_yass,
 		$yass_outdir,
 		$quiet);


  if(!$quiet){
    $end_yass = time();
    my $time_diff = $end_yass - $start_yass;
    print 
      "\nElapsed time: $time_diff seconds\n\n";
  }



}



######################## ALIGNMENT STEP WITH BLAST ############################

if($blast_available){

  my $start_blast;
  my $end_blast;

  if(!$quiet){
    $start_blast = time();
    print "Running BLAST\n";
   
  }

  #Create general alignments outdir if not exists
  &create_directory($alignments_outdir);

  #Execute alignments with Blast
  &execute_blast($reference_file_mask,
 		 $other_files,
 		 $cmd_blast,
 		 $blast_outdir,
 		 $quiet);


  if(!$quiet){
    $end_blast = time();
    my $time_diff = $end_blast - $start_blast;
    print 
      "\nElapsed time: $time_diff seconds\n\n";
  }


}




######################## FIND CONSERVED REGIONS ############################

if($cg_seq_available){

  my $start_cg_seq;
  my $end_cg_seq;

  if(!$quiet){
      print "\n____________________________________\n\n"; 
      print "STEP 3: FINDING CONSERVED REGIONS \n\n";
      $start_cg_seq = time();
      print "Running CGseqcore\n";
  }

  #Create general conserved regions outdir if not exists
  &create_directory($cg_seq_outdir);

  &find_conserved_regions($reference_file_mask,
 			  $other_files,
 			  $cmd_cg_seq,
 			  $alignments_outdir,
			  $yass_available,
			  $blast_available,
 			  $yass_outdir_name,
 			  $blast_outdir_name,
 			  $cg_seq_outdir,
 			  $quiet);

  if(!$quiet){
    $end_cg_seq = time();
    my $time_diff = $end_cg_seq - $start_cg_seq;
    print "\nElapsed time: $time_diff seconds\n\n";
  }

}




######################## STRUCTURE PREDICTION #################################



if($rnaz_available || $carnac_available){




  my $start_structure;
  my $end_structure;


  if(!$quiet){
      $start_structure = time();
      print "\n____________________________________\n\n"; 
      print "STEP 4: RNA STRUCTURE PREDICTION \n\n";    
      
      if($rnaz_available && $carnac_available){
	  print "Running caRNAc and RNAz\n";
      } 
      else{
	  if($carnac_available){
	      print "Running caRNAc\n";
	  }
	  else{
	      if($rnaz_available){
		  print "Running RNAz\n";
	      }
	  }
      } 
      
  }
  
  #Create general structure and RNA prediction outdirs
  &create_directory($structure_outdir);
  &create_directory($rna_predicted_outdir);

  #Execute carnac and/or RNAz
  &execute_structure($carnac_available,
		     $rnaz_available,
		     $cmd_carnac,
		     $cmd_rnaz,
		     $cmd_clustalw,
		     $cmd_rnazwindow,
		     $cmd_rnaplot,
		     $cg_seq_outdir,
		     $yass_available,
		     $blast_available,
		     $yass_outdir_name,
		     $blast_outdir_name,
		     $carnac_outdir,
		     $rnaz_outdir,
		     $rna_predicted_outdir,
		     $quiet);


  if(!$quiet){
    $end_structure = time();
    my $time_diff = $end_structure - $start_structure;
    print 
      "\nElapsed time: $time_diff seconds\n\n";
  }


}







################################## RESULTS FILES ##############################

if($rnaz_available || $carnac_available){

  my $start_result;
  my $end_result;

  if(!$quiet){
    $start_result = time();
    print "\n____________________________________\n\n";
    print "RESULTS\n";
     }

  #Get end time
  my $end_pipeline = time();

  #Global time difference
  my $time_diff_pipeline = $end_pipeline - $start;

  #Warning file
  my $warning_file = File::Spec->catfile($outdir,
					 $warning_filename);;
  if(!($content_warning eq "")){

    &write_file($content_warning,$warning_file);

  }
  else{
    if(-e $warning_file){
      unlink($warning_file);
    }
    $warning_file = "";
  }

  #produce result files
  &produce_result_files($reference_file,
			$original_other_files,
			$mask_coding_available,
			$mask_cds_wanted,
			$mask_rna_wanted,
			$mask_redundancy_available,
			$yass_available,
			$blast_available,
			$cg_seq_available,
			$carnac_available,
			$rnaz_available,
			$all,
			$yass_outdir_name,
			$blast_outdir_name,
			$rna_predicted_outdir,
			$outdir,
			$warning_file,
			$time_diff_pipeline);

  if(!$quiet){
    $end_result = time();
    my $time_diff = $end_result - $start_result;
   
    print "\nAll output files can be found in directory \"$outdir\" \n";
    print "See \"results.html\" and worksheets (*.csv)\n";

    if(!($content_warning eq "")){
      print "\nSome warnings occurred during execution and can be found in the ";
      print "file \"$warning_file\"\n";

    }

  }



}

############################ EXECUTION TIME ###################################


if(!$quiet){
  #Get end time
  $end = time();

  #Global time difference
  my $time_diff = $end - $start;

  print "\n\n Total execution time: $time_diff seconds\n\n";

}



###############################################################################
###############################################################################
##
##
##			   USAGE AND POD DOCUMENTATION
##
##
###############################################################################
###############################################################################


=head1 NAME

C<CG-seq.pl> - Execute the comparative genomics pipeline CG-seq

=head1 SYNOPSIS

CG-seq.pl <target> <species_1..species_N> -o output_directory [options]

=over 8

=item B<target>

Location of the target FASTA file.

=item B<species1..species_N>

Location of the FASTA files or directories containing  FASTA files corresponding 
to other species to be compared to the target file. You should provide one file or 
one directory per species. The program will consider that all FASTA files located 
in a given directory come from the same species. 

=item B<-o>=OUTPUT DIRECTORY

Location of the output directory. 

=back

=head1 OPTIONS

=over 8

=item B<--all>

Process all software according to the configuration files provided in the 
configuration directory cfg (default).

=item B<--mask-regions>=CONF

Mask annotated regions (known CDSs and/or known noncoding RNAs)
according to parameters given in the configuration file CONF.  If no
configuration file is specified, it uses the default configuration
file located in the directory 'cfg': mask_region.conf.  This option
requires that you provide GENBANK annotation files with annotated
features. GENBANK files must be in the same directory than their
respective genome FASTA files and have the same name (e.g. genome.fna
<-> genome.gbk) with a suffix .gbk or .gb.  This option uses the
script 'mask_coding.pl' that can be used independently from CG-seq.

=item B<--mask-redundancy>=CONF

Clean up redundant regions according to the parameters given in the
configuration file CONF. If no configuration file is specified, it
uses the default configuration file located in the directory 'cfg':
mask_redundancy.conf

This option is implemented with the script mask_redundancy.pl.
mask_redundancy.pl can be used independently from CG-seq.  But note
that it also needs the configuration file named yass_redundancy.conf,
because YASS is invoked by the script to compare sequences.


=item B<--yass>=CONF

Use YASS software (L<http://bioinfo.lifl.fr/yass/index.php>) for the
alignment step according to the parameters given in the configuration
file CONF.  If no configuration file is specified, it uses the default
configuration file located in the directory cfg: yass.conf.  YASS
source files are located inside the 'software' directory, if needed.

=item B<--blast>=CONF

Use BLAST software (L<http://www.ncbi.nlm.nih.gov/BLAST/>) for the
alignment step according to the parameters in the configuration file
CONF.  If no configuration file is specified, it uses the default
configuration file located in the directory cfg: blast.conf

=item B<--conserved>=CONF

Use the CGseqcore software for the search of conserved regions step
according to the configuration file CONF. If no configuration file is
specified, it uses the default configuration file located in the
directory cfg: cgseqcore.conf.  The CGseqcore source files are located
inside the directory software if needed.

=item B<--RNAz>=CONF

Use RNAz software (L<http://www.tbi.univie.ac.at/~wash/RNAz/>) to
search for consensus conserved secondary structures.  It uses the
parameters given in the configuration file CONF.  If no configuration
file is psecified, it uses the default configuration file located in
the directory cfg : rnaz.conf. This software requires clustalW or
clustalW2 (L<http://www.ebi.ac.uk/clustalw/>) to do a multiple
sequence alignment.  The configuration file used for clustalW is
located inside the directory cfg as well under the name
clustalw.conf. RNAzWindow.pl script can be used too to deal with to
long sequences. Its configuration file is located inside the directory
cfg under the filename rnazwindow.conf.  To generate image files for
structures, RNAplot, part of the Vienna package, is needed
(L<http://www.tbi.univie.ac.at/RNA/>). Its configuration file
rnaplot.cfg is also located inside the directory cfg.

=item B<--carnac>=CONF

Use caRNAc software (L<http://bioinfo.lifl.fr/RNA/carnac/index.php>)
to search for consensus conserved secondary structures.  It uses the
parameters according to the configuration file CONF.  If no
configuration file is specified, it uses the default configuration
file located in the directory cfg: carnac.conf.  This software
requires clustalW or clustalW2 (L<http://www.ebi.ac.uk/clustalw/>) to
do a multiple sequence alignment.  The configuration file used for
clustalW is located inside the directory cfg as well under the name
clustalw.conf.  To generate image files for structures, RNAplot, part
of the Vienna package, is needed
(L<http://www.tbi.univie.ac.at/RNA/>). Its configuration file
rnaplot.cfg is also located inside the directory cfg.

=item B<--quiet>

Print no message at all

=item B<-h, --help>

Print a short help message and exit.

=back

=head1 DESCRIPTION

The goal of CG-seq is find new putative noncoding RNAs on a genomic sequence
by comparative analysis. The pipeline includes four categories of tools: 

=over 2

=item 1

Preprocessing 
-> mask_coding.pl (provided);
-> mask_redundancy.pl (provided);

=item 2 

Alignment
-> Yass (provided) and/or Blast;

=item 3

Find conserved regions
-> CGseqcore (provided)

=item 4

Prediction of RNA structures
-> caRNAc (provided) and/or RNAz.

=back

Result files: 

=over 2


=item *

HTML file: result.html, located in the output directory;

=item *

CSV files;

=back

=head1 AUTHORS

=over 2

=item *

Benjamin Grenier-Boley <benjamin.grenier-boley@inria.fr>

=item *

Helene Touzet <helene.touzet@lifl.fr>

=back

=cut
