###############################################################################
###############################################################################
##
## This file is part of CG-seq.
##
##  CG-seq is free software: you can redistribute it and/or modify
##  it under the terms of the GNU General Public License as published by
##  the Free Software Foundation, either version 3 of the License, or
##  (at your option) any later version.
##
##  CG-seq is distributed in the hope that it will be useful,
##  but WITHOUT ANY WARRANTY; without even the implied warranty of
##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##  GNU General Public License for more details.
##
##  You should have received a copy of the GNU General Public License
##  along with CG-seq.  If not, see <http://www.gnu.org/licenses/>.
##
###############################################################################
###############################################################################


###############################################################################
###############################################################################
##
## 	Package : configuration_handler.pm
##	Author : Benjamin Grenier-Boley <benjamin.grenier-boley@inria.fr>
##
##	Creation : 11/03/09
##     	Last modification : 03/06/09
##
##	Goal : Contains all functions to handle configuration files used
##             in the pipeline 
##
###############################################################################
###############################################################################


package configuration_handler;


use strict;
use warnings;
use File::Spec;
use File::Basename;
use FindBin;
use lib "$FindBin::Bin";

use general_functions;

require Exporter;


###############################################################################
###############################################################################
##
##
##		               MODULE EXPORT
##
##
###############################################################################
###############################################################################

our @ISA = qw(Exporter);


our @EXPORT = qw (check_provided_files
		  get_software_availability
		  test_configuration_file);




###############################################################################
###############################################################################
##
##
##		                 FUNCTIONS
##
##
###############################################################################
###############################################################################


###############################################################################
#
#	Function check_provided_files
#
#	Goal : check the reference file and the other files or directories
#              provided by the user. If there are some directories : get the
#              fasta files within it.
#
#	Param :
#               - $ref : the location of the reference file
#               - $ref_other : the reference on the array containing other
#                 files or directories
#               - $content_warning : the warning file content
#
#	Return :
#		a reference on an hash table containing other files :
#                - the hash table key is the directory (or file if is file)
#                - the hash table value is files within directory (or file if
#                  it's not a drectory)
#
###############################################################################


sub check_provided_files{

  my %other_files;

  (my $ref, my $ref_other, my $content_warning) = @_;


  #Check reference file
  if($ref eq ""){
    exit_script("\nERROR : You must provide a reference file\n");
  }

  if(! -e $ref){
    exit_script("\nERROR : The reference file \"$ref\" doesn't exist\n");
  }

  if(! -r $ref){
    exit_script("\nERROR : The reference file \"$ref\" isn't readable\n");
  }

  if(-d $ref){
    exit_script("\nERROR : The reference file \"$ref\" is a directory, not a file\n");
  }
  (my $id, my $seq) = &read_fasta($ref);
  if(($id eq "") || ($seq eq "")){
    exit_script("\nERROR : The reference file \"$ref\" is not in fasta format\n");
  }

  #Test other genomes provided
  my $nb_others = @$ref_other;
  if($nb_others == 0){
    exit_script("\nERROR : You must provide other fasta file(s) or directory(ies)\n");
  }

  foreach my $other(@$ref_other){

    if(! -e $other){
      exit_script("\nERROR : The other file or directory \"$other\" doesn't exist\n");
    }

    #File provided
    if(! -d $other){

      if(! -r $other){
	exit_script("\nERROR : The other fasta file \"$other\" isn't readable\n");
      }
      else{
	(my $name,
	 my $dir,
	 my $suffix) = fileparse($other,qr/\.[^.]*/);
	$dir =~ s/[\/]$//;
	$other_files{$dir} = $name.$suffix;
      }

    }
    #Directory provided
    else{
      my $search_dir = File::Spec->catfile($other,"*");
      if($search_dir =~ /\s/){
	$search_dir = quotemeta($search_dir);
      }
      my @search_files = glob("$search_dir");
      my $count_fasta_files = 0;
      foreach my $file(@search_files){
	(my $name, my $dir, my $suffix) = fileparse($file,qr/\.[^.]*/);
	if(($suffix eq ".fna") ||
	   ($suffix eq ".fasta") ||
	   ($suffix eq ".fa") ||
	   ($suffix eq ".dna") ||
	   ($suffix eq ".seq")){

	  if(! -r $file){
	    exit_script("\nERROR : Fasta file \"$file\" isn't readable\n");
	  }
	  $dir =~ s/[\/|\\]$//;
	  $other_files{$dir} .= $name.$suffix."\t";
	  $count_fasta_files++;
	}
      }

      #No fasta files within directory
      if($count_fasta_files == 0){
	my $error = "\nWARNING : There's no fasta files within directory \"";
	$error .= "$other\". Fasta files must suffixed by \".fa\", \".fasta\"";
	$error .= " ,\".fna\", \".dna\" or \".seq\" : it won't be taken into ";
	$error .= "account\n\n";
	$$content_warning .= $error;
      }
    }
  }

  #Check if there's at least one good other file or directory
  my $count_other = 0;
  foreach my $other(keys(%other_files)){
    $count_other++;
  }
  if($count_other == 0){
    exit_script("\nERROR : No other files provided : process will stop\n");
  }

  return \%other_files;


}



###############################################################################
#
#	Function test_software
#
#	Goal : The function will test if the path to execute the software is
#              good (existence + execution)
#
#	Param :
#               - $exe_path : the path to test
#	Returns :
#               - the rigth executable path (in case @PATH, or under script
#                 directory) or "" if not found or executable
#
###############################################################################

sub test_software{

  (my $exe_path) = @_;

  #First test : full path
  if((-e $exe_path)){
    return "\"".$exe_path."\"";
  }
  #Second test : check under script directory
  else{

    my $file_script_dir = File::Spec->catfile(dirname($0),
					      $exe_path);

    if(-e $file_script_dir){
      return "\"".$file_script_dir."\"";
    }
    #Last test : look under $PATH
    else{
      my @paths = File::Spec->path();
      foreach my $path(@paths){
	my $real_path = File::Spec->catfile($path,
					    $exe_path);

	if(-e $real_path){
	  return "\"".$real_path."\"";
	}

      }
    }
  }

  return "";

}



###############################################################################
#
#	Function test_configuration_file
#
#	Goal : The function will execute 2 tests on the configuration file :
#              presence and the fact that the file is readable
#
#	Param :
#		- $configuration_file : the configuration file to test
#
#	Returns :
#		- 1 if tests were successful, 0 otherwise
#
###############################################################################

sub test_configuration_file{

  my $return_code = 1;

  (my $configuration_file) = @_;


  if (! -e $configuration_file){
    print "\nERROR: The configuration file \"$configuration_file\" does not ";
    print "exists\n\n";
    $return_code = 0;
  }

  if(($return_code) && ! (-r $configuration_file)){
    print "\nERROR: The configuration file \"$configuration_file\" is not ";
    print "readable";
    $return_code = 0;
  }


  return $return_code;


}



###############################################################################
#
#	Function get_software_availability
#
#	Goal : The function will read the configuration file which contains
#              both the command line to execute a software, and its options
#
#	Param :
#		- $configuration_file : the configuration file
#               - $content_warning: the content of the warning file
#
#	Returns :
#               - $available : the boolean that says if the software is
#                 available or not
#		- $command_line : the command line to execute according to
#                 exe path and options read or "" if an error occured during
#                 the file parsing
#
###############################################################################

sub get_software_availability{

  (my $configuration_file, my $content_warning) = @_;


  #Software availability
  my $available = 1;

  #Command line to return
  my $command_line = "";

  #Exe path of the software to execute
  my $exe_path = "";

  #Array of options
  my @options;

  #Read the configuration file
  open(CONF, $configuration_file) or
    exit_script("\nERROR : Could not read the configuration file \"$configuration_file\"\n");

  while(my $line = <CONF>){

    #Don't take commentary and blank lines
    if ($line !~ /(^#|^$)/) {

      #Path to execute the software
      if ($line =~ /\s*exe_path = "?(.+)"?/) {
	$exe_path = $1;
	$exe_path =~ s/"//g;
      }
      #Software options
      else{
	chomp($line);
	push(@options,$line);

      }

    }

  }

  #No exe path : exit
  if($exe_path eq ""){
    $$content_warning .= 
      "\nWARNING : There's no \"exe_path\" in the configuration file ";
    $$content_warning .= "\"$configuration_file\"\n\n";
    return (0,$command_line);
  }

  #Test path to execute the software and get real path
  my $real_path = &test_software($exe_path);

  if($real_path ne ""){

    #Generate command line
    $command_line = $real_path;
    foreach my $option(@options){
      $command_line .= " ".$option;
    }
    my $dev_null = File::Spec->devnull();
    $command_line .= " 1>$dev_null 2>&1";

  }
  else{
    $$content_warning .= 
      "\nWARNING : The path to execute the software : \"$exe_path\"";
    $$content_warning .= " is not correct or the command is not executable\n\n";
    $available = 0;
  }

  return ($available,$command_line);


}




1;



###############################################################################
###############################################################################
##
##
##			      POD DOCUMENTATION
##
##
###############################################################################
###############################################################################

=head1 NAME

Package I<configuration_handler> - Handle configuration files of the CG-seq

=head1 SYNOPSIS

Contains all functions to handle configuration files used  in the CG-seq.

Provided functions :

=over 2

=item *

check_provided_files

=item *

read_configuration_file

=item *

test_software

=item *

test_configuration_file

=back

=head1 EXPORT

Exported functions :

=over 2

=item *

check_provided_files

=item *

read_configuration_file

=item *

test_configuration_file

=back

=head1 FUNCTIONS

=head2 check_provided_file

Check the target file and the other files or directorie provided by the user.
If there are some directories : get the Fasta files within it.

=over 2

=item *

parameters :

=over 2

=item *

$ref, the location of the target file

=item *

$ref_other, the reference on the array containing other files or directories

=item *

$content_warning, the content of the warning file

=back

=item *

returns : a reference on an hash table containing other files

=back

=head2 get_software_availability

The function will read the configuration file which contains  both the command
line to execute a software and its options

=over 2

=item *

parameters :

=over 2

=item *

$configuration_file, the configuration file

=item *

$content_warning, the content of the warning file

=back

=item *

returns : the command line to execute according to exe path and options read 
or the empty string if an error occured during the file parsing and the 
software availability

=back

=head2 test_software

The function will test if the path to execute the software is good 
(existence + execution).

=over 2

=item *

parameters : $exe_path, the path to test

=item *

returns : the right executable path (in case @PATH, or under script directory)
or empty string if not found or not executable

=back

=head2 test_configuration_file

The function will execute two tests on the configuration file:  does it exist ? 
is is readable ? 

=over 2

=item *

parameters : $configuration_file, the configuration file to test

=item *

returns : 1 if tests were successful, 0 otherwise

=back

=head1 AUTHOR

Benjamin Grenier-Boley <benjamin.grenier-boley@inria.fr>

=cut

