###############################################################################
###############################################################################
##
## This file is part of CG-seq.
##
##  CG-seq is free software: you can redistribute it and/or modify
##  it under the terms of the GNU General Public License as published by
##  the Free Software Foundation, either version 3 of the License, or
##  (at your option) any later version.
##
##  CG-seq is distributed in the hope that it will be useful,
##  but WITHOUT ANY WARRANTY; without even the implied warranty of
##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##  GNU General Public License for more details.
##
##  You should have received a copy of the GNU General Public License
##  along with CG-seq.  If not, see <http://www.gnu.org/licenses/>.
##
###############################################################################
###############################################################################


###############################################################################
###############################################################################
##
## 	Package : general_fucntions.pm
##	Author : Benjamin Grenier-Boley <benjamin.grenier-boley@inria.fr>
##
##	Creation : 16/03/09
##     	Last modification : 03/06/09
##
##	Goal : Contains general functions used CG-seq such as sorting
##             files by number or getting the reverse of a nucleotide base or
##             reading or writing a file
##
###############################################################################
###############################################################################


package general_functions;


use strict;
use warnings;
use File::Basename;
use File::Path;

require Exporter;


###############################################################################
###############################################################################
##
##
##		               MODULE EXPORT
##
##
###############################################################################
###############################################################################

our @ISA = qw(Exporter);


our @EXPORT = qw (exit_script
		  read_fasta
		  read_file
		  write_file
		  create_directory
		  erase_existing_files
		  sort_by_num
		  sort_by_pos
		  get_ratio_paired_bases
		  get_reverse_base
		  execute_reverse_complementary
		  produce_result_files);




###############################################################################
###############################################################################
##
##
##		                 FUNCTIONS
##
##
###############################################################################
###############################################################################


###############################################################################
#
#	Function exit_script
#
#	Goal : The function will exit the script with the given error message
#
#	Param :
#		- $message : the message to display
#
#	Returns :
#               Nothing but the script exited
#
###############################################################################


sub exit_script{

  (my $message) = @_;
  print $message;
  die "\n\n";

}

###############################################################################
#
#	Function read_fasta
#
#	Goal : read a fasta file and get the id and the sequence
#
#	Param :
#               - $fasta : fasta file to read
#
#	Return :
#               - $id : the id of the fasta file
#               - $seq : the sequence
#               If the file is not in fasta format returns ("","")
#
###############################################################################

sub read_fasta{

  #Location of the fasta file
  my $fasta;

  #Content of the fasta file
  my $content = "";

  #Id of the fasta file
  my $id = "";

  #Sequence
  my $seq = "";

  #Regular expression matching id and sequence
  my $regexp_fasta = "^>(.+)[\\n\\r]";
  $regexp_fasta .= "([ATCGNURYKMSWBDHVatcgnurykmswbdhv\\n\\r]+)\$";

  ($fasta) = @_;

  $content = &read_file($fasta);

  #Get id and sequence
  if($content =~ /$regexp_fasta/){
    $id = $1;
    $seq = $2;
    #Deleting end line characters in sequence
    $seq =~ s/[\n|\r]//g;
  }

  return ($id,$seq);



}


###############################################################################
#
#	Function read_file
#
#	Goal : Read a file and return its content
#
#	Param :
#               - $file : file to read
#
#	Return :
#               - $content : the content of the file
#
###############################################################################

sub read_file{

  #Content that will be returned
  my $content = "";

  (my $file) = @_;

  open(FILE,$file) or exit_script("\nERROR: Could not open file \"$file\"\n\n");
  while(<FILE>){
    $content .= $_;
  }

  close(FILE);

  return $content;



}


###############################################################################
#
#	Function write_file
#
#	Goal : Write a content into a file
#
#	Param :	
#               - $content : the content to write
#               - $file : file to write 
#
#	Return :
#               Nothing but the file is written
#
###############################################################################

sub write_file{


  (my $content, my $file) = @_;

  open(FILE,">$file") or
    exit_script("\nERROR: Could not write in file \"$file\"\n\n");
  print FILE $content;
  close FILE;

}


###############################################################################
#
#	Function create_directory
#
#	Goal : Create a directory
#
#	Param :
#               - $dir : the directory to create
#
#	Return :
#               Nothing but the directory is created
#
###############################################################################

sub create_directory{

  (my $dir) = @_;

  if(! -d $dir){
    mkdir $dir or
      exit_script("\nERROR: Could not create directory \"$dir\"\n\n");
  }

}


###############################################################################
#
#	Function erase_existing_files
#
#	Goal : Erase all existing files and subdirectories under the provided
#              directory
#
#	Param :
#               - $dir : the directory containing files to erase
#
#	Return :
#               Nothing but all files and subdirectories are erased
#
###############################################################################

sub erase_existing_files{

  (my $dir) = @_;

  my $search_all = File::Spec->catdir($dir,
				      "*");
  if($search_all =~ /\s/){
    $search_all = quotemeta($search_all);
  }
  my @all_files = glob("$search_all");
  #Get all files and subdirectories
  foreach my $file(@all_files){
    #If it's a directory recursivly erase it
    if(-d $file){
      &erase_existing_files($file);
      rmtree($file);
    }
    #Else erase the file
    else{
      unlink($file);
    }
  }
}




###############################################################################
#
#	Function sort_by_num
#
#	Goal : Comparison function used to sort files by their number (Files
#              must have a name composed by digits)
#
#	Param :
#		- $a, $b representative of 2 file names
#
#	Return :
#		- 1,0 or -1 ($a > $b, $a = $b, $a < $b)
#
###############################################################################

sub sort_by_num($$){


  #Expression that matches the number
  my $regexp_num = "([0-9]+)";

  my $a_num;
  my $b_num;

  (my $a,my $b) = @_;
  $a = basename($a);
  $b = basename($b);
  $a =~ s/\..+$//g;
  $b =~ s/\..+$//g;

  #Get the number
  if($a =~ /$regexp_num/){
    $a_num = $1;
  }
  if($b =~ /$regexp_num/){
    $b_num = $1;
  }

  if ($a_num < $b_num){
    return -1;
  }
  else{
    if ($a_num == $b_num){
	return 0;
    }
    else{
      return 1;
    }
  }
}




###############################################################################
#
#	Function get_ratio_paired_base
#
#	Goal : Get the ratio paired-bases/length sequence in carnac result
#              file for the first sequence only : the reference sequence
#	Param :
#               - $file : the carnac output file "carnac.out" to read
#	Returns :
#               - $ratio : the ratio paired-bases/length sequence
#
###############################################################################

sub get_ratio_paired_bases{

  (my $file) = @_;

  #Get its content
  open(FILE,$file)  or 
    exit_script("\nERROR : Could not read \"$file\"\n\n");

  #Get first sequence length and paired bases number
  my $length_ref_seq = 0;
  my $paired_bases = 0;
  my $content = "";
  while (my $line = <FILE>){
    $content .= $line;
  }
  close(FILE);

  if($content =~ />.+\s+[AUCGTN]+\s([\.\(\)]+)/){
    my $ref_line = $1;
    $length_ref_seq = length($ref_line);
    while($ref_line =~ /[\(\)]/g){
      $paired_bases++;
    }
  }

  #Ratio
  my $ratio = $paired_bases/$length_ref_seq;

  return $ratio;


}


###############################################################################
#
#	Function sort_by_pos
#
#	Goal : Comparison function used to sort files by their position on the
#              reference sequence
#
#	Param :
#		- $a, $b representative of 2 set names
#
#	Return :
#		- 1,0 or -1 ($a > $b, $a = $b, $a < $b)
#
###############################################################################

sub sort_by_pos($$){



  (my $a,my $b) = @_;

  my $a_pos = -1;
  my $b_pos = -1;

  #Look for position in first fasta file found
  my $a_fasta_search = File::Spec->catfile($a,"*.f*a*");
  if($a_fasta_search =~ /\s/){
      $a_fasta_search = quotemeta($a_fasta_search);
    }
  my @fasta_files = glob("$a_fasta_search");

  open(FASTA, $fasta_files[0]) or 
    exit_script("ERROR : Could not open $fasta_files[0]\n\n");
  while(($a_pos == -1) && (my $line = <FASTA>)){

    if($line =~ />.+\s+([0-9]+)\s+[0-9]+\s+[-+]/) {

      $a_pos = $1;
    }
  }
  close(FASTA);


  #Look for position in second fasta file found
  my $b_fasta_search = File::Spec->catfile($b,"*.f*a*");
  if($b_fasta_search =~ /\s/){
      $b_fasta_search = quotemeta($b_fasta_search);
    }
  @fasta_files = glob("$b_fasta_search");

  open(FASTA, $fasta_files[0]) or 
    exit_script("ERROR : Could not open $fasta_files[0]\n\n");
  while(($b_pos == -1) && (my $line = <FASTA>)){

    if($line =~ />.+\s+([0-9]+)\s+[0-9]+\s+[-+]/) {

      $b_pos = $1;
    }
  }
  close(FASTA);

  #Compare the positions
  if ($a_pos < $b_pos){
    return -1;
  }
  else{
    if ($a_pos == $b_pos){
	return 0;
    }
    else{
      return 1;
    }
  }
}



###############################################################################
#
#	Function get_reverse_base
#
#	Goal : Return the complementary base of the provided one
#
#	Param :
#               - $base : the base to get its complementary
#
#	Return :
#               - $complementary : the complementary base (or N if unknown)
#
###############################################################################

sub get_reverse_base{

  my $base;

  ($base) = @_;

  $base = uc($base);

  if ( $base  eq "A")  { return  "T";}
  if ( $base  eq "T")  { return  "A";}
  if ( $base  eq "C")  { return  "G";}
  if ( $base  eq "G")  { return  "C";}
  if ( $base  eq "U")  { return  "A";}
  if ( $base  eq "N")  { return  "N";}
  return "N";
}

###############################################################################
#
#	Function execute_reverse_complementary
#
#	Goal : The function will create the reverse complementary set for all
#              sets provided in the paths
#	Param :
#               - $file : the file to get the reverse complementary
#               - $outfile : the outfile to  write the complementary file
#	Returns :
#               Nothing but the reversed file is created
#
###############################################################################

sub execute_reverse_complementary{

  (my $file,
   my $outfile) = @_;


  #Read each sequence and produce its reverse
  my $content_reverse = "";
  my $content_original = "";
  open(FILE,$file) or
    exit_script("\nERROR :Could not read \"$file\" to produce its reverse\n\n");
  while(<FILE>){
    $content_original .= $_;
  }
  close(FILE);

  #Process each sequence
  while ($content_original =~ 
	 /(.+\s+[0-9]+[-\s][0-9]+\s+)([-+])\s*\n([NATCGnatcg\n]+)/g) {
    my $name_id_pos = $1;
    my $strand = $2;
    my $seq = $3;
    $seq =~ s/[\n\r]//g;

    #Change strand
    if ($strand eq "+") {
      $strand = "-";
    } else {
      $strand = "+";
    }

    #Construction of the reverse sequence
    my $length = length($seq);
    my $base = "";
    my $reverse_base = "";
    my $seq_reverse = "";

    for (my $i = $length - 1; $i >= 0; $i--) {
      $base = substr($seq, $i, 1);
      $reverse_base = &get_reverse_base($base);
      if (((($length - 1 - $i)%70) == 0) || ($i == $length -1)) {
	$seq_reverse .= "\n";
      }
      $seq_reverse .= $reverse_base;
    }

    $content_reverse .= $name_id_pos.$strand.$seq_reverse."\n";

  }

  #Write reverse file
  open(OUT,">$outfile") or
    exit_script("\nERROR : Could not write reverse into \"$outfile\"\n\n");
  print OUT $content_reverse;
  close(OUT);

}


###############################################################################
#
#	Function get_info_genbank
#
#	Goal : Get information on genes, RNAs in a genbank file : positions,
#              name, locus, product and strand
#
#	Param :
#		- $genbank : the genbank file
#
#	Return :
#               - $ref_pos : the reference on the positions array
#		- $ref_pos_name : the reference on a hash table that links
#                 positions and name
#		- $ref_pos_locus : the reference on a hash table that links
#                 positions and locus
#		- $ref_pos_prod : the reference on a hash table that links
#                 positions and product
#		- $ref_pos_strand : the reference on a hash table that links
#                 positions and strand
#
###############################################################################


sub get_info_genbank{

  #Genbank content
  my $content = "";

  #Reference on positions array
  my $ref_pos;

  #Reference on hash table that links positions and name
  my $ref_pos_name;

  #Reference on hash table that links positions and locus
  my $ref_pos_locus;

  #Reference on hash table that links positions and product
  my $ref_pos_prod;

  #Reference on hash table that links positions and strand
  my $ref_pos_strand;

  (my $genbank) = @_;

  #Reading of the file
  $content = &read_file($genbank);

  my $regexp = "(\\s{5}(CDS|tRNA|rRNA|misc_RNA)\\s+(complement\\()?";
  $regexp .= "(join\\()?([0-9]+\\.\\.";
  $regexp .= "[0-9]+,?\\s*)+\\){0,2}\\s?";
  $regexp .= "(\\s{21}\/.+=\".+\\n?(\\s{21}.+\\n)*)+)";

  my $regexp_pos = "([0-9]+\\.\\.[0-9]+)";
  my $regexp_gene = "gene=\"(.+)\"";
  my $regexp_locus_tag = "locus_tag=\"(.+)\"";
  my $regexp_product = "product=\"(.+|.+\\s+.+)\"";
  my $regexp_strand = "complement\\(";

  #Get each feature : CDS, RNAs
  while($content =~ /$regexp/g){
     my $part = $1;
 
      my $pos = " ";
      my $name = " ";
      my $locus_tag = " ";
      my $prod = " ";
      my $strand = " ";

      if($part =~ /join\(/){

        if($part =~ /\(([0-9]+\.\.)/){
  	$pos .= $1;
        }
        if($part =~ /\.\.([0-9]+)\)/){
  	$pos .= $1;
        }

      }
      else{
        if($part =~ /$regexp_pos/){
  	$pos = $1;
        }
      }

      if($part =~ /$regexp_gene/){
        $name = $1;
      }
      if($part =~ /$regexp_locus_tag/){
        $locus_tag = $1;
      }
      if($part =~ /$regexp_product/){
        $prod = $1;
        $prod =~ s/\s+/ /g;

      }

      if($part =~ /$regexp_strand/){
        $strand = "-";
      }
      else{
        $strand = "+";
      }


      #Add info in respective tables
      push(@$ref_pos, $pos);

      $ref_pos_name->{$pos} = $name;
      $ref_pos_locus->{$pos} = $locus_tag;
      $ref_pos_prod->{$pos} = $prod;
      $ref_pos_strand->{$pos} = $strand;

    }

  return ($ref_pos,
	  $ref_pos_name,
	  $ref_pos_locus,
	  $ref_pos_prod,
	  $ref_pos_strand);


}


###############################################################################
#
#	Function get_pos_genes
#
#	Goal : Retrieve positions of the gene located before the pos_start
#              and the positions of the gene located after the pos_end
#
#	Param :
#		- $pos_start : the prediction starting position
#               - $pos_end : the prediciotn ending position
#               - $pos : the reference on array containing positions
#
#	Return :
#		- $pos_gene_5prime, $pos_gene_3prime : the positions of the
#                 genes
#
###############################################################################

sub get_pos_genes{


  (my $pos_start, my $pos_end,my $pos) = @_;

  my $genes_found = 0;
  my $pos_gene_5prime = "";
  my $pos_gene_3prime = "";
  my $nb_pos = @$pos;

  #Process each genes until positions are found
  for(my $ i =0; (!$genes_found) && ($i < $nb_pos); $i++){

    my $current_pos = $pos->[$i];
    my $current_start = "";
    my $current_end = "";
    if($current_pos =~ /([0-9]+)\.\.([0-9]+)/){
      $current_start = $1;
      $current_end = $2;
    }

    if($pos_start > $current_end){

      $pos_gene_5prime = $pos->[$i];
    }

    if($pos_end < $current_start){

      $pos_gene_3prime = $pos->[$i];
      $genes_found = 1;
    }

  }


  return ($pos_gene_5prime,$pos_gene_3prime);


}



###############################################################################
#
#	Function produce_result_files
#
#	Goal : The function will create the result files
#
#	Param :
#               - $reference_file : the reference file location
#               - $other_files : reference on array containing other files
#               - $mask_coding_available : masking regions step or not
#               - $mask_cds_wanted : masking CDSs wanted
#               - $mask_rna_wanted : masking RNAs wanted
#               - $masking_redundancy_available : mask redundancy step or not
#               - $yass_available : yass alignment step or not
#               - $blast_available : blast alignment step or not
#               - $cg_seq_available : find conserved regions or not
#               - $carnac_available : carnac prediction or not
#               - $rnaz_available : RNAz prediction or not
#               - $all : all  wanted or not
#               - $yass_outdir_name : the name of yass outdir
#               - $blast_outdir_name : the name of blast outdir
#               - $rna_predicted_outdir : the location of RNA predictions
#               - $outdir : the outdir
#               - $warning_file : the warning file if warning occured or ""
#               - $time_diff : execution time 
#
#	Returns :
#               Nothing. But the result files are created
#
###############################################################################

sub produce_result_files{


  (my $reference_file,
   my $other_files,
   my $mask_coding_available,
   my $mask_cds_wanted,
   my $mask_rna_wanted,
   my $mask_redundancy_available,
   my $yass_available,
   my $blast_available,
   my $cg_seq_available,
   my $carnac_available,
   my $rnaz_available,
   my $all,
   my $yass_outdir_name,
   my $blast_outdir_name,
   my $rna_predicted_outdir,
   my $outdir,
   my $warning_file,
   my $time_diff) = @_;

  #Boolean that says if the genbank is present
  my $has_genbank = 1;

  #Location of the genbank file
  my $gbk = "";
  my $gb = "";
  my $real_genbank = "";

  #Content of the current HTML result file
  my $content_html = "";

  #Content of the current CSV result file for yass
  my $content_csv_yass = "";

  #Content of the current CSV result file for blast
  my $content_csv_blast = "";

  #Reference on array containing interesting positions
  my $ref_pos;

  #Reference on hashtable that links positions <-> name
  my $ref_pos_name;

  #Reference on hashtable that links positions <-> locus
  my $ref_pos_locus;

  #Reference on hashtable that links positions <-> product
  my $ref_pos_prod;

  #Reference on hashtable that links positions <-> strand
  my $ref_pos_strand;



  #Colors and fonts for HTML output
  my $color_line_white = "#FFFFFF";
  my $color_line_other = " #FAFAFA";
  my $font_title = "<font  face=\"arial\" size=\"4\">";
  my $font_title_less = "<b><font face=\"arial\" size=\"2\">";
  my $font_info_set = "<font face=\"arial\" size=\"3\">";
  my $font_info_gene = "<font face=\"arial\" size=\"1\">";
  my $end_font = "</font>";
  my $end_font_bold = "</font></b>";



  #Check for the genbank file
  (my $name_ref,
   my $dir_ref,
   my $suffix) = fileparse($reference_file,qr/\.[^.]*/);
  $gbk = File::Spec->catfile($dir_ref,
			     $name_ref.".gbk");
  $gb = File::Spec->catfile($dir_ref,
			    $name_ref.".gb");

  if ((! -e $gbk) && (! -e $gb)) {
    $has_genbank = 0;
  }
  else{
    if (-e $gbk) {
      $real_genbank = $gbk;
    }
    if (-e $gb) {
      $real_genbank = $gb;
    }
  }

  my $outdir_title = $outdir;
  $outdir =~ s/\\/\\\\/g;

  #Deal with warning file location
  $warning_file =~ s/$outdir//g;
  $warning_file =~ s/^[\/|\\]//g;

  #Recuperate genbank genes information on positions, name, locus, product
  #and strand
  if ($has_genbank) {
    ($ref_pos,
     $ref_pos_name,
     $ref_pos_locus,
     $ref_pos_prod,
     $ref_pos_strand) = &get_info_genbank($real_genbank);

  }

  #Date
  my ($secs, $min, $hour, $day, $month,
      $year, $day_of_week, $calendary_day, $summer_timeshift) = localtime(time);
  my @months = ("January","February","March","April","May","June","July",
		"August","September","October","November","December");
  if ($day < 10) {
    $day = "0".$day;
  }
  $year += 1900;
  $month = $months[$month];

  #First part of result file: general heading 
  $content_html = "<html><head><title>CG-seq results</title></head>\n";
  $content_html .= "<center><h1><font color=\"#CC0000\"> CG-seq results</font></h1></center>\n";
  $content_html .= "<br><br><br><h2><font color=\"#CC0000\">Execution summary</font></h2>\n";
  $content_html .= "<ol>\n<li><b>Date</b>\n<ul>\n  <li>Results generated on ";
  $content_html .= "$day $month $year (".$hour."H".$min."min".$secs."s)</li>\n";
  $content_html .= "  <li>Total execution time : $time_diff seconds</li>\n \n </ul>\n </li>\n";
  $content_html .= "<li><b>Species</b>\n<ul>\n";
  $content_html .= "  <li>Target sequence: $reference_file</li>\n";
  $content_html .= "  <li>Other species (files or directories):\n   <ul>\n";
  foreach my $other (keys(%$other_files)) {
    $content_html .= "    <li>$other</li>\n";
  }
  $content_html .= "   </ul>\n</ul>\n</li>\n";
  $content_html .= "<li><b>Output directory</b> \n <ul><li>\"$outdir_title\"</li></ul>\n";
  $content_html.= "</ul></ul><li><b>Options and parameters</b>\n<ul>\n";
  if ($mask_coding_available || $all) {
    if($mask_cds_wanted && $mask_rna_wanted){
      $content_html .= "<li>Masking CDSs and RNAs\n";
    }
    else{
      if($mask_cds_wanted){
	$content_html .= "<li>Masking CDSs\n";
      }
      else{
	if($mask_rna_wanted){
	  $content_html .= "<li>Masking RNAs\n";
	}
      }
    }
  }
  if($mask_redundancy_available || $all){
    $content_html .= "<li>Masking redundant regions intra species\n";
  }
  if ($all || ($yass_available && $blast_available)) {
    $content_html .= "<li>Alignment step with Yass ans Blast\n";
  } else {
    if ($yass_available) {
      $content_html .= "<li>Alignment step with Yass\n";
    } else {
      if ($blast_available) {
	$content_html .= "<li>Alignment step with Blast\n";
      }
    }
  }
  if ($all || $cg_seq_available) {
    $content_html .= "<li>Find conserved regions with CGseqcore\n";
  }
  if ($all || ($carnac_available && $rnaz_available)) {
    $content_html .= "<li>Structure prediction with caRNAc and RNAz\n";
  } else {
    if ($carnac_available) {
      $content_html .= "<li>Structure prediction with caRNAc\n";
    } else {
      if ($rnaz_available) {
	$content_html .= "<li>Structure prediction with RNAz\n";
      }
    }
  }
  $content_html .= "</ul>";

  if ($warning_file ne "") {
    $content_html .= "<br><li><b>Warning :</b>\n<ul><li>Warning(s) occured ";
    $content_html .= "during pipeline execution. See ";
    $content_html .= "<a href=\"file:$warning_file\">";
    $content_html .= "warning file</a> to have some explanations.\n</ul>";

  }


  $content_html .="</ol>\n<br><br><br>\n";


  #Get Yass and blast sets
  my $yass_set = File::Spec->catdir($rna_predicted_outdir,
				    $yass_outdir_name);
  my $blast_set = File::Spec->catdir($rna_predicted_outdir,
				     $blast_outdir_name);

  my $yass_search_set = File::Spec->catdir($yass_set,
					   "set_*");
  my $blast_search_set = File::Spec->catdir($blast_set,
					    "set_*");

  if ($yass_search_set =~ /\s/) {
    $yass_search_set = quotemeta($yass_search_set);
  }
  if ($blast_search_set =~ /\s/) {
    $blast_search_set = quotemeta($blast_search_set);
  }

  my @yass_sets = glob("$yass_search_set");
  my @blast_sets = glob("$blast_search_set");

  #Sets were found
  if (@yass_sets + @blast_sets != 0) {


    #Beginning of CSV
    if($yass_available){
      $content_csv_yass = "Num;Start position;End position;Length;";
      $content_csv_yass .= "Strand(s);Nb of sequences; Organisms(Nb)";
      if ($has_genbank) {
	$content_csv_yass .= ";Distance from 5' gene;5'gene start position;";
	$content_csv_yass .= "5'gene end position;";
	$content_csv_yass .= "5'gene strand;5'gene ID;5'gene product ";
	$content_csv_yass .= "(name);Distance from 3' gene;3'gene start position;";
	$content_csv_yass .= "3'gene end position;3'gene strand;";
	$content_csv_yass .= "3'gene ID;3'gene product (name)";
      }
      $content_csv_yass .= ";Prediction tools\n";
    }

    if($blast_available){
      $content_csv_blast = "Num;Start position;End position;Length;";
      $content_csv_blast .= "Strand(s);Nb of sequences; Organisms(Nb)";
      if ($has_genbank) {
	$content_csv_blast .= ";Distance from 5' gene;5'gene start position;";
	$content_csv_blast .= "5'gene end position;";
	$content_csv_blast .= "5'gene strand;5'gene ID;5'gene product ";
	$content_csv_blast .= "(name);Distance from 3' gene;3'gene start position;";
	$content_csv_blast .= "3'gene end position;3'gene strand;";
	$content_csv_blast .= "3'gene ID;3'gene product (name)";
      }
      $content_csv_blast .= ";Prediction tools\n";
    }


    ###Result parts#####
    $content_html .= "<br><br><br><h2><font color=\"#CC0000\">List of putative RNAs found</font></h2>\n";
    $content_html .= "<table BORDER BORDERCOLOR=\"black\" CELLSPACING = \"0\"";
    $content_html .= " align=\"center\">\n";
    $content_html .= "<tr align=\"center\" valign=\"middle\" ";
    $content_html .= "bgcolor=\"#EEEEEE\">\n<td colspan=\"6\">";
    $content_html .= "$font_title Information on prediction $end_font</td>\n";

    #Gene information
    if ($has_genbank) {
      $content_html .= "<td colspan=\"5\">$font_title_less ";
      $content_html .= "Information on 5' gene$end_font_bold</td>\n";
      $content_html .= "<td colspan=\"5\">$font_title_less ";
      $content_html .= "Information on 3' gene$end_font_bold</td>\n";
    }
    $content_html .= "<td>$font_title_less Information on tools $end_font_bold";
    $content_html .= "</td></tr>\n<tr align=\"center\" ";
    $content_html .= "valign=\"middle\" bgcolor=\"#EEEEEE\">";
    $content_html .= "<td>$font_title Num$end_font</td>\n";
    $content_html .= "<td>$font_title Positions$end_font</td>\n";
    $content_html .= "<td>$font_title Length$end_font</td>\n";
    $content_html .= "<td>$font_title Strand(s)$end_font</td>\n";
    $content_html .= "<td>$font_title Nb of sequences$end_font";
    $content_html .= "</td>\n<td>$font_title Organisms (Nb)";
    $content_html .= "$end_font</td>\n";

    #Gene information
    if ($has_genbank) {
      $content_html .= "<td>$font_title_less ";
      $content_html .= "Distance from gene</td>\n";
      $content_html .= "<td>$font_title_less ";
      $content_html .= "Positions$end_font_bold</td>\n<td>";
      $content_html .= "$font_title_less Strand$end_font_bold</td>\n";
      $content_html .= "<td>$font_title_less ID$end_font_bold</td>\n";
      $content_html .= "<td>$font_title_less Product (Gene name)";
      $content_html .= "$end_font_bold</td>\n";
      $content_html .= "<td>$font_title_less ";
      $content_html .= "Distance from gene</td>\n";
      $content_html .= "<td>$font_title_less ";
      $content_html .= "Positions$end_font_bold</td>\n<td>";
      $content_html .= "$font_title_less Strand$end_font_bold</td>\n";
      $content_html .= "<td>$font_title_less ID$end_font_bold</td>\n";
      $content_html .= "<td>$font_title_less Product (Gene name)";
      $content_html .= "$end_font_bold</td>\n";
    }
    $content_html .= "<td>$font_title_less Prediction tools$end_font_bold</td>";
    $content_html .= "</tr>\n";

    #Concat yass and blast sets and them by position on the reference
    #sequence
    my @all_sets = (@yass_sets,@blast_sets);
    @all_sets = sort sort_by_pos @all_sets;

    my $count_line = 1;
    #Process each set
    foreach my $current_set (@all_sets) {

      my $html_link = $current_set;
      $html_link =~ s/$outdir//g;
      $html_link =~ s/^[\/|\\]//g;

      my $nb_set = $count_line;

      my $nb_set_csv;
      if ($current_set =~ /_([0-9]+)$/) {
	$nb_set_csv = $1;
      }

      #Get fasta file(s)
      my $fasta_search = File::Spec->catfile($current_set,"*.f*a*");
      if ($fasta_search =~ /\s/) {
	$fasta_search = quotemeta($fasta_search);
      }
      my @fasta_files = glob("$fasta_search");

      my $id = "";
      my $pos_start = "";
      my $pos_end = "";
      my $strand = "";
      my $length = "";
      my $nb_seq = 0;
      my $organisms = "";
      my $organisms_csv = "";
      my $ref_processed = 0;

      #Get information on first fasta file
      open(FASTA,$fasta_files[0]);
      while (my $line = <FASTA>) {
	#Reference information
	if ((!$ref_processed) && ($line =~ />.+/)) {
	  $id = $line;
	  $ref_processed = 1;
	  $nb_seq++;
	}
	#Other organisms information
	else {
	  if ($line =~ />(.+)\s[0-9]+\s+[0-9]+\s+[-+]/) {
	    my $current_org = $1;
	    $nb_seq++;
	    my $current_org_name = "";
	    if ($current_org =~ /\|/) {
	      my @split_name = split('\|',$current_org);
	      my $nb_split = @split_name;
	      $current_org_name = $split_name[$nb_split - 1];
	    } else {
	      $current_org_name = $current_org;
	    }
	    $current_org_name =~ s/^_/ /g;
	    if ($organisms =~ /(.*)$current_org_name\(([0-9]+)\)(.*)/) {
	      my $begin = $1;
	      my $count = $2;
	      my $end = $3;
	      $count++;
	      $organisms = $begin.$current_org_name."(".$count.")";
	      $organisms .= $end;
	    } else {
	      $organisms .= $current_org_name."(1)<BR>";
	    }
	  }
	}
      }
      close(FASTA);

      $organisms_csv = $organisms;
      $organisms_csv =~ s/<BR>/ /g;

      #Check reference positions and strand
      if ($id =~ /([0-9]+)\s+([0-9]+)\s+([+-])/) {
	$pos_start = $1;
	$pos_end = $2;
	$strand = $3;
      }

      #To get strand prediction : look at number of aln files if exist, if 
      #not, only carnac was run and so both strand prediction
      my $aln_search = File::Spec->catfile($current_set,"*.aln");
      if ($aln_search =~ /\s/) {
	$aln_search = quotemeta($aln_search);
      }
      my @aln_files = glob("$aln_search");
      if((@aln_files == 0) || (@aln_files == 2)){
	$strand = "+/-";
      }
      else{
	if($aln_files[0]=~ /reverse/){
	  $strand = "-";
	}
	else{
	  $strand = "+";
	}

      }

      $length = $pos_end - $pos_start + 1;

      #Deal with colors in HTML output
      my $color_current_line = "";
      if (($count_line % 2) != 0) {
	$color_current_line = $color_line_white;
      } else {
	$color_current_line = $color_line_other;
      }
      $count_line++;

      #Add content to HTML result file
      # modifie par Helene pour retirer les files
      $content_html .= "<tr align=\"center\" valign=\"middle\" ";
      $content_html .= "bgcolor=\"$color_current_line\">\n";
      $content_html .= "<td><a href=\"$html_link\">$font_info_set";
      $content_html .= "$nb_set$end_font</a></td>\n";
      $content_html .= "<td><a href=\"$html_link\">$font_info_set";
      $content_html .= "$pos_start\.\.$pos_end$end_font</a></td>\n";
      $content_html .= "<td>$font_info_set$length$end_font</td>\n";
      $content_html .= "<td>$font_info_set$strand$end_font</td>\n";
      $content_html .= "<td>$font_info_set$nb_seq$end_font</td>\n";
      $content_html .= "<td>$font_info_gene$organisms$end_font</td>\n";

      #Add content to CSV result file
      if($current_set =~ /$yass_outdir_name/){
	$content_csv_yass .= "$nb_set_csv;$pos_start;$pos_end;$length;";
	$content_csv_yass .= "$strand;$nb_seq;$organisms_csv";
      }
      else{
	$content_csv_blast .= "$nb_set_csv;$pos_start;$pos_end;$length;";
	$content_csv_blast .= "$strand;$nb_seq;$organisms_csv";
      }

      #Gene information
      if ($has_genbank) {

	#Get gene positions
	my $pos_gene_5prime = "";
	my $pos_gene_3prime = "";
	($pos_gene_5prime,$pos_gene_3prime) = 
	  &get_pos_genes($pos_start, $pos_end, $ref_pos);

	#Get all genes information and update content
	my $name_gene = "&nbsp;";
	my $id_gene = "&nbsp;";
	my $product_gene = "&nbsp;";
	my $strand_gene = "&nbsp;";

	my $name_gene_csv = "";
	my $id_gene_csv = "";
	my $product_gene_csv = "";
	my $strand_gene_csv = "";
	my $pos_start_csv = "";
	my $pos_end_csv ="";

	if ($pos_gene_5prime ne "") {

	  if (exists ($ref_pos_name->{$pos_gene_5prime})) {
	    $name_gene = $ref_pos_name->{$pos_gene_5prime};
	    $name_gene_csv = $name_gene;
	  }

	  if (exists ($ref_pos_locus->{$pos_gene_5prime})) {
	    $id_gene = $ref_pos_locus->{$pos_gene_5prime};
	    $id_gene_csv = $id_gene;
	  }

	  if (exists ($ref_pos_prod->{$pos_gene_5prime})) {
	    $product_gene = $ref_pos_prod->{$pos_gene_5prime};
	    $product_gene_csv = $product_gene;
	  }

	  if (exists ($ref_pos_strand->{$pos_gene_5prime})) {
	    $strand_gene = $ref_pos_strand->{$pos_gene_5prime};
	    $strand_gene_csv = $strand_gene;
	  }

	  if ($pos_gene_5prime =~ /([0-9]+)\.\.([0-9]+)/) {
	    $pos_start_csv = $1;
	    $pos_end_csv = $2;
	  }

	} else {
	  $pos_gene_5prime = "&nbsp;";
	}

	if (($name_gene ne " ") && ($name_gene ne "&nbsp;")) {
	  $name_gene_csv = " ($name_gene)";
	  $name_gene = "&nbsp;(".$name_gene.")";
	}
	my $distance_html = "&nbsp;";
	my $distance_csv = "";
	if ($pos_end_csv ne "") {
	  $distance_html = $pos_start - $pos_end_csv;
	  $distance_csv = $pos_start - $pos_end_csv;
	}
	$content_html .= "<td>$font_info_gene$distance_html$end_font</td>\n";
	$content_html .= "<td>$font_info_gene$pos_gene_5prime";
	$content_html .= "$end_font</td>\n<td>$font_info_gene";
	$content_html .= "$strand_gene$end_font</td>\n";
	$content_html .= "<td>$font_info_gene$id_gene$end_font";
	$content_html .= "</td>\n<td>$font_info_gene";
	$content_html .= "$product_gene$name_gene$end_font</td>\n";

	if($current_set =~ /$yass_outdir_name/){
	  $content_csv_yass .= ";$distance_csv;$pos_start_csv;$pos_end_csv;";
	  $content_csv_yass .= "$strand_gene_csv;$id_gene_csv;";
	  $content_csv_yass .= "$product_gene_csv$name_gene_csv";
	}
	else{
	  $content_csv_blast .= ";$distance_csv;$pos_start_csv;$pos_end_csv;";
	  $content_csv_blast .= "$strand_gene_csv;$id_gene_csv;";
	  $content_csv_blast .= "$product_gene_csv$name_gene_csv";
	}


	$name_gene = "&nbsp;";
	$id_gene = "&nbsp;";
	$product_gene = "&nbsp;";
	$strand_gene = "&nbsp;";

	$name_gene_csv = "";
	$id_gene_csv = "";
	$product_gene_csv = "";
	$strand_gene_csv = "";
	$pos_start_csv = "";
	$pos_end_csv ="";

	if ($pos_gene_3prime ne "") {

	  if (exists ($ref_pos_name->{$pos_gene_3prime})) {
	    $name_gene = $ref_pos_name->{$pos_gene_3prime};
	    $name_gene_csv = $name_gene;
	  }

	  if (exists ($ref_pos_name->{$pos_gene_3prime})) {
	    $id_gene = $ref_pos_locus->{$pos_gene_3prime};
	    $id_gene_csv = $id_gene;
	  }

	  if (exists ($ref_pos_name->{$pos_gene_3prime})) {
	    $product_gene = $ref_pos_prod->{$pos_gene_3prime};
	    $product_gene_csv = $product_gene;
	  }

	  if (exists ($ref_pos_name->{$pos_gene_3prime})) {
	    $strand_gene = $ref_pos_strand->{$pos_gene_3prime};
	    $strand_gene_csv = $strand_gene;
	  }

	  if ($pos_gene_3prime =~ /([0-9]+)\.\.([0-9]+)/) {
	    $pos_start_csv = $1;
	    $pos_end_csv = $2;
	  }

	} else {
	  $pos_gene_3prime = "&nbsp;";
	}

	if (($name_gene ne " ") && ($name_gene ne "&nbsp;")) {
	  $name_gene_csv = " ($name_gene)";
	  $name_gene = "&nbsp;(".$name_gene.")";
	}

	$distance_html = "&nbsp;";
	$distance_csv = "";
	if ($pos_start_csv ne "") {
	  $distance_html = $pos_start_csv - $pos_end;
	  $distance_csv = $pos_start_csv - $pos_end;
	}
	$content_html .= "<td>$font_info_gene$distance_html$end_font</td>\n";
	$content_html .= "<td>$font_info_gene$pos_gene_3prime";
	$content_html .= "$end_font</td>\n<td>$font_info_gene";
	$content_html .= "$strand_gene$end_font</td>\n";
	$content_html .= "<td>$font_info_gene$id_gene$end_font";
	$content_html .= "</td>\n<td>$font_info_gene";
	$content_html .= "$product_gene$name_gene$end_font</td>\n";

	if($current_set =~ /$yass_outdir_name/){
	  $content_csv_yass .= ";$distance_csv;$pos_start_csv;$pos_end_csv;";
	  $content_csv_yass .= "$strand_gene_csv;$id_gene_csv;";
	  $content_csv_yass .= "$product_gene_csv$name_gene_csv";
	}
	else{
	  $content_csv_blast .= ";$distance_csv;$pos_start_csv;$pos_end_csv;";
	  $content_csv_blast .= "$strand_gene_csv;$id_gene_csv;";
	  $content_csv_blast .= "$product_gene_csv$name_gene_csv";
	}
      }


      #Prediction tools part
      my $prediction_tools = "";
      if($current_set =~ /$yass_outdir_name/){
	$prediction_tools = "Yass<br>";
      }
      else{
	$prediction_tools = "Blast<br>";
      }

      #If there are some .out files : carnac was used
      my $out_search = File::Spec->catfile($current_set,"*.out");
      if ($out_search =~ /\s/) {
	$out_search = quotemeta($out_search);
      }
      my @out_files = glob("$out_search");

      if(@out_files != 0){
	$prediction_tools .= "Carnac<br>";
      }

      #If there's at least one alignment file : RNAz was used
      if(@aln_files != 0){
	$prediction_tools .= "RNAz<br>";

      }

      $content_html .= "<td>$font_info_gene$prediction_tools$end_font";
      $content_html .= "</td></tr>\n";

      $prediction_tools =~ s/<br>/ /g;

      if($current_set =~ /$yass_outdir_name/){
	$content_csv_yass .= ";$prediction_tools\n";
      }
      else{
	$content_csv_blast .= ";$prediction_tools\n";
      }
    }


    $content_html .="</table>\n";

  } else {
    $content_html .= "<br><center><h3>No predicted sets</h3></center><br>\n";
  }

  $content_html .= "</body></html>";

  my $outfile_html = File::Spec->catfile($outdir,
					 "results.html");
  &write_file($content_html, $outfile_html);

  if($yass_available){
    my $outfile_yass_csv = File::Spec->catfile($outdir,
					       "yass_results.csv");
    &write_file($content_csv_yass, $outfile_yass_csv);

  }
  if($blast_available){
    my $outfile_blast_csv = File::Spec->catfile($outdir,
						"blast_results.csv");
    &write_file($content_csv_blast, $outfile_blast_csv);

  }

}






###############################################################################
###############################################################################
##
##
##			      POD DOCUMENTATION
##
##
###############################################################################
###############################################################################

=head1 NAME

Package I<general_functions> - General functions

=head1 SYNOPSIS

Contains all functions to handle configuration files used  in the CG-seq pipeline.

Provided functions:

=over 2

=item *

create_directory

=item *

execute_reverse_complementary

=item *

exit_script

=item *

get_info_genbank

=item *

get_pos_genes

=item *

get_reverse_base

=item *

get_ratio_paired_bases

=item *

produce_result_files

=item *

produce_result_pipeline

=item *

read_fasta

=item *

read_file

=item *

sort_by_num

=item *

write_file

=back

=head1 EXPORT

Exported functions:

=over 2

=item *

create_directory

=item *

execute_reverse_complementary

=item *

exit_script

=item *

get_ratio_paired_bases

=item *

get_reverse_base

=item *

produce_result_files

=item *

read_fasta

=item *

read_file

=item *

sort_by_num

=item *

sort_by_pos

=item *

write_file

=back

=head1 FUNCTIONS

=head2 create_directory

Create a directory.

=over 2

=item *

parameters : $dir, the directory to create

=item *

returns: Nothing but the directory is created

=back

=head2 erase_existing_files

Erase all existing files and subdirectories in the specified directory

=over 2

=item *

parameters : $dir : the directory containing files to erase

=item *

returns: Nothing but all files and subdirectories are erased

=back

=head2 execute_reverse_complementary

The function will create the reverse complementary set for all sets provided 
in the paths.

=over 2

=item *

parameters :

=over 2

=item *

$file, the file to get the reverse complementary

=item *

$outfile, the outfile to  write the complementary file

=back

=item *

returns: Nothing but the reversed file is created

=back

=head2 exit_script

Prints an error message and exit the script

=over 2

=item *

parameters : $message, the error message to print

=item *

returns : nothing but the script exited

=back

=head2 get_info_genbank

Get information on genes, RNAs in a genbank file : positions, name, locus, 
product and strand.

=over 2

=item *

parameters : $genbank, the genbank file

=item *

returns :

=over 2

=item *

$ref_pos, the reference on the positions array

=item *

$ref_pos_name, the reference on a hash table that links positions and name

=item *

$ref_pos_locus, the reference on a hash table that links positions and locus

=item *

$ref_pos_prod, the reference on a hash table that links positions and product

=item *

$ref_pos_strand, the reference on a hash table that links positions and strand

=back

=back

=head2 get_pos_genes

Retrieve positions of the gene located before the pos_start and the positions 
of the gene located after the pos_end.

=over 2

=item *

parameters :

=over 2

=item *

$pos_start, the prediction starting position

=item *

$pos_end, the prediction ending position

=item *

$pos, the reference on array containing positions

=back

=item *

returns : 

=over 2

=item *

$pos_gene_5prime, the positions of the gene located in 5'

=item *

$pos_gene_3prime, the positions of the gene located in 3'

=back

=back

=head2 get_ratio_paired_bases

Get the ratio paired-bases/length sequence in carnac result file for the first
sequence only : the target sequence.

=over 2

=item *

parameters, $file, the carnac output file "carnac.out" to read

=item *

returns, the ratio paired-bases/length sequence.

=back

=head2 get_reverse_base

Return the complementary base of the provided one.

=over 2

=item *

parameters : $base, the base to get its complementary

=item *

returns : the complementary base (or N if unknown)

=back

=head2 produce_result_files

The function will create the result files.

=over 2

=item *

parameters :

=over 2

=item *

$reference_file, the reference file location

=item *

$other_files, reference on array containing other files 

=item *

$mask_coding_available, masking regions step or not

=item *

$mask_cds_wanted, masking CDSs wanted

=item *

$mask_rna_wanted, masking RNAs wanted

=item *

$mask_redundancy_available, masking redundant regions step or not

=item *

$yass_available, yass alignment step or not

=item *

$blast_available, blast alignment step or not

=item *

$cg_seq_available, find conserved regions or not

=item *

$carnac_available, carnac prediction or not

=item *

$rnaz_available, RNAz prediction or no

=item *

$all, all pipeline available or not

=item *

$yass_outdir_name, the name of yass output directory

=item *

$blast_outdir_name, the name of blast output directory

=item *

$rna_predicted_outdir, the location of RNA predictions

=item *

$outdir, the outdir

=item *

$warning_file, the warning file if warning occured or ""

=item *

$time_diff, the pipeline execution time

=back

=item *

returns : nothing but the result files are created.

=back

=head2 read_fasta

Read a Fasta file and return its ID and sequence.

=over 2

=item *

parameters : $fasta, the Fasta file to read

=item *

returns : the ID and sequence or nothing if no Fasta format.

=back

=head2 read_file

Read a file and return its content.

=over 2

=item *

parameters : $file, the file to read

=item *

returns : the content of the file.

=back

=head2 sort_by_num

Comparison function used to sort files by their number (Files must have a name
like by digits).

=over 2

=item *

parameters : $a, $b representative of 2 file names

=item *

returns : 1,0 or -1 ($a > $b, $a = $b, $a < $b).

=back

=head2 sort_by_pos

Comparison function used to sort sets by their position on the reference
sequence

=over 2

=item *

parameters : $a, $b representative of 2 set names

=item *

returns : 1,0 or -1 

=back

=head2 write_file

Write a content into a file.

=over 2

=item *

parameters :

=over 2

=item *

$content, the content to write

=item *

$file, file to write

=back

=item *

returns : Nothing but the file is written.

=back

=head1 AUTHOR

Benjamin Grenier-Boley <benjamin.grenier-boley@inria.fr>

=cut


1;
