###############################################################################
###############################################################################
##
## This file is part of CG-seq.
##
##  CG-seq is free software: you can redistribute it and/or modify
##  it under the terms of the GNU General Public License as published by
##  the Free Software Foundation, either version 3 of the License, or
##  (at your option) any later version.
##
##  CG-seq is distributed in the hope that it will be useful,
##  but WITHOUT ANY WARRANTY; without even the implied warranty of
##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##  GNU General Public License for more details.
##
##  You should have received a copy of the GNU General Public License
##  along with CG-seq.  If not, see <http://www.gnu.org/licenses/>.
##
###############################################################################
###############################################################################


###############################################################################
###############################################################################
##
## 	Package : software_handler.pm
##	Author : Benjamin Grenier-Boley <benjamin.grenier-boley@inria.fr>
##
##	Creation : 16/03/09
##     	Last modification : 03/06/09
##
##	Goal : Contains all functions to deal with extern pipeline softwares
##
###############################################################################
###############################################################################


package software_handler;


use strict;
use warnings;
use File::Spec;
use File::Copy;
use File::Path;
use File::Basename;
use FindBin;
use lib "$FindBin::Bin";

use configuration_handler;
use general_functions;

require Exporter;


###############################################################################
###############################################################################
##
##
##		               MODULE EXPORT
##
##
###############################################################################
###############################################################################

our @ISA = qw(Exporter);


our @EXPORT = qw (execute_masking_coding
		  execute_masking_redundancy
		  execute_yass
		  execute_blast
		  find_conserved_regions
		  execute_structure);




###############################################################################
###############################################################################
##
##
##		                 FUNCTIONS
##
##
###############################################################################
###############################################################################





###############################################################################
#
#	Function execute_masking_coding
#
#	Goal : The function will execute the masking coding step on the
#              reference file and the other genomes if there's a genbank
#              annotation file aside
#
#	Param :
#               - $reference_file : the location of the reference file
#		- $other_files : the reference on the hash table containing
#                 other files
#               - $cmd_mask : the command to execute the masking script
#               - $current_mask_outdir : the outdir of masked files
#               - $content_warning : the content of the warning file
#               - $quiet : the quiet mode
#
#	Returns :
#               - $reference_file_mask, the location of the reference file
#                 after masking
#               - $mask_other : reference on the  update the hash table of
#                 other file by putting the right directory to find them
#
###############################################################################

sub execute_masking_coding{

  (my $reference_file,
   my $other_files,
   my $cmd_mask,
   my $mask_outdir,
   my $content_warning,
   my $quiet) = @_;

  #Location of the reference file after masking
  my $reference_file_mask = "";

  #Hash table with new location of other file masked
  my %mask_others;

  #If there are existing files : erase them
  if (-d $mask_outdir) {
    &erase_existing_files($mask_outdir);
  }

  #Create outdir
  &create_directory($mask_outdir);


  #####Reference masking######
  if (!$quiet) {
    print "   file: $reference_file\n";
  }
  (my $name_ref,
   my $dir_ref,
   my $suffix) = fileparse($reference_file,qr/\.[^.]*/);

  #Check if a genbank file exists
  my $gbk = File::Spec->catfile($dir_ref,$name_ref.".gbk");
  my $gb = File::Spec->catfile($dir_ref,$name_ref.".gb");
  my $real_genbank = "";

  if (-e $gbk) {
    $real_genbank = $gbk;
  }
  if (-e $gb) {
    $real_genbank = $gb;
  }

  #Genbank file exists : process reference
  if ($real_genbank ne "") {

    my $ref_masking_dir = File::Spec->catdir($mask_outdir,
 					     "reference");
    $reference_file_mask = File::Spec->catfile($ref_masking_dir,
 					       $name_ref."_mask".$suffix);
    &create_directory($ref_masking_dir);

    #Complete command line
    my $cmd = $cmd_mask." \"$reference_file\" \"$real_genbank\" ";
    $cmd .= "--output \"$reference_file_mask\"";

    #Execute masking
    `$cmd`;

  } else {
    $$content_warning .= 
      "\nWARNING : Reference file \"$reference_file\" has no associated ";
    $$content_warning .= "genbank file (.gbk, .gb) : it won't be masked\n\n";
    $reference_file_mask = $reference_file;
  }


  #####Other files masking######
  foreach my $dir_other (keys(%$other_files)) {

    #Create outdir
    my $other_outdir = "";

    #Case current directory
    if ($dir_other eq ".") {
      (my $name_other,
       my $dir,
       my $suffix) = fileparse($other_files->{$dir_other},qr/\.[^.]*/);
      $other_outdir = File::Spec->catdir($mask_outdir,
 					 $name_other);
    } else {
      my @dirs = File::Spec->splitdir($dir_other);
      my $nb_elements = @dirs;
      $other_outdir = File::Spec->catdir($mask_outdir,
 					 $dirs[$nb_elements - 1]);
    }

    &create_directory($other_outdir);

    #Recuperate each fasta file within directory
    my @fasta_files = split('\t',$other_files->{$dir_other});
    foreach my $fasta (@fasta_files) {
      $fasta = File::Spec->catfile($dir_other,$fasta);

      if (!$quiet) {
 	print "   file: $fasta\n";
      }

      (my $name_other,
       my $dir,
       my $suffix) = fileparse($fasta,qr/\.[^.]*/);

      #Check if a genbank file exists
      my $gbk = File::Spec->catfile($dir,$name_other.".gbk");
      my $gb = File::Spec->catfile($dir,$name_other.".gb");
      my $real_genbank = "";

      if (-e $gbk) {
 	$real_genbank = $gbk;
      }
      if (-e $gb) {
 	$real_genbank = $gb;
      }

      #Genbank file exists : process file
      if ($real_genbank ne "") {

 	my $other_file_mask = File::Spec->catfile($other_outdir,
 						  $name_other."_mask".$suffix);


 	#Complete command line
 	my $cmd = $cmd_mask." \"$fasta\" \"$real_genbank\" ";
 	$cmd .= "--output \"$other_file_mask\"";

 	#Execute masking
 	`$cmd`;

 	#Add masked fasta file into temporary hash table
 	$mask_others{$other_outdir} .= $name_other."_mask".$suffix."\t";


      } else {
 	$$content_warning .= "\nWARNING : File \"$fasta\" has no associated ";
 	$$content_warning .= "genbank file (.gbk, .gb) : it won't be masked\n\n";
 	#Copy it to outdir to keep it with the other files for the rest
 	#of the pipeline
 	(my $name_other,
 	 my $dir,
 	 my $suffix) = fileparse($fasta,qr/\.[^.]*/);
 	copy($fasta,$other_outdir) or
 	  exit_script("\nERROR : Could not copy \"$fasta\" into \"$other_outdir\"\n\n");
 	$mask_others{$other_outdir} .= $name_other.$suffix."\t";
      }

    }


  }

  return ($reference_file_mask,\%mask_others);


}


###############################################################################
#
#	Function execute_masking_redundancy
#
#	Goal : The function will execute the masking coding step on the
#              reference file and the other genomes if there's a genbank
#              annotation file aside
#
#	Param :
#		- $other_files : the reference on the hash table containing
#                 other files
#               - $cmd_mask : the command to execute the masking script
#               - $current_mask_outdir : the outdir of masked files
#               - $content_warning : the content of the warning file
#               - $quiet : the quiet mode
#
#	Returns :
#               - $mask_other : reference on the  update the hash table of
#                 other file by putting the right directory to find them
#
###############################################################################

sub execute_masking_redundancy{

  (my $other_files,
   my $cmd_mask,
   my $mask_outdir,
   my $content_warning,
   my $quiet) = @_;


  #Hash table with new location of other file masked
  my %mask_others;

  #If there are existing files : erase them
  if (-d $mask_outdir) {
    &erase_existing_files($mask_outdir);
  }

  #Create outdir
  &create_directory($mask_outdir);

  #####Other files masking######
  foreach my $dir_other (keys(%$other_files)) {

    if (!$quiet) {
      print "   file: $dir_other\n";
    }

    #Create outdir
    my $other_outdir = "";

    #Case current directory
    if ($dir_other eq ".") {
      (my $name_other,
       my $dir,
       my $suffix) = fileparse($other_files->{$dir_other},qr/\.[^.]*/);
      $other_outdir = File::Spec->catdir($mask_outdir,
 					 $name_other);
    } else {
      my @dirs = File::Spec->splitdir($dir_other);
      my $nb_elements = @dirs;
      $other_outdir = File::Spec->catdir($mask_outdir,
 					 $dirs[$nb_elements - 1]);
    }

    &create_directory($other_outdir);

    #Complete command line
    my $cmd = $cmd_mask." \"$dir_other\" \"$other_outdir\"";

    #Execute masking
    `$cmd`;

    #Update mask_other hash table
    $mask_others{$other_outdir} = $other_files->{$dir_other};
    delete($mask_others{$dir_other});



  }

  return(\%mask_others);

}




###############################################################################
#
#	Function execute_yass
#
#	Goal : The function will execute the alignment step with Yass on the
#              reference against all other files
#
#	Param :
#               - $reference_file : the location of the reference file
#		- $other_files : the reference on the hash table containing
#                 other files
#               - $cmd_yass : the command to execute Yass
#               - $yass_outdir : the outdir of alignment files
#               - $quiet : the quiet mode
#
#	Returns :
#               nothing
#
###############################################################################

sub execute_yass{

  (my $reference_file,
   my $other_files,
   my $cmd_yass,
   my $yass_outdir,
   my $quiet) = @_;


  #If there are existing files : erase them
  if (-d $yass_outdir) {
    &erase_existing_files($yass_outdir);
  }

  #Create yass alignments outdir if not exists
  &create_directory($yass_outdir);

  #Add reference on the command line
  $cmd_yass .= " \"$reference_file\" ";

  #Compare the reference against all other genomes
  foreach my $dir_other (keys(%$other_files)) {

    my @split_files = split('\t',$other_files->{$dir_other});

    foreach my $file (@split_files) {
      my $fasta_other = File::Spec->catfile($dir_other,
					    $file);
      if (!$quiet) {
	print "   seq1: $reference_file\n";
	print "   seq2: $fasta_other\n\n";
      }

      (my $name_other,
       my $dir,
       my $suffix) = fileparse($fasta_other,qr/\.[^.]*/);

      my $outfile = File::Spec->catfile($yass_outdir,
					$name_other.".yass");

      #Generate current command line : -d 2 -> blast tabular output
      my $cmd = $cmd_yass."\"$fasta_other\" -o \"$outfile\" -d 2";


      #Execute comparison
      `$cmd`;
    }

  }


}


###############################################################################
#
#	Function execute_blast
#
#	Goal : The function will execute the alignment step with Blast on the
#              reference against all other files
#
#	Param :
#               - $reference_file : the location of the reference file
#		- $other_files : the reference on the hash table containing
#                 other files
#               - $cmd_blast : the command to execute Blast
#               - $blast_outdir : the outdir of alignment files
#               - $quiet : the quiet mode
#
#	Returns :
#               nothing
#
###############################################################################

sub execute_blast{

  (my $reference_file,
   my $other_files,
   my $cmd_blast,
   my $blast_outdir,
   my $quiet) = @_;


  #If there are existing files : erase them
  if (-d $blast_outdir) {
    &erase_existing_files($blast_outdir);
  }

  #Create blast alignments outdir if not exists
  &create_directory($blast_outdir);

  #Add reference on the command line
  $cmd_blast .= " -i \"$reference_file\" ";

  #Get reference ID in fasta file for the last step : change names
  #inside blast result file
  open(REF,$reference_file) or
    exit_script("\nERROR : Could not open reference file \"$reference_file\"\n\n");
  my $id_ref = "";
  while (($id_ref eq "") && (my $line = <REF>)) {
    if ($line =~ />(.+)[\n\r]/) {
      $id_ref = $1;
    }
  }
  close(REF);
  $id_ref =~ s/ /_/g;

  #Compare the reference against all other genomes
  foreach my $dir_other (keys(%$other_files)) {

    my @split_files = split('\t',$other_files->{$dir_other});

    foreach my $file (@split_files) {
      my $fasta_other = File::Spec->catfile($dir_other,
					    $file);
      if (!$quiet) {
	print "   seq1: $reference_file\n";
	print "   seq2: $fasta_other\n\n";
      }

      (my $name_other,
       my $dir,
       my $suffix) = fileparse($fasta_other,qr/\.[^.]*/);

      my $outfile = File::Spec->catfile($blast_outdir,
					$name_other.".blast");

      #Generate current command line : -D 1 -> blast tabular output
      # -p blastn for nucleotide comparison
      my $cmd = $cmd_blast."-j \"$fasta_other\" -o \"$outfile\" -D 2";
      $cmd .= " -p blastn";

      #Execute comparison
      `$cmd`;

      #Change subject and query name inside result file because
      #blast troncate them

      #Get other ID
      open(OTHER,$fasta_other) or
	exit_script("\nERROR : Could not open other file \"$fasta_other\"\n\n");
      my $id_other = "";
      while (($id_other eq "") && (my $line = <OTHER>)) {
	if ($line =~ />(.+)[\n\r]/) {
	  $id_other = $1;
	}
      }
      close(OTHER);
      $id_other =~ s/ /_/g;

      #Replace names : first 2 columns
      my $new_content = "";
      open(RES,$outfile) or
	exit_script("\nERROR : Coulnd not read blast result file \"$outfile\"\n\n");
      while (my $line = <RES>) {
	if ($line !~ /^#/) {
	  my @split = split('\t',$line);
	  my $nb_elements = @split; 
	  $new_content .= $id_ref."\t".$id_other."\t";
	  for (my $i = 2; $i < $nb_elements - 1; $i++) {
	    $new_content .= $split[$i]."\t";
	  }
	  $new_content .= $split[$nb_elements - 1]."\n";
	} else {
	  $new_content .= $line;
	}
      }
      close(RES);

      #Write new result file
      &write_file($new_content,$outfile);

    }


  }

}

###############################################################################
#
#	Function find_conserved_regions
#
#	Goal : The function will execute the finding conserved regions step
#              according to alignments (Yass and/or Blast)
#	Param :
#               - $reference_file : the location of the reference file
#		- $other_files : the reference on the hash table containing
#                 other files
#               - $cmd_cg_seq : the command to execute CGseqcore
#               - $alignments_outdir : the outdir of alignment files
#               - $yass_available : yass wanted or not
#               - $blast_available : blast wanted or not
#               - $yass_outdir_name : name of the outdir of yass
#               - $blast_outdir_name : name of the outdir of Blast
#               - $cg_seq_outdir : outdir of conserved regions
#               - $quiet : the quiet mode
#
#	Returns :
#               Nothing
#
###############################################################################

sub find_conserved_regions{


  (my $reference_file,
   my $other_files,
   my $cmd_cg_seq,
   my $alignments_outdir,
   my $yass_available,
   my $blast_available,
   my $yass_outdir_name,
   my $blast_outdir_name,
   my $cg_seq_outdir,
   my $quiet) = @_;


  #command line to execute cg_seq
  my $command_line = "";

  #Alignment files from yass output
  my @yass_aln;
  #Number of yass alignment files
  my $nb_elements_yass = 0;

  #Alignment files from blast output
  my @blast_aln;
  #Number of blast alignment files
  my $nb_elements_blast = 0;
  my $no_error = 1;

  my $dir_aln_yass = File::Spec->catdir($alignments_outdir,
					$yass_outdir_name,
					"*.yass");
  if($dir_aln_yass =~ /\s/){
    $dir_aln_yass = quotemeta($dir_aln_yass);
  }


  if($yass_available){
    @yass_aln = glob("$dir_aln_yass");
    $nb_elements_yass = @yass_aln;
  }

  my $dir_aln_blast = File::Spec->catdir($alignments_outdir,
					 $blast_outdir_name,
					 "*.blast");
  if($dir_aln_blast =~ /\s/){
    $dir_aln_blast = quotemeta($dir_aln_blast);
  }

  if($blast_available){
    @blast_aln = glob("$dir_aln_blast");
    $nb_elements_blast = @blast_aln;
  }

  #No alignments found -> step skipped
  if (($nb_elements_yass + $nb_elements_blast)== 0) {
    print "\nERROR: No alignments found in directory \"$alignments_outdir\"";
    #print " -> this step will be skipped\n";
    #print "You may have to run Yass and/or Blast to correct this\n\n";
    $no_error = 0;

  }

  if($no_error){

    #Construct other files parameter
    my $others = "";
    foreach my $dir_other(keys(%$other_files)){
      $others .= "-f \"$dir_other\" ";
    }

    #Add fasta output to command line, reference file and other genomes
    $cmd_cg_seq .= " -o 2 -r \"$reference_file\" $others";

    #Yass alignments part
    if($yass_available && ($nb_elements_yass != 0)){

      my $cmd = "";

      foreach my $aln_file(@yass_aln){
	$cmd .= "\"$aln_file\" ";

      }

      #if(!$quiet){
      #	print "->Proceed with Yass alignments\n";
      #}

      my $yass_conserved_outdir = File::Spec->catdir($cg_seq_outdir,
						     $yass_outdir_name);

      #If there are existing files : erase them
      if (-d $yass_conserved_outdir) {
	&erase_existing_files($yass_conserved_outdir);
      }

      #Create output
      &create_directory($yass_conserved_outdir);

      $cmd .= " -d \"$yass_conserved_outdir\"";
      $cmd = $cmd_cg_seq.$cmd;
      `$cmd`;

      if(!$quiet){
	#Check the number of conserved regions
	my $search_conserved = File::Spec->catfile($yass_conserved_outdir,
						   "*.fna");
	if($search_conserved =~ /\s/){
	  $search_conserved = quotemeta($search_conserved);
	}
	my @conserved_files = glob("$search_conserved");
	my $nb_conserved = @conserved_files;
	print "Found ".($nb_conserved)." conserved regions with YASS\n\n";

      }

    }

    #Blast alignments part
    if($blast_available && ($nb_elements_blast != 0)){

      my $cmd = "";

      foreach my $aln_file(@blast_aln){
	$cmd .= "\"$aln_file\" ";

      }

      #if(!$quiet){
      #	print "->Proceed with Blast alignments\n";
      #}

      my $blast_conserved_outdir = File::Spec->catdir($cg_seq_outdir,
						     $blast_outdir_name);

      #If there are existing files : erase them
      if (-d $blast_conserved_outdir) {
	&erase_existing_files($blast_conserved_outdir);
      }

      #Create output
      &create_directory($blast_conserved_outdir);

      $cmd .= " -d \"$blast_conserved_outdir\"";
      $cmd = $cmd_cg_seq.$cmd;

      `$cmd`;

      if(!$quiet){
	#Check the number of conserved regions
	my $search_conserved = File::Spec->catfile($blast_conserved_outdir,
						   "*.fna");
	if($search_conserved =~ /\s/){
	  $search_conserved = quotemeta($search_conserved);
	}
	my @conserved_files = glob("$search_conserved");
	my $nb_conserved = @conserved_files;
	print "Found ".($nb_conserved)." conserved regions with BLAST\n\n";

      }

    }

  }


}




###############################################################################
#
#	Function execute_structure
#
#	Goal : The function will execute carnac and/or RNAz on 
#              conserved regions found by the previous step
#
#	Param :
#               - $carnac_available : carnac wanted and usable
#               - $rnaz_available : RNAz wanted and usable
#               - $cmd_carnac : the command to execute carnac
#               - $cmd_rnaz : the command to execute RNAz
#               - $cmd_clustalw : the command to execute clustalw
#               - $cmd_rnazwindow : the command to execute rnazWindow.pl
#               - $cmd_rnaplot : the command to execute RNAplot
#               - $cg_seq_outdir : the outdir of conserved regions
#               - $yass_available : yass software wanted or not
#               - $blast_available : blast software wanted or not
#               - $yass_outdir_name : name of the outdir of yass
#               - $blast_outdir_name : name of the outdir of Blast
#               - $carnac_outdir : carnac outdir
#               - $rnaz_outdir : rnaz outdir
#               - $predicted_outdir : outdir for predicted RNAs
#               - $quiet : the quiet mode
#
#	Returns :
#               Nothing
#
###############################################################################


sub execute_structure{

  (my $carnac_available,
   my $rnaz_available,
   my $cmd_carnac,
   my $cmd_rnaz,
   my $cmd_clustalw,
   my $cmd_rnazwindow,
   my $cmd_rnaplot,
   my $cg_seq_outdir,
   my $yass_available,
   my $blast_available,
   my $yass_outdir_name,
   my $blast_outdir_name,
   my $carnac_outdir,
   my $rnaz_outdir,
   my $predicted_outdir,
   my $quiet) = @_;


  #Conserved regions from yass output
  my @yass_sets;

  #Number of regions found by yass
  my $nb_yass_sets = 0;

  #Conserved regions from blast output
  my @blast_sets;

  #Number of regions found by blast
  my $nb_blast_sets = 0;

  #Boolean that says if rnazwindow is available to deal with too long
  #sequences
  my $is_rnazwindow = 1;
  my $rnazwindow_used = 0;

  #Boolean that says if RNAplot is available for structure image generation
  my $is_rnaplot = 1;

  #Ratio paired bases/length sequence to consider an interesting prediction
  my $ratio_threshold = 0.1;

  #Boolean that says if no error occured
  my $no_error = 1;

  #Location of carnac outdirs
  my $carnac_yass_general_outdir = "";
  my $carnac_blast_general_outdir = "";

  #Location of rnaz outdirs
  my $rnaz_yass_general_outdir = "";
  my $rnaz_blast_general_outdir = "";

  #Location of RNA predicted outdir
  my $yass_predicted_RNA = "";
  my $blast_predicted_RNA = "";

  #Length to consider a too long sequence for RNAz
  my $too_long = 300;

  #Suffix for rnaplot generated images
  my $rnaplot_image_suffix = "";


  #Check RNAz window and RNAplot availability
  if($cmd_rnazwindow eq ""){
    $is_rnazwindow = 0;
  }
  if($cmd_rnaplot eq ""){
    $is_rnaplot = 0;
  }
  else{
    #Check suffix image
    if($cmd_rnaplot =~ /-o\s+(ps|gml|svg|xrna)/){
      $rnaplot_image_suffix = $1;
    }
    else{
      $rnaplot_image_suffix = "ps";
    }

  }

  #Strand prediction for rnaz (0->forward,1->reverse,2->both)
  my $strand_process = 2;
  if($cmd_rnaz =~ /(\-r|\-\-reverse)/){
    $strand_process = 1;
  }
  else{
    if($cmd_rnaz =~ /(\-f|\-\-forward)/){
      $strand_process = 0;
    }
  }


  #Test if there are some fasta files under the alignments outdir
  #in yass or blast outdirs

  my $dir_set_yass = File::Spec->catdir($cg_seq_outdir,
					$yass_outdir_name,
					"*.fna");
  if($dir_set_yass =~ /\s/){
    $dir_set_yass = quotemeta($dir_set_yass);
  }

  if($yass_available){
    @yass_sets = glob("$dir_set_yass");
    $nb_yass_sets = @yass_sets;
  }

  my $dir_set_blast = File::Spec->catdir($cg_seq_outdir,
					 $blast_outdir_name,
					 "*.fna");
  if($dir_set_blast =~ /\s/){
    $dir_set_blast = quotemeta($dir_set_blast);
  }

  if($blast_available){
    @blast_sets = glob("$dir_set_blast");
    $nb_blast_sets = @blast_sets;
  }


  #No sets found -> step skipped
  if (($nb_yass_sets + $nb_blast_sets)== 0) {
    print "\nERROR: No sets found in directory \"$cg_seq_outdir\"";
    #print " -> this step will be skipped\n";
    #print "You may have to run cg_seq\n\n";
    $no_error = 0;

  }


  #Directory management for carnac outputs
  if($carnac_available){
    &create_directory($carnac_outdir);

    $carnac_yass_general_outdir = File::Spec->catdir($carnac_outdir,
						     $yass_outdir_name);
    $carnac_blast_general_outdir = File::Spec->catdir($carnac_outdir,
						     $blast_outdir_name);


    #If there are existing files : erase them
    if (-d $carnac_yass_general_outdir) {
      &erase_existing_files($carnac_yass_general_outdir);
    }
    if (-d $carnac_blast_general_outdir) {
      &erase_existing_files($carnac_blast_general_outdir);
    }

    if($yass_available){
      &create_directory($carnac_yass_general_outdir);
    }

    if($blast_available){
      &create_directory($carnac_blast_general_outdir);
    }

  }

  #Directory management for rnaz outputs
  if($rnaz_available){
    &create_directory($rnaz_outdir);

    $rnaz_yass_general_outdir = File::Spec->catdir($rnaz_outdir,
						     $yass_outdir_name);
    $rnaz_blast_general_outdir = File::Spec->catdir($rnaz_outdir,
						     $blast_outdir_name);


    #If there are existing files : erase them
    if (-d $rnaz_yass_general_outdir) {
      &erase_existing_files($rnaz_yass_general_outdir);
    }
    if (-d $rnaz_blast_general_outdir) {
      &erase_existing_files($rnaz_blast_general_outdir);
    }

    if($yass_available){
      &create_directory($rnaz_yass_general_outdir);
    }

    if($blast_available){
      &create_directory($rnaz_blast_general_outdir);
    }

  }

  #Directory management for RNA predicted
  $yass_predicted_RNA = File::Spec->catdir($predicted_outdir,
					   $yass_outdir_name);

  $blast_predicted_RNA = File::Spec->catdir($predicted_outdir,
					    $blast_outdir_name);


  if (-d $yass_predicted_RNA) {
    &erase_existing_files($yass_predicted_RNA);
  }
  if (-d $blast_predicted_RNA) {
    &erase_existing_files($blast_predicted_RNA);
  }

  if ($yass_available) {
    &create_directory($yass_predicted_RNA);
  }
  if ($blast_available) {
    &create_directory($blast_predicted_RNA);
  }


  #Begin of structure prediction
  if ($no_error) {

    #Yass sets
    if ($yass_available && ($nb_yass_sets != 0)) {

      #if (!$quiet) {
      #	print "-> Processing structure prediction on the ".($nb_yass_sets);
      #	print " sets found upon yass alignments\n";
      #}

      my $nb_sets = 0;

      #Sort files by their names
      @yass_sets = sort sort_by_num @yass_sets;

      #Process each fasta
      foreach my $fasta (@yass_sets) {
	(my $name,
	 my $dir,
	 my $suffix) = fileparse($fasta,qr/\.[^.]*/);

	#Boolean that says if the current file is predicted as RNA by carnac
	my $is_RNA_carnac = 0;

	#Boolean that says if the current file is predicted as RNA by RNAz
	my $is_RNA_rnaz = 0;

	#Current outdir for Carnac
	my $outdir_carnac = "";

	#Outdir for Carnac
	my $carnac_out = "";

	#Current outdir for RNAz
	my $outdir_rnaz = "";

	#Current file for RNAz
	my $outfile_rnaz = "";

	#Current outfile for clustalw
	my $outfile_clustal = "";

	#Content of the RNAz file
	my $content_rnaz = "";


	####
	# Carnac
	####
	if ($carnac_available) {

	  $outdir_carnac = File::Spec->catdir($carnac_yass_general_outdir,
					      $name);

	  my $cmd = $cmd_carnac." \"$fasta\" -o \"$outdir_carnac\"";

	  #Execute carnac
	  `$cmd`;

	  #Check it ratio paired bases / length sequence
	  $carnac_out = File::Spec->catfile($outdir_carnac,
					    "carnac.out");
	  my $ratio = &get_ratio_paired_bases($carnac_out);

	  #Interesting prediction
	  if ($ratio >= $ratio_threshold) {
	    $is_RNA_carnac = 1;
	  }

	}

	####
	# RNAz
	####
	if ($rnaz_available) {

	  $outdir_rnaz = File::Spec->catdir($rnaz_yass_general_outdir,
					    $name);

	  &create_directory($outdir_rnaz);


	  #Execute clustalw
	  $outfile_clustal = File::Spec->catfile($outdir_rnaz,
						 $name.".aln");
	  my $guided_tree_clustal = File::Spec->catfile($cg_seq_outdir,
							$yass_outdir_name,
							$name.".dnd");
	  my $cmd = $cmd_clustalw." -infile=\"$fasta\" ";
	  $cmd .= "-outfile=\"$outfile_clustal\" -outorder=\"input\"";
	  `$cmd`;
	  unlink($guided_tree_clustal);


	  #Execute RNAz or rnazWindow.pl for too long sequences if available
	  $outfile_rnaz = File::Spec->catfile($outdir_rnaz,
					      $name.".rnaz");
	  my $current_cmd_rnaz = $cmd_rnaz." \"$outfile_clustal\" ";
	  $current_cmd_rnaz .= "-o \"$outfile_rnaz\"";
	  $rnazwindow_used = 0;

	  if ($is_rnazwindow) {
	    open(FASTA,$fasta) or
	      exit_script("\nERROR : Could not open \"$fasta\" for reading \n\n");
	    my $content_fasta = "";
	    my $length_seq = -1;
	    while (($length_seq == -1) && (my $line = <FASTA>)) {
	      $content_fasta .= $line;
	      if ($content_fasta =~ />.+[\n\r]([ATCGNatcgn\n\r]+)>.+/) {
		my $seq_ref = $1;
		$seq_ref =~ s/[\n\r]//g;
		$length_seq = length($seq_ref);
	      }
	    }
	    close(FASTA);
	    if ($length_seq >= $too_long) {
	      $current_cmd_rnaz = $cmd_rnazwindow." \"$outfile_clustal\" | ";
	      $current_cmd_rnaz .= $cmd_rnaz." -o \"$outfile_rnaz\"";
	      $rnazwindow_used = 1;
	    }


	  }

	  `$current_cmd_rnaz`;

	  #Check result file
	  $content_rnaz = &read_file($outfile_rnaz);

	  if ($content_rnaz =~ /Prediction: RNA/) {
	    $is_RNA_rnaz = 1;

	  }


	}

	#If predicted RNA : make set
	if ($is_RNA_carnac || $is_RNA_rnaz) {

	  $nb_sets++;

	  my $outdir_set = File::Spec->catdir($yass_predicted_RNA,
					      "set_".$nb_sets);
	  &create_directory($outdir_set);

	  #Copy fasta
	  copy($fasta,$outdir_set);


	  #RNAz result part
	  if ($is_RNA_rnaz) {

	    #Copy RNAz result file
	    copy($outfile_rnaz,$outdir_set);

	    #Check how much predictions
	    my $nb_RNA = 0;
	    while ($content_rnaz =~ /Prediction: RNA/g) {
	      $nb_RNA++;
	    }

	    #Producing forward strand result and reverse strand
	    my $produce_forward = 0;
	    my $produce_reverse = 0;


	    #One prediction
	    if ($nb_RNA == 1) {


	      #If no rnazWindow : one strand prediction
	      if (!$rnazwindow_used) {

		#Forward
		if ($strand_process == 0) {
		  $produce_forward = 1;
		}

		#Reverse
		if ($strand_process == 1) {
		  $produce_reverse = 1;
		}

		#Both strands : check wich strand
		if ($strand_process == 2) {
		  my $strand = "+";

		  my @split_rnaz = split(qr/#+\s+RNAz\s[0-9]+\.[0-9]+\s+#+/,
					 $content_rnaz);
		  foreach my $split_part (@split_rnaz) {
		    if (($split_part =~ /Reading direction: reverse/) &&
			($split_part =~ /Prediction: RNA/)) {
		      $strand = "-";
		    }
		  }

		  #Forward
		  if ($strand eq "+") {
		    $produce_forward = 1;
		  }
		  #Reverse
		  else{
		    $produce_reverse = 1;
		  }
		}
	      }
	      #Else rnazwindow.pl used : check part predicted as RNA
	      else {

		if ($content_rnaz =~ /Reading direction: reverse/) {
		  $produce_reverse = 1;
		}
		else{
		  $produce_forward = 1;
		}
	      }

	    }
	    #More than one prediction
	    else {

	      #If no rnazWindow.pl used : both strand prediction
	      #Generate reverse fasta, reverse alignment and copy them
	      #and the rnaz result into the set directory
	      if ((!$rnazwindow_used) && ($strand_process == 2)) {

		$produce_forward = 1;
		$produce_reverse = 1;

	      }

	      else{

		#Split file
		my @split_rnaz = split(qr/#+\s+RNAz\s[0-9]+\.[0-9]+\s+#+/,
				       $content_rnaz);
		foreach my $split_part (@split_rnaz) {

		  if ($split_part =~ /Prediction: RNA/) {

		    if ($split_part =~ /Reading direction: reverse/) {
		      $produce_reverse = 1;
		    }
		    else{
		      $produce_forward = 1;
		    }
		  }
		}

	      }
	    }

	    #Produce forward result files
	    if($produce_forward){

	      #Fasta and RNAz result file already copied : clustalw are only
	      #missing
	      copy($outfile_clustal,$outdir_set);

	    }

	    #Produce reverse result files
	    if($produce_reverse){
	      my $reverse_fasta = File::Spec->catfile($outdir_set,
						      $name."_reverse".$suffix);
	      &execute_reverse_complementary($fasta,
					     $reverse_fasta);

	      my $outfile_clustal_reverse = 
		File::Spec->catfile($outdir_set,
 				  $name."_reverse.aln");
	      my $guided_tree_clustal_reverse =
		File::Spec->catfile($outdir_set,
				    $name."_reverse.dnd");
	      my $cmd_clustal_reverse = $cmd_clustalw;
	      $cmd_clustal_reverse .= " -infile=\"$reverse_fasta\" ";
	      $cmd_clustal_reverse .= "-outfile=\"$outfile_clustal_reverse\" ";
	      $cmd_clustal_reverse .= "-outorder=\"input\"";
	      `$cmd_clustal_reverse`;
	      unlink($guided_tree_clustal_reverse);



	    }

	    #Image part
	    if($is_rnaplot){

	      my @split_rnaz = split(qr/#+\s+RNAz\s[0-9]+\.[0-9]+\s+#+/,
				     $content_rnaz);

	      #Produce images for the reference sequence which
	      #starts with >1
	      foreach my $split_part(@split_rnaz){
		my $cpt = 1;
		my $strand_name = "";
		if($split_part =~ /Prediction: RNA/){
		  if($split_part =~ /Reading direction: (forward|reverse)/){
		    if($1 eq "forward"){
		      $strand_name = "";
		    }
		    else{
		      $strand_name = "_reverse";
		    }
		  }

		  if($split_part =~ 
		       /(>1\|.+[\n\r][-CUGUA\._]+[\n\r][-\.\(\)]+)/g) {
		    my $ref_struct = $1;
		    $ref_struct =~ s/\|/_/g;
		    my $temp_file = File::Spec->catfile("temp.txt");
		    &write_file($ref_struct,$temp_file);
		    my $cmd = $cmd_rnaplot." < \"$temp_file\"";
		    `$cmd`;
		    my $ref_image = 
		      File::Spec->catfile("1*.".$rnaplot_image_suffix);
		    my @search_images = glob("$ref_image");
		    my $new_image_file = 
		      File::Spec->catfile($outdir_set,
					  $name.$strand_name."_rnaz.".$rnaplot_image_suffix);
		    while(-e $new_image_file){
		      $new_image_file = 
			File::Spec->catfile($outdir_set,
					    $name.$strand_name."_rnaz_".$cpt.".".$rnaplot_image_suffix);
		      $cpt++;
		    }
		    move($search_images[0],$new_image_file) or
		      exit_script("\nERROR : Could not move \"$search_images[0]\"\n\n");
		    unlink($temp_file);
		  }
		}
	      }
	    }
	  }


	  #Carnac result part
	  if ($is_RNA_carnac) {

	    #Result from carnac
	    my $res_carnac = File::Spec->catfile($outdir_set,
						 $name."_carnac.out");
	    copy($carnac_out,$res_carnac);


	    my $reverse_fasta = "";

	    #Reverse set part
	    $reverse_fasta = 
	      File::Spec->catfile($outdir_set,
				  $name."_reverse".$suffix);

	    &execute_reverse_complementary($fasta,
					   $reverse_fasta);

	    #Image
	    if ($is_rnaplot) {

	      #Replace '|' character in carnac.out file because
	      #RNAplot under windows throws an error if that character is found
	      my $temp_carnac_out = File::Spec->catfile($outdir_carnac,
							"temp_carnac.out");
	      my $content_temp = &read_file($carnac_out);
	      $content_temp =~ s/\|/_/g;
	      &write_file($content_temp,$temp_carnac_out);


	      #Image for forward strand
	      my $cmd = $cmd_rnaplot." < \"$temp_carnac_out\"";

	      `$cmd`;
	      my $ref_image = File::Spec->catfile("1*.".$rnaplot_image_suffix);
	      my @search_images = glob("$ref_image");
	      my $new_image_file = 
		File::Spec->catfile($outdir_set,
				    $name."_carnac.".$rnaplot_image_suffix);

	      move($search_images[0],$new_image_file) or
		exit_script("\nERROR : Could not move \"$search_images[0]\"\n\n");
	      my $other_images = 
		File::Spec->catfile("*.".$rnaplot_image_suffix);
	      my @search_other = glob("$other_images");
	      unlink(@search_other);
	      unlink($temp_carnac_out);

	      #Image for reverse strand
	      my $outdir_carnac_reverse = File::Spec->catdir($outdir_set,
							     "reverse");
	      my $out_carnac_reverse = 
		File::Spec->catfile($outdir_carnac_reverse,
				    "carnac.out");
	      $cmd = $cmd_carnac." \"$reverse_fasta\" ";
	      $cmd .= "-o \"$outdir_carnac_reverse\"";
	      `$cmd`;

	      #copy result from carnac for reverse strand
	      my $res_reverse_carnac = File::Spec->catfile($outdir_set,
							   $name."_reverse_carnac.out");
	      copy($out_carnac_reverse,$res_reverse_carnac);

	      #Replace '|' character in carnac.out file because
	      #RNAplot under windows throws an error if that character is found
	      my $reverse_temp_carnac_out = 
		File::Spec->catfile($outdir_carnac_reverse,
				    "reverse_temp_carnac.out");
	      my $reverse_content_temp = &read_file($out_carnac_reverse);
	      $reverse_content_temp =~ s/\|/_/g;
	      &write_file($reverse_content_temp,$reverse_temp_carnac_out);

	      my $current_cmd_rnaplot .= 
		$cmd_rnaplot." < \"$reverse_temp_carnac_out\"";
	      `$current_cmd_rnaplot`;

	      $ref_image = File::Spec->catfile("1*.".$rnaplot_image_suffix);
	      @search_images = glob("$ref_image");
	      $new_image_file = 
		File::Spec->catfile($outdir_set,
				    $name."_reverse_carnac.".$rnaplot_image_suffix);
	      move($search_images[0],$new_image_file) or
		exit_script("\nERROR : Could not move \"$search_images[0]\"\n\n");
	      $other_images = File::Spec->catfile("*.".$rnaplot_image_suffix);
	      @search_other = glob("$other_images");
	      unlink(@search_other);
	      unlink($reverse_temp_carnac_out);
	      rmtree($outdir_carnac_reverse);
	    }
	  }


	}

      }

      if (!$quiet) {
	print "Found $nb_sets putative ncRNAS from YASS data\n\n";

      }
    }


    #Blast sets
    if ($blast_available && ($nb_blast_sets != 0)) {

      #if (!$quiet) {
      #	print "-> Processing structure prediction on the ".($nb_blast_sets);
      # print " sets found upon blast alignments\n";
      #}

      my $nb_sets = 0;

      #Sort files by their names
      @blast_sets = sort sort_by_num @blast_sets;

      #Process each fasta
      foreach my $fasta (@blast_sets) {
	(my $name,
	 my $dir,
	 my $suffix) = fileparse($fasta,qr/\.[^.]*/);

	#Boolean that says if the current file is predicted as RNA by carnac
	my $is_RNA_carnac = 0;

	#Boolean that says if the current file is predicted as RNA by RNAz
	my $is_RNA_rnaz = 0;

	#Current outdir for Carnac
	my $outdir_carnac = "";

	#Outdir for Carnac
	my $carnac_out = "";

	#Current outdir for RNAz
	my $outdir_rnaz = "";

	#Current file for RNAz
	my $outfile_rnaz = "";

	#Current outfile for clustalw
	my $outfile_clustal = "";

	#Content of the RNAz file
	my $content_rnaz = "";


	####
	# Carnac
	####
	if ($carnac_available) {

	  $outdir_carnac = File::Spec->catdir($carnac_blast_general_outdir,
					      $name);

	  my $cmd = $cmd_carnac." \"$fasta\" -o \"$outdir_carnac\"";

	  #Execute carnac
	  `$cmd`;

	  #Check it ratio paired bases / length sequence
	  $carnac_out = File::Spec->catfile($outdir_carnac,
					    "carnac.out");
	  my $ratio = &get_ratio_paired_bases($carnac_out);

	  #Interesting prediction
	  if ($ratio >= $ratio_threshold) {
	    $is_RNA_carnac = 1;
	  }

	}

	####
	# RNAz
	####
	if ($rnaz_available) {

	  $outdir_rnaz = File::Spec->catdir($rnaz_blast_general_outdir,
					    $name);

	  &create_directory($outdir_rnaz);


	  #Execute clustalw
	  $outfile_clustal = File::Spec->catfile($outdir_rnaz,
						 $name.".aln");
	  my $guided_tree_clustal = File::Spec->catfile($cg_seq_outdir,
							$blast_outdir_name,
							$name.".dnd");
	  my $cmd = $cmd_clustalw." -infile=\"$fasta\" ";
	  $cmd .= "-outfile=\"$outfile_clustal\" -outorder=\"input\"";
	  `$cmd`;
	  unlink($guided_tree_clustal);


	  #Execute RNAz or rnazWindow.pl for too long sequences if available
	  $outfile_rnaz = File::Spec->catfile($outdir_rnaz,
					      $name.".rnaz");
	  my $current_cmd_rnaz = $cmd_rnaz." \"$outfile_clustal\" ";
	  $current_cmd_rnaz .= "-o \"$outfile_rnaz\"";
	  $rnazwindow_used = 0;

	  if ($is_rnazwindow) {

	    open(FASTA,$fasta) or
	      exit_script("\nERROR : Could not open \"$fasta\" for reading \n\n");
	    my $content_fasta = "";
	    my $length_seq = -1;
	    while (($length_seq == -1) && (my $line = <FASTA>)) {
	      $content_fasta .= $line;
	      if ($content_fasta =~ />.+[\n\r]([ATCGNatcgn\n\r]+)>.+/) {
		my $seq_ref = $1;
		$seq_ref =~ s/[\n\r]//g;
		$length_seq = length($seq_ref);
	      }
	    }
	    close(FASTA);
	    if ($length_seq >= $too_long) {
	      $current_cmd_rnaz = $cmd_rnazwindow." \"$outfile_clustal\" | ";
	      $current_cmd_rnaz .= $cmd_rnaz." -o \"$outfile_rnaz\"";
	      $rnazwindow_used = 1;
	    }


	  }

	  `$current_cmd_rnaz`;

	  #Check result file
	  $content_rnaz = &read_file($outfile_rnaz);

	  if ($content_rnaz =~ /Prediction: RNA/) {
	    $is_RNA_rnaz = 1;

	  }


	}

	#If predicted RNA : make set
	if ($is_RNA_carnac || $is_RNA_rnaz) {

	  $nb_sets++;

	  my $outdir_set = File::Spec->catdir($blast_predicted_RNA,
					      "set_".$nb_sets);
	  &create_directory($outdir_set);

	  #Copy fasta
	  copy($fasta,$outdir_set);


	  #RNAz result part
	  if ($is_RNA_rnaz) {

	    #Copy RNAz result file
	    copy($outfile_rnaz,$outdir_set);

	    #Check how much predictions
	    my $nb_RNA = 0;
	    while ($content_rnaz =~ /Prediction: RNA/g) {
	      $nb_RNA++;
	    }

	    #Producing forward strand result and reverse strand
	    my $produce_forward = 0;
	    my $produce_reverse = 0;


	    #One prediction
	    if ($nb_RNA == 1) {


	      #If no rnazWindow : one strand prediction
	      if (!$rnazwindow_used) {

		#Forward
		if ($strand_process == 0) {
		  $produce_forward = 1;
		}

		#Reverse
		if ($strand_process == 1) {
		  $produce_reverse = 1;
		}

		#Both strands : check wich strand
		if ($strand_process == 2) {
		  my $strand = "+";

		  my @split_rnaz = split(qr/#+\s+RNAz\s[0-9]+\.[0-9]+\s+#+/,
					 $content_rnaz);
		  foreach my $split_part (@split_rnaz) {
		    if (($split_part =~ /Reading direction: reverse/) &&
			($split_part =~ /Prediction: RNA/)) {
		      $strand = "-";
		    }
		  }

		  #Forward
		  if ($strand eq "+") {
		    $produce_forward = 1;
		  }
		  #Reverse
		  else{
		    $produce_reverse = 1;
		  }
		}
	      }
	      #Else rnazwindow.pl used : check part predicted as RNA
	      else {

		if ($content_rnaz =~ /Reading direction: reverse/) {
		  $produce_reverse = 1;
		}
		else{
		  $produce_forward = 1;
		}
	      }

	    }
	    #More than one prediction
	    else {

	      #If no rnazWindow.pl used : both strand prediction
	      #Generate reverse fasta, reverse alignment and copy them
	      #and the rnaz result into the set directory
	      if ((!$rnazwindow_used) && ($strand_process == 2)) {

		$produce_forward = 1;
		$produce_reverse = 1;

	      }

	      else{

		#Split file
		my @split_rnaz = split(qr/#+\s+RNAz\s[0-9]+\.[0-9]+\s+#+/,
				       $content_rnaz);
		foreach my $split_part (@split_rnaz) {

		  if ($split_part =~ /Prediction: RNA/) {

		    if ($split_part =~ /Reading direction: reverse/) {
		      $produce_reverse = 1;
		    }
		    else{
		      $produce_forward = 1;
		    }
		  }
		}

	      }
	    }

	    #Produce forward result files
	    if($produce_forward){

	      #Fasta and RNAz result file already copied : clustalw are only
	      #missing
	      copy($outfile_clustal,$outdir_set);

	    }

	    #Produce reverse result files
	    if($produce_reverse){
	      my $reverse_fasta = File::Spec->catfile($outdir_set,
						      $name."_reverse".$suffix);
	      &execute_reverse_complementary($fasta,
					     $reverse_fasta);

	      my $outfile_clustal_reverse = 
		File::Spec->catfile($outdir_set,
 				  $name."_reverse.aln");
	      my $guided_tree_clustal_reverse =
		File::Spec->catfile($outdir_set,
				    $name."_reverse.dnd");
	      my $cmd_clustal_reverse = $cmd_clustalw;
	      $cmd_clustal_reverse .= " -infile=\"$reverse_fasta\" ";
	      $cmd_clustal_reverse .= "-outfile=\"$outfile_clustal_reverse\" ";
	      $cmd_clustal_reverse .= "-outorder=\"input\"";
	      `$cmd_clustal_reverse`;
	      unlink($guided_tree_clustal_reverse);



	    }

	    #Image part
	    if($is_rnaplot){

	      my @split_rnaz = split(qr/#+\s+RNAz\s[0-9]+\.[0-9]+\s+#+/,
				     $content_rnaz);

	      #Produce images for the reference sequence which
	      #starts with >1
	      foreach my $split_part(@split_rnaz){
		my $cpt = 1;
		my $strand_name = "";
		if($split_part =~ /Prediction: RNA/){
		  if($split_part =~ /Reading direction: (forward|reverse)/){
		    if($1 eq "forward"){
		      $strand_name = "";
		    }
		    else{
		      $strand_name = "_reverse";
		    }
		  }

		  if($split_part =~ 
		       /(>1\|.+[\n\r][-CUGUA\._]+[\n\r][-\.\(\)]+)/g) {
		    my $ref_struct = $1;
		    $ref_struct =~ s/\|/_/g;
		    my $temp_file = File::Spec->catfile("temp.txt");
		    &write_file($ref_struct,$temp_file);
		    my $cmd = $cmd_rnaplot." < \"$temp_file\"";
		    `$cmd`;
		    my $ref_image = 
		      File::Spec->catfile("1*.".$rnaplot_image_suffix);
		    my @search_images = glob("$ref_image");
		    my $new_image_file = 
		      File::Spec->catfile($outdir_set,
					  $name.$strand_name."_rnaz.".$rnaplot_image_suffix);
		    while(-e $new_image_file){
		      $new_image_file = 
			File::Spec->catfile($outdir_set,
					    $name.$strand_name."_rnaz_".$cpt.".".$rnaplot_image_suffix);
		      $cpt++;
		    }
		    move($search_images[0],$new_image_file) or
		      exit_script("\nERROR : Could not move \"$search_images[0]\"\n\n");
		    unlink($temp_file);
		  }
		}
	      }
	    }
	  }


	  #Carnac result part
	  if ($is_RNA_carnac) {

	    #Result from carnac
	    my $res_carnac = File::Spec->catfile($outdir_set,
						 $name."_carnac.out");
	    copy($carnac_out,$res_carnac);


	    my $reverse_fasta = "";

	    #Reverse set part
	    $reverse_fasta = 
	      File::Spec->catfile($outdir_set,
				  $name."_reverse".$suffix);

	    &execute_reverse_complementary($fasta,
					   $reverse_fasta);

	    #Image
	    if ($is_rnaplot) {

	      #Replace '|' character in carnac.out file because
	      #RNAplot under windows throws an error if that character is found
	      my $temp_carnac_out = File::Spec->catfile($outdir_carnac,
							"temp_carnac.out");
	      my $content_temp = &read_file($carnac_out);
	      $content_temp =~ s/\|/_/g;
	      &write_file($content_temp,$temp_carnac_out);

	      #Image for forward strand
	      my $cmd = $cmd_rnaplot." < \"$temp_carnac_out\"";

	      `$cmd`;
	      my $ref_image = File::Spec->catfile("1*.".$rnaplot_image_suffix);
	      my @search_images = glob("$ref_image");
	      my $new_image_file = 
		File::Spec->catfile($outdir_set,
				    $name."_carnac.".$rnaplot_image_suffix);

	      move($search_images[0],$new_image_file) or
		exit_script("\nERROR : Could not move \"$search_images[0]\"\n\n");
	      my $other_images = 
		File::Spec->catfile("*.".$rnaplot_image_suffix);
	      my @search_other = glob("$other_images");
	      unlink(@search_other);
	      unlink($temp_carnac_out);

	      #Image for reverse strand
	      my $outdir_carnac_reverse = File::Spec->catdir($outdir_set,
							     "reverse");
	      my $out_carnac_reverse = 
		File::Spec->catfile($outdir_carnac_reverse,
				    "carnac.out");
	      $cmd = $cmd_carnac." \"$reverse_fasta\" ";
	      $cmd .= "-o \"$outdir_carnac_reverse\"";
	      `$cmd`;

	      #copy result from carnac for reverse strand
	      my $res_reverse_carnac = File::Spec->catfile($outdir_set,
							   $name."_reverse_carnac.out");
	      copy($out_carnac_reverse,$res_reverse_carnac);

	      #Replace '|' character in carnac.out file because
	      #RNAplot under windows throws an error if that character is found
	      my $reverse_temp_carnac_out = 
		File::Spec->catfile($outdir_carnac_reverse,
				    "reverse_temp_carnac.out");
	      my $reverse_content_temp = &read_file($out_carnac_reverse);
	      $reverse_content_temp =~ s/\|/_/g;
	      &write_file($reverse_content_temp,$reverse_temp_carnac_out);


	      my $current_cmd_rnaplot .= 
		$cmd_rnaplot." < \"$reverse_temp_carnac_out\"";
	      `$current_cmd_rnaplot`;

	      $ref_image = File::Spec->catfile("1*.".$rnaplot_image_suffix);
	      @search_images = glob("$ref_image");
	      $new_image_file = 
		File::Spec->catfile($outdir_set,
				    $name."_reverse_carnac.".$rnaplot_image_suffix);
	      move($search_images[0],$new_image_file) or
		exit_script("\nERROR : Could not move \"$search_images[0]\"\n\n");
	      $other_images = File::Spec->catfile("*.".$rnaplot_image_suffix);
	      @search_other = glob("$other_images");
	      unlink(@search_other);
	      unlink($reverse_temp_carnac_out);
	      rmtree($outdir_carnac_reverse);
	    }
	  }


	}

      }

      if (!$quiet) {
	print "Found $nb_sets putative ncRNAs from BLAST data\n\n";

      }
    }



  }

}

1;




###############################################################################
###############################################################################
##									     ##
##									     ##
##			      POD DOCUMENTATION				     ##
##									     ##
##									     ##
###############################################################################
###############################################################################

=head1 NAME

Package I<software_handler> - Handler for pipeline software

=head1 SYNOPSIS

Contains all functions to deal with extern pipeline softwares

Provided functions :

=over 2

=item *

execute_blast

=item *

execute_carnac

=item *

execute_masking

=item *

execute_rnaz

=item *

execute_yass

=item *

find_conserved_regions

=item *

get_results_rnaz

=back

=head1 EXPORT

Exported functions :

=over 2

=item *

execute_blast

=item *

execute_masking

=item *

execute_structure

=item *

execute_yass

=item *

find_conserved_regions

=back

=head1 FUNCTIONS

=head2 execute_blast

The function will execute the alignment step with Blast on the reference 
against all other files.

=over 2

=item *

parameters :

=over 2

=item *

$reference_file, the location of the reference file

=item *

$other_files, the reference on the hash table containing other files

=item *

$cmd_blast, the command to execute Blast

=item *

$blast_outdir, the outdir of alignment files

=item *

$quiet, the quiet mode

=back

=item *

returns : nothing.

=back

=head2 execute_carnac

The function will carnac on conserved regions found by the previous step.

=over 2

=item *

parameters :

=over 2

=item *

$cmd_carnac, the command to execute caRNAc

=item *

$cmd_rnaplot, the command to execute RNAplot

=item *

$cg_seq_outdir, the outdir of conserved region

=item *

$yass_available, yass software wanted or not

=item *

$blast_available, blast software wanted or not

=item *

$yass_outdir_name, name of the outdir of yass

=item *

$blast_outdir_name, name of the outdir of Blast

=item *

$carnac_outdir, carnac outdir

=item *

$carnac_predicted_outdir, carnac outdir for predicted RNA

=item *

$quiet, the quiet mode

=back

=item *

returns : nothing.

=back

=head2 execute_masking

The function will execute the masking step on the reference file and the 
other genomes if there's a genbank annotation file aside.

=over 2

=item *

parameters :

=over 2

=item *

$reference_file, the location of the reference file

=item *

$other_files, the reference on the hash table containing other files

=item *

$cmd_mask, the command to execute the masking script

=item *

$current_mask_outdir, the outdir of masked files

=item *

$content_warning, the content of the warning file

=item *

$quiet, the quiet mode

=back

=item *

returns : nothing.

=back

=head2 execute_rnaz

The function will RNAz on conserved regions found by the previous step.

=over 2

=item *

parameters :

=over 2

=item *

$cmd_rnaz, the command to execute RNAz

=item *

$cmd_clustalw, the command to execute clustalw

=item *

$cmd_rnazwindow, the command to execute rnazWindow.pl

=item *

$cmd_rnaplot, the command to execute RNAplot

=item *

$cg_seq_outdir, the outdir of conserved regions

=item *

$yass_available, yass software wanted or not

=item *

$blast_available, blast software wanted or not

=item *

$yass_outdir_name, name of the outdir of yass

=item *

$blast_outdir_name, name of the outdir of Blast

=item *

$rnaz_outdir, rnaz outdir

=item *

$rnaz_predicted_outdir, rnaz outdir for predicted RNA

=item *

$quiet, the quiet mode

=back

=item *

returns : nothing

=back

=head2 execute_structure

The function will execute carnac and/or RNAz on conserved regions found 
by the previous step.

=over 2

=item *

parameters :

=over 2

=item *

$carnac_available, carnac wanted and usable

=item *

$rnaz_available, RNAz wanted and usable

=item *

$cmd_carnac, the command to execute carnac

=item *

$cmd_rnaz, the command to execute RNAz

=item *

$cmd_clustalw, the command to execute clustalw

=item *

$cmd_rnazwindow, the command to execute rnazWindow.pl

=item *

$cmd_rnaplot, the command to execute RNAplot

=item *

$cg_seq_outdir, the outdir of conserved regions

=item *

$yass_available, yass software wanted or not

=item *

$blast_available, blast software wanted or not

=item *

$yass_outdir_name, name of the outdir of yass

=item *

$blast_outdir_name, name of the outdir of Blast

=item *

$carnac_outdir, carnac outdir

=item *

$rnaz_outdir, rnaz outdir

=item *

$predicted_outdir, outdir for predicted RNAs

=item *

$quiet, the quiet mode

=back

=item *

returns : nothing.

=back 

=head2 execute_yass

The function will execute the alignment step with Yass on the reference 
against all other files.

=over 2

=item *

parameters :

=over 2

=item *

$reference_file, the location of the reference file

=item *

$other_files, the reference on the hash table containing other files

=item *

$cmd_yass, the command to execute Yass

=item *

$yass_outdir, the outdir of alignment files

=item *

$quiet, the quiet mode

=back

=item *

returns : nothing.

=back

=head2 find_conserved_regions

The function will execute the finding conserved regions step according to 
alignments (Yass and/or Blast).

=over 2

=item *

parameters :

=over 2

=item *

$reference_file, the location of the reference file

=item *

$other_files, the reference on the hash table containing other files

=item *

$cmd_cg_seq, the command to execute CGseqcore

=item *

$alignments_outdir, the outdir of alignment files

=item *

$yass_available, yass software wanted or not

=item *

$blast_available, blast software wanted or not

=item *

$yass_outdir_name, name of the outdir of yass

=item *

$blast_outdir_name, name of the outdir of Blast

=item *

$cg_seq_outdir, outdir of conserved regions

=item *

$quiet, the quiet mode

=back

=item *

returns : nothing.

=back

=head2 get_results_rnaz

The function will execute and deal with RNA predicted files from RNAz.

=over 2

=item *

parameters :

=over 2

=item *

$cmd_rnaz, command line to execute RNAz

=item *

$cmd_clustalw, command line to execute clustalw

=item *

$cmd_rnazwindow, command line  to execute rnazWindow.pl

=item *

$is_rnazwindow, rnazWindow.pl executable or not

=item *

$general_outdir, the outdir for result file

=item *

$set_outdir, the outdir for RNA predicted sets

=item *

$cg_seq_outdir, the outdir of conserved regions found

=item *

$outdir_name, the name of subdirectory

=item *

$sets, the reference on conserved regions found to process

=back

=item *

returns : the number of sets predicted as RNA.

=back

=head1 AUTHOR

Benjamin Grenier-Boley <benjamin.grenier-boley@inria.fr>

=cut
