#!/usr/bin/perl -w

###############################################################################
###############################################################################
##
## This file is part of CG-seq.
##
##  CG-seq is free software: you can redistribute it and/or modify
##  it under the terms of the GNU General Public License as published by
##  the Free Software Foundation, either version 3 of the License, or
##  (at your option) any later version.
##
##  CG-seq is distributed in the hope that it will be useful,
##  but WITHOUT ANY WARRANTY; without even the implied warranty of
##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##  GNU General Public License for more details.
##
##  You should have received a copy of the GNU General Public License
##  along with CG-seq.  If not, see <http://www.gnu.org/licenses/>.
##
###############################################################################
###############################################################################


###############################################################################
###############################################################################
##
## 	Script : mask_redundancy.pl
##	Author : Benjamin Grenier-Boley <benjamin.grenier-boley@inria.fr>
##
##	Creation : July 2009
##
##	Goal : Mask redundancy regions between sequences found in provided
##             directory. It will use Yass to compare each sequence
##
###############################################################################
###############################################################################


use strict;
use warnings;
use Getopt::Long qw(:config noauto_abbrev);
use Pod::Usage;
use File::Spec;
use File::Basename;
use File::Copy;
use FindBin;


###############################################################################
###############################################################################
##
##
##			     GLOBAL VARIABLES
##
##
###############################################################################
###############################################################################

#Indir
my $indir;

#Yass configuration file
my $yass_conf = File::Spec->catfile($FindBin::Bin,
				    "..",
				    "cfg",
				    "yass_redundancy.conf");

#Minimum length of an alignment to be taken into account
my $min_length = 30;

#Minimum identity of an alignment to be taken into account
my $min_id = 95;

#Outdir
my $outdir;

#Alignment directory name
my $outdir_al_name = "alignments";

#Help
my $help = 0;

#Quiet mode
my $quiet = 0;

#Regular expression matching bed lines
my $regexp_bed = ".+\t([0-9]+)\t([0-9]+)\t[-+]\t([0-9]+)\t([0-9]+)\t";
$regexp_bed .= "[-0-9\.e+]+\t[-0-9\.e+]+\t([0-9\.]+)";



###############################################################################
###############################################################################
##
##
##				     FUNCTIONS
##
##
###############################################################################
###############################################################################

###############################################################################
#
#	Function exit_script
#
#	Goal : The function will exit the script with the given error message
#
#	Param :
#		- $message : the message to display
#
#	Returns :
#               Nothing but the script exited
#
###############################################################################


sub exit_script{

  (my $message) = @_;
  print $message;
  die "\n\n";

}


###############################################################################
#
#	Function test_software
#
#	Goal : The function will test if the path to execute the software is
#              good (existence + execution)
#
#	Param :
#               - $exe_path : the path to test
#	Returns :
#               - the rigth executable path (in case @PATH, or under script
#                 directory) or "" if not found or executable
#
###############################################################################

sub test_software{

  (my $exe_path) = @_;

  #First test : full path
  if((-e $exe_path)){
    return "\"".$exe_path."\"";
  }
  #Second test : check under script directory
  else{

    my $file_script_dir = File::Spec->catfile(dirname($0),
					      $exe_path);
    if(-e $file_script_dir){
      return "\"".$file_script_dir."\"";
    }
    #Last test : look under $PATH
    else{
      my @paths = File::Spec->path();
      foreach my $path(@paths){
	my $real_path = File::Spec->catfile($path,
					    $exe_path);

	if(-e $real_path){
	  return "\"".$real_path."\"";
	}

      }
    }
  }

  return "";

}



###############################################################################
#
#	Function test_configuration_file
#
#	Goal : The function will execute 2 tests on the configuration file :
#              presence and the fact that the file is readable
#
#	Param :
#		- $configuration_file : the configuration file to test
#
#	Returns :
#		- 1 if tests were successful, 0 otherwise
#
###############################################################################

sub test_configuration_file{

  my $return_code = 1;

  (my $configuration_file) = @_;


  if (! -e $configuration_file){
    print "\nERROR : The configuration file \"$configuration_file\" doesn't ";
    print "exists\n\n";
    $return_code = 0;
  }

  if(($return_code) && ! (-r $configuration_file)){
    print "\nERROR : The configuration file \"$configuration_file\" isn't ";
    print "readable";
    $return_code = 0;
  }


  return $return_code;


}




###############################################################################
#
#	Function get_software_availability
#
#	Goal : The function will read the configuration file which contains
#              both the command line to execute a software, and its options
#
#	Param :
#		- $configuration_file : the configuration file
#
#	Returns :
#               - $available : the boolean that says if the software is
#                 available or not
#		- $command_line : the command line to execute according to
#                 exe path and options read or "" if an error occured during
#                 the file parsing
#
###############################################################################

sub get_software_availability{

  (my $configuration_file) = @_;


  #Software availability
  my $available = 1;

  #Command line to return
  my $command_line = "";

  #Exe path of the software to execute
  my $exe_path = "";

  #Array of options
  my @options;

  #Read the configuration file
  open(CONF, $configuration_file) or
    exit_script("\nERROR : Could not read the configuration file \"$configuration_file\"");

  while(my $line = <CONF>){

    #Don't take commentary and blank lines
    if ($line !~ /(^#|^$)/) {

      #Path to execute the software
      if ($line =~ /\s*exe_path = "?(.+)"?/) {
	$exe_path = $1;
	$exe_path =~ s/"//g;
      }
      #Software options
      else{
	chomp($line);
	push(@options,$line);

      }

    }

  }

  #No exe path : exit
  if($exe_path eq ""){
    print"\nWARNING: There is no \"exe_path\" in the configuration file ";
    print "\"$configuration_file\"\n\n";
    return (0,$command_line);
  }

  #Test path to execute the software and get real path
  my $real_path = &test_software($exe_path);

  if($real_path ne ""){

    #Generate command line
    $command_line = $real_path;
    foreach my $option(@options){
      $command_line .= " ".$option;
    }
    my $dev_null = File::Spec->devnull();
    $command_line .= " 1>$dev_null 2>&1";

  }
  else{
    print "\nWARNING : The path to execute the software : \"$exe_path\"";
    print " is not correct or the command is not executable\n\n";
    $available = 0;
  }

  return ($available,$command_line);


}


###############################################################################
#
#	Function get_fasta_files
#
#	Goal : The function will get all fasta files within the directory
#              provided. Fasta file must be suffixed with ".fa", ".fna", ".dna"
#              ".seq" or ".fasta"
#
#	Param :
#		- $dir : the directory to search in
#
#	Returns :
#               - an array containing all fasta files found or empty array
#                 if no fasta files found
#
###############################################################################

sub get_fasta_files{

  (my $dir) = @_;

  my $search_files = File::Spec->catfile($dir,
					 "*");

  if($search_files =~ /\s/){
    $search_files = quotemeta($search_files);
  }

  #Get all files and select those with good suffix
  my @fasta_files;
  my @search_files = glob("$search_files");
  foreach my $file(@search_files){
    (my $name, my $dir, my $suffix) = fileparse($file,qr/\.[^.]*/);
    if(($suffix eq ".fna") ||
       ($suffix eq ".fasta") ||
       ($suffix eq ".fa") ||
       ($suffix eq ".dna") ||
       ($suffix eq ".seq")){

      if(! -r $file){
	exit_script("\nERROR : Fasta file \"$file\" is not readable\n");
      }
      push(@fasta_files,$file);

    }
  }

  return @fasta_files;


}

###############################################################################
#									      #
#	Function sort_by_size						      #
#									      #
#	Goal : comparison function used to sort file by size   		      #
#									      #
#	Param :								      #
#		- $a, $b representative of 2 files      		      #
#									      #
#	Return : 							      #
#		- 1,0,1 ($a > $b, $a = $b, $a < $b)			      #
#									      #
###############################################################################

sub sort_by_size{

  my $taille_a = -s "$a";
  my $taille_b = -s "$b";

  if($taille_a > $taille_b){
    return -1;
  }
  else{
    if($taille_b > $taille_a){
      return 1;
    }
    else{
      return 0;
    }
  }

}
###############################################################################
#
#	Function read_fasta
#
#	Goal : read a fasta file and get the id and the sequence
#
#	Param :
#               - $fasta : fasta file to read
#
#	Return :
#               - $id : the id of the fasta file
#               - $seq : the sequence
#               If the file is not in fasta format returns ("","")
#
###############################################################################

sub read_fasta{

  #Location of the fasta file
  my $fasta;

  #Content of the fasta file
  my $content = "";

  #Id of the fasta file
  my $id = "";

  #Sequence
  my $seq = "";

  #Regular expression matching id and sequence
  my $regexp_fasta = "^>(.+)[\\n\\r]";
  $regexp_fasta .= "([ATCGNURYKMSWBDHVatcgnurykmswbdhv\\n\\r]+)\$";

  ($fasta) = @_;

  $content = &read_file($fasta);

  #Get id and sequence
  if($content =~ /$regexp_fasta/){
    $id = $1;
    $seq = $2;
    #Deleting end line characters in sequence
    $seq =~ s/[\n|\r]//g;
  }

  return ($id,$seq);



}


###############################################################################
#
#	Function read_file
#
#	Goal : read a file and return its content
#
#	Param :
#               - $file : file to read
#
#	Return :
#               - $content : the content of the file
#
###############################################################################

sub read_file{

  #Content that will be returned
  my $content = "";

  (my $file) = @_;

  open(FILE,$file) or exit_script("\nCould not open file \"$file\"\n\n");
  while(<FILE>){
    $content .= $_;
  }
  close(FILE);

  return $content;



}





###############################################################################
#									      #
#	Function write_fasta						      #
#									      #
#	Goal : write the fasta file                                           #
#									      #
#	Param :	- $file : the location of the fasta file to write 	      #
#               - $id : the id to write                                       #
#               - $seq : the sequence to write                                #
#									      #
#	Return : 							      #
#		- nothing but the file is created		              #
#									      #
###############################################################################

sub write_fasta{

  #Length of the sequence
  my $lg;

  #Get the parameters
  (my $outfile, my $id, my $seq) = @_;


  $lg = length($seq);
  $seq = uc($seq);

  chomp($id);
  $id = ">".$id;

  #Writing of the new file in fasta format
  open(OUT, ">$outfile") or 
    exit_script("\nERROR : Could not write into file \"$outfile\"\n\n");
  print OUT $id;
  for(my $i = 0; $i < $lg; $i++){
    if(($i%70) == 0){
      print OUT "\n";
    }
    print OUT substr($seq,$i,1);
  }
  close(OUT);

}


###############################################################################
###############################################################################
##
##
##				     MAIN
##
##
###############################################################################
###############################################################################

#Parse options
GetOptions('yass=s' => \$yass_conf,
	   'length=i' => \$min_length,
	   'id=f' => \$min_id,
	   'quiet' => \$quiet,
	   'help' => \$help,
	   'h' => \$help) or pod2usage(2);


#Help
pod2usage(1) if $help;


#If no good number of parameters : exit
my $nb_args = @ARGV;

if($nb_args != 2){
  pod2usage(1);
}

$indir = $ARGV[0];
$outdir = $ARGV[1];

#Test values
if(! -d $indir){
  print "\nThe directory \"$indir\" does not exist or is not a directory\n\n";
  pod2usage(1);
}
if(! -e $yass_conf){
  print "\nThe yass configuration file \"$yass_conf\" does not exist\n\n";
  pod2usage(1);
}
if($min_length <= 0){
  print "\nThe minimum length to consider a region must be above 1\n\n";
  pod2usage(1);
}
if(($min_id <= 0) || ($min_id > 100)){
  print "\nThe minimum identity to consider a redundant region must be between 1 and 100\n\n";
  pod2usage(1);
}

#Indir and outdir must be different in order to not overwrite original files
$outdir =~ s/[\\\/]$//;
$indir =~ s/[\\\/]$//;

if($indir eq $outdir){
  exit_script("\nERROR : Indirectory and outdirectory must be different in order to not overwrite files\n\n");

}


#creation of outdirs
if(! -d $outdir){
  mkdir($outdir) or die "\nERROR : Could not create directory \"$outdir\"\n\n";
}
my $outdir_alignment = File::Spec->catdir($outdir,
					  $outdir_al_name);
if(! -d $outdir_alignment){
  mkdir($outdir_alignment) or 
    exit_script("\nERROR : Could not create directory \"$outdir_alignment\"\n\n");
}



#Get Yass command and parameters
(my $yass_available, my $cmd_yass) = &get_software_availability($yass_conf);

if(! $yass_available){
  exit_script("\nERROR : yass is not available. You have to check your configuration file\n\n");
}

#Add sort and output format (BED) for yass command
$cmd_yass .= " -s 4 -d 4";


#Get fasta files
my @fasta_files = &get_fasta_files($indir);

if(@fasta_files == 0){
  my $error = "\nERROR : There is no fasta files within directory \"";
  $error .= "$indir\". Fasta files must suffixed by \".fa\", \".fasta\"";
  $error .= " ,\".fna\", \".dna\" or \".seq\" : it will not  be taken into ";
  $error .= "account\n";
  exit_script($error);
}

#Sort them by size
@fasta_files = sort sort_by_size @fasta_files;


#Get id and sequence from each fasta file and
#copy files to outdir
my %ids;
my %sequences;
my @names;

for(my $i = 0; $i < @fasta_files; $i++){

  (my $id, my $seq) = &read_fasta($fasta_files[$i]);
  $ids{$i} = $id;
  $sequences{$i} = $seq;

  my $file = $fasta_files[$i];
  (my $name, my $dir, my $suffix) = fileparse($file,qr/\.[^.]*/);

  my $copy_file = File::Spec->catfile($outdir,
				      $name.$suffix);

  copy($file,$copy_file) or
    exit_script("\nERROR : Could not copy \"$file\" to \"$copy_file\"\n\n");

  $fasta_files[$i] = $copy_file;
  push(@names,$name);

}

my $nb_cmp = 0;
my $content_result = "";

#Compare each fasta against each other
for (my $i = 0; $i < (@fasta_files - 1); $i++) {

  for (my $j = $i + 1; $j < @fasta_files; $j++) {

    my $id1 = $ids{$i};
    my $id2 = $ids{$j};

    chomp($id1);
    chomp($id2);

    $id1 =~ s/>//g;
    $id2 =~ s/>//g;


    $nb_cmp++;
    if(!$quiet){
      print "Comparison $nb_cmp :\n\t- $id1\n\t- $id2\n";
    }

    $content_result .= "COMPARISON $nb_cmp\nSEQ1 : $id1\nSEQ2 TO MASK: $id2\n";
    $content_result .= "Start_seq1\tEnd_seq1\tStart_seq2\tEnd_seq2\tIdentity\n";

    my $outname = File::Spec->catfile($outdir_alignment,
				      $names[$i]."-".$names[$j].".bed");

    if(! $quiet){
      print "\t-> executing YASS\n";
    }

    #Execute yass
    my $current_cmd = 
      $cmd_yass." \"$fasta_files[$i]\" \"$fasta_files[$j]\" -o \"$outname\"";
    `$current_cmd`;

    if(! $quiet){
      print "\t-> executing masking on $id2\n";
    }

    #Number of masked regions
    my $nb_mask = 0;

    #Read the alignment file
    open(BED,$outname) or 
      exit_script("\nERROR : Could not open \"$outname\"\n\n");
    while (my $line = <BED>) {
      if ($line =~ /$regexp_bed/) {

	my $pos_start_seq1 = $1;
	my $pos_end_seq1 = $2 - 1;
	my $pos_start_seq2 = $3;
	my $pos_end_seq2 = $4 - 1;
	my $id_align = $5;
	my $length_seq1 = $pos_end_seq1 - $pos_start_seq1 + 1;
	my $length_seq2 = $pos_end_seq2 - $pos_start_seq2 + 1;

	#If the identity respects the threshold and the length is long
	#enough : mask region on second file
	if (($id_align >= $min_id) &&
	    ($length_seq2 >= $min_length) &&
	    ($length_seq1 >= $min_length)) {
	  $nb_mask++;

	  my $seq_to_mask = $sequences{$j};

	  my $mask_region = "";

	  #Build a mask region
	  for (my $k = 0; $k < $length_seq2; $k++) {
	    $mask_region .= "N";
	  }

	  #Replace the region by the masked one
	  my $new_seq = substr($seq_to_mask,
			       $pos_start_seq2,
			       $length_seq2,
			       $mask_region);

	  $sequences{$j} = $seq_to_mask;


	  #Add the information in the result file
	  $content_result .= "$pos_start_seq1\t$pos_end_seq1\t$pos_start_seq2\t$pos_end_seq2\t$id_align\n";

	}

      }
    }
    $content_result .= "\nNb masked regions : $nb_mask\n\n";
    close(BED);

    #Writing new fasta file
    &write_fasta($fasta_files[$j], $ids{$j}, $sequences{$j});

  }

}

#Writing result file
my $res_file = File::Spec->catfile($outdir,
				   "masking_redundancy_result.txt");
open(RES,">$res_file");
print RES $content_result;
close(RES);




###############################################################################
###############################################################################
##
##
##				     USAGE
##
##
###############################################################################
###############################################################################


__END__


=head1 NAME

C<mask_redundancy.pl> - Mask redundancy regions between all sequences included
in the provided directory.

=head1 SYNOPSIS

mask_redundancy.pl <directory> <outdirectory> [options]

=over 8

=item B<directory>

Directory containing sequences to eliminate redundancy in

=item B<outdirectory>

Directory where to put sequences after eliminiation of redundancy

=head1 OPTIONS

=over 8

=item B<--yass>=CONF

Use provided configuration file for yass software (Default
../cfg/yass_redundancy.conf)

=item B<--length>=INT

Minimum length to consider a redundant region (Default 30)

=item B<--id>=[1-100]

Minimum identity to consider a redundant region (Default 95)

=item B<--quiet>

Print no message at all

=item B<-h, --help>

Prints a short help message and exits.

=back

=head1 AUTHORS

Benjamin Grenier-Boley <benjamin.grenier-boley@inria.fr>

=cut
