#!/usr/bin/perl -w

###############################################################################
###############################################################################
##
## This file is part of CG-seq.
##
##  CG-seq is free software: you can redistribute it and/or modify
##  it under the terms of the GNU General Public License as published by
##  the Free Software Foundation, either version 3 of the License, or
##  (at your option) any later version.
##
##  CG-seq is distributed in the hope that it will be useful,
##  but WITHOUT ANY WARRANTY; without even the implied warranty of
##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##  GNU General Public License for more details.
##
##  You should have received a copy of the GNU General Public License
##  along with CG-seq.  If not, see <http://www.gnu.org/licenses/>.
##
###############################################################################
###############################################################################


###############################################################################
###############################################################################
##
## 	Script : mask_regions.pl
##	Author : Benjamin Grenier-Boley <benjamin.grenier-boley@inria.fr>
##
##	Date : July 2009
##
##	Goal : Mask wanted  regions according to annotations in a
##             genbank file (CDSs and/or RNAs). The masking nucleotide is "N"
##
###############################################################################
###############################################################################


use strict;
use warnings;
use Getopt::Long qw(:config noauto_abbrev);
use Pod::Usage;
use File::Spec;
use File::Basename;



###############################################################################
###############################################################################
##
##
##			     FUNCTIONS
##
##
###############################################################################
###############################################################################

###############################################################################
#
#	Function exit_script
#
#	Goal : The function will exit the script with the given error message
#
#	Param :
#		- $message : the message to display
#
#	Returns :
#               Nothing but the script exited
#
###############################################################################


sub exit_script{

  (my $message) = @_;
  print $message;
  die "\n\n";

}

###############################################################################
#
#	Function read_file
#
#	Goal : read a file and return its content
#
#	Param :
#               - $file : file to read
#
#	Return :
#               - $content : the content of the file
#
###############################################################################

sub read_file{

  #Content that will be returned
  my $content = "";

  (my $file) = @_;

  open(FILE,$file) or exit_script("\nCould not open file \"$file\"\n\n");
  while(<FILE>){
    $content .= $_;
  }
  close(FILE);

  return $content;



}


###############################################################################
#
#	Function read_fasta
#
#	Goal : read a fasta file and get the id and the sequence
#
#	Param :
#               - $fasta : fasta file to read
#
#	Return :
#               - $id : the id of the fasta file
#               - $seq : the sequence
#               If the file is not in fasta format returns ("","")
#
###############################################################################

sub read_fasta{

  #Location of the fasta file
  my $fasta;

  #Content of the fasta file
  my $content = "";

  #Id of the fasta file
  my $id = "";

  #Sequence
  my $seq = "";

  #Regular expression matching id and sequence
  my $regexp_fasta = "^>(.+)[\\n\\r]";
  $regexp_fasta .= "([ATCGNURYKMSWBDHVatcgnurykmswbdhv\\n\\r]+)\$";

  ($fasta) = @_;

  $content = &read_file($fasta);

  #Get id and sequence
  if($content =~ /$regexp_fasta/){
    $id = $1;
    $seq = $2;
    #Deleting end line characters in sequence
    $seq =~ s/[\n|\r]//g;
  }

  return ($id,$seq);



}


###############################################################################
#
#	Function write_fasta
#
#	Goal : write a fasta file
#
#	Param :
#               - $file : the location of the file where to write
#               - $id : the id of the fasta file (without > at the beginning)
#               - $seq : the sequence
#
#	Return :
#		- nothing but the file is created
#
###############################################################################

sub write_fasta{

  #Length of the sequence
  my $lg;

  (my $file, my $id, my $seq) = @_;

  $lg = length($seq);
  $seq = uc($seq);

  chomp($id);
  $id = ">".$id;

  #Writing of the new file in fasta format
  open(OUT, ">$file") or 
    exit_script("\nCould not write in file \"$file\"\n\n");
  print OUT $id;
  for(my $i = 0; $i < $lg; $i++){
    if(($i%70) == 0){
      print OUT "\n";
    }
    print OUT substr($seq,$i,1);
  }
  close(OUT);

}



###############################################################################
###############################################################################
##
##
##			     GLOBAL VARIABLES
##
##
###############################################################################
###############################################################################

#Location of the fasta file to mask
my $fasta = "";

#Location of the genbank file
my $gbk = "";


#Masking mode : 0 -> CDSs
#               1 -> RNAs (tRNA, rRNA, misc_RNA)
#               2 -> CDSs + RNAs (Default)
my $mode = 2;

#Ouptut file
my $output = "";

#Help
my $help = 0;



###############################################################################
###############################################################################
##
##
##				     MAIN
##
##
###############################################################################
###############################################################################



#Parse options
GetOptions('mode=i' => \$mode,
	   'output=s' => \$output,
	   'help' => \$help,
	   'h' => \$help) or pod2usage(2);

#Help
pod2usage(1) if $help;

#If no good number of parameters : exit
my $nb_args = @ARGV;

if($nb_args != 2){
  pod2usage(1);
}

$fasta = $ARGV[0];
$gbk = $ARGV[1];


#Check parameters
if($fasta eq ""){
  print "\nYou must provide a FASTA file.\n\n";
  pod2usage(1);
}
if(! -e $fasta){
  print "\nThe file \"$fasta\" doesn't exist.\n\n";
  pod2usage(1);
}
if($gbk eq ""){
  print "\nYou must provide a GENBANK file.\n\n";
  pod2usage(1);
}
if(! -e $gbk){
  print "\nThe  file \"$gbk\" doesn't exist.\n\n";
  pod2usage(1);
}
if(($mode < 0) || ($mode > 2)){
  print "\nThe masking mode has to be 0 (CDSs), 1 (RNAs) or 2 (CDSs + RNAs)\n\n";
  pod2usage(1);
}
if($output eq ""){
  my $fasta_dir = dirname($fasta);
  my $fasta_name = basename($fasta);
  if($fasta_name =~ /(.+)\.(.+)/){
    $output = File::Spec->catfile($fasta_dir,
				  $1."_masked.".$2);
  }
  else{
    exit_script("\nCould not determine automaticaly the ouptut name\n\n");
  }
}

#Get id and sequence from the fasta file
(my $id, my $seq) = &read_fasta($fasta);
if(($id eq "") || ($seq eq "")){
  exit_script("\nThe file \"$fasta\" is not in a fasta format\n\n");
}

#Get masking positions from the genbank file
my @positions;
my $regexp_gbk = "";
if($mode == 0){
  $regexp_gbk .= "(\\s+CDS";
}
else{
  if($mode == 1){
   $regexp_gbk = "(\\s+([tr]RNA|misc_RNA)"; 
  }
  else{
    $regexp_gbk .= "(\\s+(CDS|[tr]RNA|misc_RNA)";
  }
}
$regexp_gbk .= "\\s+(complement\\()?(join\\()?(<?[0-9]+\\.\\.>?";
$regexp_gbk .= "[0-9]+,?(\\s+)?)+\\)?)";

my $content_gbk = &read_file($gbk);
while($content_gbk =~ /$regexp_gbk/g){

  my $line = $1;
  while($line =~ /([0-9]+)\.\.>?([0-9]+)/g){
    push(@positions,$1."..".$2);
  }
}

#Case no annotation or not in genbank format
my $nb_pos = @positions;
if($nb_pos == 0){
  print("\nWarning: The file \"$gbk\" doesn't contain any annotation,  or is not in GENBANK format. \n\n");
  &write_fasta($output,$id,$seq);
}

#Process all positions and mask all regions
foreach my $pos(@positions){
  my $start = 0;
  my $end = 0;
  if($pos =~ /([0-9]+)\.\.([0-9]+)/){
    $start = $1;
    $end = $2;
  }

  #Build mask region
  my $mask = "";
  for(my $i = 0; $i < $end - $start + 1; $i++){
    $mask .= "N";
  }

  #Replace region by the masked one
  my $sub = substr($seq,
		   $start - 1,
		   $end -$start + 1,
		   $mask);
}

#Write new fasta
&write_fasta($output,$id,$seq);





###############################################################################
###############################################################################
##
##
##				     USAGE
##
##
###############################################################################
###############################################################################

__END__

=head1 NAME

C<mask_regions.pl> - Mask wanted coding regions according to annotations of a
genbank file.

=head1 SYNOPSIS

mask_coding.pl <fasta> <genbank> [options]

=over 8

=item B<fasta>

Fasta file to mask coding regions

=item B<genbank>

Genbank file containing annotations

=head1 OPTIONS

=over 8

=item B<--mode>=[0,1,2]

Masking mode : 0 -> CDSs ; 1 -> RNAs (tRNA, rRNA, misc_RNA); 2 -> CDS + RNAs (Default : 2)

=item B<--output>=FILE

Location of the ouput file (Default : same fasta name with "_masked")

=item B<-h, --help>

Prints a short help message and exits.

=back

=head1 AUTHORS

Benjamin Grenier-Boley <benjamin.grenier-boley@inria.fr>

=cut
