/*
 * This file is part of CGseqcore.
 *
 *  CGseqcore is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  CGseqcore is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with CGseqcore.  If not, see <http://www.gnu.org/licenses/>.
 */




/**
 *
 * \file alignment.c
 *
 * \author Benjamin Grenier-Boley <benjamin.grenier-boley@inria.fr>
 *
 * \brief File that defines all functions related to alignments
 *
 * \version 1.0
 *
 * \date July 2009
 *
 *
 * This file will define all fucntions related to alignments
 *
 */


/*Basic inclusions*/
#include <math.h>
#include <ctype.h>

/*Intern inclusions*/
#include "alignment.h"
#include "macros.h"
#include "io.h"
#include "options.h"
#include "blast_interface.h"
#include "sequence.h"
#include "aln_matrices.h"


/**
 *
 * \def PROG_DYN_LINEAR_INDEX_REF(Ii,Jj)
 *
 * \brief Macros that obtains the index in the alignment array according
 *to the positions on sequence I (the reference) and sequence J
 *
 */

#define PROG_DYN_LINEAR_INDEX_REF(Ii,Jj)  ((Ii)+((Jj)*(length_ref + 1)))


/**
 *
 * \def PROG_DYN_LINEAR_INDEX(Ii,Jj)
 *
 * \brief Macros that obtains the index in the alignment array according
 *to the positions on sequence I and sequence J
 *
 */

#define PROG_DYN_LINEAR_INDEX(Ii,Jj)  ((Ii)+((Jj)*(length_seq1 + 1)))



/**
 *
 * \def GAPOPEN
 *
 * \brief Macros that defines the cost of a gap opening
 *
 */


#define GAPOPEN -16.0


/**
 *
 * \def GAPEXT
 *
 * \brief Macros that defines the cost of a gap extension
 *
 */


#define GAPEXT -4


/**
 *
 * \fn int compare_sort_pos(const void *a, const void *b)
 *
 * \brief Function used to compare two alignments according to the 
 * starting position on the reference
 *
 * \param[in] a : first alignment to compare
 * \param[in] b : second alignment to compare
 *
 *
 * Function will compare two alignments according to the starting position 
 * on the reference and sort the array of alignments
 *
 *
 */

int compare_sort_pos(const void *a, const void *b)
{
  /*Args retrievment*/
  p_alignment_info *arg1 = (p_alignment_info *) a;
  p_alignment_info *arg2 = (p_alignment_info *) b;
  /*Position comparison*/
  if((*arg1)->pos_start_ref > (*arg2)->pos_start_ref) return 1;
  else if ((*arg1)->pos_start_ref == (*arg2)->pos_start_ref) return 0;
  else return -1;
}




/**
 *
 * \fn void destroy_alignment(p_alignment_info al)
 * 
 * \brief Function used to destroy a p_alignment_info structure
 *
 * \param[in,out] al : the p_alignment_info to destroy
 *
 *
 * This function will destroy all allocated memory for a p_alignment_info
 * structure
 *
 *
 */

void destroy_alignment(p_alignment_info al)
{

  /*Destroy the ref name*/
  if(al->name_ref){
    DESTROY(al->name_ref);
  }

  /*Destroy the query name*/
  if(al->name_query){
    DESTROY(al->name_query);
  }

  DESTROY(al);

}


/**
 *
 * \fn void copy_alignment(p_alignment_info al1, p_alignment_info al2)
 *
 * \brief Function used to copy one alignment into another
 *
 * \param[in,out] al1 : the p_alignment_info that will be filled with the
 * information of the second alignment
 * \param[in] al2 : the p_alignment_info to copy
 *
 *
 * This function will copy every attribute of the second alignment structure
 * into the first one
 *
 *
 */

void copy_alignment(p_alignment_info al1, p_alignment_info al2)
{

  /*Memory allocation*/
  NEW(al1->name_ref, char, strlen(al2->name_ref) + 1);
  NEW(al1->name_query, char, strlen(al2->name_query) + 1);

  /*Copy the attributes*/
  strcpy(al1->name_ref, al2->name_ref);
  strcpy(al1->name_query, al2->name_query);

  al1->index_ref = al2->index_ref;
  al1->index_query = al2->index_query;
  al1->identity = al2->identity;
  al1->al_length = al2->al_length;
  al1->nb_mismatches = al2->nb_mismatches;
  al1->nb_gaps_open = al2->nb_gaps_open;
  al1->pos_start_ref = al2->pos_start_ref;
  al1->pos_end_ref = al2->pos_end_ref;
  al1->pos_start_query = al2->pos_start_query;
  al1->pos_end_query = al2->pos_end_query;
  al1->evalue = al2->evalue;
  al1->bit_score = al2->bit_score;
  al1->strand_ref = al2->strand_ref;
  al1->strand_query = al2->strand_query;

}


/**
 *
 * \fn void read_alignments_files(p_main_structure m)
 *
 * \brief This function will read the alignments file and get the
 * the information within
 *
 * \param[in,out] m : the main structure that will contain the information
 *
 * This function will be used to extract all information on alignments files
 * in blast tabular format by using the parser
 *
 */


void read_alignments_files(p_main_structure m) 
{ 

  /*Indexes*/
  int i,j;
  /*File descriptor*/
  FILE *fd = NULL;
  /*Number of alignments information*/
  int nb_al_info = 0;
  /*Temporary alignments array for the current alignment file processed*/
  p_alignment_info *tmp_al_array = NULL;
  /*Temporary number of alignments in the final array*/
  int nb_al_info_final = 0;
  /*Evalue of the current alignment*/
  float evalue = 0.0;
  /*Length of the current alignment*/
  int length = 0;
  /*Identity of the current alignment*/
  float identity = 0;
  /*Current number of alignments*/
  int temp_nb = 0;

  /*Try to read each file*/
  for(i = 0; i < OPTS_alignments->nb_chars; i++){

    if((fd = fopen(OPTS_alignments->chars[i], "r")) == NULL) {
      ERROR_ "Unable to open '%s' for reading...\n",
	OPTS_alignments->chars[i] _ERROR;

    }
    else{
      
      if(OPTS_verbose){
	print_verbose_start_alignments_file(OPTS_alignments->chars[i]);
      }
	
      
      /*Destroy the last temporary array if exists*/
      if(tmp_al_array != NULL){
	for(j = 0; j < nb_al_info; j++){
	  destroy_alignment(tmp_al_array[j]);
	}
	DESTROY(tmp_al_array);
      }

      nb_al_info = 0;

      /*Parse the file to extract the sequences*/
      blast_file_name = OPTS_alignments->chars[i];   
      blastrestart(fd); 
      blastparse(); 
    
      fclose(fd);
      free_flex_mem_blast();
    
      /*If there were some errors print error message*/
      if(blast_nb_errors != 0) {
	ERROR_ "Alignment file '%s' has some errors\n",
	  OPTS_alignments->chars[i] _ERROR;
      }
      blast_nb_errors = 0;
      /*Retrievment of the alignment array*/
      tmp_al_array = blast_result;

      nb_al_info = 0;
      
      if((tmp_al_array != NULL) && (tmp_al_array[0] != NULL)){

	/*Counting the number of alignments*/
	while(tmp_al_array[nb_al_info] != NULL) nb_al_info++;

	/*If there was not enough aligments don't take into account*/
	if(nb_al_info < 1) {
	  ERROR_ "Alignment file '%s' don't have alignments, it won't be taken into account\n\n",
	    OPTS_alignments->chars[i] _ERROR;
	} 
	/*Else copy the alignments into the final array according to
	 selection parameters*/
	else{
	  
	  /*No selection parameter case*/
	  if((OPTS_evalue == -1) && (OPTS_min_id_alignment == 0.0) &&
	     (OPTS_min_length_alignment == 0)){

	    nb_al_info_final = m->array_of_al_info->nb_al_info;
	    RENEW(m->array_of_al_info->al_info,
		  p_alignment_info,
		  nb_al_info_final + nb_al_info);
	    for(j = 0; j < nb_al_info; j++){
	      NEW(m->array_of_al_info->al_info[j + nb_al_info_final],
		  alignment_info_s,
		  1);
	      copy_alignment(m->array_of_al_info->al_info[j + nb_al_info_final],
			     tmp_al_array[j]);
 
	    }	    


	    m->array_of_al_info->nb_al_info = nb_al_info + nb_al_info_final;


	    if(OPTS_verbose){
	      print_verbose_end_alignments_file(nb_al_info);
	    }
	  }
	 
	  /*Selection of alignments according to selection parameters*/
	  else{
	    temp_nb = 0;
	    
	    for(j = 0; j < nb_al_info; j++){
	      evalue = tmp_al_array[j]->evalue;
	      length = tmp_al_array[j]->al_length;
	      identity = tmp_al_array[j]->identity;

	      if((evalue <= OPTS_evalue) && 
		 (length >= OPTS_min_length_alignment) &&
		 (identity >= OPTS_min_id_alignment)){
       
		RENEW(m->array_of_al_info->al_info,
		      p_alignment_info,
		      m->array_of_al_info->nb_al_info + 1);
		NEW(m->array_of_al_info->
		    al_info[m->array_of_al_info->nb_al_info],
		    alignment_info_s,
		    1);
		copy_alignment(m->array_of_al_info->
			       al_info[m->array_of_al_info->nb_al_info],
			       tmp_al_array[j]);
		
		m->array_of_al_info->nb_al_info++;   
		temp_nb++;

	      }


	    }
	    if(OPTS_verbose){
	      print_verbose_end_alignments_file(temp_nb);
	    }
	    

	  }

	  
	}
      }
      else{
	ERROR_ "Alignment file '%s' don't have alignments, it won't be taken into account\n\n",
	  OPTS_alignments->chars[i] _ERROR;
      }
      
    }
    
  }

  if((tmp_al_array) && (blast_nb_errors == 0) && (nb_al_info > 1)){
    for(j = 0; j < nb_al_info; j++){
      if(tmp_al_array[j]){
	destroy_alignment(tmp_al_array[j]);
      }
    }
    DESTROY(tmp_al_array);
  }


}



/**
 *
 * \fn void complete_alignments_attributes(alignment_info_array *al_info,
 *                                       sequence_array *seqs)
 *
 * \brief This function will complete all missing attributes
 *
 * \param[in,out] al_info : the array of alignments that will be updated
 * \param[in] seqs : array of seqences that contains all information on 
 * sequences
 *
 * \return an 1 for success, 0 otherwise (sequence name not found for 
 * example)
 *      
 * This function will complete all missing attributes like the strand, it
 * will also invert start pos and end pos for minus strand in order to
 * have always an interval in the same way, it will also check the
 * subject / query name : the reference name will be put into the query
 * fields and finally, it will complete the index attributes that link that
 * array with the sequence array. It will also sort the array according to
 * the position on the reference sequence
 *
 */


int complete_alignments_attributes(alignment_info_array *al_info,
				    sequence_array *seqs){

  /*Index*/
  int i;
  /*Temporary string*/
  char *temp_string = NULL;
  /*Temporary start position*/
  int temp_pos_start = 0;
  /*Temporary end position*/
  int temp_pos_end = 0;
  /*Current index in the sequence array for the subject or the query*/
  int current_index = -1;

  /*Get all alignments*/
  for(i = 0; i < al_info->nb_al_info; i++){

    /*Convert the alignments positions which goes from 1 to N -> 0 to N-1*/
    al_info->al_info[i]->pos_start_ref--;
    al_info->al_info[i]->pos_end_ref--;
    al_info->al_info[i]->pos_start_query--;
    al_info->al_info[i]->pos_end_query--;

    /*Check the names : the reference will be considered as ref and not
     query*/
    if(strcmp(al_info->al_info[i]->name_ref,seqs->seqs[0]->name)){

      /*Permute query and subject names*/
      NEW(temp_string,
	  char,
	  strlen(al_info->al_info[i]->name_ref) + 1);
      strcpy(temp_string,al_info->al_info[i]->name_ref); 
 
      DESTROY(al_info->al_info[i]->name_ref);
      NEW(al_info->al_info[i]->name_ref, 
	  char,
	  strlen(al_info->al_info[i]->name_query) + 1);
      strcpy(al_info->al_info[i]->name_ref,
	     al_info->al_info[i]->name_query);

      DESTROY(al_info->al_info[i]->name_query);
      NEW(al_info->al_info[i]->name_query, 
	  char,
	  strlen(temp_string) + 1);
      strcpy(al_info->al_info[i]->name_query,
	     temp_string);

      DESTROY(temp_string);

      /*Permute positions*/
      temp_pos_start = al_info->al_info[i]->pos_start_ref;
      temp_pos_end = al_info->al_info[i]->pos_end_ref;
      al_info->al_info[i]->pos_start_ref = 
	al_info->al_info[i]->pos_start_query;
      al_info->al_info[i]->pos_end_ref = 
	al_info->al_info[i]->pos_end_query;
      al_info->al_info[i]->pos_start_query = temp_pos_start;
      al_info->al_info[i]->pos_end_query = temp_pos_end;

    }

    /*Ckeck positions and update strand for query*/
    if(al_info->al_info[i]->pos_start_ref >
       al_info->al_info[i]->pos_end_ref){
      /*case minus strand*/
      temp_pos_start = al_info->al_info[i]->pos_start_ref;
      al_info->al_info[i]->pos_start_ref = 
	al_info->al_info[i]->pos_end_ref;
      al_info->al_info[i]->pos_end_ref = temp_pos_start;
      al_info->al_info[i]->strand_ref = '-';
    }
    else{
      /*case plus strand*/
      al_info->al_info[i]->strand_ref = '+';
    }

    /*Ckeck positions and update strand for subject*/
    if(al_info->al_info[i]->pos_start_query >
       al_info->al_info[i]->pos_end_query){
      /*case minus strand*/
      temp_pos_start = al_info->al_info[i]->pos_start_query;
      al_info->al_info[i]->pos_start_query = 
	al_info->al_info[i]->pos_end_query;
      al_info->al_info[i]->pos_end_query = temp_pos_start;
      al_info->al_info[i]->strand_query = '-';
    }
    else{
      /*case plus strand*/
      al_info->al_info[i]->strand_query = '+';
    }

    /*Get indexes in the sequence array for query and subject */
    current_index = is_present_array_sequence(al_info->al_info[i]->name_ref,
					      seqs);
    if(current_index == -1){
      ERROR_ "The sequence name '%s' wasn't found in the sequence array\n\n",
	al_info->al_info[i]->name_ref _ERROR;
      return 0;
    }
    al_info->al_info[i]->index_ref = current_index;
   
    current_index = is_present_array_sequence(al_info->al_info[i]->name_query,
					      seqs);
    if(current_index == -1){
      ERROR_ "The sequence name '%s' wasn't found in the sequence array\n\n",
	al_info->al_info[i]->name_query _ERROR;
      return 0;
    }
    al_info->al_info[i]->index_query = current_index;
    


  }

  /*Sort the array of alignments according to the starting position on the
   reference sequence*/
  qsort(al_info->al_info,
	al_info->nb_al_info,
	sizeof(p_alignment_info),
	compare_sort_pos);

  return 1;

}



/**
 *
 * \fn int is_ref_intersect(p_alignment_info alignment, p_interval interval)
 *
 * \brief This function that tells if the alignment overlap on the reference
 * sequence for the interval given
 *
 * \param[in] alignment : the alignment
 * \param[in] interval : the interval
 *
 * \return 1 if they overlap, 0 otherwise
 *
 * This function will compare position on the reference sequence for the
 * alignment and those in the interval and check if they intersect
 *
 */

int is_ref_intersect(p_alignment_info alignment, p_interval interval)
{
  
  /*Check the intersection*/
  if((alignment->pos_start_ref <= interval->pos_end) &&
     (alignment->pos_end_ref >= interval->pos_start)){
    return 1;
  }
  else{
    return 0;
  }

}


/**
 *
 * \fn int get_length_ref_intersect(p_alignment_info alignment, 
 *                                p_interval interval)
 *
 * \brief This function will return the length of the intersection on the
 * reference sequence for the alignment and the interval given
 *
 * \param[in] alignment : the alignment
 * \param[in] interval : the interval
 *
 * \return the length of the intersection
 *
 * This function will compare positions on the reference sequence for the 
 * alignment and positions of the interval given and calculate the 
 * length of the intersection
 *
 */

int get_length_ref_intersect(p_alignment_info alignment, p_interval interval)
{

  /*Length of the intersection*/
  int length;

  /*Check the length of the instersection*/
  if(interval->pos_start >=
     alignment->pos_start_ref){
    
    if(interval->pos_end <=
       alignment->pos_end_ref){
      
      length = 
	interval->pos_end -
	interval->pos_start + 1;
      
    }
    else{
      length = 
	alignment->pos_end_ref -
	interval->pos_start + 1;
      
    }
      
  }
  
  else{
    if(alignment->pos_end_ref <=
       interval->pos_end){
      length = 
	alignment->pos_end_ref -
	alignment->pos_start_ref + 1;
    }
    else{
      length =
	interval->pos_end -
	alignment->pos_start_ref + 1;
      
      
    }

  }
  

  return length;


}



/**
 *
 * \fn void small_in_large(char *refseq, p_interval_query query)
 *
 * \brief This function will align the query sequence against the
 * reference one using a small in large algorithm
 *
 * \param[in] refseq : the reference sequence
 * \param[in,out] query : the query structure
 *
 * This function will align the query sequence against the reference one 
 * using a small in large algorithm. It will complete the id_ref attribute
 * and refine its positions by updateing its positions and sequence
 *
 */


void small_in_large(char *refseq, p_interval_query query)
{

  /*Indexes*/
  int i,j;

  /*Length of the reference sequence*/
  int length_ref = 0;
  /*Length of the query sequence*/
  int length_query = 0;

  /*Arrays of alignments results : match, insertion, deletion*/
  float *prog_dyn_mm = NULL;
  float *prog_dyn_ins = NULL;
  float *prog_dyn_del = NULL;
  /*Blosum_dna_matrix subsmatrix;*/
  blosum_iupac_matrix subsmatrix;
  /*Index of the nucleotide in the blosum matrix (for reference)*/
  int index_matrix_ref;
  /*Index of the nucleotide in the blosum matrix (for query)*/
  int index_matrix_query;
  /*Temporary score from match, insertion and deletion arrays*/
  float temp_match, temp_ins, temp_del;
  /*Maximum score in the current alignment*/
  float max_score;
  /*State of the alignment during the traceback :
   0 -> match state, 1 -> del state, 2 ->ins state*/
  int state;

  /*Position on the query sequence where the max score has been found*/
  int pos_query = -1;
  /*Length of the alignment*/
  int lg_alignment;
  /*Number of match in the alignment*/
  int nb_match;
  /*Current score during the traceback*/
  float current_score;
  
  /*New query sequence according to the alignment result*/
  char *new_query_seq = NULL;
  /*New starting position according to the alignment result*/
  int new_pos_start = 0;


  /*Declare a comparison blosum matrix*/
  subsmatrix = &iupac_full;

  /*Get reference and query lengths*/
  length_ref = strlen(refseq);
  length_query = strlen(query->seq);

  /*Initialisation of arrays used in comparison and the max score*/
  NEW(prog_dyn_mm, float, (length_ref + 1) * (length_query + 1));
  NEW(prog_dyn_del, float, (length_ref + 1) * (length_query + 1));
  NEW(prog_dyn_ins, float, (length_ref + 1) * (length_query + 1));

  for(j = 0; j <= length_query; j++) {
    prog_dyn_mm[PROG_DYN_LINEAR_INDEX_REF(0,j)] = -INFINITY;
    prog_dyn_ins[PROG_DYN_LINEAR_INDEX_REF(0,j)] = 0.0;
    prog_dyn_del[PROG_DYN_LINEAR_INDEX_REF(0,j)] = -INFINITY;
  }
  
  prog_dyn_mm[0] = prog_dyn_ins[0] = prog_dyn_del[0] = 0.0;
  max_score = -INFINITY;

  /*Begin the comparison*/ 
  for(i=1; i <= length_ref; i++) {

    /*Initialize the first column*/
    prog_dyn_mm[PROG_DYN_LINEAR_INDEX_REF(i,0)] = -INFINITY;
    prog_dyn_ins[PROG_DYN_LINEAR_INDEX_REF(i,0)] = -INFINITY;
    prog_dyn_del[PROG_DYN_LINEAR_INDEX_REF(i,0)] = 0.0;

    /*Get the index of the current nucleotide*/
    GET_IUPAC_INDICE(refseq[i-1], index_matrix_ref);
    
    if(index_matrix_ref == -1) {
      ERROR_ "REFERENCE SEQUENCE contains invalid base at position %i!!!\n",
	i _ERROR;
      break;
    }

    /*Comparison against the query sequence*/
    for(j=1; j <= length_query; j++) {
      GET_IUPAC_INDICE(query->seq[j-1], index_matrix_query);
      if(index_matrix_query == -1) {
	ERROR_ "QUERY SEQUENCE contains invalid base at position %i!!!\n",
	 i  _ERROR;
	break;
      }

      /*Calculation of the alignment score between these 2 nucleotides*/
      prog_dyn_mm[PROG_DYN_LINEAR_INDEX_REF(i,j)] = 
	(*subsmatrix)[index_matrix_ref][index_matrix_query] + 
	MAX3(prog_dyn_mm[PROG_DYN_LINEAR_INDEX_REF(i-1,j-1)], 
	     prog_dyn_ins[PROG_DYN_LINEAR_INDEX_REF(i-1,j-1)], 
	     prog_dyn_del[PROG_DYN_LINEAR_INDEX_REF(i-1,j-1)]);

      /*Got the maximum score between match, insertion and deletion*/
      temp_match = 
	GAPOPEN 
	+ 
	prog_dyn_mm[PROG_DYN_LINEAR_INDEX_REF(i,j-1)];
      temp_ins = 
	GAPEXT 
	+ prog_dyn_ins[PROG_DYN_LINEAR_INDEX_REF(i,j-1)];
      temp_del = 
	GAPOPEN 
	+ 
	prog_dyn_del[PROG_DYN_LINEAR_INDEX_REF(i,j-1)];	 
      prog_dyn_ins[PROG_DYN_LINEAR_INDEX_REF(i,j)] = MAX3(temp_match,
						      temp_ins,
						      temp_del);

	 
      temp_match = 
	GAPOPEN
	+ 
	prog_dyn_mm[PROG_DYN_LINEAR_INDEX_REF(i-1,j)];
      temp_ins = 
	GAPOPEN 
	+ 
	prog_dyn_ins[PROG_DYN_LINEAR_INDEX_REF(i-1,j)];
      temp_del = 
	GAPEXT + 
	prog_dyn_del[PROG_DYN_LINEAR_INDEX_REF(i-1,j)];	 
      prog_dyn_del[PROG_DYN_LINEAR_INDEX_REF(i,j)] = MAX3(temp_match,
						      temp_ins, 
						      temp_del);
      
    }
  }
      
  /*Search for the maximum score*/
  max_score = -INFINITY;
  for(j = 0; j <= length_query; j++){
    if(max_score < prog_dyn_mm[PROG_DYN_LINEAR_INDEX_REF(length_ref,j)]){	
      max_score = prog_dyn_mm[PROG_DYN_LINEAR_INDEX_REF(length_ref,j)];
      pos_query = j;
    }
  }

  /*Got the starting position of the alignment*/
  i = length_ref;
  j = pos_query;

  /*Parameters initialisation*/
  lg_alignment = 0;
  nb_match = 0;
  state = 0;

  /*Traceback*/
  while((i > 0) && (j > 0)){

    current_score = prog_dyn_mm[PROG_DYN_LINEAR_INDEX_REF(i,j)];

    switch(state){
	
      /*Match state*/
    case 0:
	
      GET_IUPAC_INDICE(refseq[i-1], index_matrix_ref);
      GET_IUPAC_INDICE(query->seq[j-1], index_matrix_query);
	
      current_score = prog_dyn_mm[PROG_DYN_LINEAR_INDEX_REF(i,j)] 
	- (*subsmatrix)[index_matrix_ref][index_matrix_query];
      /*Match / Mismatch*/
      if(current_score == prog_dyn_mm[PROG_DYN_LINEAR_INDEX_REF(i-1,j-1)]) {
	state = 0; /* matrix mm */   
	prog_dyn_mm[PROG_DYN_LINEAR_INDEX_REF(i,j)] = -INFINITY;
	
      } 
      else{
	/*Deletion*/
	if(current_score == prog_dyn_del[PROG_DYN_LINEAR_INDEX_REF(i-1,j-1)]) {
	  state = 1; /* matrix del */	    
	  prog_dyn_mm[PROG_DYN_LINEAR_INDEX_REF(i,j)] = -INFINITY;
	} 
	/*Insertion*/
	else {
	  state = 2; /* matrix ins */
	  prog_dyn_mm[PROG_DYN_LINEAR_INDEX_REF(i,j)] = -INFINITY;
	}
      }
      if(toupper(refseq[i-1]) == toupper(query->seq[j-1])){
	nb_match++;
      }
      lg_alignment++;
      i--;
      j--;
      break;

      /*Deletion state*/
    case 1:
	    
      /*Match / Mismatch*/
      if(prog_dyn_del[PROG_DYN_LINEAR_INDEX_REF(i,j)] ==
	 (prog_dyn_mm[PROG_DYN_LINEAR_INDEX_REF(i-1,j)] + GAPOPEN)) {
	state = 0;
	prog_dyn_mm[PROG_DYN_LINEAR_INDEX_REF(i,j)] = -INFINITY;
      } 
      else{
	/*Insertion*/
	if(prog_dyn_del[PROG_DYN_LINEAR_INDEX_REF(i,j)] ==
	   (prog_dyn_ins[PROG_DYN_LINEAR_INDEX_REF(i-1,j)] + GAPOPEN)) {
	  state = 2;
	  prog_dyn_mm[PROG_DYN_LINEAR_INDEX_REF(i,j)] = -INFINITY;
	} 
	/*Deletion*/
	else {
	  state = 1;
	  prog_dyn_mm[PROG_DYN_LINEAR_INDEX_REF(i,j)] = -INFINITY;
	  
	}
      }
      lg_alignment++;
      i--;
      break;

      /*Insertion state*/
    case 2:
	
      /*Match / Mismatch*/
      if(prog_dyn_ins[PROG_DYN_LINEAR_INDEX_REF(i,j)] ==
	 (prog_dyn_mm[PROG_DYN_LINEAR_INDEX_REF(i,j-1)] + GAPOPEN)) {
	state = 0;
	prog_dyn_mm[PROG_DYN_LINEAR_INDEX_REF(i,j )] = -INFINITY;
      } 
      else{ 
	if (prog_dyn_ins[PROG_DYN_LINEAR_INDEX_REF(i,j)] ==
	    (prog_dyn_del[PROG_DYN_LINEAR_INDEX_REF(i,j-1)] + GAPOPEN)) {
	  state = 1;
	  prog_dyn_mm[PROG_DYN_LINEAR_INDEX_REF(i,j)] = -INFINITY;
	} 
	else {
	  state = 2;
	  prog_dyn_mm[PROG_DYN_LINEAR_INDEX_REF(i,j)] = -INFINITY;
	  
	}
      }
      lg_alignment++;
      j--;
      break;

      /*Error state*/
    default:
      ERROR_ "INVALID STATE, you've discover a bug !!\n" _ERROR;
      state = -1;
    }

  }

  new_pos_start = j;

  /*Update the sequence (according to alignment positions) and the identity*/
  if(lg_alignment > length_ref){
    current_score = (float) 100.00 * (nb_match + 0.0) / (lg_alignment + 0.0);
  }
  else{
    current_score = (float) 100.00 * (nb_match + 0.0) / (length_ref + 0.0);
  }
  query->id_ref = current_score;


  /*Update the sequence only if it verify min and max identity thresholds*/
/*   if((current_score >= OPTS_min_id) && */
/*      (current_score <= OPTS_max_id)){ */
   
 
    NEW(new_query_seq, char, pos_query - new_pos_start + 2);
    for( i = new_pos_start; i < pos_query; i++){
      new_query_seq[i - new_pos_start] = query->seq[i];
    }
    new_query_seq[pos_query - new_pos_start] = '\0';

    if(query->strand == '+'){

      query->pos_start += new_pos_start;
      query->pos_end = query->pos_start + (pos_query - new_pos_start) - 1;
    }
    else{
 
      query->pos_end = query->pos_end - new_pos_start;
      query->pos_start = query->pos_end - pos_query + 1 + new_pos_start;

     
    }

    DESTROY(query->seq);
    NEW(query->seq, char, strlen(new_query_seq) + 1);
    strcpy(query->seq, new_query_seq);
    DESTROY(new_query_seq);


/*   } */

  DESTROY(prog_dyn_mm);
  DESTROY(prog_dyn_ins);
  DESTROY(prog_dyn_del);
  

}


/**
 *
 * \fn float aln(p_interval_query seq1, p_interval_query seq2)
 *
 * \brief This function will align the first sequence against the second
 * one and return the identity
 *
 * \param[in] seq1 : the first sequence
 * \param[in] seq2 : the second structure
 *
 * \return A float repreenting the identity bewteen the 2 sequences
 *
 * This function will align the 2 sequences using a semi-global alignment
 * and return the identity
 *
 */


float aln(p_interval_query seq1, p_interval_query seq2)
{

   /*Indexes*/
  int i,j;

  /*Arrays of alignments results : match, insertion, deletion*/
  float *prog_dyn_mm = NULL;
  float *prog_dyn_ins = NULL;
  float *prog_dyn_del = NULL;
  /*Blosum_dna_matrix subsmatrix;*/
  blosum_iupac_matrix subsmatrix;
  /*Index of the nucleotide in the blosum matrix (for seq1)*/
  int index_matrix_seq1;
  /*Index of the nucleotide in the blosum matrix (for seq2)*/
  int index_matrix_seq2;
  /*Temporary score from match, insertion and deletion arrays*/
  float temp_match, temp_ins, temp_del;
  /*Maximum score in the current alignment*/
  float max_score;
  /*State of the alignment during the traceback :
    0 -> match state, 1 -> del state, 2 ->ins state*/
  int state;

  /*Position on the second sequence where the max score has been found*/
  int pos_seq2 = -1;
  /*Length of the alignment*/
  int lg_alignment;
  /*Number of match in the alignment*/
  int nb_match;
  /*Current score during the traceback*/
  float current_score;

  /*Length of the first sequence*/
  int length_seq1;
  /*Length of the second sequence*/
  int length_seq2;


  
  /*Declare a comparison blosum matrix*/
  subsmatrix = &iupac_full;
  
  /*Get sequences lengths*/
  length_seq1 = strlen(seq1->seq);
  length_seq2 = strlen(seq2->seq);

  /*Initialisation of arrays used in comparison and the max score*/
  NEW(prog_dyn_mm, float, ((length_seq1 + 1) * (length_seq2 + 1)));
  NEW(prog_dyn_del, float, ((length_seq1 + 1) * (length_seq2 + 1)));
  NEW(prog_dyn_ins, float, ((length_seq1 + 1) * (length_seq2 + 1)));

  for(j = 0; j <= length_seq2; j++) {
    prog_dyn_mm[PROG_DYN_LINEAR_INDEX(0,j)] = -INFINITY;
    prog_dyn_ins[PROG_DYN_LINEAR_INDEX(0,j)] = 0.0;
    prog_dyn_del[PROG_DYN_LINEAR_INDEX(0,j)] = -INFINITY;
  }

  prog_dyn_mm[0] = prog_dyn_ins[0] = prog_dyn_del[0] = 0.0;
  max_score = -INFINITY;

  /*Begin the comparison*/ 
  for(i=1; i <= length_seq1; i++) {

    /*Initialize the first column*/
    prog_dyn_mm[PROG_DYN_LINEAR_INDEX(i,0)] = -INFINITY;
    prog_dyn_ins[PROG_DYN_LINEAR_INDEX(i,0)] = -INFINITY;
    prog_dyn_del[PROG_DYN_LINEAR_INDEX(i,0)] = GAPOPEN + (GAPEXT * i);

    /*Get the index of the current nucleotide*/
    GET_IUPAC_INDICE(seq1->seq[i-1], index_matrix_seq1);
    
    if(index_matrix_seq1 == -1) {
      ERROR_ "SEQUENCE contains invalid base at position %i!!!\n", 
        i _ERROR;
      break;
    }

    /*Comparison against the second sequence*/
    for(j=1; j <= length_seq2; j++) {
      GET_IUPAC_INDICE(seq2->seq[j-1], index_matrix_seq2);
      if(index_matrix_seq2 == -1) {
	ERROR_ "SEQUENCE contains invalid base at position %i!!!\n", 
	   j _ERROR;
	break;
      }

      /*Calculation of the alignment score between these 2 nucleotides*/
      prog_dyn_mm[PROG_DYN_LINEAR_INDEX(i,j)] = 
	(*subsmatrix)[index_matrix_seq1][index_matrix_seq2] + 
	MAX3(prog_dyn_mm[PROG_DYN_LINEAR_INDEX(i-1,j-1)], 
	     prog_dyn_ins[PROG_DYN_LINEAR_INDEX(i-1,j-1)], 
	     prog_dyn_del[PROG_DYN_LINEAR_INDEX(i-1,j-1)]);

      /*Got the maximum score between match, insertion and deletion*/
      temp_match = 
	GAPOPEN 
	+ 
	prog_dyn_mm[PROG_DYN_LINEAR_INDEX(i,j-1)];
      temp_ins = 
	GAPEXT 
	+ prog_dyn_ins[PROG_DYN_LINEAR_INDEX(i,j-1)];
      temp_del = 
	GAPOPEN 
	+ 
	prog_dyn_del[PROG_DYN_LINEAR_INDEX(i,j-1)];	 
      prog_dyn_ins[PROG_DYN_LINEAR_INDEX(i,j)] = MAX3(temp_match,
						      temp_ins,
						      temp_del);

	 
      temp_match = 
	GAPOPEN 
	+ 
	prog_dyn_mm[PROG_DYN_LINEAR_INDEX(i-1,j)];
      temp_ins = 
	GAPOPEN 
	+ 
	prog_dyn_ins[PROG_DYN_LINEAR_INDEX(i-1,j)];
      temp_del = 
	GAPEXT + 
	prog_dyn_del[PROG_DYN_LINEAR_INDEX(i-1,j)];	 
      prog_dyn_del[PROG_DYN_LINEAR_INDEX(i,j)] = MAX3(temp_match,
						      temp_ins, 
						      temp_del);
      
    }
      

  }
  
  max_score = - INFINITY;

  /*Search the maximum score on the last column*/
  for(j = 0; j <= length_seq2; j++){

    if(max_score < prog_dyn_mm[PROG_DYN_LINEAR_INDEX(length_seq1,j)]){
	
      max_score = prog_dyn_mm[PROG_DYN_LINEAR_INDEX(length_seq1,j)];
      pos_seq2 = j;
    }
  }

  /*Got the starting positions of the alignment*/
  i = length_seq1;
  j = pos_seq2;
  /*Parameters initialisation*/
  lg_alignment = 0;
  nb_match = 0;
  state = 0;
    
  /*Traceback and mask the path taken*/
  while((i > 0) && (j > 0)){
    
    switch(state){

      /*Match state*/
    case 0:

      GET_IUPAC_INDICE(seq1->seq[i-1], index_matrix_seq1);
      GET_IUPAC_INDICE(seq2->seq[j-1], index_matrix_seq2);
	    
      current_score = prog_dyn_mm[PROG_DYN_LINEAR_INDEX(i,j)] 
	- (*subsmatrix)[index_matrix_seq1][index_matrix_seq2];
      /*Match / Mismatch*/
      if(current_score == prog_dyn_mm[PROG_DYN_LINEAR_INDEX(i-1,j-1)]) {
	state = 0; /* matrix mm */   
      } 
      else{
	/*Deletion*/
	if(current_score == prog_dyn_del[PROG_DYN_LINEAR_INDEX(i-1,j-1)]) {
	  state = 1; /* matrix del */	    
	} 
	/*Insertion*/
	else {

	  state = 2; /* matrix ins */

	}
      }
      if(toupper(seq1->seq[i-1]) == toupper(seq2->seq[j-1])){
	nb_match++;
      }
      lg_alignment++;
      i--;
      j--;
      break;

      /*Deletion state*/
    case 1:
	    
      /*Match / Mismatch*/
      if(prog_dyn_del[PROG_DYN_LINEAR_INDEX(i,j)] ==
	 (prog_dyn_mm[PROG_DYN_LINEAR_INDEX(i-1,j)] + GAPOPEN)) {
	state = 0;
      } 
      else{
	/*Insertion*/
	if(prog_dyn_del[PROG_DYN_LINEAR_INDEX(i,j)] ==
	   (prog_dyn_ins[PROG_DYN_LINEAR_INDEX(i-1,j)] + GAPOPEN)) {
	  state = 2;
	} 
	/*Deletion*/
	else {

	  state = 1;

	}
      }
      lg_alignment++;
      i--;
      break;

      /*Insertion state*/
    case 2:
	
      /*Match / Mismatch*/
      if(prog_dyn_ins[PROG_DYN_LINEAR_INDEX(i,j)] ==
	 (prog_dyn_mm[PROG_DYN_LINEAR_INDEX(i,j-1)] + GAPOPEN)) {
	state = 0;
      } 
      else{ 
	if (prog_dyn_ins[PROG_DYN_LINEAR_INDEX(i,j)] ==
	    (prog_dyn_del[PROG_DYN_LINEAR_INDEX(i,j-1)] + GAPOPEN)) {
	  state = 1;
	} 
	else {

	  state = 2;

	}
      }
      lg_alignment++;
      j--;
      break;

      /*Error state*/
    default:
      ERROR_ "INVALID STATE, you've discover a bug !!\n" _ERROR;
      state = -1;
    }

  }

  current_score = (float) 100.0 * (nb_match + 0.0) / (lg_alignment + 0.0);


  DESTROY(prog_dyn_mm);
  DESTROY(prog_dyn_ins);
  DESTROY(prog_dyn_del);

  return current_score; 


}

