/*
 * This file is part of CGseqcore.
 *
 *  CGseqcore is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  CGseqcore is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with CGseqcore.  If not, see <http://www.gnu.org/licenses/>.
 */



/**
 *
 * \file alignment.h
 *
 * \author Benjamin Grenier-Boley <benjamin.grenier-boley@inria.fr>
 *
 * \brief Header file that defines all functions related to alignments
 *
 * \version 1.0
 *
 * \date July 2009
 *
 *
 * This file will define all fucntions related to alignments  
 *
 *
 */


#ifndef __ALIGNMENT_H__
#define __ALIGNMENT_H__


/*Intern inclusion*/
#include "types.h"



/**
 *
 * \fn int compare_sort_pos(const void *a, const void *b)
 *
 * \brief Function used to compare two alignments according to the 
 * starting position on the reference
 *
 * \param[in] a : first alignment to compare
 * \param[in] b : second alignment to compare
 *
 * \return -1 if "a" starts on the reference before "b", 0 if equal and -1
 * otherwise 
 *
 * Function will compare two alignments according to the starting position 
 * on the reference and sort the array of alignments
 *
 *
 */


extern int compare_sort_pos(const void *a, const void *b);


/**
 *
 * \fn void destroy_alignment(p_alignment_info al)
 *
 * \brief Function used to destroy a p_alignment_info structure
 *
 * \param[in,out] al : the p_alignment_info to destroy
 *
 *
 * This function will destroy all allocated memory for a p_alignment_info
 * structure
 *
 *
 */

extern void destroy_alignment(p_alignment_info al);


/**
 *
 * \fn void copy_alignment(p_alignment_info al1, p_alignment_info al2)
 *
 * \brief Function used to copy one alignment into another
 *
 * \param[in,out] al1 : the p_alignment_info where to copy
 * \param[in] al2 : the p_alignment_info to copy
 *
 *
 * This function will copy every attribute of the second alignment structure
 * into the first one
 *
 *
 */

extern void copy_alignment(p_alignment_info al1, p_alignment_info al2);



/**
 *
 * \fn void read_alignments_files(p_main_structure m)
 * 
 * \brief This function will read the alignments file and get the
 * the information within
 *
 * \param[in,out] m : the main structure that will updated
 *
 * This function will be used to extract all information on alignments files
 * in blast tabular format
 *
 */


extern void read_alignments_files(p_main_structure m);


/**
 *
 * \fn void complete_alignments_attributes(alignment_info_array *al_info,
 *                                         sequence_array *seqs)
 *
 * \brief This function will complete all missing attributes
 *
 * \param[in,out] al_info : the array of alignments that will be updated
 * \param[in] seqs : array of seqences that contains all information on 
 * sequences
 *
 * \return an 1 for success, 0 otherwise (sequence name not found for 
 * example)
 *
 * This function will complete all missing attributes like the strand, it
 * will also invert start pos and end pos for minus strand in order to
 * have always an interval in the same way, it will also check the
 * subject / query name : the reference name will be put into the query
 * fields and finally, it will complete the index attributes that link that
 * array with the sequence array
 *
 */


extern int complete_alignments_attributes(alignment_info_array *al_info,
					   sequence_array *seqs);


/**
 *
 * \fn int is_ref_intersect(p_alignment_info alignment, p_interval interval)
 *
 * \brief This function that tells if the alignment overlap on the reference
 * sequence for the interval given
 *
 * \param[in] alignment : the alignment
 * \param[in] interval : the interval
 *
 * \return 1 if they overlap, 0 otherwise
 *
 * This function will compare position on the reference sequence for the
 * alignment and those in the interval and check if they intersect
 *
 */


extern int is_ref_intersect(p_alignment_info alignment, p_interval interval);



/**
 *
 * \fn int get_length_ref_intersect(p_alignment_info alignment, 
 *                                p_interval interval)
 *
 * \brief This function will return the length of the intersection on the
 * reference sequence for the alignment and the interval given
 *
 * \param[in] alignment : the alignment
 * \param[in] interval : the interval
 *
 * \return the length of the intersection
 *
 * This function will compare positions on the reference sequence for the 
 * alignment and positions of the interval given and calculate the 
 * length of the intersection
 *
 */


extern int get_length_ref_intersect(p_alignment_info alignment, 
				    p_interval interval);


/**
 *
 * \fn void small_in_large(char *refseq, p_interval_query query)
 *
 * \brief This function will align the query sequence against the
 * reference one using a small in large algorithm
 *
 * \param[in] refseq : the reference sequence
 * \param[in,out] query : the query structure
 *
 * This function will align the query sequence against the reference one 
 * using a small in large algorithm. It will complete the id_ref attribute
 * and refine its positions by updateing its positions and sequence
 *
 */


extern void small_in_large(char *refseq, p_interval_query query);


/**
 *
 * \fn float aln(p_interval_query seq1, p_interval_query seq2)
 *
 * \brief This function will align the first sequence against the second
 * one and return the identity
 *
 * \param[in] seq1 : the first sequence
 * \param[in] seq2 : the second structure
 *
 * \return A float repreenting the identity bewteen the 2 sequences
 *
 * This function will align the 2 sequences using a semi-global alignment
 * and return the identity
 *
 */



extern float aln(p_interval_query seq1, p_interval_query seq2);



#endif
