/*
 * This file is part of CGseqcore.
 *
 *  CGseqcore is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  CGseqcore is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with CGseqcore.  If not, see <http://www.gnu.org/licenses/>.
 */



/**
 *
 * \file aln_matrices.h
 *
 * \author Benjamin Grenier-Boley <benjamin.grenier-boley@inria.fr>
 *
 * \brief Header file for the management of matrices used in comparison
 *
 * \version 1.0
 *
 * \date July 2009
 *
 *
 * This file will define all structures and pointers on theses that will be 
 * used in the software
 *
 *
 */

#ifndef __ALN_MATRICES_H__
#define __ALN_MATRICES_H__



/**
 *
 * \def GET_NU_INDICE(Nu,Ii)
 *
 * \brief Macros that will link basic nucleotides to its index in the basic 
 *  matrix
 *
 */


#define GET_NU_INDICE(Nu,Ii)			\
   switch(Nu) {					\
   case 'A':					\
   case 'a':					\
      (Ii) = 0;					\
      break;					\
   case 'T':					\
   case 't':					\
   case 'U':					\
   case 'u':					\
      (Ii) = 1;					\
      break;					\
   case 'G':					\
   case 'g':					\
      (Ii) = 2;					\
      break;					\
   case 'C':					\
   case 'c':					\
      (Ii) = 3;					\
      break;					\
   case 'N':					\
   case 'n':					\
      (Ii) = 4;					\
      break;					\
   default:					\
      ERROR_ "unknown nucleotide '%c' (ascii %i)\n !!!",(Nu), (Nu)  _ERROR; \
      (Ii) = -1;				\
      break;					\
   }


/**
 *
 * \def GET_IUPAC_INDICE(Nu,Ii)
 *
 * \brief Macros that will link IUPAC nucleotides to its index in the IUPAC 
 * matrix
 *
 */

#define GET_IUPAC_INDICE(Nu,Ii)			\
   switch(Nu) {					\
   case 'A':					\
   case 'a':					\
      (Ii) = 0;					\
      break;					\
   case 'T':					\
   case 't':					\
      (Ii) = 1;					\
      break;					\
   case 'G':					\
   case 'g':					\
      (Ii) = 2;					\
      break;					\
   case 'C':					\
   case 'c':					\
      (Ii) = 3;					\
      break;					\
   case 'S':					\
   case 's':					\
      (Ii) = 4;					\
      break;					\
   case 'W':					\
   case 'w':					\
      (Ii) = 5;					\
      break;					\
   case 'R':					\
   case 'r':					\
      (Ii) = 6;					\
      break;					\
   case 'Y':					\
   case 'y':					\
      (Ii) = 7;					\
      break;					\
   case 'K':					\
   case 'k':					\
      (Ii) = 8;					\
      break;					\
   case 'M':					\
   case 'm':					\
      (Ii) = 9;					\
      break;					\
   case 'B':					\
   case 'b':					\
      (Ii) = 10;				\
      break;					\
   case 'V':					\
   case 'v':					\
      (Ii) = 11;				\
      break;					\
   case 'H':					\
   case 'h':					\
      (Ii) = 12;				\
      break;					\
   case 'D':					\
   case 'd':					\
      (Ii) = 13;				\
      break;					\
   case 'N':					\
   case 'n':					\
      (Ii) = 14;				\
      break;					\
   case 'U':					\
   case 'u':					\
      (Ii) = 15;				\
      break;					\
   default:					\
      ERROR_ "unknown nucleotide '%c' (ascii %i)\n !!!",(Nu), (Nu)  _ERROR; \
      (Ii) = -1;				\
      break;					\
   }



/**
 *
 * \struct blosum_dna_matrix_t
 *
 * \brief Structure that implements a basic blosum matrix
 *
 */


typedef int blosum_dna_matrix_t[5][5];


/**
 *
 * \struct blosum_dna_matrix
 *
 * \brief Pointer on a blosum_dna_matrix_t
 *
 */


typedef blosum_dna_matrix_t *blosum_dna_matrix;



/**
 *
 * \struct blosum_iupac_matrix_t
 *
 * \brief Structure that implements an iupac blosum matrix
 *
 */


typedef int blosum_iupac_matrix_t[16][16];


/**
 *
 * \struct blosum_iupac_matrix
 *
 * \brief Pointer on a blosum_iupac_matrix_t
 *
 */


typedef blosum_iupac_matrix_t *blosum_iupac_matrix;


/******************************************************************************
 **   The basic blosum matrix
 **     
 *****************************************************************************/


extern blosum_dna_matrix_t dna_full; /*!< Basic blosum matrix*/


/******************************************************************************
 **   The IUPAC blosum matrix
 **     
 *****************************************************************************/

extern blosum_iupac_matrix_t iupac_full; /*!< Basic IUPAC blosum matrix*/


#endif
