/*
 * This file is part of CGseqcore.
 *
 *  CGseqcore is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  CGseqcore is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with CGseqcore.  If not, see <http://www.gnu.org/licenses/>.
 */


/**
 *
 * \file blast_parser.y
 *
 * \author Benjamin Grenier-Boley <benjamin.grenier-boley@inria.fr>
 *
 * \brief Bison file (yacc) that will use the lexer to parse the blast file
 *
 * \version 1.0
 *
 * \date July 2009
 *
 *
 * This file will use the lexer grammar to parse the blast tabular file
 *
 */


%{

#include "blast_interface.h"
#include "types.h"
#include "macros.h"
#include "io.h"



  char *blast_file_name;          /*!< The file to parse*/
  int blast_nb_errors;            /*!< The number of errors*/
  p_alignment_info *blast_result; /*!< The result of the parsing : the alignments*/


%}

%union{
  p_alignment_info *alignments;
  p_alignment_info alignment;
  char *text;
  int number;
  float real;
 }




%token <text> NAME_VALUE
%token <real> REAL_VALUE
%token <number> INT_VALUE


%type <alignments> alignments lines 
%type <alignment> line


%start alignments

%%

alignments:
lines
{ blast_result = $1;
}
|
error
{
  blast_result = NULL;
}


lines: 
lines
line
{
  p_alignment_info *res = $1;
  int i = 0;

  while(res[i]) i++;

  RENEW(res, p_alignment_info, (i + 2));
  res[i] = $2;
  res[i + 1] = NULL;
  $$ = res;
}
|
line
{
  p_alignment_info *res = NULL;
  NEW(res, p_alignment_info, 2);
  res[0] = $1;
  res[1] = NULL;
  $$ = res;
}
|
lines
error
{
  p_alignment_info *res = $1;
  
  $$ = res;


}
|
error
lines
{
  p_alignment_info *res = $2;
  
  $$ = res;

}


line:
NAME_VALUE
NAME_VALUE
REAL_VALUE
INT_VALUE
INT_VALUE
INT_VALUE
INT_VALUE
INT_VALUE
INT_VALUE
INT_VALUE
REAL_VALUE
REAL_VALUE
{
  
  p_alignment_info res;

  NEW(res, alignment_info_s, 1); 
  
  res->name_ref = $1;
  res->name_query = $2;
  res->identity = $3;
  res->al_length = $4;
  res->nb_mismatches = $5;
  res->nb_gaps_open = $6;
  res->pos_start_ref = $7;
  res->pos_end_ref = $8;
  res->pos_start_query = $9;
  res->pos_end_query = $10;
  res->evalue = $11;
  res->bit_score = $12;

  $$ = res;

}
|
NAME_VALUE
NAME_VALUE
INT_VALUE
INT_VALUE
INT_VALUE
INT_VALUE
INT_VALUE
INT_VALUE
INT_VALUE
INT_VALUE
REAL_VALUE
REAL_VALUE
{
  
  p_alignment_info res;

  NEW(res, alignment_info_s, 1); 
  
  res->name_ref = $1;
  res->name_query = $2;
  res->identity = (float)$3;
  res->al_length = $4;
  res->nb_mismatches = $5;
  res->nb_gaps_open = $6;
  res->pos_start_ref = $7;
  res->pos_end_ref = $8;
  res->pos_start_query = $9;
  res->pos_end_query = $10;
  res->evalue = $11;
  res->bit_score = $12;

  $$ = res;
}
|
NAME_VALUE
NAME_VALUE
REAL_VALUE
INT_VALUE
INT_VALUE
INT_VALUE
INT_VALUE
INT_VALUE
INT_VALUE
INT_VALUE
INT_VALUE
REAL_VALUE
{
  
  p_alignment_info res;

  NEW(res, alignment_info_s, 1); 
  
  res->name_ref = $1;
  res->name_query = $2;
  res->identity = $3;
  res->al_length = $4;
  res->nb_mismatches = $5;
  res->nb_gaps_open = $6;
  res->pos_start_ref = $7;
  res->pos_end_ref = $8;
  res->pos_start_query = $9;
  res->pos_end_query = $10;
  res->evalue = (float)$11;
  res->bit_score = $12;

  $$ = res;
}
|
NAME_VALUE
NAME_VALUE
REAL_VALUE
INT_VALUE
INT_VALUE
INT_VALUE
INT_VALUE
INT_VALUE
INT_VALUE
INT_VALUE
REAL_VALUE
INT_VALUE
{
  
  p_alignment_info res;

  NEW(res, alignment_info_s, 1); 
  
  res->name_ref = $1;
  res->name_query = $2;
  res->identity = $3;
  res->al_length = $4;
  res->nb_mismatches = $5;
  res->nb_gaps_open = $6;
  res->pos_start_ref = $7;
  res->pos_end_ref = $8;
  res->pos_start_query = $9;
  res->pos_end_query = $10;
  res->evalue = $11;
  res->bit_score = (float)$12;

  $$ = res;
}
|
NAME_VALUE
NAME_VALUE
INT_VALUE
INT_VALUE
INT_VALUE
INT_VALUE
INT_VALUE
INT_VALUE
INT_VALUE
INT_VALUE
INT_VALUE
REAL_VALUE
{
  
  p_alignment_info res;

  NEW(res, alignment_info_s, 1); 
  
  res->name_ref = $1;
  res->name_query = $2;
  res->identity = (float)$3;
  res->al_length = $4;
  res->nb_mismatches = $5;
  res->nb_gaps_open = $6;
  res->pos_start_ref = $7;
  res->pos_end_ref = $8;
  res->pos_start_query = $9;
  res->pos_end_query = $10;
  res->evalue = (float)$11;
  res->bit_score = $12;

  $$ = res;
}
|
NAME_VALUE
NAME_VALUE
INT_VALUE
INT_VALUE
INT_VALUE
INT_VALUE
INT_VALUE
INT_VALUE
INT_VALUE
INT_VALUE
REAL_VALUE
INT_VALUE
{
  
  p_alignment_info res;

  NEW(res, alignment_info_s, 1); 
  
  res->name_ref = $1;
  res->name_query = $2;
  res->identity = (float)$3;
  res->al_length = $4;
  res->nb_mismatches = $5;
  res->nb_gaps_open = $6;
  res->pos_start_ref = $7;
  res->pos_end_ref = $8;
  res->pos_start_query = $9;
  res->pos_end_query = $10;
  res->evalue = $11;
  res->bit_score = (float)$12;

  $$ = res;
}
|
NAME_VALUE
NAME_VALUE
REAL_VALUE
INT_VALUE
INT_VALUE
INT_VALUE
INT_VALUE
INT_VALUE
INT_VALUE
INT_VALUE
INT_VALUE
INT_VALUE
{
  
  p_alignment_info res;

  NEW(res, alignment_info_s, 1); 
  
  res->name_ref = $1;
  res->name_query = $2;
  res->identity = $3;
  res->al_length = $4;
  res->nb_mismatches = $5;
  res->nb_gaps_open = $6;
  res->pos_start_ref = $7;
  res->pos_end_ref = $8;
  res->pos_start_query = $9;
  res->pos_end_query = $10;
  res->evalue = (float)$11;
  res->bit_score = (float)$12;

  $$ = res;
}
|
NAME_VALUE
NAME_VALUE
INT_VALUE
INT_VALUE
INT_VALUE
INT_VALUE
INT_VALUE
INT_VALUE
INT_VALUE
INT_VALUE
INT_VALUE
INT_VALUE
{
  
  p_alignment_info res;

  NEW(res, alignment_info_s, 1); 
  
  res->name_ref = $1;
  res->name_query = $2;
  res->identity = (float)$3;
  res->al_length = $4;
  res->nb_mismatches = $5;
  res->nb_gaps_open = $6;
  res->pos_start_ref = $7;
  res->pos_end_ref = $8;
  res->pos_start_query = $9;
  res->pos_end_query = $10;
  res->evalue = (float)$11;
  res->bit_score = (float)$12;

  $$ = res;
}
;


%%


void blastwarning_token(const char *s, const char *token) {
   ERROR_ "'%s' line '%i' token '%s': %s\n", 
     blast_file_name, 
     blastlineno, 
     token, 
     s _ERROR;
}

void blasterror_token(const char *s, const char *token) {
   blast_nb_errors++;
   ERROR_ "'%s' line '%i' token '%s': %s\n", 
     blast_file_name, 
     blastlineno, 
     token, 
     s _ERROR;
}
   
