/*
 * This file is part of CGseqcore.
 *
 *  CGseqcore is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  CGseqcore is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with CGseqcore.  If not, see <http://www.gnu.org/licenses/>.
 */


/**
 *
 * \file buffer.h
 *
 * \author Benjamin Grenier-Boley <benjamin.grenier-boley@inria.fr>
 *
 * \brief Header file for the management of buffers
 *
 * \version 1.0
 *
 * \date July 2009
 *
 *
 * This file will define all defines and functions for bufffers
 *
 *
 */

#ifndef __BUFFER_H__
#define __BUFFER_H__



/******************************************************************************
 **   Structures that will implement a sequence buffer and its pointer
 **     
 *****************************************************************************/


/**
 *
 * \struct sequence_buffer_s
 *
 * \brief  buffer sequence implementation
 *
 * That structure will define a buffer used to parse sequences
 *
 */

typedef struct {
  char *data;   /*!< Data of the buffer */ 
  int size;     /*!< Size of the buffer */
  int max_size; /*!< Maximum size of the buffer */ 
} sequence_buffer_s;


/**
 *
 * \struct p_sequence_buffer
 *
 * \brief Pointer on a sequence_buffer_s
 *
 * That structure will define the pointer on the sequence buffer
 *
 */

typedef sequence_buffer_s* p_sequence_buffer;



/******************************************************************************
 **   Buffer functions
 **     
 *****************************************************************************/

/**
 *
 * \def NEW_BUFFER(Buf, MSiz)
 *
 * \brief Defines a function to initialize the buffer
 *
 * \param[in,out] Buf : the buffer to initialize
 * \param[in] MSiz : the maximum buffer size
 *
 * This definition defines a function to initialize the buffer given with
 * the maximum size defined
 *
 */


#define NEW_BUFFER(Buf,MSiz)			\
  NEW((Buf), sequence_buffer_s, 1);		\
  (Buf)->size=0;				\
  (Buf)->max_size=(MSiz);			\
  NEW(((Buf)->data), char, (1 + (MSiz)));	\
  (Buf)->data[(Buf)->max_size]='\0'


/**
 *
 * \def BUFFER_EMPTY(Buf)
 *
 * \brief Defines a function to declare an empty buffer
 *
 * \param[in,out] Buf : the buffer to empty
 *
 * This definition defines a function to declare a buffer with a size of 0 
 *
 */


#define BUFFER_EMPTY(Buf)			\
  ((Buf)->size=0)


/**
 *
 * \def BUFFER_MAXSIZE(Buf)
 *
 * \brief Defines a function to get the buffer maximum size
 *
 * \param[in] Buf : the buffer to know the maximum size
 *
 * This definition defines a function which returns the current buffer
 * maximum size 
 *
 */


#define BUFFER_MAXSIZE(Buf)			\
  ((Buf)->max_size)


/**
 *
 * \def BUFFER_SIZE(Buf)
 *
 * \brief Defines a function to get the buffer size
 *
 * \param[in] Buf : the buffer to know the size
 *
 * This definition defines a function which returns the current buffer size 
 *
 */


#define BUFFER_SIZE(Buf)			\
  ((Buf)->size)


/**
 *
 * \def BUFFER_IS_FULL(Buf)
 *
 * \brief Defines a function to know if a buffer is full
 *
 * \param[in] Buf : the buffer to know if its capacity is full
 *
 * This definition defines a function which returns the fact that the buffer
 * has reached its maximum size 
 *
 */


#define BUFFER_IS_FULL(Buf)			\
  ((Buf)->size == (Buf)->max_size)


/**
 *
 * \def BUFFER_RESIZE(Buf,MSiz)
 *
 * \brief Defines a function to resize a buffer
 *
 * \param[in,out] Buf : the buffer to resize
 * \param[in] MSiz : the new size of the buffer
 *
 * This definition defines a function which will resize the buffer with the 
 * wanted size
 *
 */


#define BUFFER_RESIZE(Buf,MSiz)			\
  RENEW((Buf)->data, char, (1 + (MSiz)));	\
  (Buf)->max_size = (MSiz);			\
  (Buf)->data[(Buf)->max_size]='\0'


/**
 *
 * \def BUFFER_PUT(Buf, Val)
 *
 * \brief Defines a function to add a value in the buffer
 *
 * \param[in,out] Buf : the buffer where to add the value
 * \param[in] Val : the value to add
 *
 * This definition defines a function which will add the value in the buffer
 * and increase its actual size
 *
 */


#define BUFFER_PUT(Buf,Val)			\
  ((Buf)->data[((Buf)->size)++] = (Val))


/**
 *
 * \def BUFFER_PUT_ALL(Buf, Oth)
 *
 * \brief Defines a function to add a several values in the buffer
 *
 * \param[in,out] Buf : the buffer where to add the values
 * \param[in] Oth : the buffer containing the values to add 
 *
 * This definition defines a function which will add the values of a buffer
 * into another one
 *
 */


#define BUFFER_PUT_ALL(Buf,Oth)						\
  strncpy(&((Buf)->data[(Buf)->size]), (Oth)->data, (Oth)->size);	\
  (Buf)->size += (Oth)->size


/**
 *
 * \def BUFFER_RELEASE(Buf)
 *
 * \brief Defines a function to release a buffer
 *
 * \param[in,out] Buf : the buffer to release
 *
 * This definition defines a function which add the end string char at the 
 * end of the buffer data to release it
 *
 */


#define BUFFER_RELEASE(Buf)			\
  ((Buf)->data[(Buf)->size] = '\0')


/**
 *
 * \def BUFFER_TOSTRING(Buf)
 *
 * \brief Defines a function to get the buffer data into a string
 *
 * \param[in] Buf : the buffer to get the data
 *
 * This definition defines a function which will return the data of the
 * buffer into a string
 *
 */


#define BUFFER_TOSTRING(Buf)			\
  ((Buf)->data)


/**
 *
 * \def DESTROY_BUFFER(Buf)
 *
 * \brief Defines a function to destroy a buffer
 *
 * \param[in,out] Buf : the buffer to destriy
 *
 * This definition defines a function which will detroy the buffer data and
 * then the buffer itself
 *
 */


#define DESTROY_BUFFER(Buf)			\
  DESTROY(((Buf)->data));			\
  DESTROY((Buf))



#endif
