/*
 * This file is part of CGseqcore.
 *
 *  CGseqcore is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  CGseqcore is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with CGseqcore.  If not, see <http://www.gnu.org/licenses/>.
 */


/**
 *
 * \file carnac.c
 *
 * \author Benjamin Grenier-Boley <benjamin.grenier-boley@inria.fr>
 *
 * \brief File that contains all functions for the carnac management
 *
 * \version 1.0
 *
 * \date July 2009
 *
 * 
 * This file wil define all functions for the use of carnac as a selecting
 * parameter
 *
 *
 */


/*Basic inclusions*/
#include <regex.h>
#include <errno.h>
#include <unistd.h>
#include <stdlib.h>


/*Intern inclusions*/
#include "carnac.h"
#include "macros.h"
#include "io.h"
#include "options.h"

/******************************************************************************
 **   Regular expressions initialization 
 **     
 *****************************************************************************/

/*Regular expression that matches the energy*/
regex_t* energy_regex = NULL; 


/*Regular expression that matches the number of stems in carnac results*/
regex_t* selected_regex = NULL;



/******************************************************************************
 **   Functions definitions
 **     
 *****************************************************************************/


/**
 *
 * \fn char *read_line(FILE* fd, int* nb)
 *
 * \brief Function used to read a file and returns the string read and the 
 * number of char read
 *
 * \param[in] fd : the file descriptor
 * \param[in] nb : the number of char read
 *
 * \return the string read
 *
 * Function that will read the file and returns the both string read and its
 * length
 *
 *
 */


char *read_line(FILE* fd, int* nb)
{

  /*The characters read*/
  char *res = NULL;
  /*The buffer*/
  char *buf[80];
  /*Number of current characters read*/
  int nr;
  /*Indexes*/
  int i, j;

  *nb = 0;
  nr = -1;
 

  /*Reading the content of the file*/
  while((nr = fread(buf, sizeof(char), 80, fd)) > 0) {
    
    i = 0;
    while((i < nr) && (*(buf[i]) != '\n')) i++;
 
    if((*nb)) { RENEW(res, char, (i + *nb + 1)); }
    else { NEW(res, char, (i + 1)); }
    for(j = 0; j < i; j++) {
      
      res[(*nb) + j] = *(buf[j]);
    }
    *nb += i;
    res[*nb] = '\0';
    
    if(i < nr) {
      fseek(fd, (i - nr) + 1, SEEK_CUR);
      return res;	 
    }
  }
  
  if(feof(fd) == 0) {
    *nb = 0;
    ERROR_ "read_line failed !!!\n" _ERROR;
    return NULL;
  }
  
  return res;


}

/**
 *
 * \fn int init_carnac()
 *
 * \brief Function used to initialize the call of carnac
 *
 * \return an integer : \n
 * 1 -> an error occured \n
 * 0 -> no error
 *
 * Function  that will initialize the call of the carnac software 
 *
 *
 */

int init_carnac()
{


  /*Part that initialise the regular expression of the number of stems
   */
  if(selected_regex == NULL) {
    NEW(selected_regex, regex_t, 1);

    
    if(regcomp(selected_regex, 
	       CARNAC_SELECTED_STEMS_REGEX, 
	       REG_EXTENDED | REG_NEWLINE)) {
      ERROR_ "REGULAR EXPRESSION selected_regex IN CARNAC PROBLEM\n" _ERROR;
      return 1;
    }

    

  }
  return 0;
}


/**
 *
 * \fn int carnac_individual_energy(const char *output)
 *
 * \brief Function used to read the results of carnac
 *
 * \param[in] output : the location of the carnac result
 *
 * \return an integer corresponding to the result matched by the regular
 * expression used (eg number of stems found)
 *
 * Function  that will read the carnac results depending on the regular
 * expression used
 *
 *
 */

int carnac_individual_energy(const char *output)
{

  /*Results : number of stems*/
  int nb_res;
  /*Number of characters read*/
  int nb;
  /*The line read*/
  char *line = NULL;
  /*Results read from caRNAc*/
  char *trans = NULL, *trans2 = NULL;
  /*The file to read*/
  FILE* fd;
  /*The regular expression that matches the wanted result in caRNAc*/
  regmatch_t regmatch;


  nb_res = 0;

  /*Reading the caRNAc results file*/
  if((fd = fopen(output, "r"))) {
            
    NEW(trans, char, 500);
    NEW(trans2, char, 500);

    nb_res = 0;
      
    read_file(fd,&line);
    nb = strlen(line);

    RENEW(trans, char, (nb + 1));     
    strncpy(trans, line, nb);
    trans[nb] = '\0';
 
    /*Check if the wanted result(number of stems here) is here*/
    if(regexec(selected_regex, trans, 1, &regmatch, 0) == 0) {
      
      RENEW(trans2, char, (nb + 1));

      strncpy(trans2, &(trans[regmatch.rm_so]), regmatch.rm_eo - regmatch.rm_so - 6);
      trans2[regmatch.rm_eo - regmatch.rm_so - 6] = '\0';
      
      /*If at least one stem has been found increase the number of results*/
      if(atoi(trans2) > 0) {
	nb_res++;
      }
    }

    DESTROY(line);
    fclose(fd);
    DESTROY(trans);
    DESTROY(trans2);
  } 
  else {
    ERROR_ "unable to open %s output file '%s'\n", "carnac", output _ERROR;
    return -1;
  }

  return nb_res;

}


/**
 *
 * \fn int write_fasta_seq(FILE* output, const p_sequence* seqs, const int nb)
 *
 * \brief Function used to write sequences in a file in fasta format
 * 
 * \param[in] output : the file where to write
 * \param[in] seqs : the sequences to write
 * \param[in] nb : the number of sequences to write
 *
 * \return an integer : \n
 * 1,2 -> an error occured
 * 0 -> no error
 *
 * Function  that will write the provided sequences in the wanted file in
 * the fasta format
 *
 *
 */


int write_raw(FILE* output, const p_sequence* seqs, const int nb)
{

  /*Indexes*/
  int i = 0, j, k, l;
  /*Read buffer*/
  char buf[81];

  /*Writing all the sequences wanted*/
  while(i < nb) {
    j = 0;
    while(j < seqs[i]->length) {
      k = seqs[i]->length - j;

      if(k > 80) {
	for(l = 0; l < 80; l++){
	  buf[l] = seqs[i]->bases[j + l];
	}
	buf[80] = '\0';
	j += 80;
      } else {
	for(l = 0; l < k; l++){
	  buf[l] = seqs[i]->bases[j + l];
	}
	buf[k] = '\0';
	j += k;
      }

      if(fprintf(output, "%s", buf) < 0) return 1;
    }
    if(fprintf(output, "\n") < 0) return 1;

    i++;
  }
  return 0;

}


/**
 *
 * \fn int write_fasta_seq(FILE* output, const p_sequence* seqs, const int nb)
 *
 * \brief Function used to write sequences in a file in fasta format
 * 
 * \param[in] output : the file where to write
 * \param[in] seqs : the sequences to write
 * \param[in] nb : the number of sequences to write
 *
 * \return an integer : \n
 * 1,2 -> an error occured
 * 0 -> no error
 *
 * Function  that will write the provided sequences in the wanted file in
 * the fasta format
 *
 *
 */

int write_fasta_seq(FILE* output, const p_sequence* seqs, const int nb)
{

  /*Index*/
  int i;
  /*Writing all the sequences wanted*/
  for(i= 0; i < nb; i++) {
    if(fprintf(output, ">%s\n", seqs[i]->name) < 0) return 1;
    if(write_raw(output, &(seqs[i]), 1)) return 2;
  }

  return 0;

}


/**
 *
 * \fn int file_exists(const char *file)
 *
 * \brief Function used to test if a file exists
 *
 * \param[in] file : the file to test
 *
 * \return an integer : \n
 * 1 -> the file exists
 * 0 -> the file doesn't exist
 *
 * Function  that will test the existence of a file
 *
 *
 */

int file_exists(const char *file)
{

 /*The file to test*/
  FILE* fd = NULL;

  errno = 0;

  /*Test to open the file*/
  if((fd = fopen(file, "r"))) {
    fclose(fd);
    return 1;
  }

  return 0;

}



/**
 *
 * \fn int call_carnac(const p_sequence* seqs, const int nb_seqs)
 *
 * \brief Function used to execute carnac on all sequences provided and
 * get the results
 *
 * \param[in] seqs : the array of sequences
 * \param[in] nb_seqs : the number of sequences to compare
 *
 * \return an integer representing the result of carnac matched by the
 * wanted regular expression
 *
 * Function that will execute carnac on all sequences, get the results and
 * return them
 *
 *
 */


int call_carnac(const p_sequence *seqs, const int nb_seqs)
{

  /*The result of caRNAc*/
  int status;
  /*File count*/
  int fnum;
  /*Temporary file names*/
  char *str, *output = NULL, *agregate, *t, *z, *output_base = NULL;
  /*The file used to create temp files*/
  FILE* fd = NULL;
  /*Aggregate file*/
  char *agregate_name = "agregate";
  /*Output*/
  char *output_name = "output";
  /*Random number*/
  int random = 0;
  /*Size of the random number in char*/
  int size = 0;

  /*Init part*/
  if(init_carnac()) return -1;
    
  /*Generate random number and random sequence name*/
  random = (rand() & 32767);
  get_size_number(random,&size);

  NEW(agregate, 
      char, 
      (1 + strlen(agregate_name) + strlen(OPTS_output_dir) + size));
  sprintf(agregate,
	  "%s%s%i",
	  OPTS_output_dir,
	  agregate_name,
	  random);

  while((fd = fopen(agregate, "r")) != NULL){
    DESTROY(agregate);
    
    random = (rand() & 32767);
    get_size_number(random,&size);
    
    NEW(agregate, 
	char, 
	(1 + strlen(agregate_name) + strlen(OPTS_output_dir) + size));
    sprintf(agregate,
	    "%s%s%i",
	    OPTS_output_dir,
	    agregate_name,
	    random);
    fclose(fd);

  }

  /*Create the files needed by caRNAc to execute on them*/
  if((fd = fopen(agregate, "w"))) {
    if(write_fasta_seq(fd, seqs, nb_seqs) == 0) {
      fflush(fd);
      fclose(fd);
     
      /*Generate output file*/
      NEW(output, 
	  char, 
	  (1 + strlen(output_name)+ strlen(OPTS_output_dir)) + size);
      sprintf(output,
	      "%s%s%d",
	      OPTS_output_dir,
	      output_name,
	      random);
      
      while((fd = fopen(output, "r")) != NULL){
	DESTROY(output);
	
	random = (rand() & 32767);
	get_size_number(random,&size);
	
	NEW(output, 
	    char, 
	    (1 + strlen(output_name) + strlen(OPTS_output_dir) + size));
	sprintf(output,
		"%s%s%i",
		OPTS_output_dir,
		output_name,
		random);
	 fclose(fd);
	
      }
     

      
      /*Determine the command to use*/

      NEW(str, char, (1 + 11 + 
		      strlen(CARNAC_EXE) + 
		      strlen(agregate) + 
		      strlen(output)));
      sprintf(str, "%s%s '%s' > '%s'", CARNAC_EXE," -C", agregate, output);
      

	    

      if((status = system(str)) < 0) {
	ERROR_ 
	  "system error while calling '%s' : exit status %i\n", 
	  str, 
	  status 
	  _ERROR;
	return -1;
      }
      
      DESTROY(str);
      
      NEW(t, char, (1 + strlen(agregate) + 6)); /* <agregate>00x.ct|eq */
      NEW(z, char, 21); /* Z_00x.<seq>.out */
      
      fnum = 1;
      
      /*Parse all results of caRNAc*/
      for(status = 0; status < nb_seqs; status++) {
	sprintf(z, "Z_%.3i.%.10s.out", fnum, seqs[status]->name);
	if(file_exists(z)) {
	  fnum++;
	  unlink(z);
	}
	
	sprintf(t, "%s%.3i.ct", agregate, (status + 1));
	if(file_exists(t)) {
	  unlink(t);
	}
	
	sprintf(t, "%s%.3i.eq", agregate, (status + 1));
	if(file_exists(t)) {
	  unlink(t);
	}
      }
      DESTROY(z);
      DESTROY(t);
      
      unlink("alignment.out");
      unlink("graph.out");
      unlink("stems.out");
 
      unlink(agregate);
      DESTROY(agregate);
    }
    else {
      ERROR_ 
	"an error occured while writing sequences in fasta format\n" 
	_ERROR;
      return -1;
    }
    
  } 
  else {
    ERROR_ "unable to open '%s' for writing\n", agregate _ERROR;
    return -1;
  }
  
  /*Get the wanted result of caRNAc (here the number of stems)*/
  status = carnac_individual_energy(output);
 
  if(output) {
    unlink(output);
    DESTROY(output);
  }

  if(output_base) {
    DESTROY(output_base);
  }

  /*Error management*/
  if(status < 0) {
    ERROR_ "an error occured during %s result analysis\n", "carnac" _ERROR;
  }

  if(agregate != NULL){
    DESTROY(agregate);
  }
 
  
  return status;

}


/**
 *
 * \fn void destroy_regex()
 *
 * \brief Function used to destroy the regular expression selected
 *
 * Function  that will destroy the regular expression 
 *
 *
 */

void destroy_regex()
{

  /*If there's one selected regular expression destroy it*/
  if(selected_regex != NULL){
    regfree(selected_regex);
    DESTROY(selected_regex);
  }

}
