/*
 * This file is part of CGseqcore.
 *
 *  CGseqcore is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  CGseqcore is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with CGseqcore.  If not, see <http://www.gnu.org/licenses/>.
 */


/**
 *
 * \file carnac.h
 *
 * \author Benjamin Grenier-Boley <benjamin.grenier-boley@inria.fr>
 *
 * \brief Header file for the carnac management
 *
 * \version 1.0
 *
 * \date July 2009
 *
 * 
 * This file wil define all functions for the use of carnac as a selecting
 * parameter
 *
 *
 */


#ifndef __CARNAC_H__
#define __CARNAC_H__



/*Basic inclusions*/
#include <stdio.h>


/*Intern inclusions*/
#include "types.h"


/******************************************************************************
 **   Regular expression definitions
 **     
 *****************************************************************************/


/**
 *
 * \def CARNAC_SELECTED_STEMS_REGEX
 *
 * \brief Define the regular expression matching the number of stems in 
 * carnac results
 *
 */


#define CARNAC_SELECTED_STEMS_REGEX "[0-9]?[0-9]* stems\\)"





/******************************************************************************
 **   Functions definitions
 **     
 *****************************************************************************/


/**
 *
 * \fn int call_carnac(const p_sequence* seqs, const int nb_seqs)
 *
 * \brief Function used to execute carnac on all sequences provided and
 * get the results
 *
 * \param[in] seqs : the array of sequences
 * \param[in] nb_seqs : the number of sequences to compare
 *
 * \return an integer representing the result of carnac matched by the
 * wanted regular expression
 *
 * Function that will execute carnac on all sequences, get the results and
 * return them
 *
 *
 */

extern int call_carnac(const p_sequence *seqs, const int nb_seqs);


/**
 *
 * \fn char *read_line(FILE* fd, int* nb)
 *
 * \brief Function used to read a file and returns the string read and the 
 * number of char read
 *
 * \param[in] fd : the file descriptor
 * \param[in] nb : the number of char read
 *
 * \return the string read
 *
 * Function that will read the file and returns the both string read and its
 * length
 *
 *
 */


char *read_line(FILE* fd, int* nb);


/**
 *
 * \fn int init_carnac()
 *
 * \brief Function used to initialize the call of carnac
 *
 * \return an integer : \n
 * 1 -> an error occured \n
 * 0 -> no error
 *
 * Function  that will initialize the call of the caranc software 
 *
 *
 */

int init_carnac();


/**
 *
 * \fn int carnac_individual_energy(const char *output)
 *
 * \brief Function used to read the results of carnac
 *
 * \param[in] output : the location of the carnac result
 *
 * \return an integer corresponding to the result matched by the regular
 * expression used (eg number of stems found)
 *
 * Function  that will read the carnac results depending on the regular
 * expression used
 *
 *
 */

int carnac_individual_energy(const char *output);


/**
 *
 *\fn int write_raw(FILE* output, const p_sequence* seqs, const int nb)
 *
 * \brief Function used to write sequences in a file
 *
 * \param[in] output : the file where to write
 * \param[in] seqs : the sequences to write
 * \param[in] nb : the number of sequences to write
 *
 * \return an integer : \n
 * 1 -> an error occured
 * 0 -> no error
 *
 * Function  that will write the provided sequences in the wanted file
 *
 *
 */

int write_raw(FILE* output, const p_sequence* seqs, const int nb);


/**
 *
 * \fn int write_fasta_seq(FILE* output, const p_sequence* seqs, const int nb)
 *
 * \brief Function used to write sequences in a file in fasta format
 * 
 * \param[in] output : the file where to write
 * \param[in] seqs : the sequences to write
 * \param[in] nb : the number of sequences to write
 *
 * \return an integer : \n
 * 1,2 -> an error occured
 * 0 -> no error
 *
 * Function  that will write the provided sequences in the wanted file in
 * the fasta format
 *
 *
 */

int write_fasta_seq(FILE* output, const p_sequence* seqs, const int nb);


/**
 *
 * \fn int file_exists(const char *file)
 *
 * \brief Function used to test if a file exists
 *
 * \param[in] file : the file to test
 *
 * \return an integer : \n
 * 1 -> the file exists
 * 0 -> the file doesn't exist
 *
 * Function  that will test the existence of a file
 *
 *
 */

int file_exists(const char *file);



/**
 *
 * \fn void destroy_regex()
 *
 * \brief Function used to destroy the regular expression selected
 *
 * Function  that will destroy the regular expression 
 *
 *
 */

extern void destroy_regex();


#endif
