/*
 * This file is part of CGseqcore.
 *
 *  CGseqcore is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  CGseqcore is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with CGseqcore.  If not, see <http://www.gnu.org/licenses/>.
 */


/**
 *
 * \file clique.c
 *
 * \author Benjamin Grenier-Boley <benjamin.grenier-boley@inria.fr>
 *
 * \brief File that defines all functions used in the clique search
 *
 * \version 1.0
 *
 * \date July 2009
 *
 *
 * This file will define all functions used in the naive and maxiclique
 * search
 *
 *
 */

/*Intern inclusions*/
#include "clique.h"
#include "options.h"
#include "macros.h"
#include "alignment.h"
#include "io.h"
#include "carnac.h"



/**
 *
 * \fn void search_naive_clique(p_main_structure m)
 *
 * \brief Function used to search the naive clique for all sets
 *
 * \param[in,out] m : the main structure containing information and that
 * will be updated
 *
 * That function will search the naive clique for all sets. It will include
 * by reference identity order each sequences and abort as soon as one
 * cannot be included
 *
 */

void search_naive_clique(p_main_structure m)
{

  /*Indexes*/
  int i,j,k;
  /*Number of sequences wanted*/
  int nb_seqs_wanted = 0;
  /*Boolean that says if the clique is good*/
  int is_clique_good = 1;
  /*Current identity*/
  float identity = 0.0;
  /*Current index of the clique sequence in the query array*/
  int index_current = -1;
  /*Number of sequences in the current clique*/
  int nb_seqs_clique = 0;
  /*Array of sequences for the carnac call*/
  p_sequence *seqs_carnac = NULL;


  /*Manage number of sequences by set wanted (ref + queries !!)*/
  if(OPTS_nb_seqs_set == -1){
    nb_seqs_wanted = 100000;
  }
  else{
    nb_seqs_wanted = OPTS_nb_seqs_set;
  }


  /*Get all sets of sequences*/
   for(i = 0; i < m->sets_of_sequences->nb_sets; i++){ 

    /*Initialize the clique structure*/
    NEW(m->sets_of_sequences->sets[i]->clique_set,
	clique_s,
	1);

    m->sets_of_sequences->sets[i]->clique_set->clique_index = NULL;
    m->sets_of_sequences->sets[i]->clique_set->nb_seqs_clique = 0;
    
    nb_seqs_clique = 0;
    is_clique_good = 1;
    
    /*Process only sets which have at least 2 query sequences*/
    if(m->sets_of_sequences->sets[i]->nb_q_intervals > 1){

      if(OPTS_verbose){
      print_verbose_start_naive_clique_set(i + 1, 
					   m->sets_of_sequences->sets[i]->
					   nb_q_intervals + 1);
      }

      /*Adding the first query sequence in the clique : it respects the 
	identity threshold with the reference sequence*/
      RENEW(m->sets_of_sequences->sets[i]->clique_set->clique_index,
	    int,
	    1);
      m->sets_of_sequences->sets[i]->clique_set->clique_index[0] = 0;
      nb_seqs_clique = 1;    

      
      /*Get all query sequences*/
      for(j = 1; 
	  is_clique_good && 
	    nb_seqs_clique < (nb_seqs_wanted - 1) &&
	    j < m->sets_of_sequences->sets[i]->nb_q_intervals;
	  j++){
	

	/*Compare the current query with all seqs already inside the clique
	  until the number of sequences wanted is reached or until a 
	  sequence can't be integrated inside the clique*/
	for(k = 0;
	    is_clique_good &&
	      k < nb_seqs_clique;
	    k++){
	
	  index_current = 
	    m->sets_of_sequences->sets[i]->clique_set->clique_index[k];

	  identity = 
	    aln(m->sets_of_sequences->sets[i]->q_intervals[j],
		m->sets_of_sequences->sets[i]->q_intervals[index_current]);


	  if((identity < OPTS_min_id) || (identity > OPTS_max_id)){
	    is_clique_good = 0;
	  }
	  /*Check carnac if wanted*/
	  else{

	    if(OPTS_carnac){
	      NEW(seqs_carnac, p_sequence, 2);
	      NEW(seqs_carnac[0], sequence_s, 1);
	      NEW(seqs_carnac[0]->name, char, 5);
	      NEW(seqs_carnac[0]->bases, 
		  char,
		  strlen(m->sets_of_sequences->sets[i]->q_intervals[j]->seq) + 1);
	      strcpy(seqs_carnac[0]->name, "seq1");
	      strcpy(seqs_carnac[0]->bases,
		     m->sets_of_sequences->sets[i]->q_intervals[j]->seq);
	      seqs_carnac[0]->length = 
		strlen(m->sets_of_sequences->sets[i]->q_intervals[j]->seq);

	      NEW(seqs_carnac[1], sequence_s, 1);
	      NEW(seqs_carnac[1]->name, char, 5);
	      NEW(seqs_carnac[1]->bases, 
		  char,
		  strlen(m->sets_of_sequences->sets[i]->q_intervals[index_current]->seq) + 1);
	      strcpy(seqs_carnac[1]->name, "seq2");
	      strcpy(seqs_carnac[1]->bases,
		     m->sets_of_sequences->sets[i]->q_intervals[index_current]->seq);
	      seqs_carnac[1]->length = 
		strlen(m->sets_of_sequences->sets[i]->q_intervals[index_current]->seq);


	      if(!call_carnac(seqs_carnac, 2)){
		is_clique_good = 0;
	      }

	      DESTROY(seqs_carnac[0]->name);
	      DESTROY(seqs_carnac[0]->bases);
	      DESTROY(seqs_carnac[1]->name);
	      DESTROY(seqs_carnac[1]->bases);
	      DESTROY(seqs_carnac[0]);
	      DESTROY(seqs_carnac[1]);
	      DESTROY(seqs_carnac);

	    }

	  }

	}

	if(is_clique_good){
	  RENEW(m->sets_of_sequences->sets[i]->clique_set->clique_index,
		int,
		nb_seqs_clique + 1);
	  m->sets_of_sequences->sets[i]->clique_set->clique_index[nb_seqs_clique] =
	    j;
	  nb_seqs_clique++;
	    
	}

      }

      if(is_clique_good){
	m->sets_of_sequences->sets[i]->clique_set->nb_seqs_clique = 
	  nb_seqs_clique;
	if(OPTS_verbose){    
	  print_verbose_end_naive_clique_set(nb_seqs_clique + 1);
	}
	
      }

      
      else{
	DESTROY(m->sets_of_sequences->sets[i]->clique_set->clique_index);
	DESTROY(m->sets_of_sequences->sets[i]->clique_set);
	if(OPTS_verbose){
	  print_verbose_end_naive_clique_set(0);
	}

      }

    }
    else{
      if(OPTS_verbose){
	DESTROY(m->sets_of_sequences->sets[i]->clique_set->clique_index);
	DESTROY(m->sets_of_sequences->sets[i]->clique_set);
	print_verbose_no_connect(i + 1);
      }
    }

  }
    
  
}


/**
 *
 * \fn void recursive_search_maxi_clique(p_set_of_seqs set,
 *                                       p_clique temp_clique,
 *                                       int index)
 *
 * \brief Function used to search recursively the maxi clique in the
 * current set
 *
 * \param[in,out] set : the current set
 * \param[in,out] temp_clique : the temporary maxi clique
 * \param[in] index : the index in the query array of the sequence to add 
 *
 * That function will search the maxi clique recursively by trying to 
 * add the query sequence in the temporary maxi clique
 *
 */

void recursive_search_maxi_clique(p_set_of_seqs set,
				  p_clique temp_clique,
				  int index)
{

  /*Index*/
  int i;
  /*Boolean that says if the current sequence is connected with every
    sequence in the clique*/
  int is_connected = 1;
  /*Temp index of a sequence*/
  int temp_index;
  /*Last length of the temporary max clique*/
  int last_length = 0;
  /*Boolean that says if the maxi clique has been found*/
  int is_finished = 0;

  if(((OPTS_nb_seqs_set != -1) && 
      ( set->clique_set->nb_seqs_clique < OPTS_nb_seqs_set - 1))||
     ((OPTS_nb_seqs_set == -1) &&
      (set->clique_set->nb_seqs_clique !=
       set->nb_q_intervals))){


    /*Check if the sequence to add in not already in the clique*/
    if(!is_present_int_array(index,
			     temp_clique->clique_index,
			     temp_clique->nb_seqs_clique)){

      /*Check if the sequence is connected with every sequences of the clique
	except the last one :  it's obliged to be a neighbor of the current one*/
      for(i = 0; is_connected && i < temp_clique->nb_seqs_clique - 1; i++){
      
	temp_index = temp_clique->clique_index[i];
	if(!is_present_int_array(index,
				 set->neighbors[temp_index]->indexes,
				 set->neighbors[temp_index]->nb_neighbors)){
	  is_connected = 0;
	}
      }

      /*If the sequence is connected with every sequences of the clique,
	add it to the temporary clique ans try to add its neighbors*/
      if(is_connected){

	RENEW(temp_clique->clique_index,
	      int,
	      temp_clique->nb_seqs_clique + 1);
	temp_clique->clique_index[temp_clique->nb_seqs_clique] = index;
	temp_clique->nb_seqs_clique++;
      
	/*If the size of the temporary clique is greater than the current
	  one in the set : copy the temp clique. The size of the clique must 
	  respect the max number of sequences wanted by the user*/
	if(temp_clique->nb_seqs_clique > set->clique_set->nb_seqs_clique){


	  DESTROY(set->clique_set->clique_index);
	  NEW(set->clique_set->clique_index,
	      int,
	      temp_clique->nb_seqs_clique);
	  for(i = 0; i < temp_clique->nb_seqs_clique; i++){
	    set->clique_set->clique_index[i] = temp_clique->clique_index[i];
	  }
	  set->clique_set->nb_seqs_clique = temp_clique->nb_seqs_clique;
	}

      
	if(((OPTS_nb_seqs_set != -1) &&
	    ( set->clique_set->nb_seqs_clique < OPTS_nb_seqs_set - 1))||
	   ((OPTS_nb_seqs_set == -1) &&
	   (set->clique_set->nb_seqs_clique !=
	    set->nb_q_intervals))){	
	  for(i = 0; !is_finished && 
		i < set->neighbors[index]->nb_neighbors; i++){
	
	    /*Search recursively with all neighbors of sequence that has just
	      been added if the max size has not been reached*/
	    if(((OPTS_nb_seqs_set != -1) && 
		( set->clique_set->nb_seqs_clique < OPTS_nb_seqs_set - 1))||
	       ((OPTS_nb_seqs_set == -1) &&
		(set->clique_set->nb_seqs_clique !=
		 set->nb_q_intervals))){
	  
	  
	      temp_index = set->neighbors[index]->indexes[i];
	  
	      last_length = temp_clique->nb_seqs_clique;
	  
	      recursive_search_maxi_clique(set,
					   temp_clique,
					   temp_index);
	  
	      /*If the temporary clique has been modified erase the last
		index added and continue the recursion*/
	      if(last_length != temp_clique->nb_seqs_clique){
		RENEW(temp_clique->clique_index, 
		      int,
		      temp_clique->nb_seqs_clique - 1);
		temp_clique->nb_seqs_clique--;
		
	      }
	      
	      
	    }
	    else{
	      is_finished = 1;
	    }
	  }
	}
	else{
	  is_finished = 1;
	}    
      }
    }
  }
}


/**
 *
 * \fn void maxi_clique_search(p_main_structure m)
 *
 * \brief Function used to search the maxi clique for all sets
 *
 * \param[in,out] m : the main structure containing information and that
 * will be updated
 *
 * That function will search the maxi clique search for all sets. If a
 * number of sequences has been provided by the user the maxi clique will
 * be at least that size
 *
 */

void maxi_clique_search(p_main_structure m)
{

  /*Indexes*/
  int i,j,k;
  /*Current identity*/
  float current_id;
  /*Current number of neighbors for the current query*/
  int nb_neighbors = 0;
  /*Temporary clique*/
  p_clique temp_clique;
  /*Boolean that says if the search is finished*/
  int is_finished = 0;
  /*Array of sequences for the carnac call*/
  p_sequence *seqs_carnac = NULL;
  /*Boolean that says if a sequence is connected with the current other
   one*/
  int is_connected = 0;


  /*Get all sets*/
  for(i = 0; i < m->sets_of_sequences->nb_sets; i++){
    /*Process only sets which have at least 2 query sequences*/
    if(m->sets_of_sequences->sets[i]->nb_q_intervals > 2){

      is_connected = 1;

      /*Intialize and update the neighbor array according to identity
	thresholds by doing every alignments in the set*/
      NEW(m->sets_of_sequences->sets[i]->neighbors,
	  p_neighbor,
	  m->sets_of_sequences->sets[i]->nb_q_intervals);
      
      if(OPTS_verbose){
	print_verbose_start_maxi_clique_set(i + 1, 
					    m->sets_of_sequences->sets[i]->
					    nb_q_intervals + 1);
      }

      for(j = 0; j < m->sets_of_sequences->sets[i]->nb_q_intervals; j++){

	NEW(m->sets_of_sequences->sets[i]->neighbors[j],
	    neighbor_s,
	    1);
	m->sets_of_sequences->sets[i]->neighbors[j]->indexes = NULL;
	m->sets_of_sequences->sets[i]->neighbors[j]->nb_neighbors = 0;
	nb_neighbors = 0;

	for(k = j + 1; k < m->sets_of_sequences->sets[i]->nb_q_intervals; k++){

	  current_id = aln(m->sets_of_sequences->sets[i]->q_intervals[j],
			   m->sets_of_sequences->sets[i]->q_intervals[k]);


	  /*The identity respects the identity thresholds : add the neighbor*/
	  if((current_id < OPTS_min_id) || (current_id > OPTS_max_id)){

	    is_connected = 0;

	  }

	  if(is_connected){

	    /*Check carnac if wanted*/
	    if(OPTS_carnac){
	    
	      NEW(seqs_carnac, p_sequence, 2);
	      NEW(seqs_carnac[0], sequence_s, 1);
	      NEW(seqs_carnac[0]->name, char, 5);
	      NEW(seqs_carnac[0]->bases, 
		  char,
		  strlen(m->sets_of_sequences->sets[i]->q_intervals[j]->seq) + 1);
	      strcpy(seqs_carnac[0]->name, "seq1");
	      strcpy(seqs_carnac[0]->bases,
		     m->sets_of_sequences->sets[i]->q_intervals[j]->seq);
	      seqs_carnac[0]->length = 
		strlen(m->sets_of_sequences->sets[i]->q_intervals[j]->seq);

	      NEW(seqs_carnac[1], sequence_s, 1);
	      NEW(seqs_carnac[1]->name, char, 5);
	      NEW(seqs_carnac[1]->bases, 
		  char,
		  strlen(m->sets_of_sequences->sets[i]->q_intervals[k]->seq) + 1);
	      strcpy(seqs_carnac[1]->name, "seq2");
	      strcpy(seqs_carnac[1]->bases,
		     m->sets_of_sequences->sets[i]->q_intervals[k]->seq);
	      seqs_carnac[1]->length = 
		strlen(m->sets_of_sequences->sets[i]->q_intervals[k]->seq);


	      if(!call_carnac(seqs_carnac, 2)){
		is_connected = 0;
	      }

	      DESTROY(seqs_carnac[0]->name);
	      DESTROY(seqs_carnac[0]->bases);
	      DESTROY(seqs_carnac[1]->name);
	      DESTROY(seqs_carnac[1]->bases);
	      DESTROY(seqs_carnac[0]);
	      DESTROY(seqs_carnac[1]);
	      DESTROY(seqs_carnac);



	    }

	    if(is_connected){

	      RENEW(m->sets_of_sequences->sets[i]->neighbors[j]->indexes,
		    int,
		    nb_neighbors + 1);
	      m->sets_of_sequences->sets[i]->neighbors[j]->indexes[nb_neighbors] =
		k;
	      nb_neighbors++;
	    }
	  }
	}

	m->sets_of_sequences->sets[i]->neighbors[j]->nb_neighbors = 
	  nb_neighbors;

      }
   

      /*Initialize the maxi clique for the current set*/
      NEW(m->sets_of_sequences->sets[i]->clique_set,
	  clique_s,
	  1);
    
      m->sets_of_sequences->sets[i]->clique_set->clique_index = NULL;
      m->sets_of_sequences->sets[i]->clique_set->nb_seqs_clique = 0;
      is_finished = 0;

      /*Search recursively for the maxi clique for the current set*/
      for(j = 0; !is_finished && 
	    j < m->sets_of_sequences->sets[i]->nb_q_intervals; j++){

	/*Stop condition : find a maxi clique that has enough
	  sequences if a number of sequences by set has been provided
	  otherwise continue until all sequences have been processed*/
	if(((OPTS_nb_seqs_set != -1) &&
	    (m->sets_of_sequences->sets[i]->clique_set->nb_seqs_clique < 
	     OPTS_nb_seqs_set - 1)) ||
	   ((OPTS_nb_seqs_set == -1) &&
	    (m->sets_of_sequences->sets[i]->clique_set->nb_seqs_clique !=
	     m->sets_of_sequences->sets[i]->nb_q_intervals))){

	  NEW(temp_clique,
	      clique_s,
	      1);
	  temp_clique->clique_index = NULL;
	  temp_clique->nb_seqs_clique = 0;
	  recursive_search_maxi_clique(m->sets_of_sequences->sets[i], 
				       temp_clique,
				       j);
	  DESTROY(temp_clique->clique_index);
	  DESTROY(temp_clique);

	}
	else{
	  is_finished = 1;
	}
      
      }

      if(OPTS_verbose){
      
	print_verbose_end_maxi_clique_set(m->sets_of_sequences->
					  sets[i]->clique_set->nb_seqs_clique + 1);
	
      }

    }

  }
}

