/*
 * This file is part of CGseqcore.
 *
 *  CGseqcore is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  CGseqcore is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with CGseqcore.  If not, see <http://www.gnu.org/licenses/>.
 */


/**
 *
 * \file io.c
 *
 * \author Benjamin Grenier-Boley <benjamin.grenier-boley@inria.fr>
 *
 * \brief File that defines the IN/OUT definitions and functions 
 *
 * \version 1.0
 *
 * \date July 2009
 *
 *
 * This file will define all I/O functions used in the software
 *
 *
 */


/*Intern inclusions*/
#include "io.h"
#include "macros.h"
#include "options.h"
#include "sequence.h"



/**
 *
 * \fn void read_file(FILE *fd, char **content)
 *
 * \brief Function used to read a file and get its content
 *
 * \param[in] fd : the file descriptor of the reading file
 * \param[out] content : the string representation of the file content
 *
 * This function will be used to read a file and return its content or NULL
 * if the file is empty
 *
 */


void read_file(FILE* fd, char **content) 
{

  /*Length of the file*/
  long length;
 
  /*Go to the end of the file to know the length of the file*/
  fseek(fd,0,SEEK_END);
  length = ftell(fd);

  /*Allocate the content and read the file*/
  fseek(fd,0,SEEK_SET);

  /*If the file is not empty read it*/
  if(length != 0){
    NEW(*content, char, length);
    fread(*content, sizeof(char), length - 1, fd);
  }
  else{
    *content = NULL;
  }
   
}


/**
 *
 * \fn int write_GFF3_star(p_main_structure m)
 *
 * \brief Function used to write GFF3 files for the star mode
 *
 * \param[in] m : the main structure containing all info needed
 *
 * \return the number of files written
 *
 * This function will produce as many GFF3 as the number of sets of 
 * sequences discovered for the star mode
 *
 */


int write_GFF3_star(p_main_structure m)
{

  /*Indexes*/
  int i,j,k;
  /*Current filename*/
  char *filename = NULL;
  /*Size in char of the number of the filename*/
  int size;
  /*Current file descriptor*/
  FILE *fd;
  /*Temporary query index*/
  int temp_index = -1;
  /*Temporary sequence index in the set of sequence*/
  int index_set;
  /*Number of sequences by set wanted*/
  int nb_seqs_set = 0;
  /*Number of current set processed*/
  int nb_set = 0;
  /*Indexes of different organism in case the OPTS_min_org_set option is
   used*/
  int_array *indexes_diff_org = NULL;
  /*Number of written sequences in the set (used when OPTS_min_org_set
   is used*/
  int nb_written = 0;
  /*Value of the OPTS_min_species_set or the OPTS_min_org_set value*/
  int value_set = 0;
  /*Current sequence length*/
  int length_seq = 0;
  /*Current length of the sequence to write*/
  int length_seq_to_write = 0;
  /*Current sequence to write*/
  char *seq_to_write = NULL;


  /*Check the number of sequences by set wanted : if -1 all seqs wanted*/
  if(OPTS_nb_seqs_set != -1){
    nb_seqs_set = OPTS_nb_seqs_set - 1;
  }
  else{
    nb_seqs_set = 100000;
  }

  if(OPTS_min_org_set != 0){
    value_set = OPTS_min_org_set;
  }
  if(OPTS_min_species_set != 0){
    value_set = OPTS_min_species_set;
  }


  /*Check if a constraint according to the minimum different organism per set
    has been defined*/
  if((OPTS_min_org_set != 0) || (OPTS_min_species_set != 0)){
     /*Get all sets*/
    for(i = 0; i < m->sets_of_sequences->nb_sets; i++){
      
      nb_written = 0;
      
      /*Check if there are enough sequences*/
      if((m->sets_of_sequences->sets[i]->nb_q_intervals >= value_set)&&
	 (((OPTS_nb_seqs_set_min != -1) && 
	   (m->sets_of_sequences->sets[i]->nb_q_intervals >= 
	    (OPTS_nb_seqs_set_min - 1))) || (OPTS_nb_seqs_set_min == -1)) &&
	 (((OPTS_nb_seqs_set_max != -1) &&
	   (m->sets_of_sequences->sets[i]->nb_q_intervals <= 
	    (OPTS_nb_seqs_set_max - 1))) || (OPTS_nb_seqs_set_max == -1))){
	
	/*Get indexes of different organisms present in the current set of
	  sequences if needed*/	  
	NEW(indexes_diff_org, int_array, 1);
	indexes_diff_org->integers = NULL;
	indexes_diff_org->nb_int = 0;
	
	get_indexes_diff_org(m,i,indexes_diff_org);
	    
	/*The constraint is respected*/
	if(indexes_diff_org->nb_int >= value_set){

	  nb_set++;
	  get_size_number(nb_set, &size);

	  if(OPTS_keep_too_close && m->sets_of_sequences->sets[i]->is_too_close){
	    NEW(filename,
		char,
		strlen(OPTS_output_dir_too_close) + size + strlen(".gff") + 1);
	    sprintf(filename,
		    "%s%i%s",
		    OPTS_output_dir_too_close,
		    nb_set,
		    ".gff");

	  }
	  else{

	    NEW(filename,
		char,
		strlen(OPTS_output_dir) + size + strlen(".gff") + 1);
	    sprintf(filename,
		    "%s%i%s",
		    OPTS_output_dir,
		    nb_set,
		    ".gff");
	  }

	  fd = fopen(filename, "w");
	  if(fd == NULL){
	    ERROR_ "Impossible to create the file %s\n\n",filename _ERROR;
	    
	  }
	  else{

	    /*Alignment part*/

	    /*Writing first lines*/
	    fprintf(fd,
		    "%s%s",
		    "##gff-version   3\n",
		    "##sequence-region   ");
	    /*Writing reference name and size*/
	    fprintf(fd,
		    "%s  %i  %i\n",
		    m->array_of_seqs->seqs[0]->name,
		    1,
		    m->array_of_seqs->seqs[0]->length);
	    
	    /*Writing reference*/
	    fprintf(fd,
		  "%s\t%s\t%s\t%i\t%i\t%s\t%c\t%s%s\n",
		  m->array_of_seqs->seqs[0]->name,
		  PACKAGE_NAME,
		  "ncRNA",
		  m->sets_of_sequences->sets[i]->ref_interval->pos_start + 1,
		  m->sets_of_sequences->sets[i]->ref_interval->pos_end + 1,
		  ".",
		  m->sets_of_sequences->sets[i]->ref_interval->strand,
		  ".\tID=1|",
		  m->array_of_seqs->seqs[0]->name);
	    
	    /*Writing all query sequences from the indexes_diff_org_array*/
	    for(j = 0; j < indexes_diff_org->nb_int; j++){
	      
	      index_set = indexes_diff_org->integers[j];
	      temp_index = 
		m->sets_of_sequences->sets[i]->q_intervals[index_set]->index;
	      
	      fprintf(fd,
		      "%s\t%s\t%s\t%i\t%i\t%f\t%c\t%s%i%s%s\n",
		      m->array_of_seqs->seqs[temp_index]->name,
		      PACKAGE_NAME,
		      "ncRNA",
		      m->sets_of_sequences->sets[i]->q_intervals[index_set]->pos_start + 1,
		      m->sets_of_sequences->sets[i]->q_intervals[index_set]->pos_end + 1,
		      m->sets_of_sequences->sets[i]->q_intervals[index_set]->id_ref,
		      m->sets_of_sequences->sets[i]->q_intervals[index_set]->strand,
		      ".\tID=",
		      j + 2,
		      "|",
		      m->array_of_seqs->seqs[temp_index]->name);

	    }
	    
	    /*Writing other sequences to complete the set*/
	    for(j = 0; (j < m->sets_of_sequences->sets[i]->nb_q_intervals) &&
		  (nb_written < nb_seqs_set - indexes_diff_org->nb_int); j++){
	      
	      /*No writing if it has already been written*/
	      if(!(is_present_int_array(j,
					indexes_diff_org->integers,
					indexes_diff_org->nb_int))){
		
		temp_index = 
		  m->sets_of_sequences->sets[i]->q_intervals[j]->index;
		
		fprintf(fd,
		      "%s\t%s\t%s\t%i\t%i\t%f\t%c\t%s%i%s%s\n",
		      m->array_of_seqs->seqs[temp_index]->name,
		      PACKAGE_NAME,
		      "ncRNA",
		      m->sets_of_sequences->sets[i]->q_intervals[j]->pos_start + 1,
		      m->sets_of_sequences->sets[i]->q_intervals[j]->pos_end + 1,
		      m->sets_of_sequences->sets[i]->q_intervals[j]->id_ref,
		      m->sets_of_sequences->sets[i]->q_intervals[j]->strand,
		      ".\tID=",
		      nb_written+2+indexes_diff_org->nb_int,
		      "|",
			m->array_of_seqs->seqs[temp_index]->name);

		

		nb_written++;
	      }

	    }


	    /*Write fasta sequences*/
	    fprintf(fd, "%s", "##FASTA\n");
	    
	    nb_written = 0;

	    /*Write reference sequence*/
	    fprintf(fd,
		    ">1|%s %i %i %c",
		    m->array_of_seqs->seqs[0]->name,
		    m->sets_of_sequences->sets[i]->ref_interval->pos_start + 1,
		    m->sets_of_sequences->sets[i]->ref_interval->pos_end + 1,
		    m->sets_of_sequences->sets[i]->ref_interval->strand);

	    length_seq = strlen(m->sets_of_sequences->sets[i]->ref_seq);

	    for(k = 0; k < length_seq; k++){
	      if((k % 60) == 0){
		RENEW(seq_to_write, char, length_seq_to_write + 2);
		seq_to_write[length_seq_to_write] = '\n';
		seq_to_write[length_seq_to_write + 1] = 
		  m->sets_of_sequences->sets[i]->ref_seq[k];
		length_seq_to_write += 2;
	      }
	      else{
		RENEW(seq_to_write, char, length_seq_to_write + 1);
		seq_to_write[length_seq_to_write] = 
		  m->sets_of_sequences->sets[i]->ref_seq[k];
		length_seq_to_write ++;
	      }
	    }
	    RENEW(seq_to_write, char, length_seq_to_write + 2);
	    seq_to_write[length_seq_to_write] = '\n';
	    seq_to_write[length_seq_to_write + 1] =  '\0';
	    fprintf(fd,"%s",seq_to_write);
	    DESTROY(seq_to_write);
	    length_seq_to_write = 0;


	    /*Writing all query sequences from the indexes_diff_org_array*/
	    for(j = 0; j < indexes_diff_org->nb_int; j++){
	      
	      index_set = indexes_diff_org->integers[j];
	      temp_index = 
		m->sets_of_sequences->sets[i]->q_intervals[index_set]->index;
	      
	      fprintf(fd,
		      ">%i%s%s %i %i %c",
		      j+2,
		      "|",
		      m->array_of_seqs->seqs[temp_index]->name,
		      m->sets_of_sequences->sets[i]->q_intervals[index_set]->pos_start + 1,
		      m->sets_of_sequences->sets[i]->q_intervals[index_set]->pos_end + 1,
		      m->sets_of_sequences->sets[i]->q_intervals[index_set]->strand);

	      length_seq = 
		strlen(m->sets_of_sequences->sets[i]->q_intervals[index_set]->seq);

	      for(k = 0; k < length_seq; k++){
		if((k % 60) == 0){
		  RENEW(seq_to_write, char, length_seq_to_write + 2);
		  seq_to_write[length_seq_to_write] = '\n';
		  seq_to_write[length_seq_to_write + 1] = 
		    m->sets_of_sequences->sets[i]->q_intervals[index_set]->seq[k];
		  length_seq_to_write += 2;
		}
		else{
		  RENEW(seq_to_write, char, length_seq_to_write + 1);
		  seq_to_write[length_seq_to_write] = 
		    m->sets_of_sequences->sets[i]->q_intervals[index_set]->seq[k];
		  length_seq_to_write ++;
		}
	      }
	      RENEW(seq_to_write, char, length_seq_to_write + 2);
	      seq_to_write[length_seq_to_write] = '\n';
	      seq_to_write[length_seq_to_write + 1] =  '\0';
	      fprintf(fd,"%s",seq_to_write);
	      DESTROY(seq_to_write);
	      length_seq_to_write = 0;


	    }
	    
	    /*Writing other sequences to complete the set*/
	    for(j = 0; (j < m->sets_of_sequences->sets[i]->nb_q_intervals) &&
		  (nb_written < nb_seqs_set - indexes_diff_org->nb_int); j++){
	      
	      /*No writing if it has already been written*/
	      if(!(is_present_int_array(j,
					indexes_diff_org->integers,
					indexes_diff_org->nb_int))){
		
		temp_index = 
		  m->sets_of_sequences->sets[i]->q_intervals[j]->index;
		
		fprintf(fd,
			">%i%s%s %i %i %c",
			nb_written+2+indexes_diff_org->nb_int,
			"|",
			m->array_of_seqs->seqs[temp_index]->name,
			m->sets_of_sequences->sets[i]->q_intervals[j]->pos_start + 1,
			m->sets_of_sequences->sets[i]->q_intervals[j]->pos_end + 1,
			m->sets_of_sequences->sets[i]->q_intervals[j]->strand);

		length_seq = 
		  strlen(m->sets_of_sequences->sets[i]->q_intervals[j]->seq);
		
		for(k = 0; k < length_seq; k++){
		  if((k % 60) == 0){
		    RENEW(seq_to_write, char, length_seq_to_write + 2);
		    seq_to_write[length_seq_to_write] = '\n';
		    seq_to_write[length_seq_to_write + 1] = 
		      m->sets_of_sequences->sets[i]->q_intervals[j]->seq[k];
		    length_seq_to_write += 2;
		  }
		  else{
		    RENEW(seq_to_write, char, length_seq_to_write + 1);
		    seq_to_write[length_seq_to_write] = 
		      m->sets_of_sequences->sets[i]->q_intervals[j]->seq[k];
		    length_seq_to_write ++;
		  }
		}
		RENEW(seq_to_write, char, length_seq_to_write + 2);
		seq_to_write[length_seq_to_write] = '\n';
		seq_to_write[length_seq_to_write + 1] =  '\0';
		fprintf(fd,"%s",seq_to_write);
		DESTROY(seq_to_write);
		length_seq_to_write = 0;

		nb_written++;
	      }

	    }

	    fclose(fd); 	    
	  }

	  DESTROY(filename);
	  	  
	}
	DESTROY(indexes_diff_org->integers);
	DESTROY(indexes_diff_org);
      }
      
    }   


  }
  /*No different organisms constraint*/
  else{

    /*Get all sets*/
    for(i = 0; i < m->sets_of_sequences->nb_sets; i++){

      /*If no queries : no output*/
      if((m->sets_of_sequences->sets[i]->nb_q_intervals != 0)&&
	 (((OPTS_nb_seqs_set_min != -1) && 
	   (m->sets_of_sequences->sets[i]->nb_q_intervals >= 
	    (OPTS_nb_seqs_set_min - 1))) || (OPTS_nb_seqs_set_min == -1)) &&
	 (((OPTS_nb_seqs_set_max != -1) &&
	   (m->sets_of_sequences->sets[i]->nb_q_intervals <= 
	    (OPTS_nb_seqs_set_max - 1))) || (OPTS_nb_seqs_set_max == -1))){

	nb_set++;
	get_size_number(nb_set, &size);

	if((OPTS_keep_too_close != -1) && m->sets_of_sequences->sets[i]->is_too_close){
	  NEW(filename,
	      char,
	      strlen(OPTS_output_dir_too_close) + size + strlen(".gff") + 1);
	  sprintf(filename,
		  "%s%i%s",
		  OPTS_output_dir_too_close,
		  nb_set,
		  ".gff");
	  
	}
	else{

	  NEW(filename,
	      char,
	      strlen(OPTS_output_dir) + size + strlen(".gff") + 1);
	  sprintf(filename,
		  "%s%i%s",
		  OPTS_output_dir,
		  nb_set,
		  ".gff");
	}

	fd = fopen(filename, "w");
	if(fd == NULL){
	  ERROR_ "Impossible to create the file %s\n\n",filename _ERROR;

	}
	else{

	  /*Alignment part*/

	  /*Writing first lines*/
	  fprintf(fd,
		  "%s%s",
		  "##gff-version   3\n",
		  "##sequence-region   ");
	  /*Writing reference name and size*/
	  fprintf(fd,
		  "%s  %i  %i\n",
		  m->array_of_seqs->seqs[0]->name,
		  1,
		  m->array_of_seqs->seqs[0]->length);

	  /*Writing reference*/
	  fprintf(fd,
		  "%s\t%s\t%s\t%i\t%i\t%s\t%c\t%s%s\n",
		  m->array_of_seqs->seqs[0]->name,
		  PACKAGE_NAME,
		  "ncRNA",
		  m->sets_of_sequences->sets[i]->ref_interval->pos_start + 1,
		  m->sets_of_sequences->sets[i]->ref_interval->pos_end + 1,
		  ".",
		  m->sets_of_sequences->sets[i]->ref_interval->strand,
		  ".\tID=1|",
		  m->array_of_seqs->seqs[0]->name);

	  /*Writing all queries alignment info (take into account the number
	    of sequences wanted by set)*/
	  for(j = 0; (j < m->sets_of_sequences->sets[i]->nb_q_intervals) &&
		(j < nb_seqs_set); j++){

	    temp_index = m->sets_of_sequences->sets[i]->q_intervals[j]->index;
	  
	    fprintf(fd,
		    "%s\t%s\t%s\t%i\t%i\t%f\t%c\t%s%i%s%s\n",
		    m->array_of_seqs->seqs[temp_index]->name,
		    PACKAGE_NAME,
		    "ncRNA",
		    m->sets_of_sequences->sets[i]->q_intervals[j]->pos_start + 1,
		    m->sets_of_sequences->sets[i]->q_intervals[j]->pos_end + 1,
		    m->sets_of_sequences->sets[i]->q_intervals[j]->id_ref,
		    m->sets_of_sequences->sets[i]->q_intervals[j]->strand,
		    ".\tID=",
		    j + 2,
		    "|",
		    m->array_of_seqs->seqs[temp_index]->name);
	  }
	
	  /*Write fasta sequences*/
	  fprintf(fd, "%s", "##FASTA\n");

	  /*Write reference sequence*/
	  fprintf(fd,
		  ">1|%s %i %i %c",
		  m->array_of_seqs->seqs[0]->name,
		  m->sets_of_sequences->sets[i]->ref_interval->pos_start + 1,
		  m->sets_of_sequences->sets[i]->ref_interval->pos_end + 1,
		  m->sets_of_sequences->sets[i]->ref_interval->strand);

	  length_seq = strlen(m->sets_of_sequences->sets[i]->ref_seq);
	    
	  for(k = 0; k < length_seq; k++){
	    if((k % 60) == 0){
	      RENEW(seq_to_write, char, length_seq_to_write + 2);
	      seq_to_write[length_seq_to_write] = '\n';
	      seq_to_write[length_seq_to_write + 1] = 
		m->sets_of_sequences->sets[i]->ref_seq[k];
	      length_seq_to_write += 2;
	    }
	    else{
	      RENEW(seq_to_write, char, length_seq_to_write + 1);
	      seq_to_write[length_seq_to_write] = 
		m->sets_of_sequences->sets[i]->ref_seq[k];
	      length_seq_to_write ++;
	    }
	  }
	  RENEW(seq_to_write, char, length_seq_to_write + 2);
	  seq_to_write[length_seq_to_write] = '\n';
	  seq_to_write[length_seq_to_write + 1] =  '\0';
	  fprintf(fd,"%s",seq_to_write);
	  DESTROY(seq_to_write);
	  length_seq_to_write = 0;


	  /*Writing all query sequences (take into account the number of 
	    sequences by set*/
	  for(j = 0; (j < m->sets_of_sequences->sets[i]->nb_q_intervals) &&
		(j < nb_seqs_set); j++){
	  
	    temp_index = m->sets_of_sequences->sets[i]->q_intervals[j]->index;
	  
	    fprintf(fd,
		    ">%i%s%s %i %i %c",
		    j + 2,
		    "|",
		    m->array_of_seqs->seqs[temp_index]->name,
		    m->sets_of_sequences->sets[i]->q_intervals[j]->pos_start + 1,
		    m->sets_of_sequences->sets[i]->q_intervals[j]->pos_end + 1,
		    m->sets_of_sequences->sets[i]->q_intervals[j]->strand);


	    length_seq = 
		  strlen(m->sets_of_sequences->sets[i]->q_intervals[j]->seq);
		
	    for(k = 0; k < length_seq; k++){
	      if((k % 60) == 0){
		RENEW(seq_to_write, char, length_seq_to_write + 2);
		seq_to_write[length_seq_to_write] = '\n';
		seq_to_write[length_seq_to_write + 1] = 
		  m->sets_of_sequences->sets[i]->q_intervals[j]->seq[k];
		length_seq_to_write += 2;
	      }
	      else{
		RENEW(seq_to_write, char, length_seq_to_write + 1);
		seq_to_write[length_seq_to_write] = 
		  m->sets_of_sequences->sets[i]->q_intervals[j]->seq[k];
		length_seq_to_write ++;
	      }
	    }
	    RENEW(seq_to_write, char, length_seq_to_write + 2);
	    seq_to_write[length_seq_to_write] = '\n';
	    seq_to_write[length_seq_to_write + 1] =  '\0';
	    fprintf(fd,"%s",seq_to_write);
	    DESTROY(seq_to_write);
	    length_seq_to_write = 0;

	  }
	  /*Close file*/
	  fclose(fd);
	}

	DESTROY(filename);
     
      
      }
    
    
    }
  
  }
  return nb_set;

}


/**
 *
 * \fn int write_GFF3_clique(p_main_structure m)
 *
 * \brief Function used to write GFF3 files for the clique modes
 *
 * \param[in] m : the main structure containing all info needed
 *
 * \return the number of files written
 *
 * This function will produce as many GFF3 as the number of sets of 
 * sequences discovered for the clique modes
 *
 */


int write_GFF3_clique(p_main_structure m)
{

  /*Indexes*/
  int i,j,k;
  /*Current filename*/
  char *filename = NULL;
  /*Size in char of the number of the filename*/
  int size;
  /*Current file descriptor*/
  FILE *fd;
  /*Temporary query index*/
  int temp_index = -1;
  /*Temporary index in the clique*/
  int index_clique;
  /*Temporary query index in the sequence array*/
  int temp_index_seq = -1;
  /*Temporary query index in the query interval array*/
  int temp_index_clique = -1;
  /*Temporary sequence index in the set of sequence*/
  int index_set;
  /*Number of sequences by set wanted*/
  int nb_seqs_set = 0;
  /*Number of current set processed*/
  int nb_set = 0;
  /*Indexes of different organism in case the OPTS_min_org_set option is
    used*/
  int_array *indexes_diff_org = NULL;
  /*Number of written sequences in the set (used when OPTS_min_org_set
    is used*/
  int nb_written = 0;
  /*Value of the OPTS_min_species_set or the OPTS_min_org_set value*/
  int value_set = 0;
  /*Current sequence length*/
  int length_seq = 0;
  /*Current length of the sequence to write*/
  int length_seq_to_write = 0;
  /*Current sequence to write*/
  char *seq_to_write = NULL;

  /*Check the number of sequences by set wanted : if -1 all seqs wanted*/
  if(OPTS_nb_seqs_set != -1){
    nb_seqs_set = OPTS_nb_seqs_set - 1;
  }
  else{
    nb_seqs_set = 100000;
  }

  if(OPTS_min_org_set != 0){
    value_set = OPTS_min_org_set;
  }
  if(OPTS_min_species_set != 0){
    value_set = OPTS_min_species_set;
  }


  /*Check if a constraint according to the minimum different organism per set
    has been defined or the minimum different species*/
  if((OPTS_min_org_set != 0) || (OPTS_min_species_set != 0)){

     /*Get all sets*/
    for(i = 0; i < m->sets_of_sequences->nb_sets; i++){

      nb_written = 0;
      
      /*Check if there are enough sequences*/
      if ((m->sets_of_sequences->sets[i]->clique_set != NULL) &&
	  (m->sets_of_sequences->sets[i]->clique_set->nb_seqs_clique != 1) &&
	  (m->sets_of_sequences->sets[i]->clique_set->nb_seqs_clique >= 
	   value_set) &&
	  (((OPTS_nb_seqs_set_min != -1) && 
	    (m->sets_of_sequences->sets[i]->clique_set->nb_seqs_clique >= 
	     (OPTS_nb_seqs_set_min - 1))) || (OPTS_nb_seqs_set_min == -1)) &&
	  (((OPTS_nb_seqs_set_max != -1) &&
	    (m->sets_of_sequences->sets[i]->clique_set->nb_seqs_clique <= 
	     (OPTS_nb_seqs_set_max - 1))) || (OPTS_nb_seqs_set_max == -1))){
	
	/*Get indexes of different organisms present in the current set of
	  sequences if needed*/	  
	NEW(indexes_diff_org, int_array, 1);
	indexes_diff_org->integers = NULL;
	indexes_diff_org->nb_int = 0;
	
	get_indexes_diff_org(m,i,indexes_diff_org);
	    
	/*The constraint is respected*/
	if(indexes_diff_org->nb_int >= value_set){

	  nb_set++;

	  /*Open the new file*/
	  get_size_number(nb_set, &size);

	  if((OPTS_keep_too_close != -1) && m->sets_of_sequences->sets[i]->is_too_close){
	    NEW(filename,
		char,
		strlen(OPTS_output_dir_too_close) + size + strlen(".fna") + 1);
	    sprintf(filename,
		    "%s%i%s",
		    OPTS_output_dir_too_close,
		    nb_set,
		    ".gff");
	  }
	  else{
	    NEW(filename,
		char,
		strlen(OPTS_output_dir) + size + strlen(".fna") + 1);
	    sprintf(filename,
		    "%s%i%s",
		    OPTS_output_dir,
		    nb_set,
		    ".gff");
	    
	  }

	  fd = fopen(filename, "w");
	  if(fd == NULL){
	    ERROR_ "Impossible to create the file %s\n\n",filename _ERROR;
	    
	  }
	  else{

	    /*Alignment part*/
	    
	    /*Writing first lines*/
	    fprintf(fd,
		    "%s%s",
		    "##gff-version   3\n",
		    "##sequence-region   ");
	    /*Writing reference name and size*/
	    fprintf(fd,
		    "%s  %i  %i\n",
		    m->array_of_seqs->seqs[0]->name,
		    1,
		    m->array_of_seqs->seqs[0]->length);
	    
	    /*Writing reference*/
	    fprintf(fd,
		    "%s\t%s\t%s\t%i\t%i\t%s\t%c\t%s%s\n",
		    m->array_of_seqs->seqs[0]->name,
		    PACKAGE_NAME,
		    "ncRNA",
		    m->sets_of_sequences->sets[i]->ref_interval->pos_start + 1,
		    m->sets_of_sequences->sets[i]->ref_interval->pos_end + 1,
		    ".",
		    m->sets_of_sequences->sets[i]->ref_interval->strand,
		    ".\tID=1|",
		    m->array_of_seqs->seqs[0]->name);


	    /*Writing all queries from the indexes_diff_org_array*/
	    for(j = 0; j < indexes_diff_org->nb_int; j++){
	    
	      index_set = indexes_diff_org->integers[j];
	      index_clique = 
		m->sets_of_sequences->sets[i]->clique_set->clique_index[index_set];
	      temp_index = 
		m->sets_of_sequences->sets[i]->q_intervals[index_clique]->index;

	      
	      fprintf(fd,
		    "%s\t%s\t%s\t%i\t%i\t%f\t%c\t%s%i%s%s\n",
		    m->array_of_seqs->seqs[temp_index]->name,
		    PACKAGE_NAME,
		    "ncRNA",
		    m->sets_of_sequences->sets[i]->q_intervals[index_clique]->pos_start + 1,
		    m->sets_of_sequences->sets[i]->q_intervals[index_clique]->pos_end + 1,
		    m->sets_of_sequences->sets[i]->q_intervals[index_clique]->id_ref,
		    m->sets_of_sequences->sets[i]->q_intervals[index_clique]->strand,
		    ".\tID=",
		    j + 2,
		    "|",
		    m->array_of_seqs->seqs[temp_index]->name);

	    }
	    
	    /*Writing other sequences to complete the set*/
	    for(j = 0; (j < m->sets_of_sequences->sets[i]->clique_set->nb_seqs_clique) &&
		  (nb_written < nb_seqs_set - indexes_diff_org->nb_int); j++){
	    
	      /*No writing if it has already been written*/
	      if(!(is_present_int_array(j,
					indexes_diff_org->integers,
					indexes_diff_org->nb_int))){


		index_clique = 
		  m->sets_of_sequences->sets[i]->clique_set->clique_index[j];
		temp_index = 
		  m->sets_of_sequences->sets[i]->q_intervals[index_clique]->index;

		fprintf(fd,
			"%s\t%s\t%s\t%i\t%i\t%f\t%c\t%s%i%s%s\n",
			m->array_of_seqs->seqs[temp_index]->name,
			PACKAGE_NAME,
			"ncRNA",
			m->sets_of_sequences->sets[i]->q_intervals[index_clique]->pos_start + 1,
			m->sets_of_sequences->sets[i]->q_intervals[index_clique]->pos_end + 1,
			m->sets_of_sequences->sets[i]->q_intervals[index_clique]->id_ref,
			m->sets_of_sequences->sets[i]->q_intervals[index_clique]->strand,
			".\tID=",
			nb_written+2+indexes_diff_org->nb_int,
			"|",
			m->array_of_seqs->seqs[temp_index]->name);
	      }
	    }

	    /*Write fasta sequences*/
	    fprintf(fd, "%s", "##FASTA\n");
	 
	    /*Write reference sequence*/
	    fprintf(fd,
		  ">1|%s %i %i %c",
		  m->array_of_seqs->seqs[0]->name,
		  m->sets_of_sequences->sets[i]->ref_interval->pos_start + 1,
		  m->sets_of_sequences->sets[i]->ref_interval->pos_end + 1,
		  m->sets_of_sequences->sets[i]->ref_interval->strand);

	    length_seq = strlen(m->sets_of_sequences->sets[i]->ref_seq);

	    for(k = 0; k < length_seq; k++){
	      if((k % 60) == 0){
		RENEW(seq_to_write, char, length_seq_to_write + 2);
		seq_to_write[length_seq_to_write] = '\n';
		seq_to_write[length_seq_to_write + 1] = 
		  m->sets_of_sequences->sets[i]->ref_seq[k];
		length_seq_to_write += 2;
	      }
	      else{
		RENEW(seq_to_write, char, length_seq_to_write + 1);
		seq_to_write[length_seq_to_write] = 
		  m->sets_of_sequences->sets[i]->ref_seq[k];
		length_seq_to_write ++;
	      }
	    }
	    RENEW(seq_to_write, char, length_seq_to_write + 2);
	    seq_to_write[length_seq_to_write] = '\n';
	    seq_to_write[length_seq_to_write + 1] =  '\0';
	    fprintf(fd,"%s",seq_to_write);
	    DESTROY(seq_to_write);
	    length_seq_to_write = 0;


   	
	    /*Writing all queries from the indexes_diff_org_array*/
	    for(j = 0; j < indexes_diff_org->nb_int; j++){
	      
	      index_set = indexes_diff_org->integers[j];
	      index_clique = 
		m->sets_of_sequences->sets[i]->clique_set->clique_index[index_set];
	      temp_index = 
		m->sets_of_sequences->sets[i]->q_intervals[index_clique]->index;

	      fprintf(fd,
		      ">%i%s%s %i %i %c",
		      j + 2,
		      "|",
		      m->array_of_seqs->seqs[temp_index]->name,
		      m->sets_of_sequences->sets[i]->q_intervals[index_clique]->pos_start + 1,
		      m->sets_of_sequences->sets[i]->q_intervals[index_clique]->pos_end + 1,
		      m->sets_of_sequences->sets[i]->q_intervals[index_clique]->strand);

	      length_seq = 
		strlen(m->sets_of_sequences->sets[i]->q_intervals[index_clique]->seq);

	      for(k = 0; k < length_seq; k++){
		if((k % 60) == 0){
		  RENEW(seq_to_write, char, length_seq_to_write + 2);
		  seq_to_write[length_seq_to_write] = '\n';
		  seq_to_write[length_seq_to_write + 1] = 
		    m->sets_of_sequences->sets[i]->q_intervals[index_clique]->seq[k];
		  length_seq_to_write += 2;
		}
		else{
		  RENEW(seq_to_write, char, length_seq_to_write + 1);
		  seq_to_write[length_seq_to_write] = 
		    m->sets_of_sequences->sets[i]->q_intervals[index_clique]->seq[k];
		  length_seq_to_write ++;
		}
	      }
	      RENEW(seq_to_write, char, length_seq_to_write + 2);
	      seq_to_write[length_seq_to_write] = '\n';
	      seq_to_write[length_seq_to_write + 1] =  '\0';
	      fprintf(fd,"%s",seq_to_write);
	      DESTROY(seq_to_write);
	      length_seq_to_write = 0;


	    }


	    /*Writing other sequences to complete the set*/
	    for(j = 0; (j < m->sets_of_sequences->sets[i]->clique_set->nb_seqs_clique) &&
		  (nb_written < nb_seqs_set - indexes_diff_org->nb_int); j++){
	    
	      /*No writing if it has already been written*/
	      if(!(is_present_int_array(j,
					indexes_diff_org->integers,
					indexes_diff_org->nb_int))){


		index_clique = 
		  m->sets_of_sequences->sets[i]->clique_set->clique_index[j];
		temp_index = 
		  m->sets_of_sequences->sets[i]->q_intervals[index_clique]->index;

	    
		fprintf(fd,
		      ">%i%s%s %i %i %c",
		      nb_written+2+indexes_diff_org->nb_int,
		      "|",
		      m->array_of_seqs->seqs[temp_index]->name,
		      m->sets_of_sequences->sets[i]->q_intervals[index_clique]->pos_start + 1,
		      m->sets_of_sequences->sets[i]->q_intervals[index_clique]->pos_end + 1,
		      m->sets_of_sequences->sets[i]->q_intervals[index_clique]->strand);

		length_seq = 
		  strlen(m->sets_of_sequences->sets[i]->q_intervals[index_clique]->seq);
		
		for(k = 0; k < length_seq; k++){
		  if((k % 60) == 0){
		    RENEW(seq_to_write, char, length_seq_to_write + 2);
		    seq_to_write[length_seq_to_write] = '\n';
		    seq_to_write[length_seq_to_write + 1] = 
		      m->sets_of_sequences->sets[i]->q_intervals[index_clique]->seq[k];
		    length_seq_to_write += 2;
		  }
		  else{
		    RENEW(seq_to_write, char, length_seq_to_write + 1);
		    seq_to_write[length_seq_to_write] = 
		      m->sets_of_sequences->sets[i]->q_intervals[index_clique]->seq[k];
		    length_seq_to_write ++;
		  }
		}
		RENEW(seq_to_write, char, length_seq_to_write + 2);
		seq_to_write[length_seq_to_write] = '\n';
		seq_to_write[length_seq_to_write + 1] =  '\0';
		fprintf(fd,"%s",seq_to_write);
		DESTROY(seq_to_write);
		length_seq_to_write = 0;

		nb_written++;
	      }

	    }

	    fclose(fd);
	  }
	  DESTROY(filename);	   

	}

	DESTROY(indexes_diff_org->integers);
	DESTROY(indexes_diff_org); 	

      }
    }
  }

  
  /*No different organisms constraint*/
  else{

    /*Get all sets */
    for(i = 0; i < m->sets_of_sequences->nb_sets; i++){

    
      /*If no queries : no output or if only one query : no clique
       has been found*/
      if((m->sets_of_sequences->sets[i]->clique_set != NULL) &&
	 (m->sets_of_sequences->sets[i]->clique_set->nb_seqs_clique != 1) &&
	 (((OPTS_nb_seqs_set_min != -1) && 
	    (m->sets_of_sequences->sets[i]->clique_set->nb_seqs_clique >= 
	     (OPTS_nb_seqs_set_min - 1))) || (OPTS_nb_seqs_set_min == -1)) &&
	  (((OPTS_nb_seqs_set_max != -1) &&
	    (m->sets_of_sequences->sets[i]->clique_set->nb_seqs_clique <= 
	     (OPTS_nb_seqs_set_max - 1))) || (OPTS_nb_seqs_set_max == -1))){


	nb_set++;

	/*Open the new file*/
	get_size_number(nb_set, &size);

	if((OPTS_keep_too_close != -1) && m->sets_of_sequences->sets[i]->is_too_close){
	    NEW(filename,
		char,
		strlen(OPTS_output_dir_too_close) + size + strlen(".fna") + 1);
	    sprintf(filename,
		    "%s%i%s",
		    OPTS_output_dir_too_close,
		    nb_set,
		    ".gff");
	}
	else{
	  NEW(filename,
	      char,
	      strlen(OPTS_output_dir) + size + strlen(".fna") + 1);
	  sprintf(filename,
		  "%s%i%s",
		  OPTS_output_dir,
		  nb_set,
		  ".gff");
	  
	}
	
	fd = fopen(filename, "w");
	if(fd == NULL){
	  ERROR_ "Impossible to create the file %s\n\n",filename _ERROR;

	}
	else{

	  /*Alignment part*/

	  /*Writing first lines*/
	  fprintf(fd,
		  "%s%s",
		  "##gff-version   3\n",
		  "##sequence-region   ");
	  /*Writing reference name and size*/
	  fprintf(fd,
		  "%s  %i  %i\n",
		  m->array_of_seqs->seqs[0]->name,
		  1,
		  m->array_of_seqs->seqs[0]->length);

	  /*Writing reference*/
	  fprintf(fd,
		  "%s\t%s\t%s\t%i\t%i\t%s\t%c\t%s%s\n",
		  m->array_of_seqs->seqs[0]->name,
		  PACKAGE_NAME,
		  "ncRNA",
		  m->sets_of_sequences->sets[i]->ref_interval->pos_start + 1,
		  m->sets_of_sequences->sets[i]->ref_interval->pos_end + 1,
		  ".",
		  m->sets_of_sequences->sets[i]->ref_interval->strand,
		  ".\tID=1|",
		  m->array_of_seqs->seqs[0]->name);


	  /*Writing all queries alignment info (take into account the number
	    of sequences wanted by set)*/
	  for(j = 0; (j < m->sets_of_sequences->sets[i]->clique_set->nb_seqs_clique) &&
		(j < nb_seqs_set); j++){

	    temp_index_clique = 
	      m->sets_of_sequences->sets[i]->clique_set->clique_index[j];
	    temp_index_seq = 
	      m->sets_of_sequences->sets[i]->q_intervals[temp_index_clique]->index;


	    fprintf(fd,
		    "%s\t%s\t%s\t%i\t%i\t%f\t%c\t%s%i%s%s\n",
		    m->array_of_seqs->seqs[temp_index_seq]->name,
		    PACKAGE_NAME,
		    "ncRNA",
		    m->sets_of_sequences->sets[i]->q_intervals[temp_index_clique]->pos_start + 1,
		    m->sets_of_sequences->sets[i]->q_intervals[temp_index_clique]->pos_end + 1,
		    m->sets_of_sequences->sets[i]->q_intervals[temp_index_clique]->id_ref,
		    m->sets_of_sequences->sets[i]->q_intervals[temp_index_clique]->strand,
		    ".\tID=",
		    j + 2,
		    "|",
		    m->array_of_seqs->seqs[temp_index_seq]->name);
	  }

	  /*Write fasta sequences*/
	  fprintf(fd, "%s", "##FASTA\n");

	  /*Write reference sequence*/
	  fprintf(fd,
		  ">1|%s %i %i %c",
		  m->array_of_seqs->seqs[0]->name,
		  m->sets_of_sequences->sets[i]->ref_interval->pos_start + 1,
		  m->sets_of_sequences->sets[i]->ref_interval->pos_end + 1,
		  m->sets_of_sequences->sets[i]->ref_interval->strand);


	  length_seq = strlen(m->sets_of_sequences->sets[i]->ref_seq);

	  for(k = 0; k < length_seq; k++){
	    if((k % 60) == 0){
	      RENEW(seq_to_write, char, length_seq_to_write + 2);
	      seq_to_write[length_seq_to_write] = '\n';
	      seq_to_write[length_seq_to_write + 1] = 
		m->sets_of_sequences->sets[i]->ref_seq[k];
	      length_seq_to_write += 2;
	    }
	    else{
	      RENEW(seq_to_write, char, length_seq_to_write + 1);
	      seq_to_write[length_seq_to_write] = 
		m->sets_of_sequences->sets[i]->ref_seq[k];
	      length_seq_to_write ++;
	    }
	  }
	  RENEW(seq_to_write, char, length_seq_to_write + 2);
	  seq_to_write[length_seq_to_write] = '\n';
	  seq_to_write[length_seq_to_write + 1] =  '\0';
	  fprintf(fd,"%s",seq_to_write);
	  DESTROY(seq_to_write);
	  length_seq_to_write = 0;


	  /*Writing all queries (take into account the number
	    of sequences wanted by set)*/
	  for(j = 0; (j < m->sets_of_sequences->sets[i]->clique_set->nb_seqs_clique) &&
		(j < nb_seqs_set); j++){

	    temp_index_clique = 
	      m->sets_of_sequences->sets[i]->clique_set->clique_index[j];
	    temp_index_seq = 
	      m->sets_of_sequences->sets[i]->q_intervals[temp_index_clique]->index;


	    fprintf(fd,
		    ">%i%s%s %i %i %c",
		    j + 2,
		    "|",
		    m->array_of_seqs->seqs[temp_index_seq]->name,
		    m->sets_of_sequences->sets[i]->q_intervals[temp_index_clique]->pos_start + 1,
		    m->sets_of_sequences->sets[i]->q_intervals[temp_index_clique]->pos_end + 1,
		    m->sets_of_sequences->sets[i]->q_intervals[temp_index_clique]->strand);


	    length_seq = 
		strlen(m->sets_of_sequences->sets[i]->q_intervals[temp_index_clique]->seq);

	    for(k = 0; k < length_seq; k++){
	      if((k % 60) == 0){
		RENEW(seq_to_write, char, length_seq_to_write + 2);
		seq_to_write[length_seq_to_write] = '\n';
		seq_to_write[length_seq_to_write + 1] = 
		  m->sets_of_sequences->sets[i]->q_intervals[temp_index_clique]->seq[k];
		length_seq_to_write += 2;
	      }
	      else{
		RENEW(seq_to_write, char, length_seq_to_write + 1);
		seq_to_write[length_seq_to_write] = 
		  m->sets_of_sequences->sets[i]->q_intervals[temp_index_clique]->seq[k];
		length_seq_to_write ++;
	      }
	    }
	    RENEW(seq_to_write, char, length_seq_to_write + 2);
	    seq_to_write[length_seq_to_write] = '\n';
	    seq_to_write[length_seq_to_write + 1] =  '\0';
	    fprintf(fd,"%s",seq_to_write);
	    DESTROY(seq_to_write);
	    length_seq_to_write = 0;

	  }


	  fclose(fd);
	}
	DESTROY(filename);
      }

    }
    
  }


  return nb_set;
}





/**
 *
 * \fn int write_GFF3(p_main_structure m)
 *
 * \brief Function used to write GFF3 files
 *
 * \param[in] m : the main structure containing all info needed
 *
 * \return the number of files written
 *
 * This function will produce as many GFF3 as the number of sets of 
 * sequences discovered
 *
 */


int write_GFF3(p_main_structure m)
{

  /*Number of current set processed*/
  int nb_set = 0;
 
  /*Star mode*/
  if(OPTS_selecting_mode == 0){
    nb_set = write_GFF3_star(m);
  }

  /*Clique mode*/
  else{
    nb_set = write_GFF3_clique(m); 
  }

  return nb_set;

}



/**
 *
 * \fn int write_multifasta_star(p_main_structure m)
 *
 * \brief Function used to write multifasta files for the star mode
 *
 * \param[in] m : the main structure containing all info needed
 *
 * \return the number of files written
 *
 * This function will produce as many multifasta as the number of sets of 
 * sequences discovered for the star mode
 *
 */


int write_multifasta_star(p_main_structure m)
{

  /*Indexes*/
  int i,j,k;
  /*Current filename*/
  char *filename = NULL;
  /*Size in char of the number of the filename*/
  int size;
  /*Current file descriptor*/
  FILE *fd;
  /*Temporary query index*/
  int temp_index;
  /*Temporary sequence index in the set of sequence*/
  int index_set;
  /*Number of sequences by set wanted*/
  int nb_seqs_set = 0;
  /*Number of current set processed*/
  int nb_set = 0;
  /*Indexes of different organism in case the OPTS_min_org_set option is
   used*/
  int_array *indexes_diff_org = NULL;
  /*Number of written sequences in the set (used when OPTS_min_org_set
   is used*/
  int nb_written = 0;
  /*Value of the OPTS_min_species_set or the OPTS_min_org_set value*/
  int value_set = 0;
  /*Current sequence length*/
  int length_seq = 0;
  /*Current length of the sequence to write*/
  int length_seq_to_write = 0;
  /*Current sequence to write*/
  char *seq_to_write = NULL;

  /*Check the number of sequences by set wanted : if -1 all seqs wanted*/
  if(OPTS_nb_seqs_set != -1){
    nb_seqs_set = OPTS_nb_seqs_set - 1;
  }
  else{
    nb_seqs_set = 100000;
  }

  if(OPTS_min_org_set != 0){
    value_set = OPTS_min_org_set;
  }
  if(OPTS_min_species_set != 0){
    value_set = OPTS_min_species_set;
  }


  /*Check if a constraint according to the minimum different organism per set
   has been defined*/
  if((OPTS_min_org_set != 0) || (OPTS_min_species_set != 0)){

    /*Get all sets*/
    for(i = 0; i < m->sets_of_sequences->nb_sets; i++){
      
      nb_written = 0;
      
      /*Check if there are enough sequences*/
      if((m->sets_of_sequences->sets[i]->nb_q_intervals >= value_set)&&
	 (((OPTS_nb_seqs_set_min != -1) && 
	   (m->sets_of_sequences->sets[i]->nb_q_intervals >= 
	    (OPTS_nb_seqs_set_min - 1))) || (OPTS_nb_seqs_set_min == -1)) &&
	 (((OPTS_nb_seqs_set_max != -1) &&
	   (m->sets_of_sequences->sets[i]->nb_q_intervals <= 
	    (OPTS_nb_seqs_set_max - 1))) || (OPTS_nb_seqs_set_max == -1))){
	
	/*Get indexes of different organisms present in the current set of
	  sequences if needed*/	  
	NEW(indexes_diff_org, int_array, 1);
	indexes_diff_org->integers = NULL;
	indexes_diff_org->nb_int = 0;
	
	get_indexes_diff_org(m,i,indexes_diff_org);
	    
	/*The constraint is respected*/
	if(indexes_diff_org->nb_int >= value_set){
	  nb_set++;

	  get_size_number(nb_set, &size);
	  
	  if((OPTS_keep_too_close != -1) && m->sets_of_sequences->sets[i]->is_too_close){
	    NEW(filename,
		char,
		strlen(OPTS_output_dir_too_close) + size + strlen(".fna") + 1);
	    sprintf(filename,
		    "%s%i%s",
		    OPTS_output_dir_too_close,
		    nb_set,
		    ".fna");
	  }
	  else{
	    NEW(filename,
		char,
		strlen(OPTS_output_dir) + size + strlen(".fna") + 1);
	    sprintf(filename,
		    "%s%i%s",
		    OPTS_output_dir,
		    nb_set,
		    ".fna");

	  }

	  fd = fopen(filename, "w");
	  if(fd == NULL){
	    ERROR_ "Impossible to create the file %s\n\n",filename _ERROR;
	    
	  }
	  else{
	    /*Write reference sequence*/
	    fprintf(fd,
		    ">1|%s %i %i %c",
		    m->array_of_seqs->seqs[0]->name,
		    m->sets_of_sequences->sets[i]->ref_interval->pos_start + 1,
		    m->sets_of_sequences->sets[i]->ref_interval->pos_end + 1,
		    m->sets_of_sequences->sets[i]->ref_interval->strand);

	    length_seq = strlen(m->sets_of_sequences->sets[i]->ref_seq);

	    for(k = 0; k < length_seq; k++){
	      if((k % 60) == 0){
		RENEW(seq_to_write, char, length_seq_to_write + 2);
		seq_to_write[length_seq_to_write] = '\n';
		seq_to_write[length_seq_to_write + 1] = 
		  m->sets_of_sequences->sets[i]->ref_seq[k];
		length_seq_to_write += 2;
	      }
	      else{
		RENEW(seq_to_write, char, length_seq_to_write + 1);
		seq_to_write[length_seq_to_write] = 
		  m->sets_of_sequences->sets[i]->ref_seq[k];
		length_seq_to_write ++;
	      }
	    }
	    RENEW(seq_to_write, char, length_seq_to_write + 2);
	    seq_to_write[length_seq_to_write] = '\n';
	    seq_to_write[length_seq_to_write + 1] =  '\0';
	    fprintf(fd,"%s",seq_to_write);
	    DESTROY(seq_to_write);
	    length_seq_to_write = 0;
	    
	    /*Writing all query sequences from the indexes_diff_org_array*/
	    for(j = 0; j < indexes_diff_org->nb_int; j++){
	      
	      index_set = indexes_diff_org->integers[j];
	      temp_index = 
		m->sets_of_sequences->sets[i]->q_intervals[index_set]->index;
	      
	      fprintf(fd,
		      ">%i%s%s %i %i %c",
		      j+2,
		      "|",
		      m->array_of_seqs->seqs[temp_index]->name,
		      m->sets_of_sequences->sets[i]->q_intervals[index_set]->pos_start + 1,
		      m->sets_of_sequences->sets[i]->q_intervals[index_set]->pos_end + 1,
		      m->sets_of_sequences->sets[i]->q_intervals[index_set]->strand);

	      length_seq = 
		strlen(m->sets_of_sequences->sets[i]->q_intervals[index_set]->seq);

	      for(k = 0; k < length_seq; k++){
		if((k % 60) == 0){
		  RENEW(seq_to_write, char, length_seq_to_write + 2);
		  seq_to_write[length_seq_to_write] = '\n';
		  seq_to_write[length_seq_to_write + 1] = 
		    m->sets_of_sequences->sets[i]->q_intervals[index_set]->seq[k];
		  length_seq_to_write += 2;
		}
		else{
		  RENEW(seq_to_write, char, length_seq_to_write + 1);
		  seq_to_write[length_seq_to_write] = 
		    m->sets_of_sequences->sets[i]->q_intervals[index_set]->seq[k];
		  length_seq_to_write ++;
		}
	      }
	      RENEW(seq_to_write, char, length_seq_to_write + 2);
	      seq_to_write[length_seq_to_write] = '\n';
	      seq_to_write[length_seq_to_write + 1] =  '\0';
	      fprintf(fd,"%s",seq_to_write);
	      DESTROY(seq_to_write);
	      length_seq_to_write = 0;
	      

	    }
	    
	    /*Writing other sequences to complete the set*/
	    for(j = 0; (j < m->sets_of_sequences->sets[i]->nb_q_intervals) &&
		  (nb_written < nb_seqs_set - indexes_diff_org->nb_int); j++){
	      
	      /*No writing if it has already been written*/
	      if(!(is_present_int_array(j,
					indexes_diff_org->integers,
					indexes_diff_org->nb_int))){
		
		temp_index = 
		  m->sets_of_sequences->sets[i]->q_intervals[j]->index;
		
		fprintf(fd,
			">%i%s%s %i %i %c",
			nb_written+2+indexes_diff_org->nb_int,
			"|",
			m->array_of_seqs->seqs[temp_index]->name,
			m->sets_of_sequences->sets[i]->q_intervals[j]->pos_start + 1,
			m->sets_of_sequences->sets[i]->q_intervals[j]->pos_end + 1,
			m->sets_of_sequences->sets[i]->q_intervals[j]->strand);

		length_seq = 
		  strlen(m->sets_of_sequences->sets[i]->q_intervals[j]->seq);
		
		for(k = 0; k < length_seq; k++){
		  if((k % 60) == 0){
		    RENEW(seq_to_write, char, length_seq_to_write + 2);
		    seq_to_write[length_seq_to_write] = '\n';
		    seq_to_write[length_seq_to_write + 1] = 
		      m->sets_of_sequences->sets[i]->q_intervals[j]->seq[k];
		    length_seq_to_write += 2;
		  }
		  else{
		    RENEW(seq_to_write, char, length_seq_to_write + 1);
		    seq_to_write[length_seq_to_write] = 
		      m->sets_of_sequences->sets[i]->q_intervals[j]->seq[k];
		    length_seq_to_write ++;
		  }
		}
		RENEW(seq_to_write, char, length_seq_to_write + 2);
		seq_to_write[length_seq_to_write] = '\n';
		seq_to_write[length_seq_to_write + 1] =  '\0';
		fprintf(fd,"%s",seq_to_write);
		DESTROY(seq_to_write);
		length_seq_to_write = 0;


		nb_written++;
	      }

	    }
	    fclose(fd);

 	    
	  }

	  DESTROY(filename);
	  
	  
	}
	DESTROY(indexes_diff_org->integers);
	DESTROY(indexes_diff_org);
      }
      
    }

  }

  /*No different organisms constraint*/
  else{

    /*Get all sets*/
    for(i = 0; i < m->sets_of_sequences->nb_sets; i++){

      /*If no queries : no output*/
      if((m->sets_of_sequences->sets[i]->nb_q_intervals != 0) &&
	 (((OPTS_nb_seqs_set_min != -1) && 
	   (m->sets_of_sequences->sets[i]->nb_q_intervals >= 
	    (OPTS_nb_seqs_set_min - 1))) || (OPTS_nb_seqs_set_min == -1)) &&
	 (((OPTS_nb_seqs_set_max != -1) &&
	   (m->sets_of_sequences->sets[i]->nb_q_intervals <= 
	    (OPTS_nb_seqs_set_max - 1))) || (OPTS_nb_seqs_set_max == -1))){
	
	  nb_set++;
	  get_size_number(nb_set, &size);

	  if((OPTS_keep_too_close != -1) && m->sets_of_sequences->sets[i]->is_too_close){
	    NEW(filename,
		char,
		strlen(OPTS_output_dir_too_close) + size + strlen(".fna") + 1);
	    sprintf(filename,
		    "%s%i%s",
		    OPTS_output_dir_too_close,
		    nb_set,
		    ".fna");
	  }
	  else{
	    NEW(filename,
		char,
		strlen(OPTS_output_dir) + size + strlen(".fna") + 1);
	    sprintf(filename,
		    "%s%i%s",
		    OPTS_output_dir,
		    nb_set,
		    ".fna");
	  }

	  fd = fopen(filename, "w");
	  if(fd == NULL){
	    ERROR_ "Impossible to create the file %s\n\n",filename _ERROR;

	  }
	  else{

	    /*Write reference sequence*/
	    fprintf(fd,
		    ">1|%s %i %i %c",
		    m->array_of_seqs->seqs[0]->name,
		    m->sets_of_sequences->sets[i]->ref_interval->pos_start + 1,
		    m->sets_of_sequences->sets[i]->ref_interval->pos_end + 1,
		    m->sets_of_sequences->sets[i]->ref_interval->strand);

	    length_seq = strlen(m->sets_of_sequences->sets[i]->ref_seq);
	    
	    for(k = 0; k < length_seq; k++){
	      if((k % 60) == 0){
		RENEW(seq_to_write, char, length_seq_to_write + 2);
		seq_to_write[length_seq_to_write] = '\n';
		seq_to_write[length_seq_to_write + 1] = 
		  m->sets_of_sequences->sets[i]->ref_seq[k];
		length_seq_to_write += 2;
	      }
	      else{
		RENEW(seq_to_write, char, length_seq_to_write + 1);
		seq_to_write[length_seq_to_write] = 
		  m->sets_of_sequences->sets[i]->ref_seq[k];
		length_seq_to_write ++;
	      }
	    }
	    RENEW(seq_to_write, char, length_seq_to_write + 2);
	    seq_to_write[length_seq_to_write] = '\n';
	    seq_to_write[length_seq_to_write + 1] =  '\0';
	    fprintf(fd,"%s",seq_to_write);
	    DESTROY(seq_to_write);
	    length_seq_to_write = 0;



	    /*Writing all query sequences (take into account the number of 
	      sequences by set)*/
	    for(j = 0; (j < m->sets_of_sequences->sets[i]->nb_q_intervals) &&
		  (j < nb_seqs_set); j++){
	      
	      temp_index = m->sets_of_sequences->sets[i]->q_intervals[j]->index;
	      
	      fprintf(fd,
		      ">%i%s%s %i %i %c",
		      j+2,
		      "|",
		      m->array_of_seqs->seqs[temp_index]->name,
		      m->sets_of_sequences->sets[i]->q_intervals[j]->pos_start + 1,
		      m->sets_of_sequences->sets[i]->q_intervals[j]->pos_end + 1,
		      m->sets_of_sequences->sets[i]->q_intervals[j]->strand);

	      length_seq = 
		  strlen(m->sets_of_sequences->sets[i]->q_intervals[j]->seq);
		
		for(k = 0; k < length_seq; k++){
		  if((k % 60) == 0){
		    RENEW(seq_to_write, char, length_seq_to_write + 2);
		    seq_to_write[length_seq_to_write] = '\n';
		    seq_to_write[length_seq_to_write + 1] = 
		      m->sets_of_sequences->sets[i]->q_intervals[j]->seq[k];
		    length_seq_to_write += 2;
		  }
		  else{
		    RENEW(seq_to_write, char, length_seq_to_write + 1);
		    seq_to_write[length_seq_to_write] = 
		      m->sets_of_sequences->sets[i]->q_intervals[j]->seq[k];
		    length_seq_to_write ++;
		  }
		}
		RENEW(seq_to_write, char, length_seq_to_write + 2);
		seq_to_write[length_seq_to_write] = '\n';
		seq_to_write[length_seq_to_write + 1] =  '\0';
		fprintf(fd,"%s",seq_to_write);
		DESTROY(seq_to_write);
		length_seq_to_write = 0;


	    }
	    /*Close file*/
	    fclose(fd);
	  }
    
	  DESTROY(filename);
      }
    }
  }

  return nb_set;

}



/**
 *
 * \fn int write_multifasta_clique(p_main_structure m)
 *
 * \brief Function used to write multifasta files for the clique modes
 *
 * \param[in] m : the main structure containing all info needed
 *
 * \return the number of files written
 *
 * This function will produce as many multifasta as the number of sets of 
 * sequences discovered for the clique modes
 *
 */


int write_multifasta_clique(p_main_structure m)
{

  /*Indexes*/
  int i,j,k;
  /*Current filename*/
  char *filename = NULL;
  /*Size in char of the number of the filename*/
  int size;
  /*Current file descriptor*/
  FILE *fd;
  /*Temporary query index*/
  int temp_index;
  /*Temporary index in the clique*/
  int index_clique;
  /*Temporary sequence index in the set of sequence*/
  int index_set;
  /*Number of sequences by set wanted*/
  int nb_seqs_set = 0;
  /*Number of current set processed*/
  int nb_set = 0;
  /*Indexes of different organism in case the OPTS_min_org_set option is
   used*/
  int_array *indexes_diff_org = NULL;
  /*Number of written sequences in the set*/
  int nb_written = 0;
  /*Value of the OPTS_min_species_set or the OPTS_min_org_set value*/
  int value_set = 0;
  /*Current sequence length*/
  int length_seq = 0;
  /*Current length of the sequence to write*/
  int length_seq_to_write = 0;
  /*Current sequence to write*/
  char *seq_to_write = NULL;

  /*Check the number of sequences by set wanted : if -1 all seqs wanted*/
  if(OPTS_nb_seqs_set != -1){
    nb_seqs_set = OPTS_nb_seqs_set - 1;
  }
  else{
    nb_seqs_set = 100000;
  }

  if(OPTS_min_org_set != 0){
    value_set = OPTS_min_org_set;
  }
  if(OPTS_min_species_set != 0){
    value_set = OPTS_min_species_set;
  }

  /*Check if a constraint according to the minimum different organism per set
    has been defined*/
  if((OPTS_min_org_set != 0) || (OPTS_min_species_set != 0)){

    /*Get all sets*/
    for(i = 0; i < m->sets_of_sequences->nb_sets; i++){

      nb_written = 0;
      
      /*Check if there are enough sequences*/
      if ((m->sets_of_sequences->sets[i]->clique_set != NULL) && 
	  (m->sets_of_sequences->sets[i]->clique_set->nb_seqs_clique != 1) &&
	  (m->sets_of_sequences->sets[i]->clique_set->nb_seqs_clique >= 
	   value_set) &&
	  (((OPTS_nb_seqs_set_min != -1) && 
	    (m->sets_of_sequences->sets[i]->clique_set->nb_seqs_clique >= 
	     (OPTS_nb_seqs_set_min - 1))) || (OPTS_nb_seqs_set_min == -1)) &&
	  (((OPTS_nb_seqs_set_max != -1) &&
	    (m->sets_of_sequences->sets[i]->clique_set->nb_seqs_clique <= 
	     (OPTS_nb_seqs_set_max - 1))) || (OPTS_nb_seqs_set_max == -1))){
      
	/*Get indexes of different organisms present in the current set of
	  sequences if needed*/	  
	NEW(indexes_diff_org, int_array, 1);
	indexes_diff_org->integers = NULL;
	indexes_diff_org->nb_int = 0;

	get_indexes_diff_org(m,i,indexes_diff_org);

	/*The constraint is respected*/
	if(indexes_diff_org->nb_int >= value_set){
	  nb_set++;
	
	  /*Open the new file*/
	  get_size_number(nb_set, &size);
	  NEW(filename,
	      char,
	      strlen(OPTS_output_dir) + size + strlen(".fna") + 1);
	  sprintf(filename,
		  "%s%i%s",
		  OPTS_output_dir,
		  nb_set,
		  ".fna");
	  fd = fopen(filename, "w");
	  if(fd == NULL){
	    ERROR_ "Impossible to create the file %s\n\n",filename _ERROR;
	  
	  }
	  else{
	    /*Write reference sequence*/
	    fprintf(fd,
		    ">1|%s %i %i %c",
		    m->array_of_seqs->seqs[0]->name,
		    m->sets_of_sequences->sets[i]->ref_interval->pos_start + 1,
		    m->sets_of_sequences->sets[i]->ref_interval->pos_end + 1,
		    m->sets_of_sequences->sets[i]->ref_interval->strand);

	    length_seq = strlen(m->sets_of_sequences->sets[i]->ref_seq);

	    for(k = 0; k < length_seq; k++){
	      if((k % 60) == 0){
		RENEW(seq_to_write, char, length_seq_to_write + 2);
		seq_to_write[length_seq_to_write] = '\n';
		seq_to_write[length_seq_to_write + 1] = 
		  m->sets_of_sequences->sets[i]->ref_seq[k];
		length_seq_to_write += 2;
	      }
	      else{
		RENEW(seq_to_write, char, length_seq_to_write + 1);
		seq_to_write[length_seq_to_write] = 
		  m->sets_of_sequences->sets[i]->ref_seq[k];
		length_seq_to_write ++;
	      }
	    }
	    RENEW(seq_to_write, char, length_seq_to_write + 2);
	    seq_to_write[length_seq_to_write] = '\n';
	    seq_to_write[length_seq_to_write + 1] =  '\0';
	    fprintf(fd,"%s",seq_to_write);
	    DESTROY(seq_to_write);
	    length_seq_to_write = 0;


	    /*Writing all query sequences from the indexes_diff_org_array*/
	    for(j = 0; j < indexes_diff_org->nb_int; j++){
	    
	      index_set = indexes_diff_org->integers[j];
	      index_clique = 
		m->sets_of_sequences->sets[i]->clique_set->clique_index[index_set];
	      temp_index = 
		m->sets_of_sequences->sets[i]->q_intervals[index_clique]->index;

	      fprintf(fd,
		      ">%i%s%s %i %i %c",
		      j+2,
		      "|",
		      m->array_of_seqs->seqs[temp_index]->name,
		      m->sets_of_sequences->sets[i]->q_intervals[index_clique]->pos_start + 1,
		      m->sets_of_sequences->sets[i]->q_intervals[index_clique]->pos_end + 1,
		      m->sets_of_sequences->sets[i]->q_intervals[index_clique]->strand);

	      length_seq = 
		strlen(m->sets_of_sequences->sets[i]->q_intervals[index_clique]->seq);

	      for(k = 0; k < length_seq; k++){
		if((k % 60) == 0){
		  RENEW(seq_to_write, char, length_seq_to_write + 2);
		  seq_to_write[length_seq_to_write] = '\n';
		  seq_to_write[length_seq_to_write + 1] = 
		    m->sets_of_sequences->sets[i]->q_intervals[index_clique]->seq[k];
		  length_seq_to_write += 2;
		}
		else{
		  RENEW(seq_to_write, char, length_seq_to_write + 1);
		  seq_to_write[length_seq_to_write] = 
		    m->sets_of_sequences->sets[i]->q_intervals[index_clique]->seq[k];
		  length_seq_to_write ++;
		}
	      }
	      RENEW(seq_to_write, char, length_seq_to_write + 2);
	      seq_to_write[length_seq_to_write] = '\n';
	      seq_to_write[length_seq_to_write + 1] =  '\0';
	      fprintf(fd,"%s",seq_to_write);
	      DESTROY(seq_to_write);
	      length_seq_to_write = 0;


	    }

	    /*Writing other sequences to complete the set*/
	    for(j = 0; (j < m->sets_of_sequences->sets[i]->clique_set->nb_seqs_clique) &&
		  (nb_written < nb_seqs_set - indexes_diff_org->nb_int); j++){
	    
	      /*No writing if it has already been written*/
	      if(!(is_present_int_array(j,
					indexes_diff_org->integers,
					indexes_diff_org->nb_int))){

		index_clique = 
		  m->sets_of_sequences->sets[i]->clique_set->clique_index[j];
		temp_index = 
		  m->sets_of_sequences->sets[i]->q_intervals[index_clique]->index;

		fprintf(fd,
			">%i%s%s %i %i %c",
			nb_written+2+indexes_diff_org->nb_int,
			"|",
			m->array_of_seqs->seqs[temp_index]->name,
			m->sets_of_sequences->sets[i]->q_intervals[index_clique]->pos_start + 1,
			m->sets_of_sequences->sets[i]->q_intervals[index_clique]->pos_end + 1,
			m->sets_of_sequences->sets[i]->q_intervals[index_clique]->strand);

		length_seq = 
		  strlen(m->sets_of_sequences->sets[i]->q_intervals[index_clique]->seq);
		
		for(k = 0; k < length_seq; k++){
		  if((k % 60) == 0){
		    RENEW(seq_to_write, char, length_seq_to_write + 2);
		    seq_to_write[length_seq_to_write] = '\n';
		    seq_to_write[length_seq_to_write + 1] = 
		      m->sets_of_sequences->sets[i]->q_intervals[index_clique]->seq[k];
		    length_seq_to_write += 2;
		  }
		  else{
		    RENEW(seq_to_write, char, length_seq_to_write + 1);
		    seq_to_write[length_seq_to_write] = 
		      m->sets_of_sequences->sets[i]->q_intervals[index_clique]->seq[k];
		    length_seq_to_write ++;
		  }
		}
		RENEW(seq_to_write, char, length_seq_to_write + 2);
		seq_to_write[length_seq_to_write] = '\n';
		seq_to_write[length_seq_to_write + 1] =  '\0';
		fprintf(fd,"%s",seq_to_write);
		DESTROY(seq_to_write);
		length_seq_to_write = 0;
		
		nb_written++;
	      }
	    
	    }
	    
	    fclose(fd);
	    
	  }

	  DESTROY(filename);
	  
	}
	DESTROY(indexes_diff_org->integers);
	DESTROY(indexes_diff_org);
      }
    }
  }
  
  /*No different organisms constraint*/
  else{

    /*Get all sets */
    for(i = 0; i < m->sets_of_sequences->nb_sets; i++){

      /*If no queries : no output*/
      if ((m->sets_of_sequences->sets[i]->clique_set != NULL) && 
	  (m->sets_of_sequences->sets[i]->clique_set->nb_seqs_clique != 1) &&
	  (((OPTS_nb_seqs_set_min != -1) && 
	    (m->sets_of_sequences->sets[i]->clique_set->nb_seqs_clique >= 
	     (OPTS_nb_seqs_set_min - 1))) || (OPTS_nb_seqs_set_min == -1)) &&
	  (((OPTS_nb_seqs_set_max != -1) &&
	    (m->sets_of_sequences->sets[i]->clique_set->nb_seqs_clique <= 
	     (OPTS_nb_seqs_set_max - 1))) || (OPTS_nb_seqs_set_max == -1))){
	
	nb_set++;

	/*Open the new file*/
	get_size_number(nb_set, &size);
	NEW(filename,
	    char,
	    strlen(OPTS_output_dir) + size + strlen(".fna") + 1);
	sprintf(filename,
		"%s%i%s",
		OPTS_output_dir,
		nb_set,
		".fna");
	fd = fopen(filename, "w");
	if(fd == NULL){
	  ERROR_ "Impossible to create the file %s\n\n",filename _ERROR;
	  
	}
	else{

	  /*Write reference sequence*/
	  fprintf(fd,
		  ">1|%s %i %i %c",
		  m->array_of_seqs->seqs[0]->name,
		  m->sets_of_sequences->sets[i]->ref_interval->pos_start + 1,
		  m->sets_of_sequences->sets[i]->ref_interval->pos_end + 1,
		  m->sets_of_sequences->sets[i]->ref_interval->strand);
	
	  length_seq = strlen(m->sets_of_sequences->sets[i]->ref_seq);

	  for(k = 0; k < length_seq; k++){
	    if((k % 60) == 0){
	      RENEW(seq_to_write, char, length_seq_to_write + 2);
	      seq_to_write[length_seq_to_write] = '\n';
	      seq_to_write[length_seq_to_write + 1] = 
		m->sets_of_sequences->sets[i]->ref_seq[k];
	      length_seq_to_write += 2;
	    }
	    else{
	      RENEW(seq_to_write, char, length_seq_to_write + 1);
	      seq_to_write[length_seq_to_write] = 
		m->sets_of_sequences->sets[i]->ref_seq[k];
	      length_seq_to_write ++;
	    }
	  }
	  RENEW(seq_to_write, char, length_seq_to_write + 2);
	  seq_to_write[length_seq_to_write] = '\n';
	  seq_to_write[length_seq_to_write + 1] =  '\0';
	  fprintf(fd,"%s",seq_to_write);
	  DESTROY(seq_to_write);
	  length_seq_to_write = 0;

  
	  /*Writing all query sequences (take into account the number of
	    sequences by set*/
	  for(j = 0; 
	      (j < m->sets_of_sequences->sets[i]->clique_set->nb_seqs_clique) &&
		(j < nb_seqs_set); 
	      j++){
	    
	    index_clique = 
	      m->sets_of_sequences->sets[i]->clique_set->clique_index[j];
	    temp_index = 
	      m->sets_of_sequences->sets[i]->q_intervals[index_clique]->index;
	    
	    
	    fprintf(fd,
		    ">%i%s%s %i %i %c",
		    j + 2,
		    "|",
		    m->array_of_seqs->seqs[temp_index]->name,
		    m->sets_of_sequences->sets[i]->q_intervals[index_clique]->pos_start + 1,
		    m->sets_of_sequences->sets[i]->q_intervals[index_clique]->pos_end + 1,
		    m->sets_of_sequences->sets[i]->q_intervals[index_clique]->strand);

	    length_seq = 
		strlen(m->sets_of_sequences->sets[i]->q_intervals[index_clique]->seq);

	      for(k = 0; k < length_seq; k++){
		if((k % 60) == 0){
		  RENEW(seq_to_write, char, length_seq_to_write + 2);
		  seq_to_write[length_seq_to_write] = '\n';
		  seq_to_write[length_seq_to_write + 1] = 
		    m->sets_of_sequences->sets[i]->q_intervals[index_clique]->seq[k];
		  length_seq_to_write += 2;
		}
		else{
		  RENEW(seq_to_write, char, length_seq_to_write + 1);
		  seq_to_write[length_seq_to_write] = 
		    m->sets_of_sequences->sets[i]->q_intervals[index_clique]->seq[k];
		  length_seq_to_write ++;
		}
	      }
	      RENEW(seq_to_write, char, length_seq_to_write + 2);
	      seq_to_write[length_seq_to_write] = '\n';
	      seq_to_write[length_seq_to_write + 1] =  '\0';
	      fprintf(fd,"%s",seq_to_write);
	      DESTROY(seq_to_write);
	      length_seq_to_write = 0;


	  }

	  fclose(fd);

	}

	DESTROY(filename);

      }

    }

  }


  return nb_set;


}




/**
 *
 * \fn int write_multifasta(p_main_structure m)
 *
 * \brief Function used to write multifasta files
 *
 * \param[in] m : the main structure containing all info needed
 *
 * \return the number of files written
 * 
 * This function will produce as many multifasta as the number of sets of 
 * sequences discovered
 *
 */

int write_multifasta(p_main_structure m)
{

  /*Number of current set processed*/
  int nb_set = 0;
 
  /*Star mode*/
  if(OPTS_selecting_mode == 0){
    nb_set = write_multifasta_star(m);
  }

  /*Clique mode*/
  else{
    nb_set = write_multifasta_clique(m);
  }

  return nb_set;

}



/**
 *
 * \fn void print_verbose_start_process()
 *
 * \brief Function used to print starting message on screen
 *
 * This function will print on screen the starting message 
 *
 */


void print_verbose_start_process()
{
  PRINT_ "\n%s",LINE _PRINT;
  PRINT_ "\nFIND CONSERVED REGIONS FROM ALIGNMENTS\n" _PRINT;     
  PRINT_ "\n%s\n", LINE _PRINT;

}



/**
 *
 * \fn void print_verbose_start_sequences()
 *
 * \brief Function used to print the starting message on screen concerning
 * the parsing of the sequences
 *
 * This function will print the starting message on screen concerning
 * the parsing of the sequences
 *
 */


void print_verbose_start_sequences()
{

  /*index*/
  int i;

  PRINT_ "GET REFERENCE FROM FILE : %s\n\n",OPTS_ref_file _PRINT;
  PRINT_ "GET OTHER SEQUENCES FROM FILE(S) : \n" _PRINT;
  for(i = 0; i < OPTS_file_array->nb_chars; i++){
    PRINT_ "\t- %s\n",OPTS_file_array->chars[i] _PRINT;
  }
  PRINT_ "\n" _PRINT;
  
}


/**
 *
 * \fn void print_verbose_end_sequences(sequence_array *array_of_seqs)
 *
 * \brief Function used to print the ending message on screen concerning
 * the parsing of the sequences
 *
 * \param[in] array_of_seqs : the array of extracted sequences
 *
 * This function will print the ending message on screen concerning
 * the parsing of the sequences (number of sequences, their names and 
 * lengths)
 *
 */


void print_verbose_end_sequences(sequence_array *array_of_seqs)
{

  /*Index*/
  int i;

  PRINT_ "FOUND %i SEQUENCES\n",array_of_seqs->nb_seqs 
    _PRINT;
  PRINT_ "\tREFERENCE : %s (length : %i)\n",
    array_of_seqs->seqs[0]->name,
    array_of_seqs->seqs[0]->length _PRINT;	
  for(i = 1; i < array_of_seqs->nb_seqs; i++){
    PRINT_ "\tSEQUENCE %i : %s (length : %i)\n",
      i,
      array_of_seqs->seqs[i]->name,
      array_of_seqs->seqs[i]->length _PRINT;
  }
  PRINT_ "\n%s\n", LINE _PRINT;
  
}



/**
 *
 * \fn void print_verbose_start_alignments()
 *
 * \brief Function used to print the starting message on screen concerning
 * the parsing of the alignments
 *
 * This function will print the starting message on screen concerning
 * the parsing of the alignments
 *
 */


void print_verbose_start_alignments()
{
  /*Boolean that says if at aleast one parameter has been used*/
  int has_option = 0;

  PRINT_ "READ ALIGNMENTS FILES (%i files)\n\n",
    OPTS_alignments->nb_chars _PRINT;
  if(OPTS_evalue != -1){
    PRINT_ "\tMINIMUM EVALUE : %f\n",OPTS_evalue _PRINT;
    has_option = 1;
  }
  if(OPTS_min_id_alignment != 0.0){
    PRINT_ "\tMINIMUM IDENTITY : %f\n",OPTS_min_id_alignment _PRINT;
    has_option = 1;
  }
  if(OPTS_min_length_alignment != 0){
    PRINT_ "\tMINIMUM LENGTH : %i\n",OPTS_min_length_alignment _PRINT;
    has_option = 1;
  }
  if(has_option){
    PRINT_ "\n" _PRINT;
  }

}


/**
 *
 * \fn void print_verbose_start_alignments_file(char *filename)
 *
 * \brief Function used to print the starting message on screen concerning
 * the parsing of the alignments in the current file
 *
 * \param[in] filename : the name of the current alignment file
 *
 * This function will print the starting message on screen concerning
 * the parsing of the alignments in the current file
 *
 */


void print_verbose_start_alignments_file(char *filename)
{
  PRINT_ "\tPARSING ALIGNMENT FILE : %s\n",filename _PRINT;

}


/**
 *
 * \fn void print_verbose_end_alignments_file(int nb_al)
 *
 * \brief Function used to print the starting message on screen concerning
 * the parsing of the alignments in the current file
 *
 * \param[in] nb_al : number of alignment found in the current file
 *
 * This function will print the ending message on screen concerning
 * the parsing of the alignments in the current file (number of alignments
 * found)
 *
 */


void print_verbose_end_alignments_file(int nb_al)
{
  PRINT_ "\t\tAlignments found : %i\n\n",nb_al _PRINT;

}


/**
 *
 * \fn void print_verbose_end_alignments(int nb_al)
 *
 * \brief Function used to print the ending message on screen concerning
 * the parsing of the alignments
 *
 * \param[in] nb_al : the number of alignments found
 *
 * This function will print the ending message on screen concerning
 * the parsing of the alignments (number of alignments found)
 *
 */


void print_verbose_end_alignments(int nb_al)
{

  PRINT_ "ALIGNMENTS FILES READING COMPLETE : %i alignments found\n",
    nb_al _PRINT;
  PRINT_ "\n%s\n", LINE _PRINT;


}


/**
 *
 * \fn void print_verbose_start_score(sequence_array *seqs)
 *
 * \brief Function used to print the starting message on screen concerning
 * the score calculation
 *
 * \param[in] seqs : the sequences array used to link probabilities with
 * the genome name
 *
 * This function will print the starting message on screen concerning
 * the score calculation
 *
 */


void print_verbose_start_score(sequence_array *seqs)
{

  /*Index*/
  int i;
  /*Last num species*/
  int last_num_species = -1;

  PRINT_ "CALCULATE SCORE FUNCTION\n" _PRINT;
  for(i = 1; i < seqs->nb_seqs; i++){
    if(seqs->seqs[i]->num_species != last_num_species){
      PRINT_ "\n\tPROBABILITY : %f\n",
	OPTS_proba->floats[seqs->seqs[i]->num_species] _PRINT;
      last_num_species = seqs->seqs[i]->num_species;
    }
    PRINT_ "\t\t-> %s\n",
      seqs->seqs[i]->name _PRINT;
  }  

  PRINT_ "\n" _PRINT;

}



/**
 *
 * \fn void print_verbose_end_score(float mean)
 *
 * \brief Function used to print the ending message on screen concerning
 * the score calculation
 *
 * \param[in] mean : the mean number of alignments found by position
 *
 * This function will print the ending message on screen concerning
 * the score calculation (mean number of alignments)
 *
 */


void print_verbose_end_score(float mean)
{
  PRINT_ "\tMEAN NUMBER OF ALIGNMENTS BY POSITION : %f\n\n",mean _PRINT;
  PRINT_ "SCORE FUNCTION CALCULATED\n" _PRINT;
  
  PRINT_ "\n%s\n", LINE _PRINT;

}


/**
 *
 * \fn void print_verbose_start_conserved_regions()
 *
 * \brief Function used to print the starting message on screen concerning
 * the searching for conserved regions
 *
 * This function will print the starting message on screen concerning
 * the searching of the conserved regions
 *
 */


void print_verbose_start_conserved_regions()
{
  PRINT_ "FINDING CONSERVED REGIONS\n\n" _PRINT;
  PRINT_ "\tMININUM LENGTH OF A REGION : %i\n",
    OPTS_min_length_region _PRINT;
  PRINT_ "\tMAXIMUM LENGTH OF A REGION : %i\n\n",
    OPTS_max_length_region _PRINT;
  
}


/**
 *
 * \fn void print_verbose_end_conserved_regions(int nb_sets)
 *
 * \brief Function used to print the ending message on screen concerning
 * the searching for conserved regions
 *
 * \param[in] nb_sets : the number of conserved regions found
 *
 * This function will print the ending message on screen concerning
 * the searching of the conserved regions (number of regions found)
 *
 */


void print_verbose_end_conserved_regions(int nb_sets)
{

  PRINT_ "CONSERVED REGIONS FOUND : %i\n",nb_sets _PRINT;
  PRINT_ "\n%s\n", LINE _PRINT;

}


/**
 *
 * \fn void print_verbose_start_build_sets()
 *
 * \brief Function used to print the starting message on screen concerning
 * the building sets step
 *
 * This function will print the starting message on screen concerning
 * the building sets step
 *
 */


void print_verbose_start_build_sets()
{

  PRINT_ "BUILDING SETS OF SEQUENCES\n\n" _PRINT;

}


/**
 *
 * \fn void print_verbose_end_build_sets(int nb_sets)
 *
 * \brief Function used to print the ending message on screen concerning
 * the building sets step
 *
 * \param[in] nb_sets : the number of sets built
 *
 * This function will print the ending message on screen concerning
 * the building sets step (number of sets built)
 *
 */


void print_verbose_end_build_sets(int nb_sets)
{

  PRINT_ "SETS OF SEQUENCES BUILT : %i\n",nb_sets _PRINT;
  PRINT_ "\n%s\n", LINE _PRINT;
  
}


/**
 *
 * \fn void print_verbose_start_compare_ref()
 *
 * \brief Function used to print the starting message on screen concerning
 * the comparison step of all queries against the reference
 *
 * This function will print the starting message on screen concerning
 * the comparison step of all queries against the reference
 *
 */


void print_verbose_start_compare_ref()
{
  PRINT_ "COMPARING ALL QUERIES AGAINST THE REFERENCE AND ELIMINATE REDUNDANCY\n\n" _PRINT;
  PRINT_ "\tMINIMUM IDENTITY THRESHOLD : %i\n", OPTS_min_id _PRINT;
  PRINT_ "\tMAXIMUM IDENTITY THRESHOLD : %i\n", OPTS_max_id _PRINT;

  if(OPTS_carnac){
    PRINT_ "\tCOMMON SECONDARY STRUCTURE SHARED\n" _PRINT;
  }
  if(OPTS_keep_too_close != -1){
    PRINT_ "\tKEEP SETS CONTAINING ONLY TOO CLOSE SEQUENCES\n" _PRINT;
  }
  PRINT_ "\n" _PRINT;


  
}


/**
 *
 * \fn void print_verbose_start_compare_ref_set(int nb_set, int nb_seqs)
 *
 * \brief Function used to print the starting message on screen concerning
 * the comparison step of all queries against the reference for the current
 * set
 *
 * \param[in] nb_set : the number of set processed
 * \param[in] nb_seqs : the number of sequences in the set (ref + queries)
 *
 * This function will print the starting message on screen concerning
 * the comparison step of all queries against the reference for the current
 * set (nb_set). It also print the number of sequences in the set
 *
 */


void print_verbose_start_compare_ref_set(int nb_set, int nb_seqs)


{
  
  PRINT_ "\tSET %i (%i sequences) -> ",nb_set,nb_seqs _PRINT; 

}



/**
 *
 * \fn void print_verbose_end_compare_ref_set(int nb_seqs, int is_too_close)
 *
 * \brief Function used to print the ending message on screen concerning
 * the comparison step of all queries against the reference for the current
 * set
 *
 * \param[in] nb_seqs : the number of seqs in the set after the comparison
 * (ref + query)
 * \param[in] is_too_close : boolean used when the OPTS_keep_too_close
 * option is used
 *
 * This function will print the ending message on screen concerning
 * the comparison step of all queries against the reference for the current
 * set
 *
 */


void print_verbose_end_compare_ref_set(int nb_seqs, int is_too_close)
{

  if(nb_seqs != 1){
    PRINT_ " %i sequences left",nb_seqs _PRINT;
    if((OPTS_keep_too_close != -1) && is_too_close){
      PRINT_ " (too close set)" _PRINT;
    }

  }
  else{
    PRINT_ " No connected sequence" _PRINT;
  }

  PRINT_ "\n" _PRINT;

}


/**
 *
 * \fn void print_verbose_sort_identity_ref()
 *
 * \brief Function used to print the message on screen concerning the
 * sorting of queries by identities
 *
 * This function will print the message on screen concerning the
 * sorting of queries by identities
 *
 */


extern void print_verbose_sort_identity_ref()
{

  PRINT_ "\nSORTING QUERIES IN SETS BY REFERENCE IDENTITY...\n" _PRINT;

}


/**
 *
 * \fn void print_verbose_end_compare_ref()
 *
 * \brief Function used to print the ending message on screen concerning
 * the comparison step of all queries against the reference
 *
 * This function will print the ending message on screen concerning
 * the comparison step of all queries against the reference
 *
 */


void print_verbose_end_compare_ref()
{

  PRINT_ "\nCOMPARISONS DONE\n" _PRINT;
  PRINT_ "\n%s\n", LINE _PRINT;
}


/**
 *
 * \fn void print_verbose_start_check_repetitions()
 *
 * \brief Function used to print the message on screen concerning the
 * checking of species repetitions
 *
 * This function will print the message on screen concerning the
 * checking of species repetitions
 *
 */


void print_verbose_start_check_repetitions()
{
  PRINT_ "\nCHECKING SPECIES REPETITIONS\n\n" _PRINT;
  PRINT_ "\tMAXIMUM NUMBER OF SPECIES REPETITIONS : %i\n\n", 
    OPTS_max_species_repet _PRINT;

}


/**
 *
 * \fn void print_verbose_start_checking_repetitions_set(int nb_set, 
 *                                                       int nb_seqs)
 *
 * \brief Function used to print the starting message on screen concerning
 * the species repetitions for the current set
 *
 * \param[in] nb_set : the number of set processed
 * \param[in] nb_seqs : the number of sequences in the set (ref + queries)
 *
 * This function will print the starting message on screen concerning
 * the species repetitions for the current set (nb_set). 
 * It also print the number of sequences in the set
 *
 */


void print_verbose_start_checking_repetitions_set(int nb_set, int nb_seqs)


{
  
  PRINT_ "\tSET %i (%i sequences) -> ",nb_set,nb_seqs _PRINT; 

}


/**
 *
 * \fn void print_verbose_end_checking_repetitions_set(int nb_seqs)
 *
 * \brief Function used to print the ending message on screen concerning
 * the species repetitions for the current set
 *
 * \param[in] nb_seqs : the number of sequences in the set (ref + queries)
 *
 * This function will print the ending message on screen concerning
 * the species repetitions for the current set by printing the number of 
 * sequences in the set
 *
 */


void print_verbose_end_checking_repetitions_set(int nb_seqs)
{
  if(nb_seqs != 1){
    PRINT_ " OK\n" _PRINT;
  }
  else{
    PRINT_ " Species repetitions exceeded\n" _PRINT;
  }


}


/**
 *
 * \fn void print_verbose_end_check_repetitions()
 *
 * \brief Function used to print the ending message on screen concerning the
 * checking of species repetitions
 *
 * This function will print the ending message on screen concerning the
 * checking of species repetitions
 *
 */


void print_verbose_end_check_repetitions()
{

  PRINT_ "CHECKING SPECIES REPETITIONS DONE\n\n" _PRINT;


}


/**
 *
 * \fn void print_verbose_start_search_naive_clique()
 *
 * \brief Function used to print the starting message on screen concerning
 * the comparison step of the naive clique search
 *
 * This function will print the starting message on screen concerning
 * the comparison step of of the naive clique search
 *
 */


void print_verbose_start_search_naive_clique()
{
  PRINT_ "SEARCHING NAIVE CLIQUE\n\n" _PRINT;
  PRINT_ "\tMINIMUM IDENTITY THRESHOLD : %i\n", OPTS_min_id _PRINT;
  PRINT_ "\tMAXIMUM IDENTITY THRESHOLD : %i\n", OPTS_max_id _PRINT;
  if(OPTS_nb_seqs_set_max != -1){
    PRINT_ "\tMAXIMUM NUMBER OF SEQUENCES BY SET WANTED : %i\n",
      OPTS_nb_seqs_set_max _PRINT;
  }
  if(OPTS_nb_seqs_set_min != -1){
    PRINT_ "\tMINIMUM NUMBER OF SEQUENCES BY SET WANTED : %i\n",
      OPTS_nb_seqs_set_min _PRINT;
  }
  if(OPTS_carnac){
    PRINT_ "\tCOMMON SECONDARY STRUCTURE SHARED\n\n" _PRINT;
  }
  else{
    PRINT_ "\n" _PRINT;

  }
}


/**
 *
 * \fn void print_verbose_start_naive_clique_set(int nb_set, int nb_seqs)
 *
 * \brief Function used to print the starting message on screen concerning
 * the comparison step of searching the naive clique for the current
 * set
 *
 * \param[in] nb_set : the number of set processed
 * \param[in] nb_seqs : the number of sequences in the set (ref + queries)
 *
 * This function will print the starting message on screen concerning
 * the comparison step of searching the naive clique for the current
 * set (nb_set). It also print the number of sequences in the set
 *
 */


void print_verbose_start_naive_clique_set(int nb_set, int nb_seqs)
{

  PRINT_ "\tSET %i (%i sequences) -> ",nb_set,nb_seqs _PRINT; 


}



/**
 *
 * \fn void print_verbose_end_naive_clique_set(int nb_seqs)
 *
 * \brief Function used to print the ending message on screen concerning
 * the comparison step of searching the naive clique for the current
 * set
 *
 * \param[in] nb_seqs : the number of seqs in the set after the naive
 * clique serach (ref + query)
 *
 * This function will print the ending message on screen concerning
 * the comparison step of searching the naive clique for the current
 * set
 *
 */


void print_verbose_end_naive_clique_set(int nb_seqs)
{

  if(nb_seqs > 2){
    PRINT_ " %i sequences clique\n",nb_seqs _PRINT;
  }
  else{
    PRINT_ " No clique found\n" _PRINT;
  }

}


/**
 *
 * \fn void print_verbose_end_search_naive_clique()
 *
 * \brief Function used to print the ending message on screen concerning
 * the comparison step of the naive clique search
 *
 * This function will print the ending message on screen concerning
 * the comparison step of of the naive clique search
 *
 */


void print_verbose_end_search_naive_clique()
{
  PRINT_ "NAIVE CLIQUE SEARCH DONE\n" _PRINT;
  PRINT_ "\n%s\n", LINE _PRINT;
}


/**
 *
 * \fn void print_verbose_no_connect(int nb)
 *
 * \brief Function used to print the the message when a set of sequences
 * has no connected sequence
 *
 * \param nb : the number of the set concerned
 *
 * This function will print the the message when a set of sequences
 * has no connected sequence
 *
 */

void print_verbose_no_connect(int nb)
{

  PRINT_ "\tSET %i : no connected sequence with the reference\n",nb _PRINT;

}



/**
 *
 * \fn void print_verbose_start_search_naive_clique()
 *
 * \brief Function used to print the starting message on screen concerning
 * the comparison step of the maxi clique search
 *
 * This function will print the starting message on screen concerning
 * the comparison step of of the maxi clique search
 *
 */


void print_verbose_start_search_maxi_clique()
{
  PRINT_ "SEARCHING MAXI CLIQUE\n\n" _PRINT;
  PRINT_ "\tMINIMUM IDENTITY THRESHOLD : %i\n", OPTS_min_id _PRINT;
  PRINT_ "\tMAXIMUM IDENTITY THRESHOLD : %i\n", OPTS_max_id _PRINT;
  if(OPTS_nb_seqs_set_max != -1){
    PRINT_ "\tMAXIMUM NUMBER OF SEQUENCES BY SET WANTED : %i\n",
      OPTS_nb_seqs_set_max _PRINT;
  }
  if(OPTS_nb_seqs_set_min != -1){
    PRINT_ "\tMINIMUM NUMBER OF SEQUENCES BY SET WANTED : %i\n",
      OPTS_nb_seqs_set_min _PRINT;
  }
  if(OPTS_carnac){
    PRINT_ "\tCOMMON SECONDARY STRUCTURE SHARED\n\n" _PRINT;
  }
  else{
    PRINT_ "\n" _PRINT;

  }



}


/**
 *
 * \fn void print_verbose_start_maxi_clique_set(int nb_set, int nb_seqs)
 *
 * \brief Function used to print the starting message on screen concerning
 * the comparison step of searching the maxi clique for the current
 * set
 *
 * \param[in] nb_set : the number of set processed
 * \param[in] nb_seqs : the number of sequences in the set (ref + queries)
 *
 * This function will print the starting message on screen concerning
 * the comparison step of searching the maxi clique for the current
 * set (nb_set). It also print the number of sequences in the set
 *
 */


void print_verbose_start_maxi_clique_set(int nb_set, int nb_seqs)
{

  PRINT_ "\tSET %i (%i sequences) -> ",nb_set,nb_seqs _PRINT; 

}


/**
 *
 * \fn void print_verbose_end_maxi_clique_set(int nb_seqs)
 *
 * \brief Function used to print the ending message on screen concerning
 * the comparison step of searching the maxi clique for the current
 * set
 *
 * \param[in] nb_seqs : the number of seqs in the set after the maxi
 * clique serach (ref + query)
 *
 * This function will print the ending message on screen concerning
 * the comparison step of searching the maxi clique for the current
 * set
 *
 */


void print_verbose_end_maxi_clique_set(int nb_seqs)
{
  if(nb_seqs > 2){
    PRINT_ " %i sequences clique\n",nb_seqs _PRINT;
  }
  else{
    PRINT_ " No clique found\n" _PRINT;
  }

}


/**
 *
 * \fn void print_verbose_end_search_naive_clique()
 *
 * \brief Function used to print the ending message on screen concerning
 * the comparison step of the maxi clique search
 *
 * This function will print the ending message on screen concerning
 * the comparison step of of the maxi clique search
 *
 */


void print_verbose_end_search_maxi_clique()
{
  PRINT_ "MAXI CLIQUE SEARCH DONE\n" _PRINT;
  PRINT_ "\n%s\n", LINE _PRINT;

}


/**
 *
 * \fn void print_verbose_start_write_GFF3()
 *
 * \brief Function used to print the starting message on screen concerning
 * the writing of result GFF3 files
 *
 * This function will print the starting message on screen concerning
 * the writing of result GFF3 files
 *
 */


void print_verbose_start_write_GFF3()
{
  PRINT_ "WRITE SETS OF SEQUENCES IN GFF3 format\n\n" _PRINT;
  PRINT_ "\tOUTPUT DIR : %s\n",OPTS_output_dir _PRINT;
  if(OPTS_keep_too_close != -1){
    PRINT_ "\tOUTPUT DIR FOR SETS CONTAINING TOO CLOSE SEQUENCES : %s\n",
      OPTS_output_dir_too_close _PRINT;
  }
  if(OPTS_nb_seqs_set != -1){
    PRINT_ "\tNUMBER OF SEQUENCES WANTED BY SET : %i\n",
      OPTS_nb_seqs_set _PRINT;
  }
  if(OPTS_nb_seqs_set_max != -1){
    PRINT_ "\tMAXIMUM NUMBER OF SEQUENCES BY SET : %i\n",
      OPTS_nb_seqs_set_max _PRINT;
  }
  if(OPTS_nb_seqs_set_min != -1){
    PRINT_ "\tMINIMUM NUMBER OF SEQUENCES BY SET : %i\n",
      OPTS_nb_seqs_set_min _PRINT;
  }
  
  if(OPTS_min_org_set != 0){
    PRINT_ "\tMINIMUM NUMBER OF DIFFERENT ORGANISMS BY SET : %i\n",
      OPTS_min_org_set _PRINT;
  }
  if(OPTS_min_species_set != 0){
    PRINT_ "\tMINIMUM NUMBER OF DIFFERENT SPECIES BY SET : %i\n",
      OPTS_min_species_set _PRINT;
  }


  PRINT_ "\n" _PRINT;
  
}



/**
 *
 * \fn void print_verbose_end_write_GFF3(int nb_files)
 *
 * \brief Function used to print the ending message on screen concerning
 * the writing of result GFF3 files
 *
 * \param[in] nb_files : the number of GFF3 files written
 *
 * This function will print the ending message on screen concerning
 * the writing of result GFF3 files (number of files written)
 *
 */


void print_verbose_end_write_GFF3(int nb_files)
{

  PRINT_ "GFF3 FILES WRITTEN : %i\n",nb_files  _PRINT; 
  PRINT_ "\n%s\n", LINE _PRINT;
}



/**
 *
 * \fn void print_verbose_start_write_multifasta()
 *
 * \brief Function used to print the starting message on screen concerning
 * the writing of result multifasta files
 *
 * This function will print the starting message on screen concerning
 * the writing of result multifasta files
 *
 */


void print_verbose_start_write_multifasta()
{
  PRINT_ "WRITE SETS OF SEQUENCES IN MULTIFASTA FORMAT\n\n" _PRINT;
  PRINT_ "\tOUTPUT DIR : %s\n",OPTS_output_dir _PRINT;
  if(OPTS_keep_too_close != -1){
    PRINT_ "\tOUTPUT DIR FOR SETS CONTAINING TOO CLOSE SEQUENCES : %s\n",
      OPTS_output_dir_too_close _PRINT;
  }
  if(OPTS_nb_seqs_set != -1){
    PRINT_ "\tNUMBER OF SEQUENCES WANTED BY SET : %i\n",
      OPTS_nb_seqs_set _PRINT;
  }
  if(OPTS_nb_seqs_set_max != -1){
    PRINT_ "\tMAXIMUM NUMBER OF SEQUENCES BY SET : %i\n",
      OPTS_nb_seqs_set_max _PRINT;
  }
  if(OPTS_nb_seqs_set_min != -1){
    PRINT_ "\tMINIMUM NUMBER OF SEQUENCES BY SET : %i\n",
      OPTS_nb_seqs_set_min _PRINT;
  }
  if(OPTS_min_org_set != 0){
    PRINT_ "\tMINIMUM NUMBER OF DIFFERENT ORGANISMS BY SET : %i\n",
      OPTS_min_org_set _PRINT;
  }
  if(OPTS_min_species_set != 0){
    PRINT_ "\tMINIMUM NUMBER OF DIFFERENT SPECIES BY SET : %i\n",
      OPTS_min_species_set _PRINT;
  }
  PRINT_ "\n" _PRINT;
  

}


/**
 *
 * \fn void print_verbose_end_write_multifasta(int nb_files)
 *
 * \brief Function used to print the ending message on screen concerning
 * the writing of result multifasta files
 *
 * \param[in] nb_files : the number of multifasta files written
 *
 * This function will print the ending message on screen concerning
 * the writing of result multifasta files (number of files written)
 *
 */


void print_verbose_end_write_multifasta(int nb_files)
{

  PRINT_ "MULTIFASTA FILES WRITTEN : %i\n",nb_files  _PRINT; 
  PRINT_ "\n%s\n", LINE _PRINT;

}


/**
 *
 * \fn void print_verbose_end_process(long execution_time)
 *
 * \brief Function used to print the ending message
 *
 * \param[in] execution_time : the execution time spent
 *
 * This function will print the ending message (with execution time)
 *
 */

void print_verbose_end_process(long execution_time){

 PRINT_ "PROCESS FINISHED\n" _PRINT;

 /*Execution time < 1s : print time in milliseconds*/
 if(execution_time <= 1000){

   PRINT_ "\tEXECUTION TIME : %ldms\n\n",execution_time _PRINT;

 }
 /**Execution time > 1s : print time in seconds and milliseconds*/
 else{
   
   PRINT_ "\tEXECUTION TIME : %lis%ldms\n\n",
     execution_time/1000,
     execution_time%1000 _PRINT;


 }


}
