/*
 * This file is part of CGseqcore.
 *
 *  CGseqcore is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  CGseqcore is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with CGseqcore.  If not, see <http://www.gnu.org/licenses/>.
 */


/**
 *
 * \file io.h
 *
 * \author Benjamin Grenier-Boley <benjamin.grenier-boley@inria.fr>
 *
 * \brief Header file that defines the IN/OUT definitions and functions 
 *
 * \version 1.0
 *
 * \date July 2009
 *
 *
 * This file will define all I/O functions used in the software
 *
 *
 */


#ifndef __IO_H__
#define __IO_H__

/*Basic inclusions*/
#include <stdio.h>

/*Intern inclusions*/
#include "types.h"


/******************************************************************************
 **   Define the standard outputs
 **     
 *****************************************************************************/


/**
 *
 * \def DISPLAY_OUT
 *
 * \brief Define the standard output
 *
 * This definition defines the standard output used in this software
 *
 */

#define DISPLAY_OUT stdout


/**
 *
 * \def DISPLAY_ERR
 *
 * \brief Define the standard output for error
 *
 * This definition defines the standard error output used in this software
 *
 */

#define DISPLAY_ERR stderr




/******************************************************************************
 **   Define a print on screen definition to display messages
 **     
 *****************************************************************************/

/**
 *
 * \def PRINT_
 *
 * \brief Define the beginning of the fprintf function to display messages
 *
 * This definition redefines the beginning of the fprintf function linked 
 * with a fflush function to display messages on screen
 *
 */


#define PRINT_ fprintf(DISPLAY_OUT,

/**
 *
 * \def _PRINT
 *
 * \brief Define the end of the fprintf function to display messages
 *
 * This definition redefines the end of the fprintf function linked 
 * with a fflush function to display messages on screen
 *
 */

#define _PRINT ); fflush(DISPLAY_OUT)




/******************************************************************************
 **   Define a print on screen definition to display error messages 
 **     
 *****************************************************************************/

/**
 *
 * \def ERROR_
 *
 * \brief Define the beginning of the fprintf function to display error
 * messages
 *
 * This definition redefines the beginning of the fprintf function linked 
 * with a fflush function to display error messages on screen
 *
 */

#define ERROR_ fprintf(DISPLAY_ERR,"\n\nERROR: ");fprintf(DISPLAY_ERR,


/**
 *
 * \def _ERROR
 *
 * \brief Define the end of the fprintf function
 *
 * This definition redefines the end of the fprintf function linked 
 * with a fflush function to display messages on screen
 *
 */

#define _ERROR ); fflush(DISPLAY_ERR)


/******************************************************************************
 **   I/O constants
 **     
 *****************************************************************************/

/**
 *
 * \def LINE
 *
 * \brief Define a line made by "-"
 *
 * This definition defines a line represented by "-"
 *
 */

#define LINE "-------------------------------------------------------------------------------------------\n"



/******************************************************************************
 **   Read/Write functions 
 **     
 *****************************************************************************/

/**
 *
 * \fn void read_file(FILE *fd, char **content)
 *
 * \brief Function used to read a file and get its content
 *
 * \param[in] fd : the file descriptor of the reading file
 * \param[out] content : the string representation of the file content
 *
 * This function will be used to read a file and return its content
 *
 */


extern void read_file(FILE *fd, char **content);


/**
 *
 * \fn int write_GFF3_star(p_main_structure m)
 *
 * \brief Function used to write GFF3 files for the star mode
 *
 * \param[in] m : the main structure containing all info needed
 *
 * \return the number of files written
 *
 * This function will produce as many GFF3 as the number of sets of 
 * sequences discovered for the star mode
 *
 */


extern int write_GFF3_star(p_main_structure m);



/**
 *
 * \fn int write_GFF3_clique(p_main_structure m)
 *
 * \brief Function used to write GFF3 files for the clique modes
 *
 * \param[in] m : the main structure containing all info needed
 *
 * \return the number of files written
 *
 * This function will produce as many GFF3 as the number of sets of 
 * sequences discovered for the clique modes
 *
 */


extern int write_GFF3_clique(p_main_structure m);


/**
 *
 * \fn int write_GFF3(p_main_structure m)
 *
 * \brief Function used to write GFF3 files
 *
 * \param[in] m : the main structure containing all info needed
 *
 * \return the number of files written
 *
 * This function will produce as many GFF3 as the number of sets of 
 * sequences discovered
 *
 */


extern int write_GFF3(p_main_structure m);


/**
 *
 * \fn int write_multifasta_star(p_main_structure m)
 *
 * \brief Function used to write multifasta files for the star mode
 *
 * \param[in] m : the main structure containing all info needed
 *
 * \return the number of files written
 *
 * This function will produce as many multifasta as the number of sets of 
 * sequences discovered for the star mode
 *
 */


extern int write_multifasta_star(p_main_structure m);


/**
 *
 * \fn int write_multifasta_clique(p_main_structure m)
 *
 * \brief Function used to write multifasta files for the clique modes
 *
 * \param[in] m : the main structure containing all info needed
 *
 * \return the number of files written
 *
 * This function will produce as many multifasta as the number of sets of 
 * sequences discovered for the clique modes
 *
 */

extern int write_multifasta_clique(p_main_structure m);


/**
 *
 * \fn int write_multifasta(p_main_structure m)
 *
 * \brief Function used to write multifasta files
 *
 * \param[in] m : the main structure containing all info needed
 *
 * \return the number of files written
 * 
 * This function will produce as many multifasta as the number of sets of 
 * sequences discovered
 *
 */


extern int write_multifasta(p_main_structure m);



/******************************************************************************
 **   Verbose functions 
 **     
 *****************************************************************************/


/**
 *
 * \fn void print_verbose_start_process()
 *
 * \brief Function used to print starting message on screen
 *
 * This function will print on screen the starting message 
 *
 */


extern void print_verbose_start_process();


/**
 *
 * \fn void print_verbose_start_sequences()
 *
 * \brief Function used to print the starting message on screen concerning
 * the parsing of the sequences
 *
 * This function will print the starting message on screen concerning
 * the parsing of the sequences
 *
 */


extern void print_verbose_start_sequences();



/**
 *
 * \fn void print_verbose_end_sequences(sequence_array *array_of_seqs)
 *
 * \brief Function used to print the ending message on screen concerning
 * the parsing of the sequences
 *
 * \param[in] array_of_seqs : the array of extracted sequences
 *
 * This function will print the ending message on screen concerning
 * the parsing of the sequences (number of sequences, their names and 
 * lengths)
 *
 */


extern void print_verbose_end_sequences(sequence_array *array_of_seqs);



/**
 *
 * \fn void print_verbose_start_alignments()
 *
 * \brief Function used to print the starting message on screen concerning
 * the parsing of the alignments
 *
 * This function will print the starting message on screen concerning
 * the parsing of the alignments
 *
 */


extern void print_verbose_start_alignments();


/**
 *
 * \fn void print_verbose_start_alignments_file(char *filename)
 *
 * \brief Function used to print the starting message on screen concerning
 * the parsing of the alignments in the current file
 *
 * \param[in] filename : the name of the current alignment file
 *
 * This function will print the starting message on screen concerning
 * the parsing of the alignments in the current file
 *
 */


extern void print_verbose_start_alignments_file(char *filename);


/**
 *
 * \fn void print_verbose_end_alignments_file(int nb_al)
 *
 * \brief Function used to print the starting message on screen concerning
 * the parsing of the alignments in the current file
 *
 * \param[in] nb_al : number of alignment found in the current file
 *
 * This function will print the ending message on screen concerning
 * the parsing of the alignments in the current file (number of alignments
 * found)
 *
 */


extern void print_verbose_end_alignments_file(int nb_al);


/**
 *
 * \fn void print_verbose_end_alignments(int nb_al)
 *
 * \brief Function used to print the ending message on screen concerning
 * the parsing of the alignments
 *
 * \param[in] nb_al : the number of alignments found
 *
 * This function will print the ending message on screen concerning
 * the parsing of the alignments (number of alignments found)
 *
 */


extern void print_verbose_end_alignments(int nb_al);


/**
 *
 * \fn void print_verbose_start_score(sequence_array *seqs)
 *
 * \brief Function used to print the starting message on screen concerning
 * the score calculation
 *
 * \param[in] seqs : the sequences array used to link probabilities with
 * the genome name
 *
 * This function will print the starting message on screen concerning
 * the score calculation
 *
 */


extern void print_verbose_start_score(sequence_array *seqs);

/**
 *
 * \fn void print_verbose_end_score(float mean)
 *
 * \brief Function used to print the ending message on screen concerning
 * the score calculation
 *
 * \param[in] mean : the mean number of alignments found by position
 *
 * This function will print the ending message on screen concerning
 * the score calculation (mean number of alignments)
 *
 */


extern void print_verbose_end_score(float mean);


/**
 *
 * \fn void print_verbose_start_conserved_regions()
 *
 * \brief Function used to print the starting message on screen concerning
 * the searching for conserved regions
 *
 * This function will print the starting message on screen concerning
 * the searching of the conserved regions
 *
 */


extern void print_verbose_start_conserved_regions();


/**
 *
 * \fn void print_verbose_end_conserved_regions(int nb_sets)
 *
 * \brief Function used to print the ending message on screen concerning
 * the searching for conserved regions
 *
 * \param[in] nb_sets : the number of conserved regions found
 *
 * This function will print the ending message on screen concerning
 * the searching of the conserved regions (number of regions found)
 *
 */


extern void print_verbose_end_conserved_regions(int nb_sets);


/**
 *
 * \fn void print_verbose_start_build_sets()
 *
 * \brief Function used to print the starting message on screen concerning
 * the building sets step
 *
 * This function will print the starting message on screen concerning
 * the building sets step
 *
 */


extern void print_verbose_start_build_sets();


/**
 *
 * \fn void print_verbose_end_build_sets(int nb_sets)
 *
 * \brief Function used to print the ending message on screen concerning
 * the building sets step
 *
 * \param[in] nb_sets : the number of sets built
 *
 * This function will print the ending message on screen concerning
 * the building sets step (number of sets built)
 *
 */


extern void print_verbose_end_build_sets(int nb_sets);



/**
 *
 * \fn void print_verbose_start_compare_ref()
 *
 * \brief Function used to print the starting message on screen concerning
 * the comparison step of all queries against the reference
 *
 * This function will print the starting message on screen concerning
 * the comparison step of all queries against the reference
 *
 */


extern void print_verbose_start_compare_ref();

/**
 *
 * \fn void print_verbose_start_compare_ref_set(int nb_set, int nb_seqs)
 *
 * \brief Function used to print the starting message on screen concerning
 * the comparison step of all queries against the reference for the current
 * set
 *
 * \param[in] nb_set : the number of set processed
 * \param[in] nb_seqs : the number of sequences in the set (ref + queries)
 *
 * This function will print the starting message on screen concerning
 * the comparison step of all queries against the reference for the current
 * set (nb_set). It also print the number of sequences in the set
 *
 */


extern void print_verbose_start_compare_ref_set(int nb_set, int nb_seqs);



/**
 *
 * \fn void print_verbose_end_compare_ref_set(int nb_seqs, int is_too_close)
 *
 * \brief Function used to print the ending message on screen concerning
 * the comparison step of all queries against the reference for the current
 * set
 *
 * \param[in] nb_seqs : the number of seqs in the set after the comparison
 * (ref + query)
 * \param[in] is_too_close : boolean used when the OPTS_keep_too_close
 * option is used
 *
 * This function will print the ending message on screen concerning
 * the comparison step of all queries against the reference for the current
 * set
 *
 */


extern void print_verbose_end_compare_ref_set(int nb_seqs, int is_too_close);


/**
 *
 * \fn void print_verbose_sort_identity_ref()
 *
 * \brief Function used to print the message on screen concerning the
 * sorting of queries by identities
 *
 * This function will print the message on screen concerning the
 * sorting of queries by identities
 *
 */


extern void print_verbose_sort_identity_ref();


/**
 *
 * \fn void print_verbose_end_compare_ref()
 *
 * \brief Function used to print the ending message on screen concerning
 * the comparison step of all queries against the reference
 *
 * This function will print the ending message on screen concerning
 * the comparison step of all queries against the reference
 *
 */


extern void print_verbose_end_compare_ref();


/**
 *
 * \fn void print_verbose_start_check_repetitions()
 *
 * \brief Function used to print the message on screen concerning the
 * checking of species repetitions
 *
 * This function will print the message on screen concerning the
 * checking of species repetitions
 *
 */


extern void print_verbose_start_check_repetitions();


/**
 *
 * \fn void print_verbose_start_checking_repetitions_set(int nb_set, 
 *                                                       int nb_seqs)
 *
 * \brief Function used to print the starting message on screen concerning
 * the species repetitions for the current set
 *
 * \param[in] nb_set : the number of set processed
 * \param[in] nb_seqs : the number of sequences in the set (ref + queries)
 *
 * This function will print the starting message on screen concerning
 * the species repetitions for the current set (nb_set). 
 * It also print the number of sequences in the set
 *
 */


extern void print_verbose_start_checking_repetitions_set(int nb_set, int nb_seqs);


/**
 *
 * \fn void print_verbose_end_checking_repetitions_set(int nb_seqs)
 *
 * \brief Function used to print the ending message on screen concerning
 * the species repetitions for the current set
 *
 * \param[in] nb_seqs : the number of sequences in the set (ref + queries)
 *
 * This function will print the ending message on screen concerning
 * the species repetitions for the current set by printing the number of 
 * sequences in the set
 *
 */


extern void print_verbose_end_checking_repetitions_set(int nb_seqs);



/**
 *
 * \fn void print_verbose_end_check_repetitions()
 *
 * \brief Function used to print the ending message on screen concerning the
 * checking of species repetitions
 *
 * This function will print the ending message on screen concerning the
 * checking of species repetitions
 *
 */


extern void print_verbose_end_check_repetitions();



/**
 *
 * \fn void print_verbose_start_search_naive_clique()
 *
 * \brief Function used to print the starting message on screen concerning
 * the comparison step of the naive clique search
 *
 * This function will print the starting message on screen concerning
 * the comparison step of of the naive clique search
 *
 */


extern void print_verbose_start_search_naive_clique();


/**
 *
 * \fn void print_verbose_start_naive_clique_set(int nb_set, int nb_seqs)
 *
 * \brief Function used to print the starting message on screen concerning
 * the comparison step of searching the naive clique for the current
 * set
 *
 * \param[in] nb_set : the number of set processed
 * \param[in] nb_seqs : the number of sequences in the set (ref + queries)
 *
 * This function will print the starting message on screen concerning
 * the comparison step of searching the naive clique for the current
 * set (nb_set). It also print the number of sequences in the set
 *
 */


extern void print_verbose_start_naive_clique_set(int nb_set, int nb_seqs);



/**
 *
 * \fn void print_verbose_end_naive_clique_set(int nb_seqs)
 *
 * \brief Function used to print the ending message on screen concerning
 * the comparison step of searching the naive clique for the current
 * set
 *
 * \param[in] nb_seqs : the number of seqs in the set after the naive
 * clique serach (ref + query)
 *
 * This function will print the ending message on screen concerning
 * the comparison step of searching the naive clique for the current
 * set
 *
 */


extern void print_verbose_end_naive_clique_set(int nb_seqs);


/**
 *
 * \fn void print_verbose_end_search_naive_clique()
 *
 * \brief Function used to print the ending message on screen concerning
 * the comparison step of the naive clique search
 *
 * This function will print the ending message on screen concerning
 * the comparison step of of the naive clique search
 *
 */


extern void print_verbose_end_search_naive_clique();


/**
 *
 * \fn void print_verbose_no_connect(int nb)
 *
 * \brief Function used to print the the message when a set of sequences
 * has no connected sequence
 *
 * \param nb : the number of the set concerned
 *
 * This function will print the the message when a set of sequences
 * has no connected sequence
 *
 */

extern void print_verbose_no_connect(int nb);



/**
 *
 * \fn void print_verbose_start_search_naive_clique()
 *
 * \brief Function used to print the starting message on screen concerning
 * the comparison step of the maxi clique search
 *
 * This function will print the starting message on screen concerning
 * the comparison step of of the maxi clique search
 *
 */


extern void print_verbose_start_search_maxi_clique();


/**
 *
 * \fn void print_verbose_start_maxi_clique_set(int nb_set, int nb_seqs)
 *
 * \brief Function used to print the starting message on screen concerning
 * the comparison step of searching the maxi clique for the current
 * set
 *
 * \param[in] nb_set : the number of set processed
 * \param[in] nb_seqs : the number of sequences in the set (ref + queries)
 *
 * This function will print the starting message on screen concerning
 * the comparison step of searching the maxi clique for the current
 * set (nb_set). It also print the number of sequences in the set
 *
 */


extern void print_verbose_start_maxi_clique_set(int nb_set, int nb_seqs);



/**
 *
 * \fn void print_verbose_end_maxi_clique_set(int nb_seqs)
 *
 * \brief Function used to print the ending message on screen concerning
 * the comparison step of searching the maxi clique for the current
 * set
 *
 * \param[in] nb_seqs : the number of seqs in the set after the maxi
 * clique serach (ref + query)
 *
 * This function will print the ending message on screen concerning
 * the comparison step of searching the maxi clique for the current
 * set
 *
 */


extern void print_verbose_end_maxi_clique_set(int nb_seqs);


/**
 *
 * \fn void print_verbose_end_search_naive_clique()
 *
 * \brief Function used to print the ending message on screen concerning
 * the comparison step of the maxi clique search
 *
 * This function will print the ending message on screen concerning
 * the comparison step of of the maxi clique search
 *
 */


extern void print_verbose_end_search_maxi_clique();


/**
 *
 * \fn void print_verbose_start_write_GFF3()
 *
 * \brief Function used to print the starting message on screen concerning
 * the writing of result GFF3 files
 *
 * This function will print the starting message on screen concerning
 * the writing of result GFF3 files
 *
 */


extern void print_verbose_start_write_GFF3();


/**
 *
 * \fn void print_verbose_end_write_GFF3(int nb_files)
 *
 * \brief Function used to print the ending message on screen concerning
 * the writing of result GFF3 files
 *
 * \param[in] nb_files : the number of GFF3 files written
 *
 * This function will print the ending message on screen concerning
 * the writing of result GFF3 files (number of files written)
 *
 */


extern void print_verbose_end_write_GFF3(int nb_files);


/**
 *
 * \fn void print_verbose_start_write_multifasta()
 *
 * \brief Function used to print the starting message on screen concerning
 * the writing of result multifasta files
 *
 * This function will print the starting message on screen concerning
 * the writing of result multifasta files
 *
 */


extern void print_verbose_start_write_multifasta();


/**
 *
 * \fn void print_verbose_end_write_multifasta(int nb_files)
 *
 * \brief Function used to print the ending message on screen concerning
 * the writing of result multifasta files
 *
 * \param[in] nb_files : the number of multifasta files written
 *
 * This function will print the ending message on screen concerning
 * the writing of result multifasta files (number of files written)
 *
 */


extern void print_verbose_end_write_multifasta(int nb_files);



/**
 *
 * \fn void print_verbose_end_process(long execution_time)
 *
 * \brief Function used to print the ending message
 *
 * \param[in] execution_time : the execution time spent
 *
 * This function will print the ending message (with execution time)
 *
 */


extern void print_verbose_end_process(long execution_time);

#endif
