/*
 * This file is part of CGseqcore.
 *
 *  CGseqcore is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  CGseqcore is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with CGseqcore.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 *
 * \file macros.c
 *
 * \author Benjamin Grenier-Boley <benjamin.grenier-boley@inria.fr>
 *
 * \brief File that defines all macros definitions and generic 
 * functions used  inthe software
 *
 * \version 1.0
 *
 * \date July 2009
 *
 *
 * This file will redefine all macros used in this software like realloc,
 * free, max, min  etc. and also define generic functions
 *
 *
 */

/*Intern inclusions*/
#include "macros.h"
#include "sequence.h"
#include "alignment.h"
#include "options.h"
#include "carnac.h"
#include "io.h"


/**
 *
 * \fn void destroy_main_structure(p_main_structure m)
 *
 * \brief Function used to destroy all structures allocated
 *
 * \param[in,out] m : the main structure to destroy
 *
 * That function will destroy all allocated memory according to mode used
 *
 */

void destroy_main_structure(p_main_structure m)
{

  /*Indexes*/
  int i,j;

  if(m){
 
    /*Destroy the sequences*/
    if(m->array_of_seqs){
      for(i = 0; i < m->array_of_seqs->nb_seqs; i++) {
	destroy_sequence(m->array_of_seqs->seqs[i]);
      }
      DESTROY(m->array_of_seqs->seqs);
      DESTROY(m->array_of_seqs);
    
    }
  
    /*Destroy the alignments*/
    if(m->array_of_al_info){
      for(i = 0; i < m->array_of_al_info->nb_al_info; i++){
	destroy_alignment(m->array_of_al_info->al_info[i]);
      }
      DESTROY(m->array_of_al_info->al_info);
      DESTROY(m->array_of_al_info);

    }

    /*Destroy the new scores*/
    if(m->array_of_scores){
      if(m->array_of_scores->positions){
	for(i = 0; i < m->array_of_scores->length_ps; i++){
	  DESTROY(m->array_of_scores->positions[i]);
	}
	DESTROY(m->array_of_scores->positions);
      }
      if(m->array_of_scores->scores){
	DESTROY(m->array_of_scores->scores);
      }
      if(m->array_of_scores->target){
	DESTROY(m->array_of_scores->target);
      }
      if(m->array_of_scores->background){
	DESTROY(m->array_of_scores->background);
      }
      if(m->array_of_scores->genomes_proba){
	DESTROY(m->array_of_scores->genomes_proba);
      }
      DESTROY(m->array_of_scores);
    }


    /*Destroy interesting regions*/
    if(m->interesting_regions){
      if(m->interesting_regions->intervals){
	for(i = 0; i < m->interesting_regions->nb_intervals; i++){
	  if(m->interesting_regions->intervals[i]){
	    DESTROY(m->interesting_regions->intervals[i]);
	  }
	}
	DESTROY(m->interesting_regions->intervals);
      }
      DESTROY(m->interesting_regions);
    }

    /*Destroy sets of sequences*/
    if(m->sets_of_sequences){
      if(m->sets_of_sequences->sets){
	for(i = 0; i < m->sets_of_sequences->nb_sets; i++){
	  for(j = 0; j < m->sets_of_sequences->sets[i]->nb_q_intervals; j++){
	    if(m->sets_of_sequences->sets[i]->q_intervals[j]){
	      if(m->sets_of_sequences->sets[i]->q_intervals[j]->seq != NULL){
		DESTROY(m->sets_of_sequences->sets[i]->q_intervals[j]->seq);
	      }
	      if(m->sets_of_sequences->sets[i]->q_intervals[j]->name != NULL){
		DESTROY(m->sets_of_sequences->sets[i]->q_intervals[j]->name);
	      }
	      

	      DESTROY(m->sets_of_sequences->sets[i]->q_intervals[j]);
	    }
	  }	
	  DESTROY(m->sets_of_sequences->sets[i]->q_intervals);
	  DESTROY(m->sets_of_sequences->sets[i]->ref_interval);
	  if(m->sets_of_sequences->sets[i]->ref_seq != NULL){
	    DESTROY(m->sets_of_sequences->sets[i]->ref_seq);
	  }

	  /*Destroying clique if built*/
	  if(m->sets_of_sequences->sets[i]->clique_set){
	    if(m->sets_of_sequences->sets[i]->clique_set->clique_index){
	      DESTROY(m->sets_of_sequences->sets[i]->clique_set->clique_index);
	    }
	    DESTROY(m->sets_of_sequences->sets[i]->clique_set);
	  }

	  /*Destroying neighbors if built*/
	  if(m->sets_of_sequences->sets[i]->neighbors){
	    for(j = 0; j < m->sets_of_sequences->sets[i]->nb_q_intervals; j++){
	      if(m->sets_of_sequences->sets[i]->neighbors[j]){
		if(m->sets_of_sequences->sets[i]->neighbors[j]->indexes){
		  DESTROY(m->sets_of_sequences->sets[i]->neighbors[j]->indexes);
		}
		DESTROY(m->sets_of_sequences->sets[i]->neighbors[j]);
	      }
	    }
	    DESTROY(m->sets_of_sequences->sets[i]->neighbors);
	  }

	
	  DESTROY(m->sets_of_sequences->sets[i]);
	}
	DESTROY(m->sets_of_sequences->sets);    
      }
      DESTROY(m->sets_of_sequences);
    }


    /*Destroy the regexp of carnac if used*/
    if(OPTS_carnac){
      destroy_regex();
    }


    /*Destroy the main object itself*/
    DESTROY(m);
  }
}



/**
 *
 *\fn int is_present_int_array(int num, int *array, int length)
 *
 *\brief Function used to check if a number is present in an array of
 *integers
 *
 *\param[in] num : the number to search for
 *\param[in] array : the array of integers
 *\param[in] length : length of the array
 *
 *\return 1 if the number was found, 0 otherwise
 *
 *That function will check if a number is present in an array of integres
 *and return 1 if  it was found and 0 in the other case
 *
 */


int is_present_int_array(int num, int *array, int length){
  
  /*Index*/
  int i; 

  /*Search over the array*/
  for(i = 0; i < length; i++){
    if(num == array[i]){
      return 1;
    }
  }

  return 0;
}


/**
 *
 * \fn int is_present_char_array(char *string, char_array *array)
 *
 * \brief Function used to check if a string is present in a char_array
 *
 * \param[in] string : the string to search for
 * \param[in] array : the char_array
 *
 * \return 1 if the string was found, 0 otherwise
 *
 * That function will check if a string is present in an array of strings
 * (char_array) and return 1 if  it was found and 0 in the other case
 *
 */

int is_present_char_array(char *string, char_array *array){
  
  /*Index*/
  int i; 

  /*Search over the array*/
  for(i = 0; i < array->nb_chars; i++){
    if(strcmp(string,array->chars[i]) == 0){
      return 1;
    }
  }
  return 0;
}


/**
 *
 * \fn void copy_interval_query(p_interval_query interval_1,
 *                              p_interval_query interval_2)
 * 
 * \brief Function used to copy two interval queries
 *
 * \param[in,out] interval_1 : the interval to copy in
 * \param[in] interval_2 : the interval to copy
 *
 * That function will copy all the attributes of the interavl_2 in the 
 * first one
 *
 */


void copy_interval_query(p_interval_query interval_1,
			 p_interval_query interval_2){



  interval_1->pos_start = interval_2->pos_start;
  interval_1->pos_end = interval_2->pos_end;
  interval_1->index = interval_2->index;
  if(interval_2->seq){
    NEW(interval_1->seq,
	char,
	strlen(interval_2->seq) + 1);
    strcpy(interval_1->seq, interval_2->seq);
  }
  else{
    interval_1->seq = NULL;
  }
  if(interval_2->name){
    NEW(interval_1->name,
	char,
	strlen(interval_2->name) + 1);
    strcpy(interval_1->name, interval_2->name);

  }
  else{
    interval_2->name = NULL;
  }

  interval_1->strand = interval_2->strand;
  interval_1->id_ref = interval_2->id_ref;
  

}


/**
 *
 * \fn int compare_sort_id(const void *a, const void *b)
 *
 * \brief Function used to compare two p_interval_query by their ref 
 * identity
 *
 * \param[in] a :first query to compare
 * \param[in] b : second query to compare
 *
 * \return -1 if "a" has a lower id ref than "b", 0 if equal and -1
 * otherwise
 *
 * Function used to compare two p_interval_query by their ref 
 * identity
 *
 *
 */

int compare_sort_id(const void *a, const void *b)
{
  /*Args retrievment*/
  p_interval_query *arg1 = (p_interval_query *) a;
  p_interval_query *arg2 = (p_interval_query *) b;
  /*Position comparison*/
  if((*arg1)->id_ref < (*arg2)->id_ref) return 1;
  else{ 
    if ((*arg1)->id_ref > (*arg2)->id_ref){
      return -1;
    }
    /*Sort according to lexicographical order if same identity*/
    else {
      if(strcmp((*arg1)->name,(*arg2)->name) < 0){
	return -1;
      }
      else{
	if(strcmp((*arg1)->name,(*arg2)->name) > 0){
	  return 1;
	}
	else{
	  /*If same identity and same name : sort by position*/
	  if((*arg1)->pos_start < (*arg2)->pos_start){
	    return 1;
	  }
	  else{
	    if((*arg1)->pos_start > (*arg2)->pos_start){
	      return -1;
	    }
	    else{
	      return 0;
	    }
	  }
	  return 0;
	}
      }
    }
  }
}


/**
 *
 * \fn void get_size_number(int num, int *size)
 *
 * \brief Function used to get the number of chars taken to represent an 
 * integer into a string
 *
 * \param[in] num : the number to consider
 * \param[in,out] size : the calculated size
 *
 * This function will be used to get the number of chars taken to 
 * represent a integer into a string
 *
 */

void get_size_number(int num, int *size)
{

  /*According to the number get the size*/
  if(num < 10){
    *size = 1;
  }
  else{
    if(num < 100){
      *size = 2;
    }
    else{
      if(num < 1000){
	*size = 3;
      }
      else{
	if(num < 10000){
	  *size = 4;
	}
	else{
	  if(num < 100000){
	    *size = 5;
	  }
	  else{
	    *size = 6;
	  }
	}
      }
    }
  }

}


/**
 *
 * \fn void int_to_binary(int number, int length, char *binary)
 *
 * \brief Function used to get the string representation of an integer in
 * its binary format
 *
 * \param[in] number : the number to convert
 * \param[in] length : represents the length of the result string. Indeed
 * that length will depend on the number of genomes (minus reference) and
 * take into account the end character of strings
 * \param[in,out] binary : the string representation of its binary value
 *
 * This function will get the string representation of an integer in
 * its binary format according to its value. Note that the binary string 
 * has to be already allocated.
 *
 */


void int_to_binary(int number, int length, char *binary)
{
  
  /*Index of shift bit*/
  int i = 0;

  /*Index of the current position in the binary string : the end just
   before the '/0' character*/
  int j = length - 2;
  
  /*Process conversion*/
  while(i < length - 1){

    if ( (number >> i) & 1){
      binary[j] = '1';
    }
    else{
      binary[j]='0';
    }
    i++;
    j--; 
  }
      
  /*End character*/
  binary[length - 1] = '\0';

}
