/*
 * This file is part of CGseqcore.
 *
 *  CGseqcore is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  CGseqcore is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with CGseqcore.  If not, see <http://www.gnu.org/licenses/>.
 */


/**
 *
 * \file macros.h
 *
 * \author Benjamin Grenier-Boley <benjamin.grenier-boley@inria.fr>
 *
 * \brief Header file that defines all macros definitions and generic 
 * functions
 *
 * \version 1.0
 *
 * \date July 2009
 *
 *
 * This file will redefine all macros used in this software like realloc,
 * free, max, min  etc. and also define generic functions
 *
 *
 */


#ifndef __MACROS_H__
#define __MACROS_H__

/*Basic inclusions*/
#include <stdlib.h>
#include <assert.h>
#include <string.h>

/*Intern inclusions*/
#include "types.h"


/******************************************************************************
 **   Redefine the calloc, realloc and free functions
 **     
 *****************************************************************************/


/**
 *
 * \def NEW(Ref, Type, Nb)
 *
 * \brief Redefine the calloc function
 *
 * \param[in,out] Ref : The pointer that needs to be allocated
 * \param[in] Type : Type of the pointer
 * \param[in] Nb : size that needs to be allocated
 *
 * This definition defines the calloc function on the pointer "Ref" given 
 * of type "Type" and size "Nb" linked with an assert function
 *
 */


#define NEW(Ref, Type, Nb) \
  Ref = (Type*)calloc((Nb), sizeof(Type));	\
  assert(Ref != NULL)


/**
 *
 * \def RENEW(Ref, Type, Nb)
 *
 * \brief Redefine the realloc function
 *
 * \param[in,out] Ref : The pointer that needs to be reallocated
 * \param[in] Type : Type of the pointer
 * \param[in] Nb : Total size that needs to be reallocated
 *
 * This definition defines the realloc function on the pointer "Ref" given 
 * of type "Type" and size "Nb" linked with an assert function
 *
 */


#define RENEW(Ref, Type, Nb) \
  Ref = (Type*)realloc(Ref, (Nb) * sizeof(Type)); \
  assert(Ref != NULL)



/**
 *
 * \def DESTROY(Ref)
 *
 * \brief Redefine the free function
 *
 * \param[in,out] Ref : The pointer that needs to be destroyed
 *
 * This definition defines the free function on the pointer "Ref" given
 *
 */


#define DESTROY(Ref) \
  free(Ref); \
  Ref = NULL



/******************************************************************************
 **   Define the MIN, MAX functions
 **     
 *****************************************************************************/


/**
 *
 * \def MAX(Aa, Bb)
 *
 * \brief Defines a maximum function between 2 numbers
 *
 * \param[in] Aa : First number to compare
 * \param[in] Bb : Second number to compare
 *
 * \return The max number between the 2 numbers
 *
 * This definition defines a function that will determine the maximum
 * number between two numbers
 *
 */


#define MAX(Aa, Bb) \
  ((Aa > Bb) ? (Aa) : (Bb))



/**
 *
 * \def MIN(Aa, Bb)
 *
 * \brief Defines a minimum function between 2 numbers
 *
 * \param[in] Aa : First number to compare
 * \param[in] Bb : Second number to compare
 *
 * \return The min number between the 2 numbers
 *
 * This definition defines a function that will determine the minimum
 * number between two numbers
 *
 */


#define MIN(Aa, Bb) \
  ((Aa < Bb) ? (Aa) : (Bb))



/**
 *
 * \def MAX3(Aa, Bb)
 *
 * \brief Defines a maximum function between 3 numbers
 *
 * \param[in] Aa : First number to compare
 * \param[in] Bb : Second number to compare
 * \param{in] Cc : Third number to compare
 *
 * \return The max number between the 3 numbers
 *
 * This definition defines a function that will determine the maximum
 * number between three numbers
 *
 */
	    

#define MAX3(Aa, Bb, Cc)			\
  ((Aa < Bb) ?					\
   ((Cc < Bb) ? (Bb) : (Cc))			\
   :						\
   ((Cc < Aa) ? (Aa) : (Cc)))



/******************************************************************************
 **   Define generic functions
 **     
 *****************************************************************************/


/**
 *
 * \fn void destroy_main_structure(p_main_structure m)
 *
 * \brief Function used to destroy all structures allocated
 *
 * \param[in,out] m : the main structure to destroy
 *
 * That function will destroy all allocated memory according to mode used
 *
 */


extern void destroy_main_structure(p_main_structure m);



/**
 *
 *\fn int is_present_int_array(int num, int *array, int length)
 *
 *\brief Function used to check if a number is present in an array of
 *integers
 *
 *\param[in] num : the number to search for
 *\param[in] array : the array of integers
 *\param[in] length : length of the array
 *
 *\return 1 if the number was found, 0 otherwise
 *
 *That function will check if a number is present in an array of integres
 *and return 1 if  it was found and 0 in the other case
 *
 */

extern int is_present_int_array(int num, int *array, int length);



/**
 *
 * \fn int is_present_char_array(char *string, char_array *array)
 *
 * \brief Function used to check if a string is present in a char_array
 *
 * \param[in] string : the string to search for
 * \param[in] array : the char_array
 *
 * \return 1 if the string was found, 0 otherwise
 *
 * That function will check if a string is present in an array of strings
 * (char_array) and return 1 if  it was found and 0 in the other case
 *
 */

extern int is_present_char_array(char *string, char_array *array);


/**
 *
 * \fn void copy_interval_query(p_interval_query interval_1,
 *                              p_interval_query interval_2)
 * 
 * \brief Function used to copy two interval queries
 *
 * \param[in,out] interval_1 : the interval to copy in
 * \param[in] interval_2 : the interval to copy
 *
 * That function will copy all the attributes of the interavl_2 in the 
 * first one
 *
 */

extern void copy_interval_query(p_interval_query interval_1,
				p_interval_query interval_2);



/**
 *
 * \fn int compare_sort_id(const void *a, const void *b)
 *
 * \brief Function used to compare two p_interval_query by their ref 
 * identity
 *
 * \param[in] a :first query to compare
 * \param[in] b : second query to compare
 *
 * \return -1 if "a" has a lower id ref than "b", 0 if equal and -1
 * otherwise
 *
 * Function used to compare two p_interval_query by their ref 
 * identity
 *
 *
 */

extern int compare_sort_id(const void *a, const void *b);



/**
 *
 * \fn void get_size_number(int num, int *size)
 *
 * \brief Function used to get the number of chars taken to represent an 
 * integer into a string
 *
 * \param[in] num : the number to consider
 * \param[in,out] size : the calculated size
 *
 * This function will be used to get the number of chars taken to 
 * represent a integer into a string
 *
 */

extern void get_size_number(int num, int *size);


/**
 *
 * \fn void int_to_binary(int number, int length, char *binary)
 *
 * \brief Function used to get the string representation of an integer in
 * its binary format
 *
 * \param[in] number : the number to convert
 * \param[in] length : represents the length of the result string. Indeed
 * that length will depend on the number of genomes (minus reference) and
 * take into account the end character of strings
 * \param[in,out] binary : the string representation of its binary value
 *
 * This function will get the string representation of an integer in
 * its binary format according to its value. Note that the binary string 
 * has to be already allocated.
 *
 */


extern void int_to_binary(int number, int length, char *binary);


#endif
