/*
 * This file is part of CGseqcore.
 *
 *  CGseqcore is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  CGseqcore is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with CGseqcore.  If not, see <http://www.gnu.org/licenses/>.
 */


/**
 *
 * \file main.c
 *
 * \author Benjamin Grenier-Boley <benjamin.grenier-boley@inria.fr>
 *
 * \brief File that defines the main software function
 *
 * \version 1.0
 *
 * \date July 2009
 *
 *
 * This file represents the main entrance of the software and contains the
 * main software algorithm
 *
 *
 */


/*Basic Inclusions*/
#include <sys/time.h>
#include <math.h>
#include <unistd.h>
#include <time.h>

/*Intern inclusions*/
#include "io.h"
#include "types.h"
#include "clique.h"
#include "macros.h"
#include "options.h"
#include "sequence.h"
#include "alignment.h"
#include "score.h"
#include "clique.h"


/**
 *
 * \fn int main(int argc, char **argv)
 *
 * \brief Main function
 *
 * \param[in] argc : number of parameters
 * \param[in] argv : the parameters
 *
 * \return an integer representing the main exit status
 *
 * This function contains the main step algorithm of the software
 *
 *
 */

      

int main(int argc, char **argv) 
{
 
  /*Status*/
  int status = 1;
  /*Number of result files*/
  int nb_files = 0;

  /*Start time of the process*/
  struct timeval *start_time = NULL;
  /*End time of the process*/
  struct timeval *end_time = NULL;
  /*Execution time spent in milliseconds*/
  long execution_time;

  /*Generate random numbers using time*/
  srand(time(NULL));

  /*Main sofwtare structure that will contain all information*/
  p_main_structure main_struct = NULL;

  NEW(start_time, struct timeval, 1);
  NEW(end_time, struct timeval, 1);

  /*Get start time*/
  gettimeofday(start_time,NULL);

  /*Get the parameters*/
  status = parse_params(argc, argv);

  /*Check the returned status*/
  if(status == 2){
    usage();
  }
  else{
    /*Check parameters validity*/
    if(!check_parameters_validity()){
      usage();
    }
    else{
      /*Create the main structure*/
      NEW(main_struct, main_structure_s, 1);
      NEW(main_struct->array_of_seqs, sequence_array, 1);
      NEW(main_struct->array_of_al_info, alignment_info_array,1);
      NEW(main_struct->array_of_scores, score_array, 1);
      NEW(main_struct->interesting_regions, interval_array, 1);
      NEW(main_struct->sets_of_sequences, sets_of_seqs_array, 1);

      main_struct->array_of_seqs->nb_seqs = 0;
      main_struct->array_of_al_info->nb_al_info = 0;
      main_struct->interesting_regions->nb_intervals = 0;
      main_struct->sets_of_sequences->nb_sets = 0;

      if(OPTS_verbose){
	print_verbose_start_process();
	print_verbose_start_sequences();
      }
     

      /*Get the sequences from files*/
      get_sequences(main_struct->array_of_seqs);

      /*Complete unknown attributes like species, put the sequence
       in uppercase, replace "space" by "_" in sequence name etc.*/
      if(main_struct->array_of_seqs->seqs != NULL){
	complete_sequences_attributes(main_struct->array_of_seqs);
      }

      /*Sequence parser error -> stop*/
      if(main_struct->array_of_seqs->seqs != NULL){
	
	if(OPTS_verbose){
	  print_verbose_end_sequences(main_struct->array_of_seqs);
	}

	if(OPTS_verbose){
	  print_verbose_start_alignments();
	}
	
	/*Read the alignments file(s) and get the information*/
	read_alignments_files(main_struct);

	/*If alignments were found*/
	if(main_struct->array_of_al_info->al_info != NULL){

	  /*Complete missing info about alignments (strand, query/subject,
	    indexes and sorting)*/
	  status = complete_alignments_attributes(main_struct->array_of_al_info,
						  main_struct->array_of_seqs);
	  if(status == 0){
	    ERROR_ "An error occured during completing alignment files : process will stop\n\n"
	      _ERROR;
 	  }
	  else{
	    if(OPTS_verbose){
	      print_verbose_end_alignments(main_struct->array_of_al_info->
					   nb_al_info);
	      print_verbose_start_score(main_struct->array_of_seqs);
	    }
	    

	    /*Calculate the target array according to the OPTS_proba*/
	    main_struct->array_of_scores->length_tb =
	      pow(2, file_positions->integers[file_positions->nb_int - 1] + 1);

	    main_struct->array_of_scores->nb_other_genomes =
	      main_struct->array_of_seqs->nb_seqs - 1;

	    NEW(main_struct->array_of_scores->target,
		float,
		main_struct->array_of_scores->length_tb);
	    calculate_target_array(main_struct);

	    /*Calculate the alignments by position*/
	    NEW(main_struct->array_of_scores->positions,
		char *,
		main_struct->array_of_seqs->seqs[0]->length);
	    main_struct->array_of_scores->length_ps =
	      main_struct->array_of_seqs->seqs[0]->length;
	    calculate_al_function(main_struct);
	
	    /*Calculate background array and the genomes background
	     probability*/
	    NEW(main_struct->array_of_scores->genomes_proba,
		float,
		OPTS_proba->nb_float);
	    NEW(main_struct->array_of_scores->background,
		float,
		main_struct->array_of_scores->length_tb);
	    calculate_background_array(main_struct);

    
	    /*Calculate score function*/
	    NEW(main_struct->array_of_scores->scores,
		float,
		main_struct->array_of_scores->length_ps);
	    calculate_score_function(main_struct);

	    /*Determine the intervals of interesting regions*/
	    get_interesting_regions(main_struct);

	    if(OPTS_verbose){
	      print_verbose_end_conserved_regions(main_struct->
						  interesting_regions->
						  nb_intervals);
	      print_verbose_start_build_sets();
	    }


	    /*Get the corresponding sets of sequences*/
	    get_sets_of_sequences(main_struct);
	  
	    if(OPTS_verbose){
	      print_verbose_end_build_sets(main_struct->
					   sets_of_sequences->
					   nb_sets);
	    }

	    /*Compare for all sets, the queries against the reference
	      to refine the bounds and eliminate those who don't respect
	      the identity thresholds*/

	    if(OPTS_verbose){
	      print_verbose_start_compare_ref();
	    }
	
	    compare_all_against_ref(main_struct);
	  
	    if(OPTS_verbose){
	      print_verbose_sort_identity_ref();
	    }

	    sort_identity_ref(main_struct);

  
	    /*Check species repetitions if wanted*/
	    if(OPTS_max_species_repet != -1){
	      if(OPTS_verbose){
		print_verbose_start_check_repetitions();
	      }

	      check_species_repetitions(main_struct);

	      if(OPTS_verbose){
		print_verbose_end_check_repetitions();
	      }
	    
	    }

	    if(OPTS_verbose){
	      print_verbose_end_compare_ref();
	    }


	    /*Naive clique search*/
	    if(OPTS_selecting_mode == 1){
	    
	      if(OPTS_verbose){
		print_verbose_start_search_naive_clique();
	      }

	      search_naive_clique(main_struct);

	      if(OPTS_verbose){
		print_verbose_end_search_naive_clique();
	      }


	    }
	    else{
	      /*Maxi clique search*/
	      if(OPTS_selecting_mode == 2){
	      
		if(OPTS_verbose){
		  print_verbose_start_search_maxi_clique();
		}

		maxi_clique_search(main_struct);


		if(OPTS_verbose){
		  print_verbose_end_search_maxi_clique();
		}
	      }
	    }
	    /*Write GFF3 output*/
	    if(OPTS_output_format == 1){
	  
	      if(OPTS_verbose){
		print_verbose_start_write_GFF3();

	      }

	      nb_files = write_GFF3(main_struct);
  
	      if(OPTS_verbose){
		print_verbose_end_write_GFF3(nb_files);
	      }
	    }
	    /*Multifasta output*/
	    else{
	      if(OPTS_verbose){
		print_verbose_start_write_multifasta();
	      
	      }
	    
	      nb_files = write_multifasta(main_struct);
	    
	      if(OPTS_verbose){
		print_verbose_end_write_multifasta(nb_files);
	      }



	    }


	    if(OPTS_verbose){

	      gettimeofday(end_time, NULL);

	      /*Execution time calculation*/
	      execution_time = ((end_time->tv_sec - start_time->tv_sec) *
				1000L) +
		((end_time->tv_usec - start_time->tv_usec)/1000L) ;
	    
	      print_verbose_end_process(execution_time);
	    }

	  }
	}

      }

    }
 
    /*Destroy main structure*/
    destroy_main_structure(main_struct);
    /*Destroy options*/
    destroy_options();

    
   
  }

  DESTROY(start_time);
  DESTROY(end_time);

  return status;


}

