/*
 * This file is part of CGseqcore.
 *
 *  CGseqcore is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  CGseqcore is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with CGseqcore.  If not, see <http://www.gnu.org/licenses/>.
 */



/**
 *
 * \file options.c
 *
 * \author Benjamin Grenier-Boley <benjamin.grenier-boley@inria.fr>
 *
 * \brief File that defines the software options
 *
 * \version 1.0
 *
 * \date July 2009
 *
 *
 * This file will define all options that can be used in the software
 *
 *
 */


/*Basic inclusions*/
#include <getopt.h>
#include <dirent.h>

/*Intern inclusions*/
#include "options.h"
#include "macros.h"
#include "io.h"

/******************************************************************************
 **
 **                         MAIN OPTIONS
 **     
 *****************************************************************************/

/******************************************************************************
 **   Reference file
 **     
 *****************************************************************************/

char *OPTS_ref_file = NULL; 


/******************************************************************************
 **   Other species provided other files and directories
 **     
 *****************************************************************************/

char_array *OPTS_other_files = NULL;


/******************************************************************************
 **   Array of other file sequences in fasta format
 **     
 *****************************************************************************/

char_array *OPTS_file_array = NULL;
int_array *file_positions = NULL;


/******************************************************************************
 **   Output directory (Default : current directory)
 **     
 *****************************************************************************/

char *OPTS_output_dir = "./";


/******************************************************************************
 **   Output directory for too close sets (if OPTS_keep_too_close)
 **   (Default : too_close inside current directory
 **     
 *****************************************************************************/

char *OPTS_output_dir_too_close = "too_close/";


/******************************************************************************
 **   Output format (Default : GFF3 ( = 1))
 **     
 *****************************************************************************/

int OPTS_output_format = 1; 


/******************************************************************************
 **   Verbose option (Default : no)
 **     
 *****************************************************************************/

int OPTS_verbose = 0;


/******************************************************************************
 **   Maximum number of sequences to write by set option (Default -1 -> not used)
 **     
 *****************************************************************************/

int OPTS_nb_seqs_set = -1;


/******************************************************************************
 **
 **                         ALIGNMENTS SELECTION
 **     
 *****************************************************************************/

/******************************************************************************
 **   Maximum E-value to consider an alignment (Default : not used)
 **     
 *****************************************************************************/

float OPTS_evalue = -1;


/******************************************************************************
 **   Minimum identity to consider an alignment (Default : not used)
 **     
 *****************************************************************************/

float OPTS_min_id_alignment = 0.0; 


/******************************************************************************
 **   Minimum length to consider an alignment (Default : not used)
 **     
 *****************************************************************************/

int OPTS_min_length_alignment = 0;



/******************************************************************************
 **
 **                         FINDING CONSERVED REGIONS OPTIONS
 **     
 *****************************************************************************/



/******************************************************************************
 **   Lambda parameter : the paramter lambda used in the calculation of
 **   scores by position (Default : 1)
 **     
 *****************************************************************************/

int OPTS_lambda = 1; 


/******************************************************************************
 **   Probabilities parameter : the probability used in the new score
 **   calculation. And the probability by default (default : 0.8). The string
 **   will contain the probabilities as typed by the user : e.i : 0.8,1.0,0.5
 **     
 *****************************************************************************/

float_array *OPTS_proba = NULL;

float default_proba = 0.8;

char *probabilities = NULL;


/******************************************************************************
 **   Array of alignments files
 **     
 *****************************************************************************/

char_array *OPTS_alignments = NULL;


/******************************************************************************
 **   Minimal length of a region (Default : 30)
 **     
 *****************************************************************************/

int OPTS_min_length_region = 30;

/******************************************************************************
 **   Maximal length of a region (Default : 1000)
 **     
 *****************************************************************************/

int OPTS_max_length_region = 1000;


/******************************************************************************
 **   Filter on species (Default : No)
 **     
 *****************************************************************************/

int OPTS_filter_species = 0;


/******************************************************************************
 **   Length to extend a region up to (Default : 60bp)
 **     
 *****************************************************************************/

int OPTS_extend_length_region = 60;


/******************************************************************************
 **
 **                         SELECTING SEQUENCES
 **     
 *****************************************************************************/


/******************************************************************************
 **   Minimum identity threshold (Default 50%)
 **     
 *****************************************************************************/

int OPTS_min_id = 50; 


/******************************************************************************
 **   Maximum identity threshold (Default 100%)
 **     
 *****************************************************************************/

int OPTS_max_id = 100; 


/******************************************************************************
 **   Max number of sequences by set (reference + queries) (Default -1 -> not used
 **     
 *****************************************************************************/

int OPTS_nb_seqs_set_max = -1; 


/******************************************************************************
 **   Min number of sequences by set (reference + queries) (Default -1 -> not used)
 **     
 *****************************************************************************/

int OPTS_nb_seqs_set_min = -1; 


/******************************************************************************
 **   Selecting mode (Default 0 -> star)
 **     
 *****************************************************************************/

int OPTS_selecting_mode = 0; 


/******************************************************************************
 **   Maximum number of repetitions by species (Default -1 -> not used)
 **     
 *****************************************************************************/

int OPTS_max_species_repet = -1;  


/******************************************************************************
 **   Carnac selected parameter (Default 0 -> not used)
 **     
 *****************************************************************************/

int OPTS_carnac = 0;


/******************************************************************************
 **   Minimum number of different organisms in output files
 **     
 *****************************************************************************/

int OPTS_min_org_set = 0;


/******************************************************************************
 **   Minimum number of different species by outfile (Default 0 -> not used)
 **     
 *****************************************************************************/

int OPTS_min_species_set = 0; 


/******************************************************************************
 **   Carnac
 **     
 *****************************************************************************/

char *PROG_CARNAC = NULL;


/******************************************************************************
 **   Identity threshold to consider a too close sequence  (Defaut : Not used)
 **     
 *****************************************************************************/

int OPTS_keep_too_close = -1;





/******************************************************************************
 **
 **                         FUNCTIONS
 **     
 *****************************************************************************/



/**
 *
 * \fn void usage()
 *
 * \brief Function that defines the software options
 *
 * This function will be printed on the screen to help users deal with 
 * options. It will be printed also when user's options are not correct
 *
 *
 */


void usage()
{

  fprintf(DISPLAY_ERR,"\nUSAGE:\n");
  fprintf(DISPLAY_ERR,"  " PACKAGE_NAME " -r <reference file> -f <other sequence file or directory> <alignments \"blast tabulated\" files> [options]\n\n");
  fprintf(DISPLAY_ERR,"     -h        Display this help\n");
  fprintf(DISPLAY_ERR,"     -v        Verbose\n");
  fprintf(DISPLAY_ERR,"     -r <str>  Reference file in fasta format\n");
  fprintf(DISPLAY_ERR,"     -f <str>  Other genome file or directory\n");
  fprintf(DISPLAY_ERR,"               Note that :\n");
  fprintf(DISPLAY_ERR,"                - You have to use that option for every files or directories provided\n"); 
  fprintf(DISPLAY_ERR,"                - If a directory is provided : every fasta or multifasta within it will be read\n");
  fprintf(DISPLAY_ERR,"                - Every fasta or multifasta file in a directory provided will be considered as part of the same species.\n");
  fprintf(DISPLAY_ERR,"                - Each separated fasta or directory will be considered as part from different species\n");
  fprintf(DISPLAY_ERR,"     -d <str>  Directory output (Default : write in current directory)\n");
  fprintf(DISPLAY_ERR,"     -o <int>  Output format :\n");
  fprintf(DISPLAY_ERR,"              1 : GFF3 (Default)\n");
  fprintf(DISPLAY_ERR,"              2 : Multifasta\n");
  fprintf(DISPLAY_ERR,"     -m <int>  Maximum number of sequences to write by set (ref + queries) (Default : all)\n");
  fprintf(DISPLAY_ERR,"     -D <str>  Directory ouptut for too close sets (Default : write inside a directory named \"too_close\" created in current directory)\n");
  fprintf(DISPLAY_ERR,"               Note that this option is only used when the -k option is set\n\n");
  fprintf(DISPLAY_ERR,"  1) Alignments selection from blast tabulated files:\n");
  fprintf(DISPLAY_ERR,"     -E <real> Maximum E-value to consider an alignment (Default : not used)\n");
  fprintf(DISPLAY_ERR,"     -t <real> Minimum identity to consider an alignment (Default : not used)\n");
  fprintf(DISPLAY_ERR,"     -g <int>  Minimum length to consider an alignment (Default : not used)\n\n");
  fprintf(DISPLAY_ERR,"  2) Find conserved regions from alignments options :\n");
  fprintf(DISPLAY_ERR,"     -p <str>  Probabilities for each other species or sequence separated by \",\" provided by the \"-f\" option (Default : 0.8 for each sequence)\n");
  fprintf(DISPLAY_ERR,"               Note that you must give as many probabilities than genomes and/or directory provided with the \"-f\" option\n");
  fprintf(DISPLAY_ERR,"     -L <int>  Maximal length of a conserved region (Default : 1000)\n");
  fprintf(DISPLAY_ERR,"     -l <int>  Minimal length of a conserved region (Default : 30)\n");
  fprintf(DISPLAY_ERR,"     -R <int>  Maximum number of repetitions by species (Default : not used)\n\n");
  fprintf(DISPLAY_ERR,"  3) Select sets and/or sequences according to parameters options :\n");
  fprintf(DISPLAY_ERR,"     -M <int>  Selecting mode\n");
  fprintf(DISPLAY_ERR,"              0 : star (Default)\n");
  fprintf(DISPLAY_ERR,"              1 : naive clique\n");
  fprintf(DISPLAY_ERR,"              2 : maxi clique\n");
  fprintf(DISPLAY_ERR,"     -i <int>  Minimum identity threshold in percent (Default : 50)\n");
  fprintf(DISPLAY_ERR,"     -I <int>  Maximum identity threshold in percent (Default : 100)\n");
  fprintf(DISPLAY_ERR,"     -S <int>  Minimum number of different species by set (Default : not used)\n");
  fprintf(DISPLAY_ERR,"               If set to -1 : \n");
  fprintf(DISPLAY_ERR,"                - force each species provided to be present in the set if the number of species is less than the number of sequences wanted by set (option -m)\n");
  fprintf(DISPLAY_ERR,"              Or\n");
  fprintf(DISPLAY_ERR,"                - force each sequences in the set to be part of different species if the number of species is greater or equal to the number of sequences by set (option -m)\n");
  fprintf(DISPLAY_ERR,"     -O <int>  Minimum number of different organisms by set (Default : not used)\n");
  fprintf(DISPLAY_ERR,"     -N <int>  Maximum number of sequences by set (ref + queries) (Default : not used)\n");
  fprintf(DISPLAY_ERR,"     -n <int>  Minimum number of sequences by set (ref + queries) (Default : not used)\n");  
  fprintf(DISPLAY_ERR,"     -c        Common secondary structure shared using Carnac (Default : not used)\n");
  fprintf(DISPLAY_ERR,"     -k <int>  Keep sets with only too close sequences according to that threshold identity (Default : not used)\n");
  fprintf(DISPLAY_ERR,"               Note that this option is only available for the star mode (-M 0)\n");
  fprintf(DISPLAY_ERR,"               Note also that these too close sets will be placed under a directory named according to the -D option\n\n");
  
}



/**
 *
 * \fn int parse_params(int argc, char **argv)
 *
 * \brief Function used to parse user parameters
 *
 * \param[in] argc : number of parameters
 * \param[in] argv : the parameters
 *
 * \return an integer representing the status : \n
 * 1 -> exit success \n
 * 0 -> an error occured
 *
 * This function will parse all user parameters and will check for their 
 * validity
 *
 *
 */

int parse_params(int argc, char **argv)
{
  /*Status*/
  int status = 1;
  /*The temporary character read*/
  char char_read;
  /*Number of parameters read*/
  int nb_param = 0;
  /*Index*/
  int i;

  /*Don't print automatic error messages*/
  opterr = 0;

  /*Read the parameters*/
 
  while(((char_read = 
	  getopt(argc, 
		 argv, 
		 "hvcr:f:d:L:l:i:m:I:N:n:M:R:o:E:t:g:S:O:p:k:D:")) != EOF) 
	&&
	(status != 2)){

    switch(char_read) {
      /*Help case*/
    case 'h':     
      status = 2;
      nb_param += 1;
      break;

      /*Verbose case*/
    case 'v':     
      OPTS_verbose = 1;
      nb_param += 1;
      break;

      /*Output format*/
    case 'o':
      OPTS_output_format = atoi(optarg);
      if((OPTS_output_format < 1) ||
	 (OPTS_output_format > 2)){
	ERROR_ "The output format must be 1 (GFF3) or 2(Multifasta)\n\n" _ERROR;
	status = 2;
      }
      nb_param += 2;
      break;
      
      /*Reference case*/
    case 'r' :
      if(OPTS_ref_file == NULL){
	NEW(OPTS_ref_file, char, strlen(optarg) + 1);
	strcpy(OPTS_ref_file, optarg);
      }
      else{
	ERROR_ "You must provide only one reference\n\n" _ERROR;
	status = 2;
      }
      nb_param += 2;
      break;

      /*Other files or directories case*/
    case 'f' : 
      if(OPTS_other_files == NULL){
	NEW(OPTS_other_files,
	    char_array,
	    1);
	OPTS_other_files->chars = NULL;
	OPTS_other_files->nb_chars = 0;
      }
      RENEW(OPTS_other_files->chars,
	    char *,
	    OPTS_other_files->nb_chars + 1);
      NEW(OPTS_other_files->chars[OPTS_other_files->nb_chars],
	  char,
	  strlen(optarg) + 1);
      strcpy(OPTS_other_files->chars[OPTS_other_files->nb_chars],
	     optarg);
      OPTS_other_files->nb_chars++;
      nb_param += 2;
      break;

      /*Maximum number of sequences to write*/
    case 'm':
      OPTS_nb_seqs_set = atoi(optarg);
      if(OPTS_nb_seqs_set < 2){
	ERROR_ "The maximum number of sequences to write by set must be above 2 (ref + 1 query)\n\n" 
	  _ERROR;
	status = 2;
      }
      nb_param += 2;
      break;

     /*Evalue case*/
    case 'E' : 
    OPTS_evalue = atof(optarg);
      if(OPTS_evalue < 0){
	ERROR_ "The maximum E-value must be higher or equal to 0\n\n" _ERROR;
	status = 2;
      }
      nb_param += 2;
      break;
 
      /*Minimum alignment identity case*/
    case 't' : 
    OPTS_min_id_alignment = atof(optarg);
    if((OPTS_min_id_alignment < 0.0) || (OPTS_min_id_alignment > 100.0 )){
	ERROR_ "The minimum alignment identity must be between 0 and 100\n\n" _ERROR;
	status = 2;
      }
      nb_param += 2;
      break;
      
      /*Minimum length identity case*/
    case 'g' : 
    OPTS_min_length_alignment = atoi(optarg);
    if(OPTS_min_length_alignment <=0 ){
	ERROR_ "The minimum alignment length must be positive\n\n" _ERROR;
	status = 2;
      }
      nb_param += 2;
      break;


      /*Probabilities case*/
    case 'p' :
      NEW(probabilities,char, strlen(optarg) + 1);
      strcpy(probabilities, optarg);
      nb_param += 2;
      break;


      /*Directory output case*/
    case 'd' :
      /*Check if there's a "/" at the end of the optarg*/
      if((optarg[strlen(optarg) - 1] == '/') ||
	 (optarg[strlen(optarg) - 1] == '\\')){
	NEW(OPTS_output_dir, char, strlen(optarg) + 1);
	strcpy(OPTS_output_dir, optarg);
      }
      else{
	NEW(OPTS_output_dir, char, strlen(optarg) + 2);
	sprintf(OPTS_output_dir,
		"%s%c",
		optarg,
		'/');
      }
      
      nb_param += 2;
      break;

      /*Too close directory output case*/
    case 'D' :
      /*Check if there's a "/" at the end of the optarg*/
      if((optarg[strlen(optarg) - 1] == '/') ||
	 (optarg[strlen(optarg) - 1] == '\\')){
	NEW(OPTS_output_dir_too_close, char, strlen(optarg) + 1);
	strcpy(OPTS_output_dir_too_close, optarg);
      }
      else{
	NEW(OPTS_output_dir_too_close, char, strlen(optarg) + 2);
	sprintf(OPTS_output_dir_too_close,
		"%s%c",
		optarg,
		'/');
      }
      
      nb_param += 2;
      break;

      /*Maximal length of a conserved region case*/
    case 'L' :
      OPTS_max_length_region = atoi(optarg);
      if(OPTS_max_length_region <= 0){
	ERROR_ "The maximal length of a conserved region must be positive\n\n" _ERROR;
	status = 2;
      }
      nb_param += 2;
      break;

      /*Minimal length of a conserved region case*/
    case 'l' :
      OPTS_min_length_region = atoi(optarg);
      if(OPTS_min_length_region <= 0){
	ERROR_ "The minimal length of a conserved region must be positive\n\n" _ERROR;
	status = 2;
      }
      nb_param += 2;
      break;

      /*Carnac case*/
    case 'c' :
      OPTS_carnac = 1;
      nb_param += 1;
      break;

      /*Minimum identity threshold case*/
    case 'i' :
      OPTS_min_id = atoi(optarg);
      if((OPTS_min_id <= 0) || (OPTS_min_id >= 100)){
	ERROR_ "The minimal identity threshold must be bewteen 0 and 99\n\n" _ERROR;
	status = 2;
      }
      nb_param += 2;
      break;

      /*Maximum identity threshold case*/
    case 'I' :
      OPTS_max_id = atoi(optarg);
      if((OPTS_max_id <= 1) || (OPTS_max_id > 100)){
	ERROR_ "The maximal identity threshold must be between 1 and 100\n\n" _ERROR;
	status = 2;
      }
      nb_param += 2;
      break;

      /*Selecting mode case*/
    case 'M' :
      OPTS_selecting_mode = atoi(optarg);
      if((OPTS_selecting_mode < 0) || (OPTS_selecting_mode > 2)){
	ERROR_ "The selecting mode must be 0 (star), 1(naive clique) or 2 (maxi clique)\n\n" _ERROR;
	status = 2;
      }
      nb_param += 2;
      break;

      /*Maximum number of sequences by set case*/
    case 'N' :
      OPTS_nb_seqs_set_max = atoi(optarg);
      if(OPTS_nb_seqs_set_max <= 1){
	ERROR_ "The maximum number of sequences wanted by set must be above 2 (ref + 1 query)\n\n" _ERROR;
	status = 2;
      }
      nb_param += 2;
      break;


      /*Minimum number of sequences by set case*/
    case 'n' :
      OPTS_nb_seqs_set_min = atoi(optarg);
      if(OPTS_nb_seqs_set_min <= 1){
	ERROR_ "The minimum number of sequences wanted by set must be above 2 (ref + 1 query)\n\n" _ERROR;
	status = 2;
      }
      nb_param += 2;
      break;

      /*Minimum number of different organisms by set case*/
    case 'O' :
      OPTS_min_org_set = atoi(optarg);
      if(OPTS_min_org_set < 2){
	ERROR_ "The minimum number of different organisms wanted by set must be at least two\n\n" _ERROR;
	status = 2;
      }
      nb_param += 2;
      break;

      /*Minimum number of different species by set case*/
    case 'S' :
      OPTS_min_species_set = atoi(optarg);
      if((OPTS_min_species_set < 2) && (OPTS_min_species_set != -1)){
	ERROR_ "The minimum number of different species wanted by set must be at least two\n\n" _ERROR;
	status = 2;
      }
      nb_param += 2;
      break;

      /*Maximum number of repetitions by organism*/
    case 'R' :
      OPTS_max_species_repet = atoi(optarg);
      if(OPTS_max_species_repet < 1){
	ERROR_ "The maximum number of repeptitions by species must be at least 1\n\n" _ERROR;
	status = 2;
      }
      nb_param += 2;
      break;   

      /*Keep too close sets*/
    case 'k' :
      OPTS_keep_too_close = atoi(optarg);
      if((OPTS_keep_too_close <= 1) || (OPTS_keep_too_close > 100)){
	ERROR_ "The threshold for too close sequence must be between 1 and 100\n\n" 
	  _ERROR;
	status = 2;
      }
      nb_param += 2;
      break;

      /*Other cases*/
    case '?' :
      if ((optopt == 'o') || (optopt == 'r') || (optopt == 'f') ||
	  (optopt == 'd') || (optopt == 'l') ||
	  (optopt == 'L') || (optopt == 'i') || (optopt == 'I') ||
	  (optopt == 'n') || (optopt == 'R') || (optopt == 'E') ||
	  (optopt == 't') || (optopt == 'g') || (optopt == 'S') ||
	  (optopt == 'O') || (optopt == 'N') || (optopt == 'p') ||
	  (optopt == 'D') || (optopt == 'k')){
	ERROR_ "Option -%c requires an argument\n\n",optopt _ERROR;
      }
      else{
	ERROR_ "Unrecognized option -%c\n\n",optopt _ERROR;
      }
      status = 2;
      break;
    }

   
  }

  if(status != 2){

    /*Initialize the array*/
    NEW(OPTS_alignments, char_array, 1);
    OPTS_alignments->chars = NULL;
    OPTS_alignments->nb_chars = 0;

    /*Check if there are alignments files*/
    if((argc - nb_param - 1) == 0){
      ERROR_ "You must give alignment files\n" _ERROR;
      status = 2;
    }
    else{
   
      /*Get all alignments files*/
      for(i = 0; i < argc - nb_param -1; i++){
	if(argv[optind + i] != NULL){
	  RENEW(OPTS_alignments->chars, 
		char*, 
		OPTS_alignments->nb_chars + 1);
	  NEW(OPTS_alignments->chars[OPTS_alignments->nb_chars],
	      char,
	      strlen(argv[optind + i]) + 1);
	  strcpy(OPTS_alignments->chars[OPTS_alignments->nb_chars],
		 argv[optind + i]);
	  OPTS_alignments->nb_chars++;
	}
      }

    }
 
  }


  /*Return the status*/
  return status;

}


/**
 *
 * \fn int check_parameters_validity
 *
 * \brief Function used to check parameters entered by the user
 *
 * \return an integer representing the validity : \n
 * 1 -> exit success \n
 * 0 -> an error occured
 *
 * This function will check parameters validity and returns the result of
 * the checking
 *
 *
 */

int check_parameters_validity()
{

  /*Index*/
  int i;
  /*File descriptor*/
  FILE *fd;
  /*String used to parse probabilities*/
  char *s_parse = NULL;
  /*Long used in the probabilities parsing*/
  long last_pos = 0;
  /*String used in the probability parsing*/
  char *temp_proba = NULL;
  /*Structure used to read directories*/
  struct dirent * ent;
  /*Array of filename considered to be fasta or multifasta*/
  char_array *fasta_filenames = NULL;
  /*File argument counter*/
  int file_count = 0;
  /*Directory to create*/
  char *dir_to_create = NULL;
  /*Command line to create a directory*/
  char *command_line_dir = NULL;
  /*Error code for the commande line*/
  int err_code;


  /*Fasta or multifasta files : .fa .mfas .fasta .fna .fas .seq .dna*/
  NEW(fasta_filenames, char_array, 1);
  fasta_filenames->nb_chars = 7;
  NEW(fasta_filenames->chars, char *, fasta_filenames->nb_chars);
  NEW(fasta_filenames->chars[0], char, 4);
  strcpy(fasta_filenames->chars[0],".fa");
  NEW(fasta_filenames->chars[1], char, 5);
  strcpy(fasta_filenames->chars[1],".fas");
  NEW(fasta_filenames->chars[2], char, 5);
  strcpy(fasta_filenames->chars[2],".fna");
  NEW(fasta_filenames->chars[3], char, 7);
  strcpy(fasta_filenames->chars[3],".fasta");
  NEW(fasta_filenames->chars[4], char, 6);
  strcpy(fasta_filenames->chars[4],".mfas");
  NEW(fasta_filenames->chars[5], char, 5);
  strcpy(fasta_filenames->chars[5],".seq");
  NEW(fasta_filenames->chars[6], char, 5);
  strcpy(fasta_filenames->chars[6],".dna");


  /*Check if there's at least one reference file and one alignment file*/
  if(OPTS_ref_file == NULL){
    ERROR_ "You must give a reference file\n\n" _ERROR;
    return 0;
  }

  if((OPTS_other_files->chars == NULL) ||
     (OPTS_other_files->nb_chars == 0)){
    ERROR_ "You must give at least one other file\n\n" _ERROR; 
    return 0;
  }

  /*Check if reference file provided exist*/
  fd = fopen(OPTS_ref_file, "r");
  if (fd == NULL){
    ERROR_ "The reference file %s doesn't exist\n\n",OPTS_ref_file _ERROR;
    return 0;
  }
  fclose(fd);
 
 
  /*Get the other sequence file(s)*/
  NEW(OPTS_file_array, char_array, 1);
  OPTS_file_array->chars = NULL;
  OPTS_file_array->nb_chars = 0;
  NEW(file_positions,int_array,1);
  file_positions->nb_int = 0;
  file_positions->integers = NULL;

  for(i = 0; i < OPTS_other_files->nb_chars; i++){
    RENEW(OPTS_file_array->chars, 
	  char*,
	  OPTS_file_array->nb_chars + 1);
    NEW(OPTS_file_array->chars[OPTS_file_array->nb_chars], 
	char,
	strlen(OPTS_other_files->chars[i]) + 1);
    strcpy(OPTS_file_array->chars[OPTS_file_array->nb_chars],
	   OPTS_other_files->chars[i]);

    /*Check if it's a directory : get all fasta files within it*/
    DIR *fdir = opendir(OPTS_file_array->chars[OPTS_file_array->nb_chars]);
    char *temp_dir_name = NULL;
    int length = 0;
    NEW(temp_dir_name,
	char,
	strlen(OPTS_file_array->chars[OPTS_file_array->nb_chars]) + 1);
    strcpy(temp_dir_name,OPTS_file_array->chars[OPTS_file_array->nb_chars]);
    length = strlen(temp_dir_name);
    if(temp_dir_name[length - 1] != '/'){
      RENEW(temp_dir_name,
	    char,
	    length + 2);
      temp_dir_name[length] = '/';
      temp_dir_name[length + 1] = '\0';
    } 

    if (fdir != NULL){
      DESTROY(OPTS_file_array->chars[OPTS_file_array->nb_chars]);
      while((ent = readdir(fdir)) != NULL){
	if((strcmp(ent->d_name,"./")) != 0 && ( strcmp(ent->d_name,"../") != 0)){
	  char *ext = strrchr(ent->d_name,'.');
	  if(ext != NULL){
	    /*If it's a fasta file add it to the file array*/
	    if(is_present_char_array(ext, fasta_filenames)){
	      RENEW(OPTS_file_array->chars,
		    char*,
		    OPTS_file_array->nb_chars + 1);
	      NEW(OPTS_file_array->chars[OPTS_file_array->nb_chars],
		  char,
		  strlen(ent->d_name) + strlen(temp_dir_name) + 1);
	      sprintf(OPTS_file_array->chars[OPTS_file_array->nb_chars],
		      "%s%s",
		      temp_dir_name,
		      ent->d_name);
	      OPTS_file_array->nb_chars++;
	      RENEW(file_positions->integers, int, file_positions->nb_int + 1);
	      file_positions->integers[file_positions->nb_int] = file_count;
	      file_positions->nb_int++;
	    }
	  }
	}

      }

    }
    else{
      OPTS_file_array->nb_chars++;
      RENEW(file_positions->integers, int, file_positions->nb_int + 1);
      file_positions->integers[file_positions->nb_int] = file_count;  
      file_positions->nb_int++;
    }
    closedir(fdir);
    DESTROY(temp_dir_name);
    file_count++;    
  }
 


  /*Check if other species file(s) exists*/
  for(i = 0; i < OPTS_file_array->nb_chars; i++){
    fd = fopen(OPTS_file_array->chars[i], "r");
    if(fd == NULL){
      ERROR_ "The sequence file %s doesn't exist\n\n",
	OPTS_file_array->chars[i] _ERROR;
      return 0;

    }
    fclose(fd);
  }

  /*Get the probabilities*/
  NEW(OPTS_proba, float_array, 1);
  OPTS_proba->floats = NULL;
  OPTS_proba->nb_float = 0;

  if(probabilities != NULL){

    last_pos = 0;
    s_parse = strchr(probabilities, ',');
        
    while(s_parse != NULL){
      
      RENEW(OPTS_proba->floats,
	    float,
	    OPTS_proba->nb_float + 1);

      NEW(temp_proba,
	  char,
	  last_pos + (s_parse - probabilities) + 1);
      
    for(i = last_pos; i < s_parse - probabilities; i++){
      temp_proba[i - last_pos] = probabilities[i];
    }
    
    temp_proba[i - last_pos] = '\0';
    last_pos = i + 1;
    OPTS_proba->floats[OPTS_proba->nb_float] = atof(temp_proba);
    OPTS_proba->nb_float++;
    DESTROY(temp_proba);

    s_parse = strchr( s_parse + 1, ',');
    }
    
    s_parse = strchr(probabilities, '\0');
    if(s_parse != NULL){
      RENEW(OPTS_proba->floats,
	    float,
	    OPTS_proba->nb_float + 1);

      NEW(temp_proba,
	  char,
	  last_pos + (s_parse - probabilities) + 1);
      
      for(i = last_pos; i < s_parse - probabilities; i++){
	temp_proba[i - last_pos] = probabilities[i];
      }
      
      temp_proba[i - last_pos] = '\0';
      last_pos = i + 1;
      OPTS_proba->floats[OPTS_proba->nb_float] = atof(temp_proba);
      OPTS_proba->nb_float++;
      DESTROY(temp_proba);
   
    }
  
    /*Check if there are enough probabilities*/
    if(OPTS_proba->nb_float != 
       (file_positions->integers[file_positions->nb_int - 1] + 1)){
      ERROR_ "You must give a probability for each other genome(file)/species(directory) provided\n\n"
	_ERROR;
      return 0;
    }

    /*Check if all probabilities are bewteen 0 and 1*/
    for(i = 0; i < OPTS_proba->nb_float; i++){
      if((OPTS_proba->floats[i] > 1.0) || (OPTS_proba->floats[i] < 0.0)){
	ERROR_ "The probabilities must be between 0.0 and 1.0\n\n" _ERROR;
	return 0;
      }

    }
  }
  /*If not entered : default probabilities for each sequence*/
  else{
    NEW(OPTS_proba->floats,
	float,
	file_positions->integers[file_positions->nb_int - 1] + 1);
    for(i = 0; 
	i < file_positions->integers[file_positions->nb_int - 1] + 1 ; 
	i++){
      OPTS_proba->floats[i] = default_proba;
    }
    OPTS_proba->nb_float = file_positions->integers[file_positions->nb_int - 1] + 1;
  }



  /*Check if alignments file(s) exist*/
  for(i = 0; i < OPTS_alignments->nb_chars; i++){
    fd = fopen(OPTS_alignments->chars[i], "r");
    if(fd == NULL){
      ERROR_ "Can't open alignment file %s\n\n",
	OPTS_alignments->chars[i] _ERROR;
      return 0;

    }
    fclose(fd);
  }
 

  /*Check if output directory exists : if not create it*/ 
  DIR *test_dir = opendir(OPTS_output_dir);
  if(test_dir == NULL){
    if(OPTS_output_dir[strlen(OPTS_output_dir) - 1] == '/'){
      NEW(dir_to_create,
	  char,
	  strlen(OPTS_output_dir));
      for(i = 0; i < strlen(OPTS_output_dir) - 1; i++){
	dir_to_create[i] = OPTS_output_dir[i];
      }
      dir_to_create[i] = '\0';
    }
    else{
      NEW(dir_to_create,
	  char,
	  strlen(OPTS_output_dir) + 1);
      strcpy(dir_to_create,OPTS_output_dir);
    }
    NEW(command_line_dir, 
	char, 
	strlen("mkdir") + strlen(dir_to_create) + 4);
    sprintf(command_line_dir,
	    "%s%s%s",
	    "mkdir \"",
	    dir_to_create,
	    "\"");
    err_code = system(command_line_dir);
    DESTROY(command_line_dir);
    DESTROY(dir_to_create);
  }
  closedir(test_dir);
  

   /*Same thing for too close directory if wanted (option -k) and
    check if the threshold is below max_id and higher than min_id*/
   if(OPTS_keep_too_close != -1){

     if((OPTS_keep_too_close > OPTS_max_id) ||
	(OPTS_keep_too_close < OPTS_min_id)){
       ERROR_ "The identity threshold to consider too close sequences must be between the minimum identity (-i) and the maximum identity (-i) values\n\n" _ERROR;
       return 0;
     }

     DIR *test_dir_too_close = opendir(OPTS_output_dir_too_close);
     if(test_dir_too_close == NULL){
       if(OPTS_output_dir_too_close[strlen(OPTS_output_dir_too_close) - 1] == 
	  '/'){
       NEW(dir_to_create,
	   char,
	   strlen(OPTS_output_dir_too_close));
       for(i = 0; i < strlen(OPTS_output_dir_too_close) - 1; i++){
	 dir_to_create[i] = OPTS_output_dir_too_close[i];
       }
       dir_to_create[i] = '\0';
       }
       else{
	 NEW(dir_to_create,
	     char,
	     strlen(OPTS_output_dir_too_close) + 1);
	 strcpy(dir_to_create,OPTS_output_dir_too_close);
       }
       NEW(command_line_dir, 
	   char, 
	   strlen("mkdir") + strlen(dir_to_create) + 2);
     sprintf(command_line_dir,
	     "%s%s",
	     "mkdir ",
	     dir_to_create);
     err_code = system(command_line_dir);
     DESTROY(command_line_dir);  
     DESTROY(dir_to_create);
     }
     closedir(test_dir_too_close);
   }
   

   /*Check if min_id is below max_id*/
   if(OPTS_min_id >= OPTS_max_id){
     ERROR_ "The mimumum identity threshold must be lower than the maximum\n\n"
       _ERROR;
     return 0;
   }

   /*Check if the max number of sequences wanted by set is above or equal to 
     the minimum*/
   if((OPTS_nb_seqs_set_min != -1) && (OPTS_nb_seqs_set_max != -1)){
     if(OPTS_nb_seqs_set_min > OPTS_nb_seqs_set_max){
       ERROR_ "The minimum number of sequences wanted by set must be less or equal to the maximum wanted\n\n"
	 _ERROR;
       return 0;

     }
   }

   /*Check if the number of sequences to write by set is above the minimum
    by set and the maximum*/
   if(OPTS_nb_seqs_set != -1){
     if((OPTS_nb_seqs_set < OPTS_nb_seqs_set_min) ||
	((OPTS_nb_seqs_set_max != -1) && 
	 (OPTS_nb_seqs_set_max < OPTS_nb_seqs_set))){

       ERROR_ "The number of sequences to write by set (-m) must be between the maximum (-N) and the minimum (-n) number of sequences by set\n\n"
	 _ERROR;
       return 0;
     }

   }

   /*Check if max length of a conserved region is above the min length*/
   if(OPTS_min_length_region > OPTS_max_length_region){
     ERROR_ "The mimumum length of a conserved region  must be lower than the maximum\n\n"
       _ERROR;
     return 0;

   }

   /*Check if carnac is available*/
   if(OPTS_carnac){
     
     /*Look if program is available*/
     if(strcmp(CARNAC_EXE,"") == 0){
       ERROR_ "Carnac is not available : the option -c cannot be used\n\n"
	 _ERROR;
       return 0;

     }     
 
   }

   /*Check if a max number of sequences by set has been provided if the clique
    search is wanted*/
   if((OPTS_selecting_mode != 0) && (OPTS_nb_seqs_set == -1)){
     ERROR_ "You must give a maximum number of sequences by set (option -m) if the clique mode is wanted\n\n"
       _ERROR;
     return 0;

   }

   /*Check if the min number of different organisms is lower than the number
    of sequences by set if wanted*/
   if((OPTS_nb_seqs_set != -1) && 
      (OPTS_min_org_set != -1) &&
      (OPTS_min_org_set > OPTS_nb_seqs_set)){
     ERROR_ "You must give a maximum number of sequences by set higher than the minimum number of different organisms by set\n\n"
       _ERROR;
     return 0;

   }

   /*Check if the number of different species is higher than the number
    of sequences by set if wanted*/
   if((OPTS_nb_seqs_set != -1) && 
      (OPTS_min_species_set != -1) &&
      (OPTS_min_species_set > OPTS_nb_seqs_set)){
     ERROR_ "You must give a maximum number of sequences by set higher than the minimum number of different species by set\n\n"
       _ERROR;
     return 0;

   }

   /*Check if both options : OPTS_min_org_set and OPTS_min_species_set
    are used together*/
   if((OPTS_min_species_set != 0) && (OPTS_min_org_set != 0)){
     ERROR_ "The options -O (minimum different organisms by set) and -S (minimum different species by set) cannot be used together\n\n"
       _ERROR;
     return 0;

   }

   /*Check minimum number of species wanted by set : if set to -1 -> 2 options
     1)the number of species is less than the number of sequences wanted by set : 
       => the number of different species wanted = the number of species 
       provided
     2)the number of species is greater or equal to the number of sequences 
     wanted by set :
       => the number of different species wanted = the number of sequences 
       wanted - 1 (minus reference sequence)*/
   if(OPTS_min_species_set == -1){

     /*First case*/
     if(file_positions->integers[file_positions->nb_int - 1] < 
	OPTS_nb_seqs_set){
       OPTS_min_species_set = 
	 file_positions->integers[file_positions->nb_int - 1] + 1;
     }
     else{
       if(OPTS_nb_seqs_set == -1){
	 OPTS_min_species_set = 
	   file_positions->integers[file_positions->nb_int - 1] + 1;
       }
       else{
	 OPTS_min_species_set = OPTS_nb_seqs_set - 1;
       }
     }
   }
   


   /*Destroy filenames array*/
   if(fasta_filenames != NULL){
     if(fasta_filenames->chars != NULL){
       for(i = 0; i < fasta_filenames->nb_chars; i++){
	   DESTROY(fasta_filenames->chars[i]);
       }
       DESTROY(fasta_filenames->chars);
     }
     DESTROY(fasta_filenames);
   }


  /*Return success*/
  return 1;


}


/**
 *
 * \fn void destroy_options
 *
 * \brief Function used to destroyed allocated options
 *
 * This function will destroy the allocated options : the reference name,
 * the array of alignments files and the location of the file in fasta or 
 * in GFF3 format
 *
 *
 */

void destroy_options()
{
  
  /*Index*/
  int i;

  /*Destroy the reference file*/
  if(OPTS_ref_file){
    DESTROY(OPTS_ref_file);
  }

  /*Destroy the file*/
  if(OPTS_other_files != NULL){
    if(OPTS_other_files->chars != NULL){
      for(i = 0; i < OPTS_other_files->nb_chars; i++){
	DESTROY(OPTS_other_files->chars[i]);
    }
    DESTROY(OPTS_other_files->chars);
    DESTROY(OPTS_other_files);
    }
  }

  /*Destroy the array of sequence file*/
  if(OPTS_file_array){
    for(i = 0; i < OPTS_file_array->nb_chars; i++){
      DESTROY(OPTS_file_array->chars[i]);
    }
    DESTROY(OPTS_file_array->chars);
    DESTROY(OPTS_file_array);

  }


   /*Destroy file_positions*/
   if(file_positions != NULL){
     if(file_positions->integers != NULL){
       DESTROY(file_positions->integers);
     }
     DESTROY(file_positions);

   }


  /*Destroy the array of alignments file*/
  if(OPTS_alignments){
    for(i = 0; i < OPTS_alignments->nb_chars; i++){
      DESTROY(OPTS_alignments->chars[i]);
    }
    DESTROY(OPTS_alignments->chars);
    DESTROY(OPTS_alignments);

  }

  /*Destroy output dir*/
  if(OPTS_output_dir){
    if(strcmp(OPTS_output_dir, "./") != 0){
      DESTROY(OPTS_output_dir);
    }
  }
  if(OPTS_output_dir_too_close){
    if(strcmp(OPTS_output_dir_too_close, "too_close/") != 0){
      DESTROY(OPTS_output_dir_too_close);
    }
  }
  


  /*Destroy the probabilities*/
  if(probabilities){
    DESTROY(probabilities);
  }

  if(OPTS_proba){
    if(OPTS_proba->floats){
      DESTROY(OPTS_proba->floats);
    }
    DESTROY(OPTS_proba);
  }

  


}
