/*
 * This file is part of CGseqcore.
 *
 *  CGseqcore is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  CGseqcore is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with CGseqcore.  If not, see <http://www.gnu.org/licenses/>.
 */


/**
 *
 * \file options.h
 *
 * \author Benjamin Grenier-Boley <benjamin.grenier-boley@inria.fr>
 *
 * \brief Header file that defines the software options
 *
 * \version 1.0
 *
 * \date July 2009
 *
 *
 * This file will define all options that can be used in the software
 *
 *
 */


#ifndef __OPTIONS_H__
#define __OPTIONS_H__

/*Intern inclusions*/
#include "types.h"




/******************************************************************************
 **
 **                         MAIN OPTIONS
 **     
 *****************************************************************************/

/******************************************************************************
 **   Reference file
 **     
 *****************************************************************************/

extern char *OPTS_ref_file; /*!< Reference file*/


/******************************************************************************
 **   Other species provided files and directories
 **     
 *****************************************************************************/

extern char_array *OPTS_other_files; /*!< Other species provided files and directories*/


/******************************************************************************
 **   Array of other file sequences in fasta format
 **     
 *****************************************************************************/

extern char_array *OPTS_file_array; /*!< Array of file(s) containing sequences*/
extern int_array *file_positions;   /*!< Array that makes the link between file and directories*/

/******************************************************************************
 **   Output directory
 **     
 *****************************************************************************/

extern char *OPTS_output_dir; /*!< Output directory*/


/******************************************************************************
 **   Output directory for too close sets (if OPTS_keep_too_close)
 **     
 *****************************************************************************/

extern char *OPTS_output_dir_too_close; /*!< Output directory for too close sets*/



/******************************************************************************
 **   Output format
 **     
 *****************************************************************************/

extern int OPTS_output_format; /*!< Output format*/



/******************************************************************************
 **   Verbose option
 **     
 *****************************************************************************/

extern int OPTS_verbose; /*!< Verbose option*/


/******************************************************************************
 **   Maximum number of sequences to write by set option (Default -1 -> not used)
 **     
 *****************************************************************************/

extern int OPTS_nb_seqs_set; /*!< Maximum number of sequences to wrtie by set*/


/******************************************************************************
 **
 **                         ALIGNMENTS SELECTION
 **     
 *****************************************************************************/

/******************************************************************************
 **   Maximum E-value to consider an alignment
 **     
 *****************************************************************************/

extern float OPTS_evalue; /*!< Maximum E-value*/


/******************************************************************************
 **   Minimum identity to consider an alignment
 **     
 *****************************************************************************/

extern float OPTS_min_id_alignment; /*!< Minimum alignment identity*/


/******************************************************************************
 **   Minimum length to consider an alignment
 **     
 *****************************************************************************/

extern int OPTS_min_length_alignment; /*!< Minimum alignment length*/



/******************************************************************************
 **
 **                         FINDING CONSERVED REGIONS OPTIONS
 **     
 *****************************************************************************/


/******************************************************************************
 **   Lambda parameter : the paramter lambda used in the calculation of
 **   scores by position 
 **     
 *****************************************************************************/

extern int OPTS_lambda; /*!< Score lambda parameter*/

/******************************************************************************
 **   Probabilities parameter : the probability used in the new score
 **   calculation. And the probability by default. The string
 **   will contain the probabilities as typed by the user : e.i : 0.8,1.0,0.5
 **     
 *****************************************************************************/

extern float_array* OPTS_proba; /*!< The array of probability for each genome/species used in the score calculation*/

extern float default_proba;     /*!< The default probability*/

extern char *probabilities;     /*!< The probabilities user string*/

/******************************************************************************
 **   Array of alignments files
 **     
 *****************************************************************************/

extern char_array *OPTS_alignments; /*!< Array of alignments file*/


/******************************************************************************
 **   Minimal length of a region
 **     
 *****************************************************************************/

extern int OPTS_min_length_region; /*!< Minimal length of a region*/


/******************************************************************************
 **   Maximal length of a region
 **     
 *****************************************************************************/

extern int OPTS_max_length_region; /*!< Maximal length of a region*/


/******************************************************************************
 **   Filter on species : an alignment will be take into account only one
 **   time for a same species
 **     
 *****************************************************************************/

extern int OPTS_filter_species; /*!< Filter on species */


/******************************************************************************
 **   Maximal length to extend a conserved region up to
 **     
 *****************************************************************************/

extern int OPTS_extend_length_region; /*!< Length to extend a conserved
				           up to*/


/******************************************************************************
 **
 **                         SELECTING SEQUENCES
 **     
 *****************************************************************************/


/******************************************************************************
 **   Minimum identity threshold 
 **     
 *****************************************************************************/

extern int OPTS_min_id; /*!< Minimum identity threshold*/


/******************************************************************************
 **   Maximum identity threshold 
 **     
 *****************************************************************************/

extern int OPTS_max_id; /*!< Minimum identity threshold*/


/******************************************************************************
 **   Max number of sequences by set (reference + queries)
 **     
 *****************************************************************************/

extern int OPTS_nb_seqs_set_max; /*!< Maximum number of sequences wanted by set*/


/******************************************************************************
 **   Min number of sequences by set (reference + queries)
 **     
 *****************************************************************************/

extern int OPTS_nb_seqs_set_min; /*!< Minimum number of sequences wanted by set*/


/******************************************************************************
 **   Selecting mode (Default 0 -> etoile)
 **     
 *****************************************************************************/

extern int OPTS_selecting_mode;  /*!< Selecting mode*/ 


/******************************************************************************
 **   Maximum number of repetitions by species (Default -1 -> not used)
 **     
 *****************************************************************************/

extern int OPTS_max_species_repet;  /*!< Maximum number of repetitions by species*/ 


/******************************************************************************
 **   Carnac selected parameter (Default 0 -> not used)
 **     
 *****************************************************************************/

extern int OPTS_carnac; /*!< Carnac selected parameter*/


/******************************************************************************
 **   Minimum number of different organisms by outfile (Default 0 -> not used)
 **     
 *****************************************************************************/

extern int OPTS_min_org_set; /*!< Minimum number of different organisms by outfile*/


/******************************************************************************
 **   Minimum number of different species by outfile (Default 0 -> not used)
 **     
 *****************************************************************************/

extern int OPTS_min_species_set; /*!< Minimum number of different species by outfile*/


/******************************************************************************
 **   Identity threshold to consider a too close sequence  (Defaut : Not used)
 **     
 *****************************************************************************/

extern int OPTS_keep_too_close; /*!< Identity threshold to consider a too close sequence*/


/******************************************************************************
 **
 **                         FUNCTIONS
 **     
 *****************************************************************************/

/**
 *
 * \fn void usage()
 *
 * \brief Function that defines the software options
 *
 * This function will be printed on the screen to help users deal with 
 * options. It will be printed also when user's options are not correct
 *
 *
 */


extern void usage();


/**
 *
 * \fn int parse_params(int argc, char **argv)
 *
 * \brief Function used to parse user parameters
 *
 * \param[in] argc : number of parameters
 * \param[in] argv : the parameters
 *
 * \return an integer representing the status : \n
 * 1 -> exit success \n
 * 0 -> an error occured
 *
 * This function will parse all user parameters and will check for their 
 * validity
 *
 *
 */


extern int parse_params(int argc, char **argv);



/**
 *
 * \fn int check_parameters_validity
 *
 * \brief Function used to check parameters entered by the user
 *
 * \return an integer representing the validity : \n
 * 1 -> exit success \n
 * 0 -> an error occured
 *
 * This function will check parameters validity and returns the result of
 * the checking
 *
 *
 */

extern int check_parameters_validity();



/**
 *
 * \fn void destroy_options
 *
 * \brief Function used to destroyed allocated options
 *
 * This function will destroy the allocated options : the reference name,
 * the array of alignments files and the location of the file in fasta or 
 * in GFF3 format
 *
 *
 */

extern void destroy_options();

#endif
