/*
 * This file is part of CGseqcore.
 *
 *  CGseqcore is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  CGseqcore is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with CGseqcore.  If not, see <http://www.gnu.org/licenses/>.
 */


/**
 *
 * \file score.c
 *
 * \author Benjamin Grenier-Boley <benjamin.grenier-boley@inria.fr>
 *
 * \brief File that defines all functions relied on score functions
 *
 * \version 1.0
 *
 * \date July 2009
 *
 *
 * This file will defines the function used in the calculation of scores
 * that will delimit interesting regions
 *
 *
 */


/*Basic inclusions*/
#include <math.h>
#include <stdlib.h>


/*Intern inclusions*/
#include "score.h"
#include "macros.h"
#include "options.h"
#include "alignment.h"
#include "io.h"



/**
 *
 * \fn float get_target_proba(char existence, int num_genome)
 *
 * \brief Function used to get the genome target probability
 *
 * \param[in] existence : a char representing if a genome exist at a
 * given position (1) or if it doesn't exist (0)
 * \param[in] num_genome : the number of the current genome in the 
 * OPTS_proba array
 *
 * That function will return the probability according to the presence/
 * absence of a genome and its probability. If the genome exists 
 * (existence = 1) then the probability will be the one link with the
 * genome. If not exist (existence=0) then it will be 1-p with p the
 * probability of the genome
 *
 */


float get_target_proba(char existence, int num_genome)
{
  
  /*Case the genome exists : return the probabilty of this genome*/
  if(existence == '1'){
    return (OPTS_proba->floats[num_genome]);
  }
  else{
    return (1.0 - OPTS_proba->floats[num_genome]);
  }
}


/**
 *
 * \fn float get_background_proba(char existence, 
 *                                int num_genome,
 *                                float *probabilities)
 *
 * \brief Function used to get the genome background probability
 *
 * \param[in] existence : an char representing if a genome exist at a
 * given position (1) or if it doesn't exist (0)
 * \param[in] num_genome : the number of the current genome in the 
 * OPTS_proba array
 * \param[in,out] probabilities : the probability according to the parameters
 *
 * That function will return the probability according to the presence/
 * absence of a genome and its background probability. If the genome exists 
 * (existence = 1) then the probability will be the one link with the
 * genome. If not exist (existence=0) then it will be 1-p with p the 
 * background probability of the genome
 *
 */

float get_background_proba(char existence, 
			   int num_genome, 
			   float *probabilities)
{
  
  /*Case the genome exists : return the probabilty of this genome*/
  if(existence == '1'){
    return (probabilities[num_genome]);
  }
  else{
    return (1.0 - probabilities[num_genome]);
  }
}


/**
 *
 * \fn void calculate_target_array(p_main_structure m)
 *
 * \brief Function used to calculate the target array
 *
 * \param[in,out] m : the main structure containing information and that
 * will be updated with defined regions
 *
 * That function will calculate every scores according to the number
 * of other genomes and the OPTS_proba probabilities in order to speed up 
 * the future calculation of the scores
 *
 * \note Example for 3 genomes with an alphabet of 2: 1 (presence alignment 
 * at that position) and 0 (absence of alignment at that position). So the
 * number of possibilities is 2^3 -> 8 solutions (000,001,010,100,011,101,
 * 110 and 111)
 *
 */

void calculate_target_array(p_main_structure m)
{

  /*Indexes*/
  int i,j;
  /*Binary string representation of the current index*/
  char * binary = NULL;
  /*Current score*/
  float score;
 
  /*Calculate probability for all possibilities*/
  for(i = 0; 
      i < m->array_of_scores->length_tb;
      i++){
    
    /*Get the binary string representation of the current index*/
    NEW(binary, char, OPTS_proba->nb_float + 1);
    int_to_binary(i, 
		  OPTS_proba->nb_float + 1, 
		  binary);
 
    /*Calculate the score*/
    score = 1.0;
    for (j = 0; 
	 j < OPTS_proba->nb_float; 
	 j++){
      score *= get_target_proba(binary[j],j);
    }

    m->array_of_scores->target[i] = score;

    /*Destroy the binary string*/
    DESTROY(binary);
  }
}



/**
 *
 * \fn void calculate_al_function(p_main_structure m)
 *
 * \brief Function used to calculate the array containing the absence/
 * presence of a every genomes alignments on that position
 *
 * \param[in,out] m : the main structure containing information and that
 * will be updated with defined regions
 *
 * That function will calculate the array containing the absence/
 * presence of a every genomes alignments on that position
 *
 * \note Example for 3 genomes with an alphabet of 2: 1 (presence alignment 
 * at that position) and 0 (absence of alignment at that position).
 *
 */


void calculate_al_function(p_main_structure m)
{

  /*Index*/
  int i,j;
  /*Current alignment starting position on the reference*/
  int start_pos = 0;
  /*Current alignment ending position on the reference*/
  int end_pos = 0;
  /*Current index of the query sequence involved in the current alignment*/
  int index_query = 0;
  /*Current number of species : representing the second index in the array
   of positions*/
  int num_species = 0;

  /*Allocate the position array to contain strings like "110101"
   which length is the number of other genomes/species and initialize
  the string with '0'*/
  for(i = 0; i < m->array_of_scores->length_ps; i++){
    NEW(m->array_of_scores->positions[i],
	char,
	OPTS_proba->nb_float + 1);
    for(j = 0; 
	j < OPTS_proba->nb_float; 
	j++){
      m->array_of_scores->positions[i][j] = '0';
    }
    m->array_of_scores->positions[i][j] = '\0';
  }

  /*Process all alignments*/
  for(i = 0; i < m->array_of_al_info->nb_al_info; i++){

    start_pos = m->array_of_al_info->al_info[i]->pos_start_ref;
    end_pos = m->array_of_al_info->al_info[i]->pos_end_ref;
    index_query = m->array_of_al_info->al_info[i]->index_query;
    num_species = m->array_of_seqs->seqs[index_query]->num_species;

    /*Put '1' in the string between each position at the correct index
     in the string*/
    for(j = start_pos; j <= end_pos; j++){
      m->array_of_scores->positions[j][num_species] = '1';
    } 
  }


}


/**
 *
 * \fn void calculate_background_array(p_main_structure m)
 *
 * \brief Function used to calculate the background  array
 *
 * \param[in,out] m : the main structure containing information and that
 * will be updated with defined regions
 *
 * That function will calculate every scores according to alignments
 * and the genomes background probabilities that will also be calculated
 * inside that function
 *
 */

void calculate_background_array(p_main_structure m)
{

  /*Indexes*/
  int i,j;
  /*Binary string representation of the current index*/
  char * binary = NULL;
  /*Current score*/
  float score;
  /*Number of '1' (presence genome or species alignment) found in the positions
   array*/
  int nb_presence;


  /*Calculate genomes/species background proabilities*/
  for(i = 0; i < OPTS_proba->nb_float; i++){

    nb_presence = 0;
    
    for(j = 0; j < m->array_of_scores->length_ps; j++){

      if( m->array_of_scores->positions[j][i] == '1'){
	nb_presence++;
      }

    }

    m->array_of_scores->genomes_proba[i] = 
      (float)(((float)nb_presence)/((float)m->array_of_seqs->seqs[0]->real_length));
    if(m->array_of_scores->genomes_proba[i] > 1.0){
      m->array_of_scores->genomes_proba[i] = 1.0;
    }
  }



  /*Calculate probability for all possibilities*/
  for(i = 0;
      i < m->array_of_scores->length_tb;
      i++){
    
    /*Get the binary string representation of the current index*/
    NEW(binary, char, OPTS_proba->nb_float + 1);
    int_to_binary(i, OPTS_proba->nb_float + 1, binary);

    /*Calculate the score*/
    score = 1.0;
    for (j = 0; j < OPTS_proba->nb_float; j++){
      score *= get_background_proba(binary[j],
				    j,
				    m->array_of_scores->genomes_proba);
    }

    m->array_of_scores->background[i] = score;
    
    /*Destroy the binary string*/
    DESTROY(binary);
  }
}



/**
 *
 * \fn void calculate_score_function(p_main_structure m)
 *
 * \brief Function used to calculate the score function
 *
 * \param[in,out] m : the main structure containing information
 *
 * That function will calculate the score function used to retrieve
 * conserved region.
 *
 */


void calculate_score_function(p_main_structure m)
{

  /*Index*/
  int i;
  /*Current "bit" string at a given position*/
  char *current_bit = NULL;
  /*Current index of the "bit" in the target and background arrays*/
  unsigned int current_bit_index;


  /*Calculate the score for the first position*/
  NEW(current_bit, char, OPTS_proba->nb_float + 1);
  strcpy(current_bit,m->array_of_scores->positions[0]);
  current_bit_index = strtoul(current_bit, NULL, 2);

  m->array_of_scores->scores[0] =
    MAX(0.0,
	log10(m->array_of_scores->target[current_bit_index] /
	      m->array_of_scores->background[current_bit_index]));

  DESTROY(current_bit);

  /*Calculate score for every position using target and background arrays*/
  for(i = 1; i < m->array_of_scores->length_ps; i++){

    /*Get the presence/absence bit for the current position*/
    NEW(current_bit, char, OPTS_proba->nb_float + 1);
    strcpy(current_bit,m->array_of_scores->positions[i]);

    /*Get its corresponding index in the target and background arrays*/
    current_bit_index = strtoul(current_bit, NULL, 2);
    
    /*Calculate the score*/
    if(m->array_of_scores->background[current_bit_index] == 0){
      m->array_of_scores->scores[i] = 0.0;
    }
    else{
      m->array_of_scores->scores[i] =
	MAX(0.0,
	    m->array_of_scores->scores[i - 1] +
	    log10(m->array_of_scores->target[current_bit_index] /
		  m->array_of_scores->background[current_bit_index]));
    }
    DESTROY(current_bit);
   

  }


}




/**
 *
 * \fn void get_interesting_regions(p_main_structure m)
 *
 * \brief Function used to determine interesting regions on the reference
 *
 * \param[in,out] m : the main structure containing information and that
 * will be updated with defined regions
 *
 * That function will use the scores calculated by the score function to
 * determine positions of conserved regions on the reference sequence
 *
 */

void get_interesting_regions(p_main_structure m)
{

  /*Index*/
  int i;
  /*Start position of the current new region*/
  int start_position = -1;
  /*End position of the current new region*/
  int end_position = -1;
  /*Current score*/
  float current_score;
  /*Local score maximum*/
  float local_max = -1.0;
  /*Index of the local score maximum*/
  int index_local_max = 0;
  /*Length of the conserved region*/
  int length_region = 0;
  /*Difference bewteen length and OPTS_extend_length_region used
   to extend "too small" regions*/
  int diff_length_region = 0;

  /*Search for positive scores in the array of scores*/
  for(i = 0; i < m->array_of_scores->length_ps; i++){
    
    current_score = m->array_of_scores->scores[i];
    
    /*An interesting score has been found*/
    if(current_score > 0.0){

      /*New start of a region*/
      if(start_position == -1){
	end_position = -1;
	start_position = i;

	local_max = current_score;
	index_local_max = i;

      }

      else{

	/*If the score is above the local maximum : continue*/
	if(current_score >= local_max){
	  local_max = current_score;
	  index_local_max = i;
	}

      }
    }
    /*Otherwise the score is 0 : whether it's an end of a region
     or we're not in an interesting region*/
    else{

      /*End of a region : new interval if its length is above the minimum
       length parameter and below max length*/
      if(start_position != -1){
	end_position = index_local_max;
	
	length_region = end_position - start_position + 1;

	if((length_region >= OPTS_min_length_region) &&
	   (length_region <= OPTS_max_length_region)){

	  /*If a region is less than the length to extend a region up
	   to (OPTS_extend_length_region) : extend it*/
	  if(length_region < OPTS_extend_length_region){

	    diff_length_region = OPTS_extend_length_region - length_region;

	    if((diff_length_region % 2) != 0){

	      if((start_position - ((diff_length_region / 2) + 1)) < 0){
		start_position = 0;
		end_position = MIN(m->array_of_seqs->seqs[0]->length - 1,
				   end_position + 
				   ((diff_length_region / 2) + 1)); 
		while((end_position < m->array_of_seqs->seqs[0]->length) &&
		      ((end_position - start_position + 1) < 
		       OPTS_extend_length_region)){
		  end_position++;
		}

	      }
	      else{
		if((end_position + ((diff_length_region / 2) + 1)) >=
		   m->array_of_seqs->seqs[0]->length){
		  end_position = m->array_of_seqs->seqs[0]->length - 1;
		  start_position = MAX(0,
				       start_position - 
				       ((diff_length_region / 2) + 1));
		  
		  while((start_position > 0) &&
			((end_position - start_position + 1) < 
			 OPTS_extend_length_region)){
		    start_position--;
		}
		
		}
		else{
		  start_position = MAX(0,start_position - 
				       ((diff_length_region / 2) + 1));
		  end_position =  MIN(end_position + 
				      ((diff_length_region / 2) + 1),
				      m->array_of_seqs->seqs[0]->length - 1);
		} 
	      }
	    }
	    else{
	      if((start_position - (diff_length_region / 2)) < 0){
		start_position = 0;
		end_position = MIN(m->array_of_seqs->seqs[0]->length - 1,
				   end_position + 
				   (diff_length_region / 2)); 
		while((end_position < m->array_of_seqs->seqs[0]->length) &&
		      ((end_position - start_position + 1) < 
		       OPTS_extend_length_region)){
		  end_position++;
		}

	      }
	      else{
		if((end_position + (diff_length_region / 2)) >=
		   m->array_of_seqs->seqs[0]->length){
		  end_position = m->array_of_seqs->seqs[0]->length - 1;
		  start_position = MAX(0,
				       start_position - 
				       (diff_length_region / 2));
		  
		  while((start_position > 0) &&
			((end_position - start_position + 1) < 
			 OPTS_extend_length_region)){
		    start_position--;
		}
		
		}
		else{
		  start_position = MAX(0,start_position - 
				       (diff_length_region / 2));
		  end_position =  MIN(end_position + 
				      (diff_length_region / 2),
				      m->array_of_seqs->seqs[0]->length - 1);
		} 
	      }
	    }

	  }


	  RENEW(m->interesting_regions->intervals,
		p_interval,
		m->interesting_regions->nb_intervals + 1);
	  NEW(m->interesting_regions->
	      intervals[m->interesting_regions->nb_intervals],
	      interval_s,
	      1);
	  m->interesting_regions->nb_intervals++;
	  m->interesting_regions->
	    intervals[m->interesting_regions->nb_intervals - 1]->pos_start = 
	    start_position;
	  m->interesting_regions->
	    intervals[m->interesting_regions->nb_intervals - 1]->pos_end = 
	    end_position;

	}
	start_position = -1;
	end_position = -1;
	local_max = -1;
	index_local_max = 0;
	 
      }

    }

  }

}
