/*
 * This file is part of CGseqcore.
 *
 *  CGseqcore is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  CGseqcore is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with CGseqcore.  If not, see <http://www.gnu.org/licenses/>.
 */


/**
 *
 * \file score.h
 *
 * \author Benjamin Grenier-Boley <benjamin.grenier-boley@inria.fr>
 *
 * \brief Header file that defines all functions relied on score functions
 *
 * \version 1.0
 *
 * \date July 2009
 *
 *
 * This file will defines the function used in the calculation of scores
 * that will delimit interesting regions
 *
 *
 */


#ifndef __SCORE_H__
#define __SCORE_H__

/*Intern inclusions*/
#include "types.h"


/**
 *
 * \fn void calculate_target_array(p_main_structure m)
 *
 * \brief Function used to calculate the target array
 *
 * \param[in,out] m : the main structure containing information and that
 * will be updated with defined regions
 *
 * That function will calculate every scores according to the number
 * of other genomes and the OPTS_proba probabilities in order to speed up 
 * the future calculation of the scores
 *
 * \note Example for 3 genomes with an alphabet of 2: 1 (presence alignment 
 * at that position) and 0 (absence of alignment at that position). So the
 * number of possibilities is 2^3 -> 8 solutions (000,001,010,100,011,101,
 * 110 and 111)
 *
 */

extern void calculate_target_array(p_main_structure m);


/**
 *
 * \fn float get_target_proba(char existence, int num_genome)
 *
 * \brief Function used to get the genome target probability
 *
 * \param[in] existence : a char representing if a genome exist at a
 * given position (1) or if it doesn't exist (0)
 * \param[in] num_genome : the number of the current genome in the 
 * OPTS_proba array
 *
 * That function will return the probability according to the presence/
 * absence of a genome and its probability. If the genome exists 
 * (existence = 1) then the probability will be the one link with the
 * genome. If not exist (existence=0) then it will be 1-p with p the
 * probability of the genome
 *
 */

extern float get_target_proba(char existence, int num_genome);


/**
 *
 * \fn float get_background_proba(char existence, 
 *                                int num_genome,
 *                                float *probabilities)
 *
 * \brief Function used to get the genome background probability
 *
 * \param[in] existence : an char representing if a genome exist at a
 * given position (1) or if it doesn't exist (0)
 * \param[in] num_genome : the number of the current genome in the 
 * OPTS_proba array
 * \param[in,out] probabilities : the probability according to the parameters
 *
 * That function will return the probability according to the presence/
 * absence of a genome and its background probability. If the genome exists 
 * (existence = 1) then the probability will be the one link with the
 * genome. If not exist (existence=0) then it will be 1-p with p the 
 * background probability of the genome
 *
 */

extern float get_background_proba(char existence, 
				  int num_genome, 
				  float *probabilities);


/**
 *
 * \fn void calculate_al_function(p_main_structure m)
 *
 * \brief Function used to calculate the array containing the absence/
 * presence of a every genomes alignments on that position
 *
 * \param[in,out] m : the main structure containing information and that
 * will be updated with defined regions
 *
 * That function will calculate the array containing the absence/
 * presence of a every genomes alignments on that position
 *
 * \note Example for 3 genomes with an alphabet of 2: 1 (presence alignment 
 * at that position) and 0 (absence of alignment at that position).
 *
 */

extern void calculate_al_function(p_main_structure m);


/**
 *
 * \fn void calculate_background_array(p_main_structure m)
 *
 * \brief Function used to calculate the background  array
 *
 * \param[in,out] m : the main structure containing information and that
 * will be updated with defined regions
 *
 * That function will calculate every scores according to alignments
 * and the genomes background probabilities that will also be calculated
 * inside that function
 *
 */

extern void calculate_background_array(p_main_structure m);



/**
 *
 * \fn void calculate_score_function(p_main_structure m)
 *
 * \brief Function used to calculate the score function
 *
 * \param[in,out] m : the main structure containing information
 *
 * That function will calculate the score function used to retrieve
 * conserved region.
 *
 */

extern void calculate_score_function(p_main_structure m);


/**
 *
 * \fn void get_interesting_regions(p_main_structure m)
 *
 * \brief Function used to determine interesting regions on the reference
 *
 * \param[in,out] m : the main structure containing information and that
 * will be updated with defined regions
 *
 * That function will use the scores calculated by the score function to
 * determine positions of conserved regions on the reference sequence
 *
 */

extern void get_interesting_regions(p_main_structure m);


#endif
