/*
 * This file is part of CGseqcore.
 *
 *  CGseqcore is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  CGseqcore is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with CGseqcore.  If not, see <http://www.gnu.org/licenses/>.
 */


/**
 *
 * \file sequence.c
 *
 * \author Benjamin Grenier-Boley <benjamin.grenier-boley@inria.fr>
 *
 * \brief File that defines all functions related to sequences
 * or set of sequences
 *
 * \version 1.0
 *
 * \date July 2009
 *
 *
 * This file will define all fucntions related to sequences  or sets of
 * sequences 
 *
 *
 */


/*Basic inclusions*/
#include <ctype.h>
#include <math.h>

/*Intern inclusions*/
#include "sequence.h"
#include "options.h"
#include "sequence_interface.h"
#include "io.h"
#include "macros.h"
#include "alignment.h"
#include "carnac.h"


/**
 *
 * \fn void get_sequences(sequence_array *array_of_seqs)
 *
 * \brief Function used to retrieve sequences contained in the fasta file
 *
 * \param[in,out] array_of_seqs : the array of sequences that will be
 * updated with sequences contained in the fasta file
 *
 *
 * This function will use the sequence parser to retrieve the sequences in
 * the fasta file
 *
 *
 */

void get_sequences(sequence_array *array_of_seqs)
{

  /*Indexes*/
  int i,j,k;
  /*File descriptor*/
  FILE *fd = NULL;
  /*Temporary sequence array*/
  p_sequence *other_sequences = NULL;
  /*Number of sequences*/
  int nb_seqs = 0;
  /*Sequences names*/
  char_array *sequences_names = NULL;
  

  /*Try to read the reference file*/
  if((fd = fopen(OPTS_ref_file, "r")) == NULL) {
    ERROR_ "Unable to open '%s' for reading...\n",OPTS_ref_file _ERROR;
    array_of_seqs->seqs = NULL;
    array_of_seqs->nb_seqs = 0;
  }
  
  /*Parse the file to extract the sequences*/
  sequence_file_name = OPTS_ref_file;   
  sequencerestart(fd); 
  sequenceparse(); 
  
  fclose(fd);
  free_flex_mem();

  /*If there were some errors print error message, return NULL
   and free all allocated memory from the parser*/
  if(sequence_nb_errors != 0) {
    ERROR_ "File '%s' has some errors : process stopped\n\n",OPTS_ref_file _ERROR; 
    array_of_seqs->seqs = NULL;
    array_of_seqs->nb_seqs = 0;
    i = 0;
    if(sequence_result != NULL){
      while(sequence_result[i] != NULL){
	if(sequence_result[i]->name != NULL){
	  DESTROY(sequence_result[i]->name);
	}
	if(sequence_result[i]->bases != NULL){
	  DESTROY(sequence_result[i]->bases);
	}
	DESTROY(sequence_result[i]);
	i++;
      }
      DESTROY(sequence_result);
    }
 
  }
  else{

    /*Retrievment of the sequences array*/
    array_of_seqs->seqs = sequence_result;

    /*Counting the number of sequences*/
    while(array_of_seqs->seqs[nb_seqs] != NULL) nb_seqs++;

    /*If there was not enough sequences return NULL or if there are 
     more than one sequence*/
    if((nb_seqs < 1) || (nb_seqs > 1)) {
      if(nb_seqs > 1){
	ERROR_ "Too many reference sequences....\n\n" _ERROR;
	array_of_seqs->seqs = NULL;
	array_of_seqs->nb_seqs = 0;

      }
      else{

	ERROR_ "No reference sequence....\n\n" _ERROR;
	array_of_seqs->seqs = NULL;
	array_of_seqs->nb_seqs = 0;
      }
      i = 0;
      if(sequence_result != NULL){
	while(sequence_result[i] != NULL){
	  if(sequence_result[i]->name != NULL){
	    DESTROY(sequence_result[i]->name);
	  }
	  if(sequence_result[i]->bases != NULL){
	    DESTROY(sequence_result[i]->bases);
	  }
	  DESTROY(sequence_result[i]);
	  i++;
	}
	DESTROY(sequence_result);
      }

      
    }
    else{
 
      /*Get the reference name*/
      NEW(sequences_names, char_array, 1);
      NEW(sequences_names->chars, char*, 1);
      NEW(sequences_names->chars[0], 
	  char, 
	  strlen(array_of_seqs->seqs[0]->name) + 1);
      strcpy(sequences_names->chars[0],array_of_seqs->seqs[0]->name);
      sequences_names->nb_chars = 1;

      /*Reference species number : -1*/
      array_of_seqs->seqs[0]->num_species = -1;
      array_of_seqs->nb_seqs = nb_seqs;
      nb_seqs = 0;

      /*Try to read the files containing other sequences*/
      for(j = 0; j < OPTS_file_array->nb_chars; j++){
	
	nb_seqs = 0;

      /*Parse the file to extract the sequences*/
	if((fd = fopen(OPTS_file_array->chars[j], "r")) == NULL) {
	  ERROR_ "Unable to open '%s' for reading...\n", 
	    OPTS_file_array->chars[j] _ERROR;
	  array_of_seqs->seqs = NULL;
	  array_of_seqs->nb_seqs = 0;
	}
	
	sequence_file_name = OPTS_file_array->chars[j];
	sequencerestart(fd); 
	sequenceparse(); 

	fclose(fd);
	free_flex_mem();
	
	/*If there were some errors print error message, return NULL
	  and free all allocated memory from the parser*/
	if(sequence_nb_errors != 0) {
	  ERROR_ "File '%s' has some errors : process stopped\n\n",
	    OPTS_file_array->chars[j] _ERROR; 
	  if(array_of_seqs->seqs != NULL){
	    destroy_sequence(array_of_seqs->seqs[0]);
	  }

	  array_of_seqs->seqs = NULL;
	  array_of_seqs->nb_seqs = 0;
	  i = 0;
	  while(sequence_result[i] != NULL){
	    if(sequence_result[i]->name != NULL){
	      DESTROY(sequence_result[i]->name);
	    }
	    if(sequence_result[i]->bases != NULL){
	      DESTROY(sequence_result[i]->bases);
	    }
	    DESTROY(sequence_result[i]);
	    i++;
	  }
	  DESTROY(sequence_result);

	  if(sequences_names != NULL){
	    if(sequences_names->chars != NULL){
	      for(j = 0; j < sequences_names->nb_chars; j++){
		if(sequences_names->chars[j] != NULL){
		  DESTROY(sequences_names->chars[j]);
		}
	      }
	      DESTROY(sequences_names->chars);
	    }
	    DESTROY(sequences_names);
	  }

	}
	else{
	  
	  /*Retrievment of the sequences array*/
	  other_sequences = sequence_result;
	  
	  /*Counting the number of sequences*/
	  while(other_sequences[nb_seqs] != NULL) nb_seqs++;
	  
	  /*If there was not enough sequences return NULL*/
	  if(nb_seqs < 1) {
	    ERROR_ "Not enough sequences....\n" _ERROR;
	    array_of_seqs->seqs = NULL;
	    array_of_seqs->nb_seqs = 0;
	  }
	  else{
	    /*Copy the other sequences in the sequences array*/
	    RENEW(array_of_seqs->seqs,
		  p_sequence,
		  nb_seqs + array_of_seqs->nb_seqs);

	    for(i = 0; i < nb_seqs; i++){

	      NEW(array_of_seqs->seqs[array_of_seqs->nb_seqs],
		  sequence_s,
		  1);
	      NEW(array_of_seqs->seqs[array_of_seqs->nb_seqs]->name,
		  char,
		  strlen(other_sequences[i]->name) + 1);
	      NEW(array_of_seqs->seqs[array_of_seqs->nb_seqs]->bases,
		  char,
		  strlen(other_sequences[i]->bases) + 1);
	      strcpy(array_of_seqs->seqs[array_of_seqs->nb_seqs]->name,
		     other_sequences[i]->name);
	      strcpy(array_of_seqs->seqs[array_of_seqs->nb_seqs]->bases,
		     other_sequences[i]->bases);
	      array_of_seqs->seqs[array_of_seqs->nb_seqs]->length =
		other_sequences[i]->length;

	      /*Value of number species due to file positions array*/
	      array_of_seqs->seqs[array_of_seqs->nb_seqs]->num_species = 
		file_positions->integers[j];

	      /*If the name is already present in the sequences names
	       array : stop processing*/
	      if(is_present_char_array(array_of_seqs->
				       seqs[array_of_seqs->nb_seqs]->name,
				       sequences_names)){
		ERROR_ "Two sequences have the name : \"%s\"\n",
		 array_of_seqs->seqs[array_of_seqs->nb_seqs]->name _ERROR;
		
		 if(array_of_seqs->seqs != NULL){
		   for(k = 0; k < (array_of_seqs->nb_seqs + nb_seqs); k++){
		     destroy_sequence(array_of_seqs->seqs[k]);
		   }
		 }

		 array_of_seqs->seqs = NULL;
		 array_of_seqs->nb_seqs = 0;

		
		if(sequences_names != NULL){
		  if(sequences_names->chars != NULL){
		    for(k = 0; k < sequences_names->nb_chars; k++){
		      if(sequences_names->chars[k] != NULL){
			DESTROY(sequences_names->chars[k]);
		      }
		    }
		    DESTROY(sequences_names->chars);
		  }
		  DESTROY(sequences_names);
		}
		i = nb_seqs + 1;
	      }
	      else{
		RENEW(sequences_names->chars,
		      char *, 
		      sequences_names->nb_chars + 1);
		NEW(sequences_names->chars[sequences_names->nb_chars],
		    char,
		    strlen(array_of_seqs->seqs[array_of_seqs->nb_seqs]->name)+1);
		
		strcpy(sequences_names->chars[sequences_names->nb_chars],
		       array_of_seqs->seqs[array_of_seqs->nb_seqs]->name);
		sequences_names->nb_chars++;	
		array_of_seqs->nb_seqs++;
	      }
	      
	    }

	    /*Destroy the temporary sequences array*/
	    for(i = 0; i < nb_seqs; i++){
	      DESTROY(other_sequences[i]->name);
	      DESTROY(other_sequences[i]->bases);
	      DESTROY(other_sequences[i]);
	      
	    }
	    DESTROY(other_sequences);

	  }
	}
      }


      /*Destroy sequences names array*/
      if(sequences_names != NULL){
	if(sequences_names->chars != NULL){
	  for(i = 0; i < sequences_names->nb_chars; i++){
	    if(sequences_names->chars[i] != NULL){
	      DESTROY(sequences_names->chars[i]);
	    }
	  }
	  DESTROY(sequences_names->chars);
	}
	DESTROY(sequences_names);
      }
      
      /*Calculation of the real length (all expect N) on all sequences*/
      if(array_of_seqs->seqs){
	for(i = 0; i < array_of_seqs->nb_seqs; i++){
	  get_real_length_sequence(array_of_seqs->seqs[i]);
	  
	}
      }

     } 
    
  } 
}


/**
 *
 * \fn void destroy_sequence(p_sequence s)
 *
 * \brief Function used to destroy a p_sequence structure
 *
 * \param[in,out] s : the p_sequence to destroy
 *
 *
 * This function will destroy all allocated memory for a p_sequence 
 * structure
 *
 *
 */

void destroy_sequence(p_sequence s)
{
  /*Destroy the sequence if existing*/
  if(s->bases) {
    DESTROY(s->bases);
  }
  
  /*Destroy the name*/
  if(s->name){
    DESTROY(s->name); 
  }
  
  DESTROY(s);
}


/**
 *
 * \fn int is_present_array_sequence(char *seq_name_to_find,
 *                                 sequence_array *array_of_seqs)
 *
 * \brief Function used to retrieve the index of a sequence according to
 * its name
 *
 * \param[in] seq_name_to_find : the name to search for
 * \param[in] array_of_seqs : the array of sequences
 *
 * \return the index in the sequence array or -1 if it wasn't found
 *
 * This function will search through the array of sequences for the name and 
 * return its idnex or -1 if it wasn't found
 *
 *
 */


int is_present_array_sequence(char *seq_name_to_find, 
			      sequence_array *array_of_seqs){

  /*Current index*/
  int i;

  /*Search over the array*/
  for(i = 0; i < array_of_seqs->nb_seqs; i++){
    if(strcmp(seq_name_to_find,array_of_seqs->seqs[i]->name) == 0){
      return i;
    }
  }

  return -1;
}


/**
 *
 * \fn void get_real_length_sequence(p_sequence seq)
 *
 * \brief Function used to calculate the real length of a sequence
 *
 * \param[in,out] seq : the reference sequence
 *
 * This function will calculate the real length of a sequence(all 
 * nucleotides except N)
 *
 *
 */


void get_real_length_sequence(p_sequence seq)
{

  /*Index*/
  int i;
  /*Number of N nucleotides*/
  int nb_N = 0;

  /*Read all nucleotides of the sequence and count the number
   of N*/
  for(i = 0; i < seq->length; i++){
    if(seq->bases[i] == 'N'){
      nb_N++;
    }
  }

  seq->real_length = seq->length - nb_N;

}



/**
 *
 * \fn char get_complementary_base(char nucleotide)
 *
 * \brief Function used to get the complementary nucleotide
 *
 * \param[in] nucleotide : the nucleotide to get its complementary
 *
 * \return the complementary nucleotide
 *
 * This function will return the complementary nucleotide of the one given
 *
 *
 */


char get_complementary_base(char nucleotide)
{

  /*Different nucleotide cases*/
  switch(nucleotide){

  case 'A':
    return 'T';
    break;
  case 'T':
    return 'A';
    break;
  case 'C':
    return 'G';
    break;
  case 'G':
    return 'C';
    break;
  case 'U':
    return 'A';
    break;
  case 'R':
    return 'Y';
    break;
  case 'Y':
    return 'R';
    break;
  case 'W':
    return 'W';
    break;
  case 'K':
    return 'M';
    break;
  case 'M':
    return 'K';
    break;
  case 'B':
    return 'V';
    break;
  case 'V':
    return 'B';
    break;
  case 'S':
    return 'S';
    break;
  case 'D':
    return'H';
    break;
  case 'H':
    return 'D';
    break;
  case 'N':
    return 'N';
    break;
    /*Error case : return N*/
  default:
    ERROR_ "Invalid nucleotide %c\n",nucleotide _ERROR;
    return 'N';
  }



}


/**
 *
 * \fn void complete_sequences_attributes(sequence_array *array_of_seqs)
 *
 * \brief Function used to complete some missing attributes in the sequence
 * array
 *
 * \param[in,out] array_of_seqs : the array containing all sequences
 * and that will be updated
 *
 * This function will complete the species attribute when possible, put in 
 * uppercase the sequence when needed and replace the spaces in the sequence
 * name by a "_"
 *
 */

void complete_sequences_attributes(sequence_array *array_of_seqs)

{

  /*Indexes*/
  int i,j;
  /*Temporary sequence name*/
  char *temp_name = NULL;

  /*Get all sequences*/
  for(i = 0; i < array_of_seqs->nb_seqs; i++){

    /*Test if the first base of the sequence is not in uppercase*/
    if(array_of_seqs->seqs[i]->bases[0] != 
       toupper(array_of_seqs->seqs[i]->bases[0])){
      for(j = 0; j < array_of_seqs->seqs[i]->length; j++){
	array_of_seqs->seqs[i]->bases[j] = 
	  toupper(array_of_seqs->seqs[i]->bases[j]);
      }
    }

    NEW(temp_name, char, strlen(array_of_seqs->seqs[i]->name) + 1);
    strcpy(temp_name,array_of_seqs->seqs[i]->name);
 
    /*Replace any space by "_" in the sequence name*/
    for(j = 0; j < strlen(array_of_seqs->seqs[i]->name); j++){
      
      if((array_of_seqs->seqs[i]->name[j] == ' ') ||
	 (array_of_seqs->seqs[i]->name[j] == '\t')){
	array_of_seqs->seqs[i]->name[j] = '_';
      }
    }

    DESTROY(temp_name);
         
  }
}


/**
 *
 * \fn void determine_pos_query(p_main_structure m, 
 *                              int index_interval, 
 *                              int index_al)
 *
 * \brief Function used to determine the positions on the query sequence
 * for the conserved region on the reference
 *
 * \param[in,out] m : the main structure containing information and that
 * will be updated
 * \param[in] index_interval : index of the reference interval in the array
 * of conserved region
 * \param[in] index_al : index of the alignment concerned 
 *
 * That function will determine the positions on the query sequence for the 
 * the conserved region on the reference. It points at the correct interval
 * by the index_interval parameter and the correct alignment by the index_al
 * parameter.
 */

void determine_pos_query(p_main_structure m, 
			 int index_interval, 
			 int index_al)
{

  /*Length of the interval on the reference sequence*/
  int length_interval_ref = 0;
  /*Length difference of the 2 starting positions*/
  int length_diff_start = 0;
  /*Length difference of the 2 ending positions*/
  int length_diff_end = 0;
  /*Max length of the query sequence : 50% gap max*/
  int max_length_query;
  /*Intersect percent*/
  float percent_intersect = 0.2;
  /*Length of the intersection*/
  int length_intersect = 0;

  /*Calculate the length of the interval and determine max length of the
   query sequence : 50%gap -> 2 * sizeofinterval */
  length_interval_ref = 
    m->interesting_regions->intervals[index_interval]->pos_end -
    m->interesting_regions->intervals[index_interval]->pos_start + 1;
  max_length_query = 2 * length_interval_ref;


  /*Calculate the length differences of the starting and ending 
    positions between the ref interval and the alignment*/
  length_diff_start = 
    m->interesting_regions->intervals[index_interval]->pos_start -
    m->array_of_al_info->al_info[index_al]->pos_start_ref;
  length_diff_end = 
    m->interesting_regions->intervals[index_interval]->pos_end -
    m->array_of_al_info->al_info[index_al]->pos_end_ref;

  /*First case : interesting regions is included inside alignments :
   positions won't change*/
  if((length_diff_start >= 0) && (length_diff_end <= 0)){

    RENEW(m->sets_of_sequences->sets[index_interval]->q_intervals,
	  p_interval_query,
	  m->sets_of_sequences->sets[index_interval]->nb_q_intervals + 1);
    NEW(m->sets_of_sequences->sets[index_interval]->
	q_intervals[m->sets_of_sequences->sets[index_interval]->nb_q_intervals],
	interval_query_s,
	1);
    m->sets_of_sequences->sets[index_interval]->
      q_intervals[m->sets_of_sequences->sets[index_interval]->nb_q_intervals]->
      pos_start = m->array_of_al_info->al_info[index_al]->pos_start_query;
    m->sets_of_sequences->sets[index_interval]->
      q_intervals[m->sets_of_sequences->sets[index_interval]->nb_q_intervals]->
      pos_end = m->array_of_al_info->al_info[index_al]->pos_end_query;
    m->sets_of_sequences->sets[index_interval]->ref_interval->strand =
      m->array_of_al_info->al_info[index_al]->strand_ref;
    
    m->sets_of_sequences->sets[index_interval]->
      q_intervals[m->sets_of_sequences->sets[index_interval]->nb_q_intervals]->
      index = m->array_of_al_info->al_info[index_al]->index_query;
    
    m->sets_of_sequences->sets[index_interval]->
      q_intervals[m->sets_of_sequences->sets[index_interval]->nb_q_intervals]->
      strand = m->array_of_al_info->al_info[index_al]->strand_query;

    m->sets_of_sequences->sets[index_interval]->nb_q_intervals++;
       
  }

  else{

    /*Case the alignment is included inside the ref interval :
      extend positions on query*/
    if((length_diff_start <= 0) && (length_diff_end >= 0)){


      RENEW(m->sets_of_sequences->sets[index_interval]->q_intervals,
	  p_interval_query,
	  m->sets_of_sequences->sets[index_interval]->nb_q_intervals + 1);
      NEW(m->sets_of_sequences->sets[index_interval]->
	  q_intervals[m->sets_of_sequences->sets[index_interval]->nb_q_intervals],
	  interval_query_s,
	  1);

      m->sets_of_sequences->sets[index_interval]->
	q_intervals[m->sets_of_sequences->sets[index_interval]->
		    nb_q_intervals]->pos_start = 
	MAX(0,
	    m->array_of_al_info->al_info[index_al]->pos_start_query -
	    (2 * fabs(length_diff_start)));
      m->sets_of_sequences->sets[index_interval]->
	q_intervals[m->sets_of_sequences->sets[index_interval]->
		    nb_q_intervals]->pos_end = 
	MIN(m->array_of_seqs->seqs[m->array_of_al_info->al_info[index_al]->
				   index_query]->length - 1,
	    m->array_of_al_info->al_info[index_al]->pos_end_query +
	    (2 * length_diff_end));
      
      m->sets_of_sequences->sets[index_interval]->ref_interval->strand =
	m->array_of_al_info->al_info[index_al]->strand_ref;
      
      m->sets_of_sequences->sets[index_interval]->
	q_intervals[m->sets_of_sequences->sets[index_interval]->nb_q_intervals]->
	index = m->array_of_al_info->al_info[index_al]->index_query;
      
      m->sets_of_sequences->sets[index_interval]->
	q_intervals[m->sets_of_sequences->sets[index_interval]->nb_q_intervals]->
	strand = m->array_of_al_info->al_info[index_al]->strand_query;
      m->sets_of_sequences->sets[index_interval]->nb_q_intervals++;
       
    }

    /*Last case : alignment and interesting region overlap : check
     if at least the overlapping represents the percent_intersect*/
    else{
      length_intersect = 
	get_length_ref_intersect(m->array_of_al_info->al_info[index_al], 
				 m->interesting_regions->intervals[index_interval]);

      
       if(length_intersect >=  
	  (percent_intersect *
	   (m->interesting_regions->intervals[index_interval]->pos_end -
	    m->interesting_regions->intervals[index_interval]->pos_start + 1))){

	 RENEW(m->sets_of_sequences->sets[index_interval]->q_intervals,
	       p_interval_query,
	       m->sets_of_sequences->sets[index_interval]->nb_q_intervals + 1);
	 NEW(m->sets_of_sequences->sets[index_interval]->
	     q_intervals[m->sets_of_sequences->sets[index_interval]->nb_q_intervals],
	     interval_query_s,
	     1);

	 if(length_diff_start <= 0){
	   m->sets_of_sequences->sets[index_interval]->
	     q_intervals[m->sets_of_sequences->sets[index_interval]->
			 nb_q_intervals]->pos_start = 
	     MAX(0,
		 m->array_of_al_info->al_info[index_al]->pos_start_query -
		 (2 * fabs(length_diff_start)));
	   m->sets_of_sequences->sets[index_interval]->
	     q_intervals[m->sets_of_sequences->sets[index_interval]->nb_q_intervals]->
	     pos_end = m->array_of_al_info->al_info[index_al]->pos_end_query; 
	   
	 }
	 
	 /*Last case : extending query on ending position*/
	 else{
	   m->sets_of_sequences->sets[index_interval]->
	     q_intervals[m->sets_of_sequences->sets[index_interval]->nb_q_intervals]->
	     pos_start = m->array_of_al_info->al_info[index_al]->pos_start_query;
	   m->sets_of_sequences->sets[index_interval]->
	     q_intervals[m->sets_of_sequences->sets[index_interval]->
			 nb_q_intervals]->pos_end = 
	  MIN(m->array_of_seqs->seqs[m->array_of_al_info->al_info[index_al]->
				     index_query]->length - 1,
	      m->array_of_al_info->al_info[index_al]->pos_end_query +
	      (2 * length_diff_end));	
	 }
	 
	 m->sets_of_sequences->sets[index_interval]->ref_interval->strand =
	   m->array_of_al_info->al_info[index_al]->strand_ref;
	 
	 m->sets_of_sequences->sets[index_interval]->
	   q_intervals[m->sets_of_sequences->sets[index_interval]->nb_q_intervals]->
	   index = m->array_of_al_info->al_info[index_al]->index_query;
	 
	 m->sets_of_sequences->sets[index_interval]->
	   q_intervals[m->sets_of_sequences->sets[index_interval]->nb_q_intervals]->
	   strand = m->array_of_al_info->al_info[index_al]->strand_query;
	 m->sets_of_sequences->sets[index_interval]->nb_q_intervals++;

       }

    }

  }

}



/**
 *
 * \fn void complete_set_sequences(p_main_structure m)
 *
 * \brief Function used to determine the sequences of the reference 
 * sequence and all query sequences of all sets
 *
 * \param[in,out] m : main structure containing all sets
 *
 * That function will determine the sequences of the reference 
 * sequence and all query sequences according to positions and strand on all
 * sets of sequences
 *
 */

void complete_set_sequences(p_main_structure m)
{

  /*Indexes*/
  int i,j,k;
  /*Temporary start position*/
  int temp_start;
  /*Temporary end position*/
  int temp_end;
  /*Temporary query index*/
  int temp_index;


  /*Get all sets of sequences*/
  for(i = 0; i < m->sets_of_sequences->nb_sets; i++){

    temp_start = m->sets_of_sequences->sets[i]->ref_interval->pos_start;
    temp_end = m->sets_of_sequences->sets[i]->ref_interval->pos_end;

    /*Determine the reference sequence*/
    NEW(m->sets_of_sequences->sets[i]->ref_seq,
	char,
	temp_end - temp_start + 2);    
    
    /*Forward strand case*/
    if(m->sets_of_sequences->sets[i]->ref_interval->strand == '+'){
      
      for(j = temp_start; j <= temp_end; j++){
	m->sets_of_sequences->sets[i]->ref_seq[j - temp_start] = 
	  m->array_of_seqs->seqs[0]->bases[j];
      }
      m->sets_of_sequences->sets[i]->ref_seq[j - temp_start] = '\0';
    }
    /*Reverse strand case*/
    else{
      for(j = temp_end; j >= temp_start; j--){
	m->sets_of_sequences->sets[i]->ref_seq[temp_end - j] =
	  get_complementary_base(m->array_of_seqs->seqs[0]->bases[j]);
      }
      m->sets_of_sequences->sets[i]->ref_seq[temp_end - j] = '\0';
      
    }

    /*Determine all query sequences of the set*/
    for(k = 0; k < m->sets_of_sequences->sets[i]->nb_q_intervals; k++){

      
      temp_start = m->sets_of_sequences->sets[i]->q_intervals[k]->pos_start;
      temp_end = m->sets_of_sequences->sets[i]->q_intervals[k]->pos_end;
      temp_index = m->sets_of_sequences->sets[i]->q_intervals[k]->index;

      NEW(m->sets_of_sequences->sets[i]->q_intervals[k]->seq,
	  char,
	  temp_end - temp_start + 2);
      
      /*Forward strand case*/
      if(m->sets_of_sequences->sets[i]->q_intervals[k]->strand == '+'){	

	for(j = temp_start; j <= temp_end; j++){
	  m->sets_of_sequences->sets[i]->q_intervals[k]->seq[j - temp_start] = 
	    m->array_of_seqs->seqs[temp_index]->bases[j];
	}
	m->sets_of_sequences->sets[i]->q_intervals[k]->seq[j - temp_start] = '\0';

      }
      /*Reverse strand case*/
      else{

	for(j = temp_end; j >= temp_start; j--){

	  m->sets_of_sequences->sets[i]->q_intervals[k]->seq[temp_end - j] =
	    get_complementary_base(m->array_of_seqs->seqs[temp_index]->bases[j]);

	}

	m->sets_of_sequences->sets[i]->q_intervals[k]->seq[temp_end - j] = '\0';
	
      }

      NEW(m->sets_of_sequences->sets[i]->q_intervals[k]->name,
	  char,
	  strlen(m->array_of_seqs->seqs[temp_index]->name) + 1);
      strcpy(m->sets_of_sequences->sets[i]->q_intervals[k]->name,
	     m->array_of_seqs->seqs[temp_index]->name);
      
    } 
  }
}


void eliminate_redundancy(p_main_structure m, int index_set){

  /*Indexes*/
  int i,j,k;
  /*Index of the current sequence*/
  int current_index = 0;
  /*Name of the current sequence*/
  char *current_name = NULL;
  /*Index of the compared sequence*/
  int compared_index = 0;
  /*New start position*/
  int new_pos_start = 0;
  /*New end position*/
  int new_pos_end = 0;

  /*Process all queries of the set*/
  for(i = 0; i < m->sets_of_sequences->sets[index_set]->nb_q_intervals; i++){

    /*Get sequence name*/
    current_index = 
      m->sets_of_sequences->sets[index_set]->q_intervals[i]->index;
    NEW(current_name,
	char,
	strlen(m->array_of_seqs->seqs[current_index]->name) + 1);
    strcpy(current_name,
	   m->array_of_seqs->seqs[current_index]->name);

    /*Check all queries that has the same name and overlap positions*/
    for(j = i + 1; j < m->sets_of_sequences->sets[index_set]->nb_q_intervals;
	j++){

      compared_index = 
	m->sets_of_sequences->sets[index_set]->q_intervals[j]->index;

      if((strcmp(current_name,
		 (m->array_of_seqs->seqs[compared_index]->name)) == 0) &&
	 (m->sets_of_sequences->sets[index_set]->q_intervals[i]->strand == 
	  m->sets_of_sequences->sets[index_set]->q_intervals[j]->strand)){

	/*Overlaping condition*/
	if(((m->sets_of_sequences->sets[index_set]->q_intervals[i]->pos_start <=
	     m->sets_of_sequences->sets[index_set]->q_intervals[j]->pos_end) &&
	    (m->sets_of_sequences->sets[index_set]->q_intervals[i]->pos_end >=
	     m->sets_of_sequences->sets[index_set]->q_intervals[j]->pos_start)) ||
	   ((m->sets_of_sequences->sets[index_set]->q_intervals[j]->pos_start <=
	     m->sets_of_sequences->sets[index_set]->q_intervals[i]->pos_end) &&
	    (m->sets_of_sequences->sets[index_set]->q_intervals[j]->pos_end >=
	     m->sets_of_sequences->sets[index_set]->q_intervals[i]->pos_start))){

	  /*Get new positions*/
	  new_pos_start = 
	    MIN(m->sets_of_sequences->sets[index_set]->q_intervals[i]->pos_start,
		m->sets_of_sequences->sets[index_set]->q_intervals[j]->pos_start);
	  new_pos_end = 
	    MAX(m->sets_of_sequences->sets[index_set]->q_intervals[i]->pos_end,
		m->sets_of_sequences->sets[index_set]->q_intervals[j]->pos_end);
	  
	  /*Update positions on interval i, delete interval j and recheck*/
	  m->sets_of_sequences->sets[index_set]->q_intervals[i]->pos_start =
	    new_pos_start;
	  m->sets_of_sequences->sets[index_set]->q_intervals[i]->pos_end =
	    new_pos_end;

	  DESTROY(m->sets_of_sequences->sets[index_set]->q_intervals[j]);
	  for(k = j;
	      k < m->sets_of_sequences->sets[index_set]->nb_q_intervals - 1;
	      k++){
	    NEW(m->sets_of_sequences->sets[index_set]->q_intervals[k],
		interval_query_s,
		1);
	    copy_interval_query(m->sets_of_sequences->sets[index_set]->q_intervals[k],
				m->sets_of_sequences->sets[index_set]->q_intervals[k + 1]);
	    DESTROY(m->sets_of_sequences->sets[index_set]->q_intervals[k + 1]);
	    
	  }
	  m->sets_of_sequences->sets[index_set]->nb_q_intervals--;
	  i = 0;
	  j = i+1;

	}
      }
    }
    DESTROY(current_name);
  }
}



/**
 *
 * \fn void get_sets_of_sequences(p_main_structure m)
 *
 * \brief Function used to determine the sets of sequences of conserved 
 * regions
 *
 * \param[in,out] m : the main structure containing information and that
 * will be updated with the sets
 *
 * That function will determine the positions on query sequences according
 * to the positions of conserved regions on the reference and the info
 * about alignments
 *
 */

void get_sets_of_sequences(p_main_structure m)
{

  /*Indexes*/
  int i,j;  
  /*Boolean that says if the current interval has been compared
   to all possible intervals in the array of alignment*/
  int comparison_done = 0;


  /*Initialize the array of sets of sequence : the number of regions*/
  NEW(m->sets_of_sequences->sets,
      p_set_of_seqs, 
      m->interesting_regions->nb_intervals);
  m->sets_of_sequences->nb_sets = m->interesting_regions->nb_intervals;

  /*Get all regions on the reference and search for sequences
   that intersect according to the array of alignments*/  
  for(i = 0; i < m->interesting_regions->nb_intervals; i++){

    comparison_done = 0;
    NEW(m->sets_of_sequences->sets[i],
	set_of_seqs_s,
	1);
    NEW(m->sets_of_sequences->sets[i]->ref_interval,
	interval_s,
	1);
    m->sets_of_sequences->sets[i]->q_intervals = NULL;
    m->sets_of_sequences->sets[i]->nb_q_intervals = 0;
    m->sets_of_sequences->sets[i]->ref_interval->pos_start =
      m->interesting_regions->intervals[i]->pos_start;
    m->sets_of_sequences->sets[i]->ref_interval->pos_end =
      m->interesting_regions->intervals[i]->pos_end;

    /*Search the alignments that intersects the current interval until
     comparison is done*/
    for(j = 0;
	(!comparison_done) &&
	  (j < m->array_of_al_info->nb_al_info);
	j++){


      /*Since the array of alignment info has been sorted : check if
       all the possible comparison has been done*/
      if(m->array_of_al_info->al_info[j]->pos_start_ref >
	 m->interesting_regions->intervals[i]->pos_end){
	comparison_done = 1;
      }
      else{
	/*Check if the current alignment intersects the actual one*/
	if(is_ref_intersect(m->array_of_al_info->al_info[j],
			    m->interesting_regions->intervals[i])){
	  
	  /*Determine positions on query*/
	  determine_pos_query(m, i, j);

	}	
      }      
    }
 
    /*Eliminate redundancy*/
    eliminate_redundancy(m,i);

 
  }

  /*Get reference sequence and all query sequences on all found sets*/
  complete_set_sequences(m);
}



/**
 *
 * \fn void sort_identity_ref(p_main_structure m)
 *
 * \brief Function used to sort all queries in each set by reference 
 * identity
 *
 * \param[in,out] m : the main structure containing information and that
 * will be updated
 *
 * Function used to sort all queries in each set by reference 
 * identity
 *
 */

void sort_identity_ref(p_main_structure m)
{

  /*Index*/
  int i;

  /*Get all sets of sequences*/
  for(i = 0; i < m->sets_of_sequences->nb_sets; i++){ 

    /*Sort all queries*/

    qsort(m->sets_of_sequences->sets[i]->q_intervals,
	  m->sets_of_sequences->sets[i]->nb_q_intervals,
	  sizeof(p_interval_query),
	  compare_sort_id);

  }
  
}



/**
 *
 * \fn void compare_all_against_ref(p_main_structure m)
 *
 * \brief Function used to compare all sequences against the reference
 * for all sets of sequences
 *
 * \param[in,out] m : the main structure containing information and that
 * will be updated
 *
 * That function will compare all sequences against the reference
 * for all sets of sequences and eliminate all queries that don't repsect
 * the identity thresholds (min and max)
 *
 */


void compare_all_against_ref(p_main_structure m)
{

  /*Indexes*/
  int i,j,k;
  /*Array of indexes of sequences in the current set to destroy*/
  int *to_destroy = NULL;
  /*Number of sequences to destroy*/
  int nb_to_destroy = 0;
  /*Current indes of the sequence to destroy*/
  int current_index = 0;
  /*Array of sequences for the carnac call*/
  p_sequence *seqs_carnac = NULL;
  /*Boolean that says if a sequence is good according to the identity
   thresholds and/or carnac*/
  int is_seq_good = 0;
  /*Boolean that says if a sequence is too close (ref OPTS_keep-too_close)*/
  int is_too_close = 0;
  /*Array of indexes of too close sequences*/
  int *too_close = NULL;
  /*Number of too close sequences*/
  int nb_too_close = 0;
  /*Number of sequences destroyed*/
  int nb_currently_destroyed = 0;

  /*Get all sets of sequences*/
  for(i = 0; i < m->sets_of_sequences->nb_sets; i++){

    if(to_destroy != NULL){
      DESTROY(to_destroy);
      nb_to_destroy = 0;
    }
    
    if(too_close != NULL){
      DESTROY(too_close);
      nb_too_close = 0;
    }


    if(OPTS_verbose){
      print_verbose_start_compare_ref_set(i + 1,
					  m->sets_of_sequences->
					  sets[i]->nb_q_intervals + 1);

    }

    /*Create an array of sequence with the reference sequence if
      carnac is wanted*/
    if(OPTS_carnac){

      if(seqs_carnac != NULL){
	
	if(seqs_carnac[1] != NULL){
	  DESTROY(seqs_carnac[1]->name);
	  DESTROY(seqs_carnac[1]->bases);
	  DESTROY(seqs_carnac[1]);

	}

	if(seqs_carnac[0] != NULL){
	  DESTROY(seqs_carnac[0]->name);
	  DESTROY(seqs_carnac[0]->bases);
	  DESTROY(seqs_carnac[0]);

	}    

	DESTROY(seqs_carnac);

      }

      
      NEW(seqs_carnac, p_sequence, 2);

      /*First sequence : the reference*/
      NEW(seqs_carnac[0], sequence_s, 1);
      NEW(seqs_carnac[0]->bases,
	  char,
	  strlen(m->sets_of_sequences->sets[i]->ref_seq) + 1);
      NEW(seqs_carnac[0]->name,
	  char,
	  4);
      strcpy(seqs_carnac[0]->bases,
	     m->sets_of_sequences->sets[i]->ref_seq);
      strcpy(seqs_carnac[0]->name, "ref");
      seqs_carnac[0]->length = 
	strlen(m->sets_of_sequences->sets[i]->ref_seq);

    }

    /*Get all queries and compare them with the reference*/
    for(j = 0; j < m->sets_of_sequences->sets[i]->nb_q_intervals; j++){

      is_seq_good = 1;
      is_too_close = 0;

      small_in_large(m->sets_of_sequences->sets[i]->ref_seq,
		     m->sets_of_sequences->sets[i]->q_intervals[j]);
     
      /*Check identity constraints*/
      if(m->sets_of_sequences->sets[i]->q_intervals[j]->id_ref < 
	 OPTS_min_id){
	is_seq_good = 0;

      }
      else{
	if((m->sets_of_sequences->sets[i]->q_intervals[j]->id_ref > 
	    OPTS_max_id) && (OPTS_keep_too_close == -1)){
	  is_seq_good = 0;
	}
	if((OPTS_keep_too_close != -1) && 
	   (m->sets_of_sequences->sets[i]->q_intervals[j]->id_ref > 
	    OPTS_keep_too_close)){
	  is_too_close = 1;
	}
      }

	/*Check carnac if wanted*/
      if((OPTS_carnac && is_seq_good && !is_too_close)){

	if(seqs_carnac[1] != NULL){

	  DESTROY(seqs_carnac[1]->name);
	  DESTROY(seqs_carnac[1]->bases);
	  DESTROY(seqs_carnac[1]);
	      
	}	  

	/*Create a temporary sequence for the current query*/
	NEW(seqs_carnac[1], sequence_s, 1);
	NEW(seqs_carnac[1]->bases, 
	    char, 
	    strlen(m->sets_of_sequences->sets[i]->q_intervals[j]->seq) + 1);
	NEW(seqs_carnac[1]->name,
	    char,
	    5);
	strcpy(seqs_carnac[1]->bases, 
	       m->sets_of_sequences->sets[i]->q_intervals[j]->seq);
	strcpy(seqs_carnac[1]->name, "temp");
	seqs_carnac[1]->length = 
	  strlen(m->sets_of_sequences->sets[i]->q_intervals[j]->seq);

	if(!call_carnac(seqs_carnac, 2)){
	  is_seq_good = 0;

	}


      }
      

      /*Destroy sequences that is not good*/
      if(!is_seq_good){

	RENEW(to_destroy, int, nb_to_destroy + 1);
	to_destroy[nb_to_destroy] = j;
	nb_to_destroy++;
      }
      /*Keep too close sequences option*/
      if((OPTS_keep_too_close != -1) && is_too_close){
	RENEW(too_close, int, nb_too_close + 1);
	too_close[nb_too_close] = j;
	nb_too_close++;
      }
      
    }

    /*Option too close region process*/
    if(OPTS_keep_too_close != -1){

      /*Check if all sequences are set too close*/
      if(m->sets_of_sequences->sets[i]->nb_q_intervals - nb_too_close == 0){

	m->sets_of_sequences->sets[i]->is_too_close = 1;
	nb_currently_destroyed = 0;

	for(j = 0; j < nb_to_destroy; j++){

	  current_index = to_destroy[j];

	  DESTROY(m->sets_of_sequences->sets[i]->q_intervals[current_index - nb_currently_destroyed]->seq);
	  DESTROY(m->sets_of_sequences->sets[i]->q_intervals[current_index - nb_currently_destroyed]);
	
	  for(k = current_index - nb_currently_destroyed;
	      k < m->sets_of_sequences->sets[i]->nb_q_intervals - 1 - nb_currently_destroyed;
	      k++){
	    NEW(m->sets_of_sequences->sets[i]->q_intervals[k],
		interval_query_s,
		1);
		
	    copy_interval_query(m->sets_of_sequences->sets[i]->q_intervals[k],
				m->sets_of_sequences->sets[i]->q_intervals[k + 1]);
	    
	    DESTROY(m->sets_of_sequences->sets[i]->q_intervals[k + 1]->seq);
	    DESTROY(m->sets_of_sequences->sets[i]->q_intervals[k + 1]);
		
	  }

	  nb_currently_destroyed++;
	    
	} 

	m->sets_of_sequences->sets[i]->nb_q_intervals -= nb_currently_destroyed;
	if(m->sets_of_sequences->sets[i]->nb_q_intervals != 0){
	  RENEW(m->sets_of_sequences->sets[i]->q_intervals,
		p_interval_query,
		m->sets_of_sequences->sets[i]->nb_q_intervals);

	}
	else{
	  DESTROY(m->sets_of_sequences->sets[i]->q_intervals);
	}
      }
      else{

	m->sets_of_sequences->sets[i]->is_too_close = 0;

	/*Destroy sequences that have been set to destroy*/
	for(j = 0; j < nb_to_destroy; j++){
	
	  current_index = to_destroy[j];
	
	  DESTROY(m->sets_of_sequences->sets[i]->q_intervals[current_index - j]->seq);
	  DESTROY(m->sets_of_sequences->sets[i]->q_intervals[current_index - j]);
	
	  for(k = current_index - j;
	      k < m->sets_of_sequences->sets[i]->nb_q_intervals - 1 - j;
	      k++){
	    NEW(m->sets_of_sequences->sets[i]->q_intervals[k],
		interval_query_s,
		1);
	    
	    copy_interval_query(m->sets_of_sequences->sets[i]->q_intervals[k],
				m->sets_of_sequences->sets[i]->q_intervals[k + 1]);
	    
	    DESTROY(m->sets_of_sequences->sets[i]->q_intervals[k + 1]->seq);
	    DESTROY(m->sets_of_sequences->sets[i]->q_intervals[k + 1]);
	  
	  }
	
	}


	m->sets_of_sequences->sets[i]->nb_q_intervals -= nb_to_destroy;
	if(m->sets_of_sequences->sets[i]->nb_q_intervals != 0){
	  RENEW(m->sets_of_sequences->sets[i]->q_intervals,
		p_interval_query,
		m->sets_of_sequences->sets[i]->nb_q_intervals);
	}
	else{
	  DESTROY(m->sets_of_sequences->sets[i]->q_intervals);
	}


      }

    }
    else{
  
      m->sets_of_sequences->sets[i]->is_too_close = 0;

      /*Destroy sequences that don't verify identity constraints with the 
	reference*/
      for(j = 0; j < nb_to_destroy; j++){
	
	current_index = to_destroy[j];
	
	DESTROY(m->sets_of_sequences->sets[i]->q_intervals[current_index - j]->seq);
	DESTROY(m->sets_of_sequences->sets[i]->q_intervals[current_index - j]);
	
	for(k = current_index - j;
	    k < m->sets_of_sequences->sets[i]->nb_q_intervals - 1 - j;
	    k++){
	  NEW(m->sets_of_sequences->sets[i]->q_intervals[k],
	      interval_query_s,
	      1);
	  
	  copy_interval_query(m->sets_of_sequences->sets[i]->q_intervals[k],
			      m->sets_of_sequences->sets[i]->q_intervals[k + 1]);
	  
	  DESTROY(m->sets_of_sequences->sets[i]->q_intervals[k + 1]->seq);
	  DESTROY(m->sets_of_sequences->sets[i]->q_intervals[k + 1]);
	  
	}
	
      }

    

      m->sets_of_sequences->sets[i]->nb_q_intervals -= nb_to_destroy;
      if(m->sets_of_sequences->sets[i]->nb_q_intervals != 0){
	RENEW(m->sets_of_sequences->sets[i]->q_intervals,
	      p_interval_query,
	      m->sets_of_sequences->sets[i]->nb_q_intervals);
      }
      else{
	DESTROY(m->sets_of_sequences->sets[i]->q_intervals);
      }
    }


    if(OPTS_verbose){
      print_verbose_end_compare_ref_set(m->sets_of_sequences->
					sets[i]->nb_q_intervals + 1,
					m->sets_of_sequences->sets[i]->is_too_close);
    }

  }

  if(to_destroy != NULL){
    DESTROY(to_destroy);
  }

  if(OPTS_carnac){

    if(seqs_carnac != NULL){
      
      if(seqs_carnac[1] != NULL){
	DESTROY(seqs_carnac[1]->name);
	DESTROY(seqs_carnac[1]->bases);
	DESTROY(seqs_carnac[1]);
	
      }
      
      if(seqs_carnac[0] != NULL){
	DESTROY(seqs_carnac[0]->name);
	DESTROY(seqs_carnac[0]->bases);
	DESTROY(seqs_carnac[0]);
	
	}    
      
      DESTROY(seqs_carnac);
      
    }
  }
  if(too_close != NULL){
    DESTROY(too_close);
  }
    
}



/**
 *
 * \fn void check_species_repetitions(p_main_structure m)
 *
 * \brief Function used to check species repetitions
 *
 * \param[in,out] m : the main structure containing information and that
 * will be updated
 *
 * That function will check if the species repetitions threshold is
 * respected
 *
 */

void check_species_repetitions(p_main_structure m)
{

  /*Indexes*/
  int i,j,k;
  /*Array of processed species*/
  int_array *processed_species = NULL;
  /*Index of the current sequence*/
  int current_index = 0;
  /*Species number of the current sequence*/
  int current_species = 0;
  /*Index of the compared sequence*/
  int compared_index = 0;
  /*Species number of the current compared sequence*/
  int compared_species = 0;
  /*Number of the current species*/
  int cpt_species = 0;
  /*Boolean that says if a species is good according to the max number
   of repetitions*/
  int is_species_good = 1;


  /*Initialize the array of processed species*/
  NEW(processed_species, int_array, 1);
  processed_species->integers = NULL;
  processed_species->nb_int = 0;

  /*Get all sets of sequences*/
  for(i = 0; i < m->sets_of_sequences->nb_sets; i++){
    if(OPTS_verbose){
      print_verbose_start_checking_repetitions_set(i + 1, 
					   m->sets_of_sequences->sets[i]->
					   nb_q_intervals + 1);
    }

    /*The set contains enough sequences to check the species*/
    if(m->sets_of_sequences->sets[i]->nb_q_intervals > OPTS_max_species_repet){

      if(processed_species->integers){
	DESTROY(processed_species->integers);
	processed_species->nb_int = 0;	
      }

      is_species_good = 1;

      /*Check all query species and stop if one doesn't respect the
       max number of species*/
      for(j = 0; is_species_good && 
	    j < m->sets_of_sequences->sets[i]->nb_q_intervals; j++){
	current_index = m->sets_of_sequences->sets[i]->q_intervals[j]->index;
	current_species = m->array_of_seqs->seqs[current_index]->num_species;

	cpt_species = 0;

	/*Check if the species has not been already processed*/
	if(!is_present_int_array(current_species,
				 processed_species->integers,
				 processed_species->nb_int)){
	  
	  /*Count the number of species*/
	  for(k = j + 1; is_species_good &&
		k < m->sets_of_sequences->sets[i]->nb_q_intervals; k++){
	    
	    compared_index = 
	      m->sets_of_sequences->sets[i]->q_intervals[k]->index;
	    compared_species = 
	      m->array_of_seqs->seqs[compared_index]->num_species;

	    if(compared_species == current_species){
	      cpt_species++;

	      /*If the maximum is reached stop*/
	      if(cpt_species > OPTS_max_species_repet){
		is_species_good = 0;
	      }

	    }
	      

	  }

	  /*Add the current species to the processed ones*/
	  if(is_species_good){
	    RENEW(processed_species->integers, 
		  int, 
		  processed_species->nb_int + 1);
	    processed_species->integers[processed_species->nb_int]= 
	      current_species;
	    processed_species->nb_int++;
	  }

	}

      }

      /*If the species is not good : destroy the queries to tell
       that this set is not good*/
      if(!is_species_good){
	for(k = 0; k < m->sets_of_sequences->sets[i]->nb_q_intervals; k++){
	  if(m->sets_of_sequences->sets[i]->q_intervals[k]->seq){
	    DESTROY(m->sets_of_sequences->sets[i]->q_intervals[k]->seq);
	  }
	  DESTROY(m->sets_of_sequences->sets[i]->q_intervals[k]);
	}

	DESTROY(m->sets_of_sequences->sets[i]->q_intervals);
	m->sets_of_sequences->sets[i]->nb_q_intervals = 0;
      }
    }


    if(OPTS_verbose){

      print_verbose_end_checking_repetitions_set(m->sets_of_sequences->
						 sets[i]->nb_q_intervals + 1);

    }
  }

  if(processed_species){
    if(processed_species->integers){
      DESTROY(processed_species->integers);
    }
    DESTROY(processed_species);
  }



}



/**
 *
 * \fn void get_indexes_diff_org(p_main_structure m,
 *                               int index_set, 
 *                               int_array *indexes)
 *
 * \brief Function used to get the indexes of different organisms present
 * in the provided set of sequences according to the OPTS_min_org_set value
 *
 * \param[in] m : the main structure
 * \param[in] index_set : the index of th set of sequences to check
 * \param[in,out] indexes : the indexes of different organisms present in 
 * the set according to the OPTS_min_org_set value
 *
 * That function will get the idnexes of different organisms present
 * in the provided set of sequences according to the OPTS_min_org_set value
 *
 */


void get_indexes_diff_org(p_main_structure m,
			  int index_set,
			  int_array *indexes)
{

  /*Index*/
  int i;
  /*Array of unique organisms found*/
  char_array *unique_org = NULL;
  /*Array of unique species number found*/
  int_array *unique_num_species = NULL;
  /*Current index of the organism in the sequnece array*/
  int current_index;
  /*Current name*/
  char * current_name;
  /*Current species number*/
  int current_num_species = 0;

  NEW(unique_org, char_array, 1);
  unique_org->chars = NULL;
  unique_org->nb_chars = 0;
  NEW(unique_num_species, int_array, 1);
  unique_num_species->integers = NULL;
  unique_num_species->nb_int = 0;

  /*Case minimum different organisms*/
  if(OPTS_min_org_set != 0){

    /*Etoile mode*/
    if(OPTS_selecting_mode == 0){

      /*Retrieve all queries until the OPTS_min_org_set is reached or 
	all queries have been processed*/
      for(i = 0; 
	  (unique_org->nb_chars < OPTS_min_org_set) && 
	    (i < m->sets_of_sequences->sets[index_set]->nb_q_intervals); 
	  i++){

	/*Get the organism index in the sequence array to retrieve its name*/
	current_index = 
	  m->sets_of_sequences->sets[index_set]->q_intervals[i]->index;
	current_name = 
	  m->array_of_seqs->seqs[current_index]->name;

	/*Check if that organism is present in the array of unique names : if false
	  add it and its index too in the corresponding arrays*/
	if(!(is_present_char_array(current_name, unique_org))){

	  RENEW(unique_org->chars, char*, unique_org->nb_chars + 1);
	  NEW(unique_org->chars[unique_org->nb_chars], 
	      char, 
	      strlen(current_name) + 1);
	  strcpy(unique_org->chars[unique_org->nb_chars], current_name);
	  unique_org->nb_chars++;

	  RENEW(indexes->integers, int, indexes->nb_int + 1);
	  indexes->integers[indexes->nb_int] = i;
	  indexes->nb_int++;

	}
      }


    }
    /*Clique mode*/
    else{
      /*Retrieve all queries until the OPTS_min_org_set is reached or 
	all queries have been processed*/
      for(i = 0; 
	  (unique_org->nb_chars < OPTS_min_org_set) && 
	    (i < m->sets_of_sequences->sets[index_set]->clique_set->nb_seqs_clique); 
	  i++){

	/*Get the organism index in the sequence array to retrieve its name*/
	current_index = 
	  m->sets_of_sequences->sets[index_set]->clique_set->clique_index[i];
	current_index = 
	  m->sets_of_sequences->sets[index_set]->q_intervals[current_index]->index;
	current_name = 
	  m->array_of_seqs->seqs[current_index]->name;

	/*Check if that organism is present in the array of unique names : 
	  if false add it and its index too in the corresponding arrays*/
	if(!(is_present_char_array(current_name, unique_org))){

	  RENEW(unique_org->chars, char*, unique_org->nb_chars + 1);
	  NEW(unique_org->chars[unique_org->nb_chars], 
	      char, 
	      strlen(current_name) + 1);
	  strcpy(unique_org->chars[unique_org->nb_chars], current_name);
	  unique_org->nb_chars++;

	  RENEW(indexes->integers, int, indexes->nb_int + 1);
	  indexes->integers[indexes->nb_int] = i;
	  indexes->nb_int++;

	}
      }

    }

  }
  
  /*Case minimum different species*/
  else{

    /*Etoile mode*/
    if(OPTS_selecting_mode == 0){

      /*Retrieve all queries until the OPTS_min_species_set is reached or 
	all queries have been processed*/
      for(i = 0; 
	  (unique_num_species->nb_int < OPTS_min_species_set) && 
	    (i < m->sets_of_sequences->sets[index_set]->nb_q_intervals); 
	  i++){

	/*Get the organism index in the sequence array to retrieve its species*/
	current_index = 
	  m->sets_of_sequences->sets[index_set]->q_intervals[i]->index;
	current_num_species = 
	  m->array_of_seqs->seqs[current_index]->num_species;

	/*Check if that species is present in the array of unique species : 
	  if false add it and its index too in the corresponding arrays*/
	if(!(is_present_int_array(current_num_species, 
				  unique_num_species->integers,
				  unique_num_species->nb_int))){

	  RENEW(unique_num_species->integers, 
		int, 
		unique_num_species->nb_int + 1);
	  unique_num_species->integers[unique_num_species->nb_int] = 
	    current_num_species;
	  unique_num_species->nb_int++;

	  RENEW(indexes->integers, int, indexes->nb_int + 1);
	  indexes->integers[indexes->nb_int] = i;
	  indexes->nb_int++;

	}
      }


    }
    /*Clique mode*/
    else{
      /*Retrieve all queries until the OPTS_min_org_species is reached or 
	all queries have been processed*/
      for(i = 0; 
	  (unique_num_species->nb_int < OPTS_min_species_set) && 
	    (i < m->sets_of_sequences->sets[index_set]->clique_set->nb_seqs_clique); 
	  i++){

	/*Get the organism index in the sequence array to retrieve its species*/
	current_index = 
	  m->sets_of_sequences->sets[index_set]->clique_set->clique_index[i];
	current_index = 
	  m->sets_of_sequences->sets[index_set]->q_intervals[current_index]->index;
	current_num_species = 
	  m->array_of_seqs->seqs[current_index]->num_species;
	
	/*Check if that species is present in the array of unique names : if false
	  add it and its index too in the corresponding arrays*/
	if(!(is_present_int_array(current_num_species, 
				  unique_num_species->integers,
				  unique_num_species->nb_int))){

	  RENEW(unique_num_species->integers, 
		int, 
		unique_num_species->nb_int + 1);
	  unique_num_species->integers[unique_num_species->nb_int]=
	    current_num_species;
	  unique_num_species->nb_int++;

	  RENEW(indexes->integers, int, indexes->nb_int + 1);
	  indexes->integers[indexes->nb_int] = i;
	  indexes->nb_int++;

	}
      }

    }

  }

  if(unique_org){
    if(unique_org->chars){
      for(i = 0; i < unique_org->nb_chars; i++){
	DESTROY(unique_org->chars[i]);
      }
      DESTROY(unique_org->chars);
    }
    DESTROY(unique_org);
  }

  if(unique_num_species){
    if(unique_num_species->integers){
      DESTROY(unique_num_species->integers);
    }
    DESTROY(unique_num_species);
  }


}
