/*
 * This file is part of CGseqcore.
 *
 *  CGseqcore is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  CGseqcore is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with CGseqcore.  If not, see <http://www.gnu.org/licenses/>.
 */


/**
 *
 * \file sequence.h
 *
 * \author Benjamin Grenier-Boley <benjamin.grenier-boley@inria.fr>
 *
 * \brief Header file that defines all functions related to sequences
 * or set of sequences
 *
 * \version 1.0
 *
 * \date July 2009
 *
 *
 * This file will define all fucntions related to sequences  or sets of
 * sequences 
 *
 *
 */


#ifndef __SEQUENCE_H__
#define __SEQUENCE_H__


/*Intern inclusion*/
#include "types.h"


/**
 *
 * \fn void get_sequences(sequence_array *array_of_seqs)
 *
 * \brief Function used to retrieve sequences contained in the fasta file
 *
 * \param[in,out] array_of_seqs : the array of sequences that will be
 * updated with sequences contained in the fasta file
 *
 *
 * This function will use the sequence parser to retrieve the sequences in
 * the fasta file
 *
 *
 */

extern void get_sequences(sequence_array *array_of_seqs);



/**
 *
 * \fn void destroy_sequence(p_sequence s)
 *
 * \brief Function used to destroy a p_sequence structure
 *
 * \param[in,out] s : the p_sequence to destroy
 *
 *
 * This function will destroy all allocated memory for a p_sequence 
 * structure
 *
 *
 */

extern void destroy_sequence(p_sequence s);



/**
 *
 * \fn int is_present_array_sequence(char *seq_name_to_find,
 *                                 sequence_array *array_of_seqs)
 *
 * \brief Function used to retrieve the index of a sequence according to
 * its name
 *
 * \param[in] seq_name_to_find : the name to search for
 * \param[in] array_of_seqs : the array of sequences
 *
 * \return the index in the sequence array or -1 if it wasn't found
 *
 * This function will search through the array of sequences for the name and 
 * return its idnex or -1 if it wasn't found
 *
 *
 */


extern int is_present_array_sequence(char *seq_name_to_find, 
				     sequence_array *array_of_seqs);




/**
 *
 * \fn void get_real_length_sequence(p_sequence seq)
 *
 * \brief Function used to calculate the real length of a sequence
 *
 * \param[in,out] seq : the reference sequence
 *
 * This function will calculate the real length of a sequence(all 
 * nucleotides except N)
 *
 *
 */


extern void get_real_length_sequence(p_sequence seq);



/**
 *
 * \fn char get_complementary_base(char nucleotide)
 *
 * \brief Function used to get the complementary nucleotide
 *
 * \param[in] nucleotide : the nucleotide to get its complementary
 *
 * \return the complementary nucleotide
 *
 * This function will return the complementary nucleotide of the one given
 *
 *
 */

extern char get_complementary_base(char nucleotide);


/**
 *
 * \fn void complete_sequences_attributes(sequence_array *array_of_seqs)
 *
 * \brief Function used to complete some missing attributes in the sequence
 * array
 *
 * \param[in,out] : array_of_seqs : the array containing all sequences
 * and that will be updated
 *
 * This function will complete the species attribute when possible, put in 
 * uppercase the sequence when needed and replace the spaces in the sequence
 * name by a "_"
 *
 */

extern void complete_sequences_attributes(sequence_array *array_of_seqs);


/**
 *
 * \fn void determine_pos_query(p_main_structure m, 
 *                              int index_interval, 
 *                              int index_al)
 *
 * \brief Function used to determine the positions on the query sequence
 * for the conserved region on the reference
 *
 * \param[in,out] m : the main structure containing information and that
 * will be updated
 * \param[in] index_interval : index of the reference interval in the array
 * of conserved region
 * \param[in] index_al : index of the alignment concerned 
 *
 * That function will determine the positions on the query sequence for the 
 * the conserved region on the reference. It points at the correct interval
 * by the index_interval parameter and the correct alignment by the index_al
 * parameter.
 *
 */
extern void determine_pos_query2(p_main_structure m, 
				int index_interval, 
				int index_aln);

extern void determine_pos_query(p_main_structure m, 
				int index_interval, 
				int index_aln);



/**
 *
 * \fn void complete_set_sequences(p_main_structure m)
 *
 * \brief Function used to determine the sequences of the reference 
 * sequence and all query sequences of all sets
 *
 * \param[in,out] m : main structure containing all sets
 *
 * That function will determine the sequences of the reference 
 * sequence and all query sequences according to positions and strand on all
 * sets of sequences
 *
 */


extern void complete_set_sequences(p_main_structure m);


/**
 *
 * \fn void get_sets_of_sequences(p_main_structure m)
 *
 * \brief Function used to determine the sets of sequences of conserved 
 * regions
 *
 * \param[in,out] m : the main structure containing information and that
 * will be updated with the sets
 *
 * That function will determine the positions on query sequences according
 * to the positions of conserved regions on the reference and the info
 * about alignments
 *
 */


extern void get_sets_of_sequences(p_main_structure m);



/**
 *
 * \fn void eliminate_redundancy_set(p_main_structure m, int index_set)
 *
 * \brief Function used to eliminate redundant sequences inside the 
 * current set
 *
 * \param[in,out] m : the main structure containing information and that
 * will be updated
 * \param[in] index_set : the index of the set to process
 *
 * That function will eliminate redundant sequences inside the set (same
 * species and overlapped positions) 
 *
 */
extern void eliminate_redundancy(p_main_structure m, int index_set);

extern void eliminate_redundancy_set(p_main_structure m, int index_set);


/**
 *
 * \fn void sort_identity_ref(p_main_structure m)
 *
 * \brief Function used to sort all queries in each set by reference 
 * identity
 *
 * \param[in,out] m : the main structure containing information and that
 * will be updated
 *
 * Function used to sort all queries in each set by reference 
 * identity
 *
 */


extern void sort_identity_ref(p_main_structure m);



/**
 *
 * \fn void compare_all_against_ref(p_main_structure m)
 *
 * \brief Function used to compare all sequences against the reference
 * for all sets of sequences
 *
 * \param[in,out] m : the main structure containing information and that
 * will be updated
 *
 * That function will compare all sequences against the reference
 * for all sets of sequences and eliminate all queries that don't repsect
 * the identity thresholds (min and max)
 *
 */


extern void compare_all_against_ref(p_main_structure m);



/**
 *
 * \fn void check_species_repetitions(p_main_structure m)
 *
 * \brief Function used to check species repetitions
 *
 * \param[in,out] m : the main structure containing information and that
 * will be updated
 *
 * That function will check if the species repetitions threshold is
 * respected
 *
 */

extern void check_species_repetitions(p_main_structure m);




/**
 *
 * \fn void get_indexes_diff_org(p_main_structure m,
 *                               int index_set, 
 *                               int_array *indexes)
 *
 * \brief Function used to get the indexes of different organisms present
 * in the provided set of sequences according to the OPTS_min_org_set value
 *
 * \param[in] m : the main structure
 * \param[in] index_set : the index of th set of sequences to check
 * \param[in,out] indexes : the indexes of different organisms present in 
 * the set according to the OPTS_min_org_set value
 *
 * That function will get the idnexes of different organisms present
 * in the provided set of sequences according to the OPTS_min_org_set value
 *
 */

extern void get_indexes_diff_org(p_main_structure m, 
				 int index_set,
				 int_array *indexes);


#endif
