/*
 * This file is part of CGseqcore.
 *
 *  CGseqcore is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  CGseqcore is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with CGseqcore.  If not, see <http://www.gnu.org/licenses/>.
 */


/**
 *
 * \file sequence_interface.h
 *
 * \author Benjamin Grenier-Boley <benjamin.grenier-boley@inria.fr>
 *
 * \brief Header file for the interface between the Flex/Bison parser
 * and this software
 *
 * \version 1.0
 *
 * \date July 2009
 *
 *
 * This file will define all functions and defines that will be used
 * to communicate with the Flex/Bison fasta parser 
 *
 *
 */


#ifndef __SEQUENCE_INTERFACE_H__
#define __SEQUENCE_INTERFACE_H__

/*Basic inclusions*/
#include <stdio.h>

/*Intern inclusions*/
#include "buffer.h"
#include "types.h"

/******************************************************************************
 **   Define local variables
 **     
 *****************************************************************************/


/**
 *
 * \def INPUT_BUFFER
 *
 * \brief Defines the buffer size
 *
 * This definition defines the buffer size that will be used by the parser
 *
 */


#define INPUT_BUFFER 20000


/**
 *
 * \def sequencetext
 *
 * \brief Defines a text interface
 *
 * This definition defines the text interface that will be used by the 
 * parser
 *
 */


#define yytext sequencetext


/******************************************************************************
 **   Displays warning and increase the current number of warnings
 **     
 *****************************************************************************/

/**
 *
 * \def sequencewarning(s)
 *
 * \brief Defines a function to display flex warnings
 *
 * \param[in] s : the warning message
 *
 * This function will be used to display flex warning messages by calling
 * the sequencewarning function
 *
 */


#define sequencewarning(s) (sequencewarning_token(s, yytext))


/******************************************************************************
 **   Displays error and increase the current number of errors
 **     
 *****************************************************************************/

/**
 *
 * \def sequenceerror(s)
 *
 * \brief Defines a function to display flex error
 *
 * \param[in] s : the error message
 *
 * This function will be used to display flex error messages by calling the
 * sequenceerror function
 *
 */


#define sequenceerror(s) (sequenceerror_token(s, yytext))




/******************************************************************************
 **   Function that mean "start processing a new file"
 **     
 *****************************************************************************/

/**
 *
 * \fn sequencerestart(FILE *f)
 *
 * \brief Function used to start processing a file
 *
 * \param[in] f : the file to process
 *
 * This function will be used to start processing a file by the parser
 *
 */


extern void sequencerestart(FILE *f);


/******************************************************************************
 **   Function that parse a sequence
 **     
 *****************************************************************************/

/**
 *
 * \fn sequenceparse()
 *
 * \brief Function used to start parsing a sequence
 *
 * \return an integer representing the parsing status
 *
 * This function will be used to start parsing a sequence by the parser
 *
 */


extern int sequenceparse();


/**
 *
 * \fn sequencelex()
 *
 * \brief Function used to start parsing a sequence
 *
 * This function will be used to start parsing a sequence by the parser
 *
 */

extern int sequencelex();

/******************************************************************************
 **   Sequence attributes used by the parser 
 **     
 *****************************************************************************/

extern int sequencelineno;         /*!< Line number*/
extern char* sequencetext;         /*!< Sequence nucleotides*/
extern FILE *sequencein;           /*!< The file desriptor*/
extern char *sequence_file_name;   /*!< The file to read*/
extern int sequence_nb_errors;     /*!< Numbers of parser's error*/
extern p_sequence* sequence_result;/*!< The result of the parser: the sequences*/


/******************************************************************************
 **   Convenient warnings
 **     
 *****************************************************************************/


/**
 *
 * \def sequencewarning_token(const char *s, const char *token)
 *
 * \brief Function used to display flex warnings
 *
 * \param[in] s : the warning message
 * \param[in] token : the token that has raised the warning
 *
 * This function will be used to display flex warnings
 *
 */


extern void sequencewarning_token(const char *s, const char *token);


/******************************************************************************
 **   Convenient errors
 **     
 *****************************************************************************/

/**
 *
 * \fn void sequenceerror_token(const char *s, const char *token)
 *
 * \brief Function used to display flex errors
 *
 * \param[in] s : the error message
 * \param[in] token : the token that has raised the error
 *
 * This function will be used to display flex errors
 *
 */


extern void sequenceerror_token(const char *s, const char *token);



/******************************************************************************
 **   Free flex memory
 **     
 *****************************************************************************/

/**
 *
 * \fn free_flex_mem()
 *
 * \brief Function used to free all memory allocated by the parser
 *
 * This function will be used to free all pointers declared by the parser
 * that have not been destroyed
 *
 */

extern void free_flex_mem();



#endif
