/*
 * This file is part of CGseqcore.
 *
 *  CGseqcore is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  CGseqcore is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with CGseqcore.  If not, see <http://www.gnu.org/licenses/>.
 */


/**
 *
 * \file sequence_parser.y
 *
 * \author Benjamin Grenier-Boley <benjamin.grenier-boley@inria.fr>
 *
 * \brief Bison file (yacc) that will use the lexer to parse the fasta file
 *
 * \version 1.0
 *
 * \date July 2009
 *
 *
 * This file will use the lexer grammar to parse the fasta file
 *
 */

%{
  
#include "sequence_interface.h"
#include "types.h"
#include "macros.h"
#include "io.h"  

  
  char *sequence_file_name;    /*!< The file to parse*/
  int sequence_nb_errors;      /*!< The number of errors*/
  p_sequence* sequence_result; /*!< The result of the parsing : the sequences*/
  
  
  
%}

%union {
  char car;                 /*!< Char read*/   
  p_sequence* sequences;    /*!< The array of sequences*/
  p_sequence sequence;      /*!< A sequence structure*/
  char *text;               /*!< Represents the parsing text*/
  int number;               /*!< Represents a number*/
  p_sequence_buffer buffer; /*!< The sequence buffer*/
}

%token <text> FASTA_TITLE_T
%token <text> SEQUENCE_ELEMENT_T
%token <text> FASTA_NAME_T


%type <buffer> sequence

%type <sequences> sequences multifasta
%type <sequence> fasta

%start sequences

%%

sequences:
multifasta
{ sequence_result = $1;}
;


multifasta:
multifasta
fasta
{
   p_sequence* res = $1;
   int i = 0;
   
   while(res[i]) i++;
   
   RENEW(res,p_sequence, (i + 2));
   res[i] = $2;
   res[i+1] = NULL;
   $$ = res;
}
|
fasta
{
   p_sequence* res = NULL;

   NEW(res,p_sequence,2);

   res[0] = $1;
   res[1] = NULL;
   $$ = res;
}
;

fasta:
FASTA_TITLE_T
FASTA_NAME_T
sequence
{
   p_sequence_buffer seq = $3;
   p_sequence res;

   NEW(res, sequence_s, 1);
   res->name = $2;

   BUFFER_RELEASE(seq);
   res->length = BUFFER_SIZE(seq);

   NEW(res->bases, char, (1+res->length));
   strncpy(res->bases, BUFFER_TOSTRING(seq), res->length);
   res->bases[res->length] = '\0';

   DESTROY_BUFFER(seq);

   $$ = res;
}
|
FASTA_TITLE_T
sequence
{
   p_sequence_buffer seq = $2;
   p_sequence res;

   NEW(res, sequence_s, 1);
   NEW(res->name, char, 8);
   strcpy(res->name,"unknown");

   BUFFER_RELEASE(seq);
   res->length = BUFFER_SIZE(seq);

   NEW(res->bases, char, (1+res->length));
   strncpy(res->bases, BUFFER_TOSTRING(seq), res->length);
   res->bases[res->length] = '\0';

   DESTROY_BUFFER(seq);

   $$ = res;
}
;


sequence:
sequence
SEQUENCE_ELEMENT_T
{
   p_sequence_buffer res = $1;
   register int i;
   int length;
   char *txt = $2;

   length = strlen(txt);

   if(BUFFER_MAXSIZE(res) < (BUFFER_SIZE(res) + length)) {
      BUFFER_RESIZE(res, (BUFFER_SIZE(res) + length + INPUT_BUFFER));
   }

   for(i = 0; i < length; i++) {
      BUFFER_PUT(res, txt[i]);
   }
   
   DESTROY(txt);

   $$ = res;
 
}
|
SEQUENCE_ELEMENT_T
{
   p_sequence_buffer res;
   register int i;
   int length;
   char *txt = $1;

   length = strlen(txt);
   NEW_BUFFER(res, (length + INPUT_BUFFER));
   for(i = 0; i < length; i++) {
      BUFFER_PUT(res, txt[i]);
   }

   DESTROY(txt);

   $$ = res;
}
;


%%


void
sequencewarning_token(const char *s, const char *token) {
   ERROR_ "'%s' line '%i' token '%s': %s\n", 
     sequence_file_name, 
     sequencelineno, 
     token, 
     s _ERROR;
}

void
sequenceerror_token(const char *s, const char *token) {
   sequence_nb_errors++;
   ERROR_ "'%s' line '%i' token '%s': %s\n", 
     sequence_file_name, 
     sequencelineno, 
     token, 
     s _ERROR;
}
   

