/*
 * This file is part of CGseqcore.
 *
 *  CGseqcore is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  CGseqcore is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with CGseqcore.  If not, see <http://www.gnu.org/licenses/>.
 */


/**
 *
 * \file types.h
 *
 * \author Benjamin Grenier-Boley <benjamin.grenier-boley@inria.fr>
 *
 * \brief Header file that defines all structures used in this software
 *
 * \version 1.0
 *
 * \date July 2009
 *
 *
 * This file will define all structures and pointers on theses that will be 
 * used in the software
 *
 *
 */


#ifndef __TYPES_H__
#define __TYPES_H__



/******************************************************************************
 **   Structure that will implement an array of char (e.g: used to save all 
 **   alignments files),array of integers and array of float
 **     
 *****************************************************************************/

/**
 *
 * \struct char_array
 *
 * \brief Array of chars
 *
 * That structure will define an array of chars
 *
 */

typedef struct
{
  char **chars; /*!< The array of chars*/

  int nb_chars; /*!< The number of chars in the array*/

} char_array;



/**
 *
 * \struct int_array
 *
 * \brief Array of integers
 *
 * That structure will define an array of integers
 *
 */

typedef struct
{
  int *integers; /*!< The array of integers*/

  int nb_int;    /*!< The number of integers in the array*/

} int_array;


/**
 *
 * \struct float_array
 *
 * \brief Array of floats
 *
 * That structure will define an array of floats
 *
 */

typedef struct
{
  float *floats;  /*!< The array of floats*/

  int nb_float;   /*!< The number of floats in the array*/

} float_array;


/******************************************************************************
 **   Structure that will implement a sequence and its array representation
 **     
 *****************************************************************************/


/**
 *
 * \struct sequence_s
 *
 * \brief Sequence structure implementation
 *
 * That structure will define all information needed on sequences
 *
 */


typedef struct 
{
  char *name;        /*!< The name of the sequence*/

  char *bases;       /*!< The nucleotides*/

  int length;        /*!< The length of the sequence*/

  int real_length;   /*!< The real length (without N)*/

  int num_species;   /*!< The number of the species*/

  int from;          /*!< The start position of the sequence on the genome*/

  int to;            /*!< The ending position of the sequence on the genome*/

  char strand;       /*!< The strand where is located the sequence*/

} sequence_s;



/**
 *
 * \struct p_sequence
 *
 * \brief Pointer on a sequence_s
 *
 * That structure will define the pointer on the sequence structure
 *
 */


typedef sequence_s *p_sequence;



/**
 *
 * \struct sequence_array
 *
 * \brief Array of sequences structure
 *
 * That structure will define an array of sequences
 *
 */

typedef struct
{
  p_sequence *seqs; /*!< The array of sequences*/

  int nb_seqs;      /*!< The number of sequences contained in the array*/

} sequence_array;




/******************************************************************************
 **   Structure that will implement a structure that will contain information
 **  about an alignment and its array representation
 **     
 *****************************************************************************/

/**
 *
 * \struct alignment_info_s
 *
 * \brief Structure that implements an alignment
 *
 * That structure will implement a structure that will contain all
 * information about an alignment extracted from the alignment files
 *
 */


typedef struct
{
  char *name_ref;       /*!< Name of the reference sequence*/

  int index_ref;        /*!< Index of the reference sequence in the sequence array*/

  char *name_query;     /*!< Name of the query sequence*/

  int index_query;      /*!< Index of the query sequence in the sequence array*/

  float identity;       /*!< Identity of the alignment*/

  int al_length;        /*!< Length of the alignment*/

  int nb_mismatches;    /*!< Number of mismatches*/

  int nb_gaps_open;     /*!< Number of gap openings*/

  int pos_start_ref;    /*!< Starting position of match in the ref sequence*/
  
  int pos_end_ref;      /*!< Starting position of match in the ref sequence*/
 
  int pos_start_query;  /*!< Starting position of match in the query sequence*/
  
  int pos_end_query;    /*!< Starting position of match in the query sequence*/
  
  float evalue;         /*!< Evalue of the alignment*/

  float bit_score;      /*!< Bit score of the alignment*/
  
  char strand_ref;      /*!< Strand of the alignment on the ref*/

  char strand_query;    /*!< Strand of the alignment on the query*/
  
}alignment_info_s;


/**
 *
 * \struct p_alignment_info
 *
 * \brief Pointer on a alignment_info_s
 *
 * That structure will define the pointer on the alignment info structure
 *
 */


typedef alignment_info_s *p_alignment_info;



/**
 *
 * \struct alignment_info_array
 *
 * \brief Array of alignments information
 *
 * That structure will define an array of all information about alignments
 *
 */

typedef struct
{
  p_alignment_info *al_info; /*!< The array of alignment information*/

  int nb_al_info;            /*!< The number of information extracted*/

} alignment_info_array;



/******************************************************************************
 **   Structure that will implement an array representation of scores
 **   calculated by the score function
 **     
 *****************************************************************************/


/**
 *
 * \struct score_array
 *
 * \brief Structure that implements the array of scores
 *
 * That structure will define the structure containing all info about the
 * score function, the score by position etc.
 *
 */

typedef struct{

  float *target;        /*!< The target array that will be calculated for every possibilities according to the OPTS_proba values*/

  float *background;    /*!< The background array that will be calculated for every possibilities according to the background probabilities calculated with alignments*/

  float *genomes_proba; /*!< The probabilities of every genomes to be present according to the alignments*/

  int length_tb;        /*!< The length of the target and background arrays : 2^x (x=number of genomes minus the reference)*/

  int nb_other_genomes; /*!< The number of genomes from other species (all except reference)*/

  int length_ps;        /*!< The length of the positions and scores arrays (= length of the reference sequence)*/

  char **positions;     /*<! The array containing the presence/absence of each genome on the format for example : 1001 (for 4 genomes)*/

  float *scores;        /*!< The array of scores by position*/
		    

}score_array;


  

/******************************************************************************
 **   Structure that will implement an interval (region) and its array 
 **   representation
 **     
 *****************************************************************************/

/**
 *
 * \struct interval_s
 *
 * \brief Structure that implements an interval
 *
 * That structure will implement an interval represented by its starting
 * position and its ending position
 *
 */

typedef struct{

  int pos_start; /*!< Starting position of the interval*/

  int pos_end;   /*!< Ending position of the interval*/

  char strand;   /*!< Strand of the interval*/

}interval_s;


/**
 *
 * \struct p_interval
 *
 * \brief Pointer on an interval_s
 *
 * That structure will define the pointer on an interval
 *
 */

typedef interval_s *p_interval;


/**
 *
 * \struct interval_array
 *
 * \brief Array of intervals
 *
 * That structure will define an array of intervals
 *
 */

typedef struct{

  p_interval *intervals; /*!< The array of intervals*/

  int nb_intervals;      /*!< Number of intervals*/

}interval_array;


/******************************************************************************
 **   Structure that will implement an interval (region) and its pointer 
 **   representation on query sequences
 **     
 *****************************************************************************/

/**
 *
 * \struct interval_query_s
 *
 * \brief Structure that implements an interval on a query
 *
 * That structure will implement an interval represented by its starting
 * position and its ending position, its index in the array of sequences
 * and the strand concerned. It also has an attribute representing its 
 * identity wit hthe reference 
 *
 */

typedef struct{

  int pos_start; /*!< Starting position of the interval*/

  int pos_end;   /*!< Ending position of the interval*/

  char *seq;     /*!< The corresponding sequence*/

  int index;     /*!< Index of the query sequence in the array of sequences*/

  char strand;   /*!< Strand concerned by the alignment*/

  float id_ref;  /*!< Identity with the reference*/

  char *name;

}interval_query_s;


/**
 *
 * \struct p_interval_query
 *
 * \brief Pointer on an interval_query_s
 *
 * That structure will define the pointer on an interval of query
 *
 */

typedef interval_query_s *p_interval_query;



/******************************************************************************
 **   Structure that will implement a clique and its pointer representation
 **     
 *****************************************************************************/

/**
 *
 * \struct clique_s
 *
 * \brief Structure that implements a clique
 *
 * That structure will implement a clique represented by an array of 
 * indexes representing the sequence indexes and a number of sequences
 *
 */

typedef struct{

  int *clique_index;  /*!< The array of indexes of sequences in the clique*/

  int nb_seqs_clique; /*!< The number of sequences in the clique*/

}clique_s;


/**
 *
 * \struct p_clique
 *
 * \brief Pointer on a clique_s
 *
 * That structure will define the pointer on a clique
 *
 */

typedef clique_s *p_clique;


/******************************************************************************
 **   Structure that will implement an array of neighbors and its pointer 
 **   representation
 **     
 *****************************************************************************/

/**
 *
 * \struct neighbor_s
 *
 * \brief Structure that implements an array of neighbors
 *
 * That structure will implement a clique represented by an array of 
 * indexes representing the sequence indexes and a number of sequences
 *
 */

typedef struct{

  int *indexes;       /*!< The array of indexes of neighbors sequences*/

  int nb_neighbors;   /*!< The number of neighbors*/

}neighbor_s;

/**
 *
 * \struct p_neighbor
 *
 * \brief Pointer on a neighbor_s
 *
 * That structure will define the pointer on a clique
 *
 */

typedef neighbor_s *p_neighbor;


/******************************************************************************
 **   Structure that will implement a set of sequences, its pointer and its 
 **   array representation
 **     
 *****************************************************************************/

/**
 *
 * \struct set_of_seqs_s
 *
 * \brief Structure that implements a set of sequences
 *
 * That structure will implement a set of sequences : an interval on the
 * reference and intervals on queries, a reference sequence and a clique
 * representation  
 *
 */

typedef struct{

  p_interval ref_interval;       /*!< Interval on the reference sequence*/

  char *ref_seq;                 /*!< The reference sequence*/

  p_interval_query *q_intervals; /*!< Array of intervals on queries*/

  int nb_q_intervals;            /*!< Number of query intervals*/

  p_clique clique_set;           /*!< Clique in the set*/

  p_neighbor *neighbors;         /*!< The array of neighbors : sequences
				   that are connected*/

  int is_too_close;              /*!< Boolean that says if the set contains
                                   only too close sequences according to the 
                                   max identity (and if OPTS_keep_too_close)
                                   is wanted*/


}set_of_seqs_s;


/**
 *
 * \struct p_set_of_seqs
 *
 * \brief Pointer on an set_of_seqs_s
 *
 * That structure will define the pointer on a set of sequences
 *
 */

typedef set_of_seqs_s *p_set_of_seqs;


/**
 *
 * \struct set_of_seqs_array
 *
 * \brief Array of sets of sequences
 *
 * That structure will define an array of sets of sequences
 *
 */

typedef struct{

  p_set_of_seqs *sets; /*!< The array of sets of sequences*/

  int nb_sets;         /*!< Number of sets*/

}sets_of_seqs_array;




/******************************************************************************
 **   Structure that will implement the main structure of the software and its
 **   pointer
 **     
 *****************************************************************************/

/**
 *
 * \struct main_structure_s
 *
 * \brief Main structure of the software
 *
 * That structure will define the main software structure that will contain
 * all others structures that will be used in the software (sequences, 
 * species, comparisons etc.)
 *
 */

typedef struct
{
  sequence_array *array_of_seqs;          /*!< The array of sequences*/

  alignment_info_array *array_of_al_info; /*!< The array of alignments information*/

  score_array *array_of_scores;           /*!< The array of scores used to retrieve interesting regions*/

  interval_array *interesting_regions;    /*!< Interesting regions on the reference sequence */

  sets_of_seqs_array *sets_of_sequences;  /*!< Sets of sequences of conserved regions*/

} main_structure_s;


/**
 *
 * \struct p_main_structure
 *
 * \brief Pointer on a main_structure_s
 *
 * That structure will define the pointer on the main software structure
 *
 */

typedef main_structure_s *p_main_structure;




#endif
